/* Generated by Cython 0.23.4 */

/* BEGIN: Cython Metadata
{
    "distutils": {
        "depends": []
    }
}
END: Cython Metadata */

#define PY_SSIZE_T_CLEAN
#include "Python.h"
#ifndef Py_PYTHON_H
    #error Python headers needed to compile C extensions, please install development version of Python.
#elif PY_VERSION_HEX < 0x02060000 || (0x03000000 <= PY_VERSION_HEX && PY_VERSION_HEX < 0x03020000)
    #error Cython requires Python 2.6+ or Python 3.2+.
#else
#define CYTHON_ABI "0_23_4"
#include <stddef.h>
#ifndef offsetof
#define offsetof(type, member) ( (size_t) & ((type*)0) -> member )
#endif
#if !defined(WIN32) && !defined(MS_WINDOWS)
  #ifndef __stdcall
    #define __stdcall
  #endif
  #ifndef __cdecl
    #define __cdecl
  #endif
  #ifndef __fastcall
    #define __fastcall
  #endif
#endif
#ifndef DL_IMPORT
  #define DL_IMPORT(t) t
#endif
#ifndef DL_EXPORT
  #define DL_EXPORT(t) t
#endif
#ifndef PY_LONG_LONG
  #define PY_LONG_LONG LONG_LONG
#endif
#ifndef Py_HUGE_VAL
  #define Py_HUGE_VAL HUGE_VAL
#endif
#ifdef PYPY_VERSION
#define CYTHON_COMPILING_IN_PYPY 1
#define CYTHON_COMPILING_IN_CPYTHON 0
#else
#define CYTHON_COMPILING_IN_PYPY 0
#define CYTHON_COMPILING_IN_CPYTHON 1
#endif
#if !defined(CYTHON_USE_PYLONG_INTERNALS) && CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x02070000
#define CYTHON_USE_PYLONG_INTERNALS 1
#endif
#if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX < 0x02070600 && !defined(Py_OptimizeFlag)
#define Py_OptimizeFlag 0
#endif
#define __PYX_BUILD_PY_SSIZE_T "n"
#define CYTHON_FORMAT_SSIZE_T "z"
#if PY_MAJOR_VERSION < 3
  #define __Pyx_BUILTIN_MODULE_NAME "__builtin__"
  #define __Pyx_PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)\
          PyCode_New(a+k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
  #define __Pyx_DefaultClassType PyClass_Type
#else
  #define __Pyx_BUILTIN_MODULE_NAME "builtins"
  #define __Pyx_PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)\
          PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
  #define __Pyx_DefaultClassType PyType_Type
#endif
#ifndef Py_TPFLAGS_CHECKTYPES
  #define Py_TPFLAGS_CHECKTYPES 0
#endif
#ifndef Py_TPFLAGS_HAVE_INDEX
  #define Py_TPFLAGS_HAVE_INDEX 0
#endif
#ifndef Py_TPFLAGS_HAVE_NEWBUFFER
  #define Py_TPFLAGS_HAVE_NEWBUFFER 0
#endif
#ifndef Py_TPFLAGS_HAVE_FINALIZE
  #define Py_TPFLAGS_HAVE_FINALIZE 0
#endif
#if PY_VERSION_HEX > 0x03030000 && defined(PyUnicode_KIND)
  #define CYTHON_PEP393_ENABLED 1
  #define __Pyx_PyUnicode_READY(op)       (likely(PyUnicode_IS_READY(op)) ?\
                                              0 : _PyUnicode_Ready((PyObject *)(op)))
  #define __Pyx_PyUnicode_GET_LENGTH(u)   PyUnicode_GET_LENGTH(u)
  #define __Pyx_PyUnicode_READ_CHAR(u, i) PyUnicode_READ_CHAR(u, i)
  #define __Pyx_PyUnicode_KIND(u)         PyUnicode_KIND(u)
  #define __Pyx_PyUnicode_DATA(u)         PyUnicode_DATA(u)
  #define __Pyx_PyUnicode_READ(k, d, i)   PyUnicode_READ(k, d, i)
#else
  #define CYTHON_PEP393_ENABLED 0
  #define __Pyx_PyUnicode_READY(op)       (0)
  #define __Pyx_PyUnicode_GET_LENGTH(u)   PyUnicode_GET_SIZE(u)
  #define __Pyx_PyUnicode_READ_CHAR(u, i) ((Py_UCS4)(PyUnicode_AS_UNICODE(u)[i]))
  #define __Pyx_PyUnicode_KIND(u)         (sizeof(Py_UNICODE))
  #define __Pyx_PyUnicode_DATA(u)         ((void*)PyUnicode_AS_UNICODE(u))
  #define __Pyx_PyUnicode_READ(k, d, i)   ((void)(k), (Py_UCS4)(((Py_UNICODE*)d)[i]))
#endif
#if CYTHON_COMPILING_IN_PYPY
  #define __Pyx_PyUnicode_Concat(a, b)      PyNumber_Add(a, b)
  #define __Pyx_PyUnicode_ConcatSafe(a, b)  PyNumber_Add(a, b)
#else
  #define __Pyx_PyUnicode_Concat(a, b)      PyUnicode_Concat(a, b)
  #define __Pyx_PyUnicode_ConcatSafe(a, b)  ((unlikely((a) == Py_None) || unlikely((b) == Py_None)) ?\
      PyNumber_Add(a, b) : __Pyx_PyUnicode_Concat(a, b))
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyUnicode_Contains)
  #define PyUnicode_Contains(u, s)  PySequence_Contains(u, s)
#endif
#define __Pyx_PyString_FormatSafe(a, b)   ((unlikely((a) == Py_None)) ? PyNumber_Remainder(a, b) : __Pyx_PyString_Format(a, b))
#define __Pyx_PyUnicode_FormatSafe(a, b)  ((unlikely((a) == Py_None)) ? PyNumber_Remainder(a, b) : PyUnicode_Format(a, b))
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyString_Format(a, b)  PyUnicode_Format(a, b)
#else
  #define __Pyx_PyString_Format(a, b)  PyString_Format(a, b)
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyBaseString_Type            PyUnicode_Type
  #define PyStringObject               PyUnicodeObject
  #define PyString_Type                PyUnicode_Type
  #define PyString_Check               PyUnicode_Check
  #define PyString_CheckExact          PyUnicode_CheckExact
#endif
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyBaseString_Check(obj) PyUnicode_Check(obj)
  #define __Pyx_PyBaseString_CheckExact(obj) PyUnicode_CheckExact(obj)
#else
  #define __Pyx_PyBaseString_Check(obj) (PyString_Check(obj) || PyUnicode_Check(obj))
  #define __Pyx_PyBaseString_CheckExact(obj) (PyString_CheckExact(obj) || PyUnicode_CheckExact(obj))
#endif
#ifndef PySet_CheckExact
  #define PySet_CheckExact(obj)        (Py_TYPE(obj) == &PySet_Type)
#endif
#define __Pyx_TypeCheck(obj, type) PyObject_TypeCheck(obj, (PyTypeObject *)type)
#if PY_MAJOR_VERSION >= 3
  #define PyIntObject                  PyLongObject
  #define PyInt_Type                   PyLong_Type
  #define PyInt_Check(op)              PyLong_Check(op)
  #define PyInt_CheckExact(op)         PyLong_CheckExact(op)
  #define PyInt_FromString             PyLong_FromString
  #define PyInt_FromUnicode            PyLong_FromUnicode
  #define PyInt_FromLong               PyLong_FromLong
  #define PyInt_FromSize_t             PyLong_FromSize_t
  #define PyInt_FromSsize_t            PyLong_FromSsize_t
  #define PyInt_AsLong                 PyLong_AsLong
  #define PyInt_AS_LONG                PyLong_AS_LONG
  #define PyInt_AsSsize_t              PyLong_AsSsize_t
  #define PyInt_AsUnsignedLongMask     PyLong_AsUnsignedLongMask
  #define PyInt_AsUnsignedLongLongMask PyLong_AsUnsignedLongLongMask
  #define PyNumber_Int                 PyNumber_Long
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyBoolObject                 PyLongObject
#endif
#if PY_MAJOR_VERSION >= 3 && CYTHON_COMPILING_IN_PYPY
  #ifndef PyUnicode_InternFromString
    #define PyUnicode_InternFromString(s) PyUnicode_FromString(s)
  #endif
#endif
#if PY_VERSION_HEX < 0x030200A4
  typedef long Py_hash_t;
  #define __Pyx_PyInt_FromHash_t PyInt_FromLong
  #define __Pyx_PyInt_AsHash_t   PyInt_AsLong
#else
  #define __Pyx_PyInt_FromHash_t PyInt_FromSsize_t
  #define __Pyx_PyInt_AsHash_t   PyInt_AsSsize_t
#endif
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyMethod_New(func, self, klass) ((self) ? PyMethod_New(func, self) : PyInstanceMethod_New(func))
#else
  #define __Pyx_PyMethod_New(func, self, klass) PyMethod_New(func, self, klass)
#endif
#if PY_VERSION_HEX >= 0x030500B1
#define __Pyx_PyAsyncMethodsStruct PyAsyncMethods
#define __Pyx_PyType_AsAsync(obj) (Py_TYPE(obj)->tp_as_async)
#elif CYTHON_COMPILING_IN_CPYTHON && PY_MAJOR_VERSION >= 3
typedef struct {
    unaryfunc am_await;
    unaryfunc am_aiter;
    unaryfunc am_anext;
} __Pyx_PyAsyncMethodsStruct;
#define __Pyx_PyType_AsAsync(obj) ((__Pyx_PyAsyncMethodsStruct*) (Py_TYPE(obj)->tp_reserved))
#else
#define __Pyx_PyType_AsAsync(obj) NULL
#endif
#ifndef CYTHON_RESTRICT
  #if defined(__GNUC__)
    #define CYTHON_RESTRICT __restrict__
  #elif defined(_MSC_VER) && _MSC_VER >= 1400
    #define CYTHON_RESTRICT __restrict
  #elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define CYTHON_RESTRICT restrict
  #else
    #define CYTHON_RESTRICT
  #endif
#endif
#define __Pyx_void_to_None(void_result) ((void)(void_result), Py_INCREF(Py_None), Py_None)

#ifndef CYTHON_INLINE
  #if defined(__GNUC__)
    #define CYTHON_INLINE __inline__
  #elif defined(_MSC_VER)
    #define CYTHON_INLINE __inline
  #elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define CYTHON_INLINE inline
  #else
    #define CYTHON_INLINE
  #endif
#endif

#if defined(WIN32) || defined(MS_WINDOWS)
  #define _USE_MATH_DEFINES
#endif
#include <math.h>
#ifdef NAN
#define __PYX_NAN() ((float) NAN)
#else
static CYTHON_INLINE float __PYX_NAN() {
  float value;
  memset(&value, 0xFF, sizeof(value));
  return value;
}
#endif


#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyNumber_Divide(x,y)         PyNumber_TrueDivide(x,y)
  #define __Pyx_PyNumber_InPlaceDivide(x,y)  PyNumber_InPlaceTrueDivide(x,y)
#else
  #define __Pyx_PyNumber_Divide(x,y)         PyNumber_Divide(x,y)
  #define __Pyx_PyNumber_InPlaceDivide(x,y)  PyNumber_InPlaceDivide(x,y)
#endif

#ifndef __PYX_EXTERN_C
  #ifdef __cplusplus
    #define __PYX_EXTERN_C extern "C"
  #else
    #define __PYX_EXTERN_C extern
  #endif
#endif

#define __PYX_HAVE__thriftrw__spec__service
#define __PYX_HAVE_API__thriftrw__spec__service
#include "stdint.h"
#ifdef _OPENMP
#include <omp.h>
#endif /* _OPENMP */

#ifdef PYREX_WITHOUT_ASSERTIONS
#define CYTHON_WITHOUT_ASSERTIONS
#endif

#ifndef CYTHON_UNUSED
# if defined(__GNUC__)
#   if !(defined(__cplusplus)) || (__GNUC__ > 3 || (__GNUC__ == 3 && __GNUC_MINOR__ >= 4))
#     define CYTHON_UNUSED __attribute__ ((__unused__))
#   else
#     define CYTHON_UNUSED
#   endif
# elif defined(__ICC) || (defined(__INTEL_COMPILER) && !defined(_MSC_VER))
#   define CYTHON_UNUSED __attribute__ ((__unused__))
# else
#   define CYTHON_UNUSED
# endif
#endif
#ifndef CYTHON_NCP_UNUSED
# if CYTHON_COMPILING_IN_CPYTHON
#  define CYTHON_NCP_UNUSED
# else
#  define CYTHON_NCP_UNUSED CYTHON_UNUSED
# endif
#endif
typedef struct {PyObject **p; char *s; const Py_ssize_t n; const char* encoding;
                const char is_unicode; const char is_str; const char intern; } __Pyx_StringTabEntry;

#define __PYX_DEFAULT_STRING_ENCODING_IS_ASCII 0
#define __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT 0
#define __PYX_DEFAULT_STRING_ENCODING ""
#define __Pyx_PyObject_FromString __Pyx_PyBytes_FromString
#define __Pyx_PyObject_FromStringAndSize __Pyx_PyBytes_FromStringAndSize
#define __Pyx_uchar_cast(c) ((unsigned char)c)
#define __Pyx_long_cast(x) ((long)x)
#define __Pyx_fits_Py_ssize_t(v, type, is_signed)  (\
    (sizeof(type) < sizeof(Py_ssize_t))  ||\
    (sizeof(type) > sizeof(Py_ssize_t) &&\
          likely(v < (type)PY_SSIZE_T_MAX ||\
                 v == (type)PY_SSIZE_T_MAX)  &&\
          (!is_signed || likely(v > (type)PY_SSIZE_T_MIN ||\
                                v == (type)PY_SSIZE_T_MIN)))  ||\
    (sizeof(type) == sizeof(Py_ssize_t) &&\
          (is_signed || likely(v < (type)PY_SSIZE_T_MAX ||\
                               v == (type)PY_SSIZE_T_MAX)))  )
#if defined (__cplusplus) && __cplusplus >= 201103L
    #include <cstdlib>
    #define __Pyx_sst_abs(value) std::abs(value)
#elif SIZEOF_INT >= SIZEOF_SIZE_T
    #define __Pyx_sst_abs(value) abs(value)
#elif SIZEOF_LONG >= SIZEOF_SIZE_T
    #define __Pyx_sst_abs(value) labs(value)
#elif defined (_MSC_VER) && defined (_M_X64)
    #define __Pyx_sst_abs(value) _abs64(value)
#elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define __Pyx_sst_abs(value) llabs(value)
#elif defined (__GNUC__)
    #define __Pyx_sst_abs(value) __builtin_llabs(value)
#else
    #define __Pyx_sst_abs(value) ((value<0) ? -value : value)
#endif
static CYTHON_INLINE char* __Pyx_PyObject_AsString(PyObject*);
static CYTHON_INLINE char* __Pyx_PyObject_AsStringAndSize(PyObject*, Py_ssize_t* length);
#define __Pyx_PyByteArray_FromString(s) PyByteArray_FromStringAndSize((const char*)s, strlen((const char*)s))
#define __Pyx_PyByteArray_FromStringAndSize(s, l) PyByteArray_FromStringAndSize((const char*)s, l)
#define __Pyx_PyBytes_FromString        PyBytes_FromString
#define __Pyx_PyBytes_FromStringAndSize PyBytes_FromStringAndSize
static CYTHON_INLINE PyObject* __Pyx_PyUnicode_FromString(const char*);
#if PY_MAJOR_VERSION < 3
    #define __Pyx_PyStr_FromString        __Pyx_PyBytes_FromString
    #define __Pyx_PyStr_FromStringAndSize __Pyx_PyBytes_FromStringAndSize
#else
    #define __Pyx_PyStr_FromString        __Pyx_PyUnicode_FromString
    #define __Pyx_PyStr_FromStringAndSize __Pyx_PyUnicode_FromStringAndSize
#endif
#define __Pyx_PyObject_AsSString(s)    ((signed char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsUString(s)    ((unsigned char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_FromCString(s)  __Pyx_PyObject_FromString((const char*)s)
#define __Pyx_PyBytes_FromCString(s)   __Pyx_PyBytes_FromString((const char*)s)
#define __Pyx_PyByteArray_FromCString(s)   __Pyx_PyByteArray_FromString((const char*)s)
#define __Pyx_PyStr_FromCString(s)     __Pyx_PyStr_FromString((const char*)s)
#define __Pyx_PyUnicode_FromCString(s) __Pyx_PyUnicode_FromString((const char*)s)
#if PY_MAJOR_VERSION < 3
static CYTHON_INLINE size_t __Pyx_Py_UNICODE_strlen(const Py_UNICODE *u)
{
    const Py_UNICODE *u_end = u;
    while (*u_end++) ;
    return (size_t)(u_end - u - 1);
}
#else
#define __Pyx_Py_UNICODE_strlen Py_UNICODE_strlen
#endif
#define __Pyx_PyUnicode_FromUnicode(u)       PyUnicode_FromUnicode(u, __Pyx_Py_UNICODE_strlen(u))
#define __Pyx_PyUnicode_FromUnicodeAndLength PyUnicode_FromUnicode
#define __Pyx_PyUnicode_AsUnicode            PyUnicode_AsUnicode
#define __Pyx_NewRef(obj) (Py_INCREF(obj), obj)
#define __Pyx_Owned_Py_None(b) __Pyx_NewRef(Py_None)
#define __Pyx_PyBool_FromLong(b) ((b) ? __Pyx_NewRef(Py_True) : __Pyx_NewRef(Py_False))
static CYTHON_INLINE int __Pyx_PyObject_IsTrue(PyObject*);
static CYTHON_INLINE PyObject* __Pyx_PyNumber_Int(PyObject* x);
static CYTHON_INLINE Py_ssize_t __Pyx_PyIndex_AsSsize_t(PyObject*);
static CYTHON_INLINE PyObject * __Pyx_PyInt_FromSize_t(size_t);
#if CYTHON_COMPILING_IN_CPYTHON
#define __pyx_PyFloat_AsDouble(x) (PyFloat_CheckExact(x) ? PyFloat_AS_DOUBLE(x) : PyFloat_AsDouble(x))
#else
#define __pyx_PyFloat_AsDouble(x) PyFloat_AsDouble(x)
#endif
#define __pyx_PyFloat_AsFloat(x) ((float) __pyx_PyFloat_AsDouble(x))
#if PY_MAJOR_VERSION < 3 && __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
static int __Pyx_sys_getdefaultencoding_not_ascii;
static int __Pyx_init_sys_getdefaultencoding_params(void) {
    PyObject* sys;
    PyObject* default_encoding = NULL;
    PyObject* ascii_chars_u = NULL;
    PyObject* ascii_chars_b = NULL;
    const char* default_encoding_c;
    sys = PyImport_ImportModule("sys");
    if (!sys) goto bad;
    default_encoding = PyObject_CallMethod(sys, (char*) "getdefaultencoding", NULL);
    Py_DECREF(sys);
    if (!default_encoding) goto bad;
    default_encoding_c = PyBytes_AsString(default_encoding);
    if (!default_encoding_c) goto bad;
    if (strcmp(default_encoding_c, "ascii") == 0) {
        __Pyx_sys_getdefaultencoding_not_ascii = 0;
    } else {
        char ascii_chars[128];
        int c;
        for (c = 0; c < 128; c++) {
            ascii_chars[c] = c;
        }
        __Pyx_sys_getdefaultencoding_not_ascii = 1;
        ascii_chars_u = PyUnicode_DecodeASCII(ascii_chars, 128, NULL);
        if (!ascii_chars_u) goto bad;
        ascii_chars_b = PyUnicode_AsEncodedString(ascii_chars_u, default_encoding_c, NULL);
        if (!ascii_chars_b || !PyBytes_Check(ascii_chars_b) || memcmp(ascii_chars, PyBytes_AS_STRING(ascii_chars_b), 128) != 0) {
            PyErr_Format(
                PyExc_ValueError,
                "This module compiled with c_string_encoding=ascii, but default encoding '%.200s' is not a superset of ascii.",
                default_encoding_c);
            goto bad;
        }
        Py_DECREF(ascii_chars_u);
        Py_DECREF(ascii_chars_b);
    }
    Py_DECREF(default_encoding);
    return 0;
bad:
    Py_XDECREF(default_encoding);
    Py_XDECREF(ascii_chars_u);
    Py_XDECREF(ascii_chars_b);
    return -1;
}
#endif
#if __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT && PY_MAJOR_VERSION >= 3
#define __Pyx_PyUnicode_FromStringAndSize(c_str, size) PyUnicode_DecodeUTF8(c_str, size, NULL)
#else
#define __Pyx_PyUnicode_FromStringAndSize(c_str, size) PyUnicode_Decode(c_str, size, __PYX_DEFAULT_STRING_ENCODING, NULL)
#if __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
static char* __PYX_DEFAULT_STRING_ENCODING;
static int __Pyx_init_sys_getdefaultencoding_params(void) {
    PyObject* sys;
    PyObject* default_encoding = NULL;
    char* default_encoding_c;
    sys = PyImport_ImportModule("sys");
    if (!sys) goto bad;
    default_encoding = PyObject_CallMethod(sys, (char*) (const char*) "getdefaultencoding", NULL);
    Py_DECREF(sys);
    if (!default_encoding) goto bad;
    default_encoding_c = PyBytes_AsString(default_encoding);
    if (!default_encoding_c) goto bad;
    __PYX_DEFAULT_STRING_ENCODING = (char*) malloc(strlen(default_encoding_c));
    if (!__PYX_DEFAULT_STRING_ENCODING) goto bad;
    strcpy(__PYX_DEFAULT_STRING_ENCODING, default_encoding_c);
    Py_DECREF(default_encoding);
    return 0;
bad:
    Py_XDECREF(default_encoding);
    return -1;
}
#endif
#endif


/* Test for GCC > 2.95 */
#if defined(__GNUC__)     && (__GNUC__ > 2 || (__GNUC__ == 2 && (__GNUC_MINOR__ > 95)))
  #define likely(x)   __builtin_expect(!!(x), 1)
  #define unlikely(x) __builtin_expect(!!(x), 0)
#else /* !__GNUC__ or GCC < 2.95 */
  #define likely(x)   (x)
  #define unlikely(x) (x)
#endif /* __GNUC__ */

static PyObject *__pyx_m;
static PyObject *__pyx_d;
static PyObject *__pyx_b;
static PyObject *__pyx_empty_tuple;
static PyObject *__pyx_empty_bytes;
static int __pyx_lineno;
static int __pyx_clineno = 0;
static const char * __pyx_cfilenm= __FILE__;
static const char *__pyx_filename;


static const char *__pyx_f[] = {
  "thriftrw/spec/service.pyx",
  "thriftrw/spec/service.pxd",
  "thriftrw/wire/value.pxd",
  "thriftrw/spec/base.pxd",
  "thriftrw/spec/field.pxd",
};

/*--- Type declarations ---*/
struct __pyx_obj_8thriftrw_4wire_5value_ValueVisitor;
struct __pyx_obj_8thriftrw_4wire_5value__ValueVisitorArgs;
struct __pyx_obj_8thriftrw_4wire_5value_Value;
struct __pyx_obj_8thriftrw_4wire_5value_BoolValue;
struct __pyx_obj_8thriftrw_4wire_5value_ByteValue;
struct __pyx_obj_8thriftrw_4wire_5value_DoubleValue;
struct __pyx_obj_8thriftrw_4wire_5value_I16Value;
struct __pyx_obj_8thriftrw_4wire_5value_I32Value;
struct __pyx_obj_8thriftrw_4wire_5value_I64Value;
struct __pyx_obj_8thriftrw_4wire_5value_BinaryValue;
struct __pyx_obj_8thriftrw_4wire_5value_FieldValue;
struct __pyx_obj_8thriftrw_4wire_5value_StructValue;
struct __pyx_obj_8thriftrw_4wire_5value_MapItem;
struct __pyx_obj_8thriftrw_4wire_5value_MapValue;
struct __pyx_obj_8thriftrw_4wire_5value_SetValue;
struct __pyx_obj_8thriftrw_4wire_5value_ListValue;
struct __pyx_obj_8thriftrw_4spec_4base_TypeSpec;
struct __pyx_obj_8thriftrw_4spec_6struct_StructTypeSpec;
struct __pyx_obj_8thriftrw_4spec_5union_UnionTypeSpec;
struct __pyx_obj_8thriftrw_4spec_5field_FieldSpec;
struct __pyx_obj_8thriftrw_4spec_7service_FunctionArgsSpec;
struct __pyx_obj_8thriftrw_4spec_7service_FunctionResultSpec;
struct __pyx_obj_8thriftrw_4spec_7service_FunctionSpec;
struct __pyx_obj_8thriftrw_4spec_7service_ServiceSpec;

/* "thriftrw/wire/value.pxd":31
 * 
 * 
 * cdef class ValueVisitor(object):             # <<<<<<<<<<<<<<
 * 
 *     cdef object visit_bool(self, bint value)
 */
struct __pyx_obj_8thriftrw_4wire_5value_ValueVisitor {
  PyObject_HEAD
  struct __pyx_vtabstruct_8thriftrw_4wire_5value_ValueVisitor *__pyx_vtab;
};


/* "thriftrw/wire/value.pxd":56
 * 
 * 
 * cdef class _ValueVisitorArgs(ValueVisitor):             # <<<<<<<<<<<<<<
 *     pass
 * 
 */
struct __pyx_obj_8thriftrw_4wire_5value__ValueVisitorArgs {
  struct __pyx_obj_8thriftrw_4wire_5value_ValueVisitor __pyx_base;
};


/* "thriftrw/wire/value.pxd":60
 * 
 * 
 * cdef class Value(object):             # <<<<<<<<<<<<<<
 * 
 *     cpdef object apply(Value self, ValueVisitor visitor)
 */
struct __pyx_obj_8thriftrw_4wire_5value_Value {
  PyObject_HEAD
  struct __pyx_vtabstruct_8thriftrw_4wire_5value_Value *__pyx_vtab;
};


/* "thriftrw/wire/value.pxd":65
 * 
 * 
 * cdef class BoolValue(Value):             # <<<<<<<<<<<<<<
 *     cdef readonly bint value
 * 
 */
struct __pyx_obj_8thriftrw_4wire_5value_BoolValue {
  struct __pyx_obj_8thriftrw_4wire_5value_Value __pyx_base;
  int value;
};


/* "thriftrw/wire/value.pxd":69
 * 
 * 
 * cdef class ByteValue(Value):             # <<<<<<<<<<<<<<
 *     cdef readonly int8_t value
 * 
 */
struct __pyx_obj_8thriftrw_4wire_5value_ByteValue {
  struct __pyx_obj_8thriftrw_4wire_5value_Value __pyx_base;
  int8_t value;
};


/* "thriftrw/wire/value.pxd":73
 * 
 * 
 * cdef class DoubleValue(Value):             # <<<<<<<<<<<<<<
 *     cdef readonly double value
 * 
 */
struct __pyx_obj_8thriftrw_4wire_5value_DoubleValue {
  struct __pyx_obj_8thriftrw_4wire_5value_Value __pyx_base;
  double value;
};


/* "thriftrw/wire/value.pxd":77
 * 
 * 
 * cdef class I16Value(Value):             # <<<<<<<<<<<<<<
 *     cdef readonly int16_t value
 * 
 */
struct __pyx_obj_8thriftrw_4wire_5value_I16Value {
  struct __pyx_obj_8thriftrw_4wire_5value_Value __pyx_base;
  int16_t value;
};


/* "thriftrw/wire/value.pxd":81
 * 
 * 
 * cdef class I32Value(Value):             # <<<<<<<<<<<<<<
 *     cdef readonly int32_t value
 * 
 */
struct __pyx_obj_8thriftrw_4wire_5value_I32Value {
  struct __pyx_obj_8thriftrw_4wire_5value_Value __pyx_base;
  int32_t value;
};


/* "thriftrw/wire/value.pxd":85
 * 
 * 
 * cdef class I64Value(Value):             # <<<<<<<<<<<<<<
 *     cdef readonly int64_t value
 * 
 */
struct __pyx_obj_8thriftrw_4wire_5value_I64Value {
  struct __pyx_obj_8thriftrw_4wire_5value_Value __pyx_base;
  int64_t value;
};


/* "thriftrw/wire/value.pxd":89
 * 
 * 
 * cdef class BinaryValue(Value):             # <<<<<<<<<<<<<<
 *     cdef readonly bytes value
 *     # TODO change to char* once BinaryProtocol knows how to write that.
 */
struct __pyx_obj_8thriftrw_4wire_5value_BinaryValue {
  struct __pyx_obj_8thriftrw_4wire_5value_Value __pyx_base;
  PyObject *value;
};


/* "thriftrw/wire/value.pxd":94
 * 
 * 
 * cdef class FieldValue(object):             # <<<<<<<<<<<<<<
 *     cdef readonly int16_t id
 *     cdef readonly int8_t ttype
 */
struct __pyx_obj_8thriftrw_4wire_5value_FieldValue {
  PyObject_HEAD
  int16_t id;
  int8_t ttype;
  PyObject *value;
};


/* "thriftrw/wire/value.pxd":100
 * 
 * 
 * cdef class StructValue(Value):             # <<<<<<<<<<<<<<
 *     cdef readonly list fields
 *     cdef readonly dict _index
 */
struct __pyx_obj_8thriftrw_4wire_5value_StructValue {
  struct __pyx_obj_8thriftrw_4wire_5value_Value __pyx_base;
  PyObject *fields;
  PyObject *_index;
};


/* "thriftrw/wire/value.pxd":105
 * 
 * 
 * cdef class MapItem(object):             # <<<<<<<<<<<<<<
 *     cdef readonly object key
 *     cdef readonly object value
 */
struct __pyx_obj_8thriftrw_4wire_5value_MapItem {
  PyObject_HEAD
  PyObject *key;
  PyObject *value;
};


/* "thriftrw/wire/value.pxd":110
 * 
 * 
 * cdef class MapValue(Value):             # <<<<<<<<<<<<<<
 *     cdef readonly int8_t key_ttype
 *     cdef readonly int8_t value_ttype
 */
struct __pyx_obj_8thriftrw_4wire_5value_MapValue {
  struct __pyx_obj_8thriftrw_4wire_5value_Value __pyx_base;
  int8_t key_ttype;
  int8_t value_ttype;
  PyObject *pairs;
};


/* "thriftrw/wire/value.pxd":116
 * 
 * 
 * cdef class SetValue(Value):             # <<<<<<<<<<<<<<
 *     cdef readonly int8_t value_ttype
 *     cdef readonly list values
 */
struct __pyx_obj_8thriftrw_4wire_5value_SetValue {
  struct __pyx_obj_8thriftrw_4wire_5value_Value __pyx_base;
  int8_t value_ttype;
  PyObject *values;
};


/* "thriftrw/wire/value.pxd":121
 * 
 * 
 * cdef class ListValue(Value):             # <<<<<<<<<<<<<<
 *     cdef readonly int8_t value_ttype
 *     cdef readonly list values
 */
struct __pyx_obj_8thriftrw_4wire_5value_ListValue {
  struct __pyx_obj_8thriftrw_4wire_5value_Value __pyx_base;
  int8_t value_ttype;
  PyObject *values;
};


/* "base.pxd":26
 * 
 * 
 * cdef class TypeSpec(object):             # <<<<<<<<<<<<<<
 * 
 *     cpdef Value to_wire(TypeSpec self, object value)
 */
struct __pyx_obj_8thriftrw_4spec_4base_TypeSpec {
  PyObject_HEAD
  struct __pyx_vtabstruct_8thriftrw_4spec_4base_TypeSpec *__pyx_vtab;
};


/* "struct.pxd":26
 * 
 * 
 * cdef class StructTypeSpec(TypeSpec):             # <<<<<<<<<<<<<<
 *     cdef readonly str name
 *     cdef readonly list fields
 */
struct __pyx_obj_8thriftrw_4spec_6struct_StructTypeSpec {
  struct __pyx_obj_8thriftrw_4spec_4base_TypeSpec __pyx_base;
  PyObject *name;
  PyObject *fields;
  PyObject *base_cls;
  int linked;
  PyObject *surface;
};


/* "union.pxd":26
 * 
 * 
 * cdef class UnionTypeSpec(TypeSpec):             # <<<<<<<<<<<<<<
 *     cdef readonly str name
 *     cdef readonly list fields
 */
struct __pyx_obj_8thriftrw_4spec_5union_UnionTypeSpec {
  struct __pyx_obj_8thriftrw_4spec_4base_TypeSpec __pyx_base;
  PyObject *name;
  PyObject *fields;
  int allow_empty;
  int linked;
  PyObject *surface;
};


/* "field.pxd":28
 * 
 * 
 * cdef class FieldSpec(object):             # <<<<<<<<<<<<<<
 *     cdef readonly int16_t id
 *     cdef readonly str name
 */
struct __pyx_obj_8thriftrw_4spec_5field_FieldSpec {
  PyObject_HEAD
  int16_t id;
  PyObject *name;
  int required;
  struct __pyx_obj_8thriftrw_4spec_4base_TypeSpec *spec;
  PyObject *default_value;
  int linked;
};


/* "thriftrw/spec/service.pxd":28
 * 
 * 
 * cdef class FunctionArgsSpec(StructTypeSpec):             # <<<<<<<<<<<<<<
 *     cdef public FunctionSpec function
 * 
 */
struct __pyx_obj_8thriftrw_4spec_7service_FunctionArgsSpec {
  struct __pyx_obj_8thriftrw_4spec_6struct_StructTypeSpec __pyx_base;
  struct __pyx_obj_8thriftrw_4spec_7service_FunctionSpec *function;
};


/* "thriftrw/spec/service.pxd":32
 * 
 * 
 * cdef class FunctionResultSpec(UnionTypeSpec):             # <<<<<<<<<<<<<<
 *     cdef public TypeSpec return_spec
 *     cdef public list exception_specs
 */
struct __pyx_obj_8thriftrw_4spec_7service_FunctionResultSpec {
  struct __pyx_obj_8thriftrw_4spec_5union_UnionTypeSpec __pyx_base;
  struct __pyx_obj_8thriftrw_4spec_4base_TypeSpec *return_spec;
  PyObject *exception_specs;
  struct __pyx_obj_8thriftrw_4spec_7service_FunctionSpec *function;
  PyObject *exception_ids;
};


/* "thriftrw/spec/service.pxd":39
 * 
 * 
 * cdef class FunctionSpec(object):             # <<<<<<<<<<<<<<
 *     cdef readonly str name
 *     cdef public FunctionArgsSpec args_spec
 */
struct __pyx_obj_8thriftrw_4spec_7service_FunctionSpec {
  PyObject_HEAD
  struct __pyx_vtabstruct_8thriftrw_4spec_7service_FunctionSpec *__pyx_vtab;
  PyObject *name;
  struct __pyx_obj_8thriftrw_4spec_7service_FunctionArgsSpec *args_spec;
  struct __pyx_obj_8thriftrw_4spec_7service_FunctionResultSpec *result_spec;
  int oneway;
  int linked;
  PyObject *surface;
  struct __pyx_obj_8thriftrw_4spec_7service_ServiceSpec *service;
};


/* "thriftrw/spec/service.pxd":51
 * 
 * 
 * cdef class ServiceSpec(object):             # <<<<<<<<<<<<<<
 *     cdef readonly str name
 *     cdef public list functions
 */
struct __pyx_obj_8thriftrw_4spec_7service_ServiceSpec {
  PyObject_HEAD
  struct __pyx_vtabstruct_8thriftrw_4spec_7service_ServiceSpec *__pyx_vtab;
  PyObject *name;
  PyObject *functions;
  PyObject *parent;
  PyObject *_functions;
  int linked;
  PyObject *surface;
};



/* "thriftrw/wire/value.pxd":31
 * 
 * 
 * cdef class ValueVisitor(object):             # <<<<<<<<<<<<<<
 * 
 *     cdef object visit_bool(self, bint value)
 */

struct __pyx_vtabstruct_8thriftrw_4wire_5value_ValueVisitor {
  PyObject *(*visit_bool)(struct __pyx_obj_8thriftrw_4wire_5value_ValueVisitor *, int);
  PyObject *(*visit_byte)(struct __pyx_obj_8thriftrw_4wire_5value_ValueVisitor *, int8_t);
  PyObject *(*visit_double)(struct __pyx_obj_8thriftrw_4wire_5value_ValueVisitor *, double);
  PyObject *(*visit_i16)(struct __pyx_obj_8thriftrw_4wire_5value_ValueVisitor *, int16_t);
  PyObject *(*visit_i32)(struct __pyx_obj_8thriftrw_4wire_5value_ValueVisitor *, int32_t);
  PyObject *(*visit_i64)(struct __pyx_obj_8thriftrw_4wire_5value_ValueVisitor *, int64_t);
  PyObject *(*visit_binary)(struct __pyx_obj_8thriftrw_4wire_5value_ValueVisitor *, PyObject *);
  PyObject *(*visit_struct)(struct __pyx_obj_8thriftrw_4wire_5value_ValueVisitor *, PyObject *);
  PyObject *(*visit_map)(struct __pyx_obj_8thriftrw_4wire_5value_ValueVisitor *, int8_t, int8_t, PyObject *);
  PyObject *(*visit_set)(struct __pyx_obj_8thriftrw_4wire_5value_ValueVisitor *, int8_t, PyObject *);
  PyObject *(*visit_list)(struct __pyx_obj_8thriftrw_4wire_5value_ValueVisitor *, int8_t, PyObject *);
};
static struct __pyx_vtabstruct_8thriftrw_4wire_5value_ValueVisitor *__pyx_vtabptr_8thriftrw_4wire_5value_ValueVisitor;


/* "thriftrw/wire/value.pxd":56
 * 
 * 
 * cdef class _ValueVisitorArgs(ValueVisitor):             # <<<<<<<<<<<<<<
 *     pass
 * 
 */

struct __pyx_vtabstruct_8thriftrw_4wire_5value__ValueVisitorArgs {
  struct __pyx_vtabstruct_8thriftrw_4wire_5value_ValueVisitor __pyx_base;
};
static struct __pyx_vtabstruct_8thriftrw_4wire_5value__ValueVisitorArgs *__pyx_vtabptr_8thriftrw_4wire_5value__ValueVisitorArgs;


/* "thriftrw/wire/value.pxd":60
 * 
 * 
 * cdef class Value(object):             # <<<<<<<<<<<<<<
 * 
 *     cpdef object apply(Value self, ValueVisitor visitor)
 */

struct __pyx_vtabstruct_8thriftrw_4wire_5value_Value {
  PyObject *(*apply)(struct __pyx_obj_8thriftrw_4wire_5value_Value *, struct __pyx_obj_8thriftrw_4wire_5value_ValueVisitor *, int __pyx_skip_dispatch);
};
static struct __pyx_vtabstruct_8thriftrw_4wire_5value_Value *__pyx_vtabptr_8thriftrw_4wire_5value_Value;


/* "thriftrw/wire/value.pxd":65
 * 
 * 
 * cdef class BoolValue(Value):             # <<<<<<<<<<<<<<
 *     cdef readonly bint value
 * 
 */

struct __pyx_vtabstruct_8thriftrw_4wire_5value_BoolValue {
  struct __pyx_vtabstruct_8thriftrw_4wire_5value_Value __pyx_base;
};
static struct __pyx_vtabstruct_8thriftrw_4wire_5value_BoolValue *__pyx_vtabptr_8thriftrw_4wire_5value_BoolValue;


/* "thriftrw/wire/value.pxd":69
 * 
 * 
 * cdef class ByteValue(Value):             # <<<<<<<<<<<<<<
 *     cdef readonly int8_t value
 * 
 */

struct __pyx_vtabstruct_8thriftrw_4wire_5value_ByteValue {
  struct __pyx_vtabstruct_8thriftrw_4wire_5value_Value __pyx_base;
};
static struct __pyx_vtabstruct_8thriftrw_4wire_5value_ByteValue *__pyx_vtabptr_8thriftrw_4wire_5value_ByteValue;


/* "thriftrw/wire/value.pxd":73
 * 
 * 
 * cdef class DoubleValue(Value):             # <<<<<<<<<<<<<<
 *     cdef readonly double value
 * 
 */

struct __pyx_vtabstruct_8thriftrw_4wire_5value_DoubleValue {
  struct __pyx_vtabstruct_8thriftrw_4wire_5value_Value __pyx_base;
};
static struct __pyx_vtabstruct_8thriftrw_4wire_5value_DoubleValue *__pyx_vtabptr_8thriftrw_4wire_5value_DoubleValue;


/* "thriftrw/wire/value.pxd":77
 * 
 * 
 * cdef class I16Value(Value):             # <<<<<<<<<<<<<<
 *     cdef readonly int16_t value
 * 
 */

struct __pyx_vtabstruct_8thriftrw_4wire_5value_I16Value {
  struct __pyx_vtabstruct_8thriftrw_4wire_5value_Value __pyx_base;
};
static struct __pyx_vtabstruct_8thriftrw_4wire_5value_I16Value *__pyx_vtabptr_8thriftrw_4wire_5value_I16Value;


/* "thriftrw/wire/value.pxd":81
 * 
 * 
 * cdef class I32Value(Value):             # <<<<<<<<<<<<<<
 *     cdef readonly int32_t value
 * 
 */

struct __pyx_vtabstruct_8thriftrw_4wire_5value_I32Value {
  struct __pyx_vtabstruct_8thriftrw_4wire_5value_Value __pyx_base;
};
static struct __pyx_vtabstruct_8thriftrw_4wire_5value_I32Value *__pyx_vtabptr_8thriftrw_4wire_5value_I32Value;


/* "thriftrw/wire/value.pxd":85
 * 
 * 
 * cdef class I64Value(Value):             # <<<<<<<<<<<<<<
 *     cdef readonly int64_t value
 * 
 */

struct __pyx_vtabstruct_8thriftrw_4wire_5value_I64Value {
  struct __pyx_vtabstruct_8thriftrw_4wire_5value_Value __pyx_base;
};
static struct __pyx_vtabstruct_8thriftrw_4wire_5value_I64Value *__pyx_vtabptr_8thriftrw_4wire_5value_I64Value;


/* "thriftrw/wire/value.pxd":89
 * 
 * 
 * cdef class BinaryValue(Value):             # <<<<<<<<<<<<<<
 *     cdef readonly bytes value
 *     # TODO change to char* once BinaryProtocol knows how to write that.
 */

struct __pyx_vtabstruct_8thriftrw_4wire_5value_BinaryValue {
  struct __pyx_vtabstruct_8thriftrw_4wire_5value_Value __pyx_base;
};
static struct __pyx_vtabstruct_8thriftrw_4wire_5value_BinaryValue *__pyx_vtabptr_8thriftrw_4wire_5value_BinaryValue;


/* "thriftrw/wire/value.pxd":100
 * 
 * 
 * cdef class StructValue(Value):             # <<<<<<<<<<<<<<
 *     cdef readonly list fields
 *     cdef readonly dict _index
 */

struct __pyx_vtabstruct_8thriftrw_4wire_5value_StructValue {
  struct __pyx_vtabstruct_8thriftrw_4wire_5value_Value __pyx_base;
};
static struct __pyx_vtabstruct_8thriftrw_4wire_5value_StructValue *__pyx_vtabptr_8thriftrw_4wire_5value_StructValue;


/* "thriftrw/wire/value.pxd":110
 * 
 * 
 * cdef class MapValue(Value):             # <<<<<<<<<<<<<<
 *     cdef readonly int8_t key_ttype
 *     cdef readonly int8_t value_ttype
 */

struct __pyx_vtabstruct_8thriftrw_4wire_5value_MapValue {
  struct __pyx_vtabstruct_8thriftrw_4wire_5value_Value __pyx_base;
};
static struct __pyx_vtabstruct_8thriftrw_4wire_5value_MapValue *__pyx_vtabptr_8thriftrw_4wire_5value_MapValue;


/* "thriftrw/wire/value.pxd":116
 * 
 * 
 * cdef class SetValue(Value):             # <<<<<<<<<<<<<<
 *     cdef readonly int8_t value_ttype
 *     cdef readonly list values
 */

struct __pyx_vtabstruct_8thriftrw_4wire_5value_SetValue {
  struct __pyx_vtabstruct_8thriftrw_4wire_5value_Value __pyx_base;
};
static struct __pyx_vtabstruct_8thriftrw_4wire_5value_SetValue *__pyx_vtabptr_8thriftrw_4wire_5value_SetValue;


/* "thriftrw/wire/value.pxd":121
 * 
 * 
 * cdef class ListValue(Value):             # <<<<<<<<<<<<<<
 *     cdef readonly int8_t value_ttype
 *     cdef readonly list values
 */

struct __pyx_vtabstruct_8thriftrw_4wire_5value_ListValue {
  struct __pyx_vtabstruct_8thriftrw_4wire_5value_Value __pyx_base;
};
static struct __pyx_vtabstruct_8thriftrw_4wire_5value_ListValue *__pyx_vtabptr_8thriftrw_4wire_5value_ListValue;


/* "base.pxd":26
 * 
 * 
 * cdef class TypeSpec(object):             # <<<<<<<<<<<<<<
 * 
 *     cpdef Value to_wire(TypeSpec self, object value)
 */

struct __pyx_vtabstruct_8thriftrw_4spec_4base_TypeSpec {
  struct __pyx_obj_8thriftrw_4wire_5value_Value *(*to_wire)(struct __pyx_obj_8thriftrw_4spec_4base_TypeSpec *, PyObject *, int __pyx_skip_dispatch);
  PyObject *(*from_wire)(struct __pyx_obj_8thriftrw_4spec_4base_TypeSpec *, struct __pyx_obj_8thriftrw_4wire_5value_Value *, int __pyx_skip_dispatch);
  struct __pyx_obj_8thriftrw_4spec_4base_TypeSpec *(*link)(struct __pyx_obj_8thriftrw_4spec_4base_TypeSpec *, PyObject *, int __pyx_skip_dispatch);
  PyObject *(*to_primitive)(struct __pyx_obj_8thriftrw_4spec_4base_TypeSpec *, PyObject *, int __pyx_skip_dispatch);
  PyObject *(*from_primitive)(struct __pyx_obj_8thriftrw_4spec_4base_TypeSpec *, PyObject *, int __pyx_skip_dispatch);
  void (*validate)(struct __pyx_obj_8thriftrw_4spec_4base_TypeSpec *, PyObject *, int __pyx_skip_dispatch);
};
static struct __pyx_vtabstruct_8thriftrw_4spec_4base_TypeSpec *__pyx_vtabptr_8thriftrw_4spec_4base_TypeSpec;


/* "struct.pxd":26
 * 
 * 
 * cdef class StructTypeSpec(TypeSpec):             # <<<<<<<<<<<<<<
 *     cdef readonly str name
 *     cdef readonly list fields
 */

struct __pyx_vtabstruct_8thriftrw_4spec_6struct_StructTypeSpec {
  struct __pyx_vtabstruct_8thriftrw_4spec_4base_TypeSpec __pyx_base;
};
static struct __pyx_vtabstruct_8thriftrw_4spec_6struct_StructTypeSpec *__pyx_vtabptr_8thriftrw_4spec_6struct_StructTypeSpec;


/* "union.pxd":26
 * 
 * 
 * cdef class UnionTypeSpec(TypeSpec):             # <<<<<<<<<<<<<<
 *     cdef readonly str name
 *     cdef readonly list fields
 */

struct __pyx_vtabstruct_8thriftrw_4spec_5union_UnionTypeSpec {
  struct __pyx_vtabstruct_8thriftrw_4spec_4base_TypeSpec __pyx_base;
};
static struct __pyx_vtabstruct_8thriftrw_4spec_5union_UnionTypeSpec *__pyx_vtabptr_8thriftrw_4spec_5union_UnionTypeSpec;


/* "thriftrw/spec/service.pyx":44
 * 
 * 
 * cdef class FunctionArgsSpec(StructTypeSpec):             # <<<<<<<<<<<<<<
 *     """Represents the parameters of a service function.
 * 
 */

struct __pyx_vtabstruct_8thriftrw_4spec_7service_FunctionArgsSpec {
  struct __pyx_vtabstruct_8thriftrw_4spec_6struct_StructTypeSpec __pyx_base;
};
static struct __pyx_vtabstruct_8thriftrw_4spec_7service_FunctionArgsSpec *__pyx_vtabptr_8thriftrw_4spec_7service_FunctionArgsSpec;


/* "thriftrw/spec/service.pyx":112
 * 
 * 
 * cdef class FunctionResultSpec(UnionTypeSpec):             # <<<<<<<<<<<<<<
 *     """Represents the result of a service function.
 * 
 */

struct __pyx_vtabstruct_8thriftrw_4spec_7service_FunctionResultSpec {
  struct __pyx_vtabstruct_8thriftrw_4spec_5union_UnionTypeSpec __pyx_base;
};
static struct __pyx_vtabstruct_8thriftrw_4spec_7service_FunctionResultSpec *__pyx_vtabptr_8thriftrw_4spec_7service_FunctionResultSpec;


/* "thriftrw/spec/service.pyx":236
 * 
 * 
 * cdef class FunctionSpec(object):             # <<<<<<<<<<<<<<
 *     """Specification of a single function on a service.
 * 
 */

struct __pyx_vtabstruct_8thriftrw_4spec_7service_FunctionSpec {
  struct __pyx_obj_8thriftrw_4spec_7service_FunctionSpec *(*link)(struct __pyx_obj_8thriftrw_4spec_7service_FunctionSpec *, PyObject *, struct __pyx_obj_8thriftrw_4spec_7service_ServiceSpec *, int __pyx_skip_dispatch);
};
static struct __pyx_vtabstruct_8thriftrw_4spec_7service_FunctionSpec *__pyx_vtabptr_8thriftrw_4spec_7service_FunctionSpec;


/* "thriftrw/spec/service.pyx":346
 * 
 * 
 * cdef class ServiceSpec(object):             # <<<<<<<<<<<<<<
 *     """Spec for a single service.
 * 
 */

struct __pyx_vtabstruct_8thriftrw_4spec_7service_ServiceSpec {
  struct __pyx_obj_8thriftrw_4spec_7service_ServiceSpec *(*link)(struct __pyx_obj_8thriftrw_4spec_7service_ServiceSpec *, PyObject *, int __pyx_skip_dispatch);
};
static struct __pyx_vtabstruct_8thriftrw_4spec_7service_ServiceSpec *__pyx_vtabptr_8thriftrw_4spec_7service_ServiceSpec;

/* --- Runtime support code (head) --- */
#ifndef CYTHON_REFNANNY
  #define CYTHON_REFNANNY 0
#endif
#if CYTHON_REFNANNY
  typedef struct {
    void (*INCREF)(void*, PyObject*, int);
    void (*DECREF)(void*, PyObject*, int);
    void (*GOTREF)(void*, PyObject*, int);
    void (*GIVEREF)(void*, PyObject*, int);
    void* (*SetupContext)(const char*, int, const char*);
    void (*FinishContext)(void**);
  } __Pyx_RefNannyAPIStruct;
  static __Pyx_RefNannyAPIStruct *__Pyx_RefNanny = NULL;
  static __Pyx_RefNannyAPIStruct *__Pyx_RefNannyImportAPI(const char *modname);
  #define __Pyx_RefNannyDeclarations void *__pyx_refnanny = NULL;
#ifdef WITH_THREAD
  #define __Pyx_RefNannySetupContext(name, acquire_gil)\
          if (acquire_gil) {\
              PyGILState_STATE __pyx_gilstate_save = PyGILState_Ensure();\
              __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__);\
              PyGILState_Release(__pyx_gilstate_save);\
          } else {\
              __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__);\
          }
#else
  #define __Pyx_RefNannySetupContext(name, acquire_gil)\
          __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__)
#endif
  #define __Pyx_RefNannyFinishContext()\
          __Pyx_RefNanny->FinishContext(&__pyx_refnanny)
  #define __Pyx_INCREF(r)  __Pyx_RefNanny->INCREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_DECREF(r)  __Pyx_RefNanny->DECREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_GOTREF(r)  __Pyx_RefNanny->GOTREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_GIVEREF(r) __Pyx_RefNanny->GIVEREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_XINCREF(r)  do { if((r) != NULL) {__Pyx_INCREF(r); }} while(0)
  #define __Pyx_XDECREF(r)  do { if((r) != NULL) {__Pyx_DECREF(r); }} while(0)
  #define __Pyx_XGOTREF(r)  do { if((r) != NULL) {__Pyx_GOTREF(r); }} while(0)
  #define __Pyx_XGIVEREF(r) do { if((r) != NULL) {__Pyx_GIVEREF(r);}} while(0)
#else
  #define __Pyx_RefNannyDeclarations
  #define __Pyx_RefNannySetupContext(name, acquire_gil)
  #define __Pyx_RefNannyFinishContext()
  #define __Pyx_INCREF(r) Py_INCREF(r)
  #define __Pyx_DECREF(r) Py_DECREF(r)
  #define __Pyx_GOTREF(r)
  #define __Pyx_GIVEREF(r)
  #define __Pyx_XINCREF(r) Py_XINCREF(r)
  #define __Pyx_XDECREF(r) Py_XDECREF(r)
  #define __Pyx_XGOTREF(r)
  #define __Pyx_XGIVEREF(r)
#endif
#define __Pyx_XDECREF_SET(r, v) do {\
        PyObject *tmp = (PyObject *) r;\
        r = v; __Pyx_XDECREF(tmp);\
    } while (0)
#define __Pyx_DECREF_SET(r, v) do {\
        PyObject *tmp = (PyObject *) r;\
        r = v; __Pyx_DECREF(tmp);\
    } while (0)
#define __Pyx_CLEAR(r)    do { PyObject* tmp = ((PyObject*)(r)); r = NULL; __Pyx_DECREF(tmp);} while(0)
#define __Pyx_XCLEAR(r)   do { if((r) != NULL) {PyObject* tmp = ((PyObject*)(r)); r = NULL; __Pyx_DECREF(tmp);}} while(0)

#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStr(PyObject* obj, PyObject* attr_name) {
    PyTypeObject* tp = Py_TYPE(obj);
    if (likely(tp->tp_getattro))
        return tp->tp_getattro(obj, attr_name);
#if PY_MAJOR_VERSION < 3
    if (likely(tp->tp_getattr))
        return tp->tp_getattr(obj, PyString_AS_STRING(attr_name));
#endif
    return PyObject_GetAttr(obj, attr_name);
}
#else
#define __Pyx_PyObject_GetAttrStr(o,n) PyObject_GetAttr(o,n)
#endif

static PyObject *__Pyx_GetBuiltinName(PyObject *name);

static void __Pyx_RaiseArgtupleInvalid(const char* func_name, int exact,
    Py_ssize_t num_min, Py_ssize_t num_max, Py_ssize_t num_found);

static void __Pyx_RaiseDoubleKeywordsError(const char* func_name, PyObject* kw_name);

static int __Pyx_ParseOptionalKeywords(PyObject *kwds, PyObject **argnames[],\
    PyObject *kwds2, PyObject *values[], Py_ssize_t num_pos_args,\
    const char* function_name);

#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_Call(PyObject *func, PyObject *arg, PyObject *kw);
#else
#define __Pyx_PyObject_Call(func, arg, kw) PyObject_Call(func, arg, kw)
#endif

#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE int __Pyx_ListComp_Append(PyObject* list, PyObject* x) {
    PyListObject* L = (PyListObject*) list;
    Py_ssize_t len = Py_SIZE(list);
    if (likely(L->allocated > len)) {
        Py_INCREF(x);
        PyList_SET_ITEM(list, len, x);
        Py_SIZE(list) = len+1;
        return 0;
    }
    return PyList_Append(list, x);
}
#else
#define __Pyx_ListComp_Append(L,x) PyList_Append(L,x)
#endif

static CYTHON_INLINE int __Pyx_TypeTest(PyObject *obj, PyTypeObject *type);

#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallMethO(PyObject *func, PyObject *arg);
#endif

static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg);

#if CYTHON_COMPILING_IN_CPYTHON
#define __Pyx_PyObject_DelAttrStr(o,n) __Pyx_PyObject_SetAttrStr(o,n,NULL)
static CYTHON_INLINE int __Pyx_PyObject_SetAttrStr(PyObject* obj, PyObject* attr_name, PyObject* value) {
    PyTypeObject* tp = Py_TYPE(obj);
    if (likely(tp->tp_setattro))
        return tp->tp_setattro(obj, attr_name, value);
#if PY_MAJOR_VERSION < 3
    if (likely(tp->tp_setattr))
        return tp->tp_setattr(obj, PyString_AS_STRING(attr_name), value);
#endif
    return PyObject_SetAttr(obj, attr_name, value);
}
#else
#define __Pyx_PyObject_DelAttrStr(o,n)   PyObject_DelAttr(o,n)
#define __Pyx_PyObject_SetAttrStr(o,n,v) PyObject_SetAttr(o,n,v)
#endif

#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE int __Pyx_PyList_Append(PyObject* list, PyObject* x) {
    PyListObject* L = (PyListObject*) list;
    Py_ssize_t len = Py_SIZE(list);
    if (likely(L->allocated > len) & likely(len > (L->allocated >> 1))) {
        Py_INCREF(x);
        PyList_SET_ITEM(list, len, x);
        Py_SIZE(list) = len+1;
        return 0;
    }
    return PyList_Append(list, x);
}
#else
#define __Pyx_PyList_Append(L,x) PyList_Append(L,x)
#endif

static CYTHON_INLINE int __Pyx_PyList_Extend(PyObject* L, PyObject* v) {
#if CYTHON_COMPILING_IN_CPYTHON
    PyObject* none = _PyList_Extend((PyListObject*)L, v);
    if (unlikely(!none))
        return -1;
    Py_DECREF(none);
    return 0;
#else
    return PyList_SetSlice(L, PY_SSIZE_T_MAX, PY_SSIZE_T_MAX, v);
#endif
}

static CYTHON_INLINE PyObject* __Pyx_PyFrozenSet_New(PyObject* it) {
    if (it) {
        PyObject* result;
#if CYTHON_COMPILING_IN_PYPY
        PyObject* args;
        args = PyTuple_Pack(1, it);
        if (unlikely(!args))
            return NULL;
        result = PyObject_Call((PyObject*)&PyFrozenSet_Type, args, NULL);
        Py_DECREF(args);
        return result;
#else
        if (PyFrozenSet_CheckExact(it)) {
            Py_INCREF(it);
            return it;
        }
        result = PyFrozenSet_New(it);
        if (unlikely(!result))
            return NULL;
        if (likely(PySet_GET_SIZE(result)))
            return result;
        Py_DECREF(result);
#endif
    }
#if CYTHON_COMPILING_IN_CPYTHON
    return PyFrozenSet_Type.tp_new(&PyFrozenSet_Type, __pyx_empty_tuple, NULL);
#else
    return PyObject_Call((PyObject*)&PyFrozenSet_Type, __pyx_empty_tuple, NULL);
#endif
}

static CYTHON_INLINE int __Pyx_PySequence_ContainsTF(PyObject* item, PyObject* seq, int eq) {
    int result = PySequence_Contains(seq, item);
    return unlikely(result < 0) ? result : (result == (eq == Py_EQ));
}

static CYTHON_INLINE PyObject *__Pyx_GetModuleGlobalName(PyObject *name);

static CYTHON_INLINE void __Pyx_ErrRestore(PyObject *type, PyObject *value, PyObject *tb);
static CYTHON_INLINE void __Pyx_ErrFetch(PyObject **type, PyObject **value, PyObject **tb);

static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb, PyObject *cause);

static CYTHON_INLINE int __Pyx_ArgTypeTest(PyObject *obj, PyTypeObject *type, int none_allowed,
    const char *name, int exact);

static PyObject* __Pyx_PyDict_GetItemDefault(PyObject* d, PyObject* key, PyObject* default_value);

static void __Pyx_call_next_tp_dealloc(PyObject* obj, destructor current_tp_dealloc);

static int __Pyx_call_next_tp_traverse(PyObject* obj, visitproc v, void *a, traverseproc current_tp_traverse);

static void __Pyx_call_next_tp_clear(PyObject* obj, inquiry current_tp_dealloc);

static void* __Pyx_GetVtable(PyObject *dict);

static int __Pyx_SetVtable(PyObject *dict, void *vtable);

static PyObject *__Pyx_Import(PyObject *name, PyObject *from_list, int level);

static PyObject* __Pyx_ImportFrom(PyObject* module, PyObject* name);

static PyObject *__Pyx_GetNameInClass(PyObject *nmspace, PyObject *name);

static PyObject *__Pyx_CalculateMetaclass(PyTypeObject *metaclass, PyObject *bases);

static PyObject *__Pyx_Py3MetaclassPrepare(PyObject *metaclass, PyObject *bases, PyObject *name, PyObject *qualname,
                                           PyObject *mkw, PyObject *modname, PyObject *doc);
static PyObject *__Pyx_Py3ClassCreate(PyObject *metaclass, PyObject *name, PyObject *bases, PyObject *dict,
                                      PyObject *mkw, int calculate_metaclass, int allow_py2_metaclass);

typedef struct {
    int code_line;
    PyCodeObject* code_object;
} __Pyx_CodeObjectCacheEntry;
struct __Pyx_CodeObjectCache {
    int count;
    int max_count;
    __Pyx_CodeObjectCacheEntry* entries;
};
static struct __Pyx_CodeObjectCache __pyx_code_cache = {0,0,NULL};
static int __pyx_bisect_code_objects(__Pyx_CodeObjectCacheEntry* entries, int count, int code_line);
static PyCodeObject *__pyx_find_code_object(int code_line);
static void __pyx_insert_code_object(int code_line, PyCodeObject* code_object);

static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename);

#include "descrobject.h"
static PyObject* __Pyx_Method_ClassMethod(PyObject *method);

static CYTHON_INLINE PyObject* __Pyx_PyInt_From_long(long value);

static CYTHON_INLINE long __Pyx_PyInt_As_long(PyObject *);

static CYTHON_INLINE int __Pyx_PyInt_As_int(PyObject *);

static int __Pyx_check_binary_version(void);

#if !defined(__Pyx_PyIdentifier_FromString)
#if PY_MAJOR_VERSION < 3
  #define __Pyx_PyIdentifier_FromString(s) PyString_FromString(s)
#else
  #define __Pyx_PyIdentifier_FromString(s) PyUnicode_FromString(s)
#endif
#endif

static PyObject *__Pyx_ImportModule(const char *name);

static PyTypeObject *__Pyx_ImportType(const char *module_name, const char *class_name, size_t size, int strict);

static int __Pyx_ImportFunction(PyObject *module, const char *funcname, void (**f)(void), const char *sig);

static int __Pyx_InitStrings(__Pyx_StringTabEntry *t);

static PyObject *__pyx_f_8thriftrw_4spec_7service_18FunctionResultSpec_from_wire(struct __pyx_obj_8thriftrw_4spec_7service_FunctionResultSpec *__pyx_v_self, struct __pyx_obj_8thriftrw_4wire_5value_Value *__pyx_v_wire_value, int __pyx_skip_dispatch); /* proto*/
static struct __pyx_obj_8thriftrw_4spec_7service_FunctionSpec *__pyx_f_8thriftrw_4spec_7service_12FunctionSpec_link(struct __pyx_obj_8thriftrw_4spec_7service_FunctionSpec *__pyx_v_self, PyObject *__pyx_v_scope, struct __pyx_obj_8thriftrw_4spec_7service_ServiceSpec *__pyx_v_service, int __pyx_skip_dispatch); /* proto*/
static struct __pyx_obj_8thriftrw_4spec_7service_ServiceSpec *__pyx_f_8thriftrw_4spec_7service_11ServiceSpec_link(struct __pyx_obj_8thriftrw_4spec_7service_ServiceSpec *__pyx_v_self, PyObject *__pyx_v_scope, int __pyx_skip_dispatch); /* proto*/

/* Module declarations from 'libc.stdint' */

/* Module declarations from 'thriftrw.wire.value' */
static PyTypeObject *__pyx_ptype_8thriftrw_4wire_5value_ValueVisitor = 0;
static PyTypeObject *__pyx_ptype_8thriftrw_4wire_5value__ValueVisitorArgs = 0;
static PyTypeObject *__pyx_ptype_8thriftrw_4wire_5value_Value = 0;
static PyTypeObject *__pyx_ptype_8thriftrw_4wire_5value_BoolValue = 0;
static PyTypeObject *__pyx_ptype_8thriftrw_4wire_5value_ByteValue = 0;
static PyTypeObject *__pyx_ptype_8thriftrw_4wire_5value_DoubleValue = 0;
static PyTypeObject *__pyx_ptype_8thriftrw_4wire_5value_I16Value = 0;
static PyTypeObject *__pyx_ptype_8thriftrw_4wire_5value_I32Value = 0;
static PyTypeObject *__pyx_ptype_8thriftrw_4wire_5value_I64Value = 0;
static PyTypeObject *__pyx_ptype_8thriftrw_4wire_5value_BinaryValue = 0;
static PyTypeObject *__pyx_ptype_8thriftrw_4wire_5value_FieldValue = 0;
static PyTypeObject *__pyx_ptype_8thriftrw_4wire_5value_StructValue = 0;
static PyTypeObject *__pyx_ptype_8thriftrw_4wire_5value_MapItem = 0;
static PyTypeObject *__pyx_ptype_8thriftrw_4wire_5value_MapValue = 0;
static PyTypeObject *__pyx_ptype_8thriftrw_4wire_5value_SetValue = 0;
static PyTypeObject *__pyx_ptype_8thriftrw_4wire_5value_ListValue = 0;

/* Module declarations from 'thriftrw.spec.base' */
static PyTypeObject *__pyx_ptype_8thriftrw_4spec_4base_TypeSpec = 0;

/* Module declarations from 'thriftrw.spec.struct' */
static PyTypeObject *__pyx_ptype_8thriftrw_4spec_6struct_StructTypeSpec = 0;

/* Module declarations from 'thriftrw.spec.union' */
static PyTypeObject *__pyx_ptype_8thriftrw_4spec_5union_UnionTypeSpec = 0;

/* Module declarations from 'thriftrw.spec.field' */
static PyTypeObject *__pyx_ptype_8thriftrw_4spec_5field_FieldSpec = 0;

/* Module declarations from 'thriftrw.spec' */

/* Module declarations from 'thriftrw.spec.check' */
static PyObject *(*__pyx_f_8thriftrw_4spec_5check_type_code_matches)(struct __pyx_obj_8thriftrw_4spec_4base_TypeSpec *, struct __pyx_obj_8thriftrw_4wire_5value_Value *, int __pyx_skip_dispatch); /*proto*/

/* Module declarations from 'thriftrw.spec.service' */
static PyTypeObject *__pyx_ptype_8thriftrw_4spec_7service_FunctionArgsSpec = 0;
static PyTypeObject *__pyx_ptype_8thriftrw_4spec_7service_FunctionResultSpec = 0;
static PyTypeObject *__pyx_ptype_8thriftrw_4spec_7service_FunctionSpec = 0;
static PyTypeObject *__pyx_ptype_8thriftrw_4spec_7service_ServiceSpec = 0;
#define __Pyx_MODULE_NAME "thriftrw.spec.service"
int __pyx_module_is_main_thriftrw__spec__service = 0;

/* Implementation of 'thriftrw.spec.service' */
static PyObject *__pyx_builtin_super;
static PyObject *__pyx_builtin_object;
static char __pyx_k_id[] = "id";
static char __pyx_k_all[] = "__all__";
static char __pyx_k_doc[] = "__doc__";
static char __pyx_k_get[] = "get";
static char __pyx_k_func[] = "func";
static char __pyx_k_init[] = "__init__";
static char __pyx_k_link[] = "link";
static char __pyx_k_main[] = "__main__";
static char __pyx_k_name[] = "name";
static char __pyx_k_spec[] = "spec";
static char __pyx_k_test[] = "__test__";
static char __pyx_k_field[] = "field";
static char __pyx_k_scope[] = "scope";
static char __pyx_k_slots[] = "__slots__";
static char __pyx_k_super[] = "super";
static char __pyx_k_utf_8[] = "utf-8";
static char __pyx_k_encode[] = "encode";
static char __pyx_k_errors[] = "errors";
static char __pyx_k_fields[] = "fields";
static char __pyx_k_import[] = "__import__";
static char __pyx_k_lineno[] = "lineno";
static char __pyx_k_module[] = "__module__";
static char __pyx_k_object[] = "object";
static char __pyx_k_oneway[] = "oneway";
static char __pyx_k_params[] = "params";
static char __pyx_k_parent[] = "parent";
static char __pyx_k_compile[] = "compile";
static char __pyx_k_prepare[] = "__prepare__";
static char __pyx_k_service[] = "service";
static char __pyx_k_success[] = "success";
static char __pyx_k_surface[] = "surface";
static char __pyx_k_function[] = "function";
static char __pyx_k_qualname[] = "__qualname__";
static char __pyx_k_required[] = "required";
static char __pyx_k_args_spec[] = "args_spec";
static char __pyx_k_from_wire[] = "from_wire";
static char __pyx_k_functions[] = "functions";
static char __pyx_k_metaclass[] = "__metaclass__";
static char __pyx_k_exceptions[] = "exceptions";
static char __pyx_k_namedtuple[] = "namedtuple";
static char __pyx_k_parameters[] = "parameters";
static char __pyx_k_parent_cls[] = "parent_cls";
static char __pyx_k_pyx_vtable[] = "__pyx_vtable__";
static char __pyx_k_ServiceSpec[] = "ServiceSpec";
static char __pyx_k_allow_empty[] = "allow_empty";
static char __pyx_k_collections[] = "collections";
static char __pyx_k_result_spec[] = "result_spec";
static char __pyx_k_result_type[] = "result_type";
static char __pyx_k_return_spec[] = "return_spec";
static char __pyx_k_return_type[] = "return_type";
static char __pyx_k_service_cls[] = "service_cls";
static char __pyx_k_service_dct[] = "service_dct";
static char __pyx_k_spec_mapper[] = "spec_mapper";
static char __pyx_k_struct_name[] = "struct_name";
static char __pyx_k_FunctionSpec[] = "FunctionSpec";
static char __pyx_k_s__s_request[] = "%s_%s_request";
static char __pyx_k_service_name[] = "service_name";
static char __pyx_k_service_spec[] = "service_spec";
static char __pyx_k_default_value[] = "default_value";
static char __pyx_k_function_name[] = "function_name";
static char __pyx_k_s__s_response[] = "%s_%s_response";
static char __pyx_k_ServiceFunction[] = "ServiceFunction";
static char __pyx_k_FunctionArgsSpec[] = "FunctionArgsSpec";
static char __pyx_k_type_spec_or_ref[] = "type_spec_or_ref";
static char __pyx_k_FunctionResultSpec[] = "FunctionResultSpec";
static char __pyx_k_ThriftCompilerError[] = "ThriftCompilerError";
static char __pyx_k_require_requiredness[] = "require_requiredness";
static char __pyx_k_resolve_service_spec[] = "resolve_service_spec";
static char __pyx_k_UnknownExceptionError[] = "UnknownExceptionError";
static char __pyx_k_thriftrw_spec_service[] = "thriftrw.spec.service";
static char __pyx_k_name_request_response_spec[] = "name request response spec";
static char __pyx_k_FunctionSpec_name_r_args_spec_r[] = "FunctionSpec(name=%r, args_spec=%r, result_spec=%r)";
static char __pyx_k_Represents_a_single_function_on[] = "Represents a single function on a service.\n\n    .. py:attribute:: name\n\n        Name of the function.\n\n    .. py:attribute:: request\n\n        Class representing requests for this function.\n\n    .. py:attribute:: response\n\n        Class representing responses for this function, or ``None`` if this\n        function is ``oneway``.\n\n    .. py:attribute:: spec\n\n        :py:class:`thriftrw.spec.FunctionSpec` for this function.\n\n    .. versionchanged:: 0.3\n\n        Added the ``spec`` attribute.\n    ";
static char __pyx_k_private_var_folders_8z_qdzjsr3n[] = "/private/var/folders/8z/qdzjsr3n5l72vdg67zr6xkjc0000gn/T/thriftrw-1.2.4-4x2dc1/gitclone/thriftrw/spec/service.pyx";
static char __pyx_k_s_received_an_unrecognized_exce[] = "\"%s\" received an unrecognized exception. Make sure your Thrift IDL is up-to-date with what the remote host is using.";
static char __pyx_k_Function_s_s_cannot_be_defined_T[] = "Function \"%s.%s\" cannot be defined. That name is already taken.";
static char __pyx_k_Function_s_s_is_oneway_It_cannot[] = "Function \"%s.%s\" is oneway. It cannot return a value.";
static char __pyx_k_ServiceSpec_name_r_functions_r_p[] = "ServiceSpec(name=%r, functions=%r, parent=%r)";
static char __pyx_k_Function_s_s_is_oneway_It_cannot_2[] = "Function \"%s.%s\" is oneway. It cannot raise exceptions.";
static PyObject *__pyx_n_u_FunctionArgsSpec;
static PyObject *__pyx_n_u_FunctionResultSpec;
static PyObject *__pyx_n_u_FunctionSpec;
static PyObject *__pyx_kp_u_FunctionSpec_name_r_args_spec_r;
static PyObject *__pyx_kp_u_Function_s_s_cannot_be_defined_T;
static PyObject *__pyx_kp_u_Function_s_s_is_oneway_It_cannot;
static PyObject *__pyx_kp_u_Function_s_s_is_oneway_It_cannot_2;
static PyObject *__pyx_kp_s_Represents_a_single_function_on;
static PyObject *__pyx_n_s_ServiceFunction;
static PyObject *__pyx_n_u_ServiceFunction;
static PyObject *__pyx_n_u_ServiceSpec;
static PyObject *__pyx_kp_u_ServiceSpec_name_r_functions_r_p;
static PyObject *__pyx_n_s_ThriftCompilerError;
static PyObject *__pyx_n_s_UnknownExceptionError;
static PyObject *__pyx_n_s_all;
static PyObject *__pyx_n_s_allow_empty;
static PyObject *__pyx_n_s_args_spec;
static PyObject *__pyx_n_s_collections;
static PyObject *__pyx_n_s_compile;
static PyObject *__pyx_n_s_default_value;
static PyObject *__pyx_n_s_doc;
static PyObject *__pyx_n_s_encode;
static PyObject *__pyx_n_s_errors;
static PyObject *__pyx_n_s_exceptions;
static PyObject *__pyx_n_s_field;
static PyObject *__pyx_n_s_fields;
static PyObject *__pyx_n_s_from_wire;
static PyObject *__pyx_n_s_func;
static PyObject *__pyx_n_s_function;
static PyObject *__pyx_n_s_function_name;
static PyObject *__pyx_n_s_functions;
static PyObject *__pyx_n_s_get;
static PyObject *__pyx_n_s_id;
static PyObject *__pyx_n_s_import;
static PyObject *__pyx_n_s_init;
static PyObject *__pyx_n_s_lineno;
static PyObject *__pyx_n_s_link;
static PyObject *__pyx_n_s_main;
static PyObject *__pyx_n_s_metaclass;
static PyObject *__pyx_n_s_module;
static PyObject *__pyx_n_s_name;
static PyObject *__pyx_kp_u_name_request_response_spec;
static PyObject *__pyx_n_s_namedtuple;
static PyObject *__pyx_n_s_object;
static PyObject *__pyx_n_s_oneway;
static PyObject *__pyx_n_s_parameters;
static PyObject *__pyx_n_s_params;
static PyObject *__pyx_n_s_parent;
static PyObject *__pyx_n_s_parent_cls;
static PyObject *__pyx_n_s_prepare;
static PyObject *__pyx_kp_s_private_var_folders_8z_qdzjsr3n;
static PyObject *__pyx_n_s_pyx_vtable;
static PyObject *__pyx_n_s_qualname;
static PyObject *__pyx_n_s_require_requiredness;
static PyObject *__pyx_n_s_required;
static PyObject *__pyx_n_s_resolve_service_spec;
static PyObject *__pyx_n_s_result_spec;
static PyObject *__pyx_n_s_result_type;
static PyObject *__pyx_n_s_return_spec;
static PyObject *__pyx_n_s_return_type;
static PyObject *__pyx_kp_u_s__s_request;
static PyObject *__pyx_kp_u_s__s_response;
static PyObject *__pyx_kp_u_s_received_an_unrecognized_exce;
static PyObject *__pyx_n_s_scope;
static PyObject *__pyx_n_s_service;
static PyObject *__pyx_n_s_service_cls;
static PyObject *__pyx_n_s_service_dct;
static PyObject *__pyx_n_s_service_name;
static PyObject *__pyx_n_s_service_spec;
static PyObject *__pyx_n_u_service_spec;
static PyObject *__pyx_n_u_slots;
static PyObject *__pyx_n_s_spec;
static PyObject *__pyx_n_s_spec_mapper;
static PyObject *__pyx_n_s_struct_name;
static PyObject *__pyx_n_u_success;
static PyObject *__pyx_n_s_super;
static PyObject *__pyx_n_s_surface;
static PyObject *__pyx_n_s_test;
static PyObject *__pyx_n_s_thriftrw_spec_service;
static PyObject *__pyx_n_s_type_spec_or_ref;
static PyObject *__pyx_kp_u_utf_8;
static int __pyx_pf_8thriftrw_4spec_7service_16FunctionArgsSpec___init__(struct __pyx_obj_8thriftrw_4spec_7service_FunctionArgsSpec *__pyx_v_self, PyObject *__pyx_v_name, PyObject *__pyx_v_params); /* proto */
static PyObject *__pyx_pf_8thriftrw_4spec_7service_16FunctionArgsSpec_2compile(PyTypeObject *__pyx_v_cls, PyObject *__pyx_v_parameters, PyObject *__pyx_v_service_name, PyObject *__pyx_v_function_name); /* proto */
static PyObject *__pyx_pf_8thriftrw_4spec_7service_16FunctionArgsSpec_4link(struct __pyx_obj_8thriftrw_4spec_7service_FunctionArgsSpec *__pyx_v_self, PyObject *__pyx_v_scope, PyObject *__pyx_v_function); /* proto */
static PyObject *__pyx_pf_8thriftrw_4spec_7service_16FunctionArgsSpec_8function___get__(struct __pyx_obj_8thriftrw_4spec_7service_FunctionArgsSpec *__pyx_v_self); /* proto */
static int __pyx_pf_8thriftrw_4spec_7service_16FunctionArgsSpec_8function_2__set__(struct __pyx_obj_8thriftrw_4spec_7service_FunctionArgsSpec *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_8thriftrw_4spec_7service_16FunctionArgsSpec_8function_4__del__(struct __pyx_obj_8thriftrw_4spec_7service_FunctionArgsSpec *__pyx_v_self); /* proto */
static int __pyx_pf_8thriftrw_4spec_7service_18FunctionResultSpec___init__(struct __pyx_obj_8thriftrw_4spec_7service_FunctionResultSpec *__pyx_v_self, PyObject *__pyx_v_name, PyObject *__pyx_v_return_spec, PyObject *__pyx_v_exceptions); /* proto */
static PyObject *__pyx_pf_8thriftrw_4spec_7service_18FunctionResultSpec_2from_wire(struct __pyx_obj_8thriftrw_4spec_7service_FunctionResultSpec *__pyx_v_self, struct __pyx_obj_8thriftrw_4wire_5value_Value *__pyx_v_wire_value); /* proto */
static PyObject *__pyx_pf_8thriftrw_4spec_7service_18FunctionResultSpec_4link(struct __pyx_obj_8thriftrw_4spec_7service_FunctionResultSpec *__pyx_v_self, PyObject *__pyx_v_scope, PyObject *__pyx_v_function); /* proto */
static PyObject *__pyx_pf_8thriftrw_4spec_7service_18FunctionResultSpec_6compile(PyTypeObject *__pyx_v_cls, PyObject *__pyx_v_return_type, PyObject *__pyx_v_exceptions, PyObject *__pyx_v_service_name, PyObject *__pyx_v_function_name); /* proto */
static PyObject *__pyx_pf_8thriftrw_4spec_7service_18FunctionResultSpec_11return_spec___get__(struct __pyx_obj_8thriftrw_4spec_7service_FunctionResultSpec *__pyx_v_self); /* proto */
static int __pyx_pf_8thriftrw_4spec_7service_18FunctionResultSpec_11return_spec_2__set__(struct __pyx_obj_8thriftrw_4spec_7service_FunctionResultSpec *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_8thriftrw_4spec_7service_18FunctionResultSpec_11return_spec_4__del__(struct __pyx_obj_8thriftrw_4spec_7service_FunctionResultSpec *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_8thriftrw_4spec_7service_18FunctionResultSpec_15exception_specs___get__(struct __pyx_obj_8thriftrw_4spec_7service_FunctionResultSpec *__pyx_v_self); /* proto */
static int __pyx_pf_8thriftrw_4spec_7service_18FunctionResultSpec_15exception_specs_2__set__(struct __pyx_obj_8thriftrw_4spec_7service_FunctionResultSpec *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_8thriftrw_4spec_7service_18FunctionResultSpec_15exception_specs_4__del__(struct __pyx_obj_8thriftrw_4spec_7service_FunctionResultSpec *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_8thriftrw_4spec_7service_18FunctionResultSpec_8function___get__(struct __pyx_obj_8thriftrw_4spec_7service_FunctionResultSpec *__pyx_v_self); /* proto */
static int __pyx_pf_8thriftrw_4spec_7service_18FunctionResultSpec_8function_2__set__(struct __pyx_obj_8thriftrw_4spec_7service_FunctionResultSpec *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_8thriftrw_4spec_7service_18FunctionResultSpec_8function_4__del__(struct __pyx_obj_8thriftrw_4spec_7service_FunctionResultSpec *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_8thriftrw_4spec_7service_18FunctionResultSpec_13exception_ids___get__(struct __pyx_obj_8thriftrw_4spec_7service_FunctionResultSpec *__pyx_v_self); /* proto */
static int __pyx_pf_8thriftrw_4spec_7service_18FunctionResultSpec_13exception_ids_2__set__(struct __pyx_obj_8thriftrw_4spec_7service_FunctionResultSpec *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_8thriftrw_4spec_7service_18FunctionResultSpec_13exception_ids_4__del__(struct __pyx_obj_8thriftrw_4spec_7service_FunctionResultSpec *__pyx_v_self); /* proto */
static int __pyx_pf_8thriftrw_4spec_7service_12FunctionSpec___init__(struct __pyx_obj_8thriftrw_4spec_7service_FunctionSpec *__pyx_v_self, PyObject *__pyx_v_name, PyObject *__pyx_v_args_spec, PyObject *__pyx_v_result_spec, PyObject *__pyx_v_oneway); /* proto */
static PyObject *__pyx_pf_8thriftrw_4spec_7service_12FunctionSpec_2compile(PyTypeObject *__pyx_v_cls, PyObject *__pyx_v_func, PyObject *__pyx_v_service_name); /* proto */
static PyObject *__pyx_pf_8thriftrw_4spec_7service_12FunctionSpec_4link(struct __pyx_obj_8thriftrw_4spec_7service_FunctionSpec *__pyx_v_self, PyObject *__pyx_v_scope, struct __pyx_obj_8thriftrw_4spec_7service_ServiceSpec *__pyx_v_service); /* proto */
static PyObject *__pyx_pf_8thriftrw_4spec_7service_12FunctionSpec_6__str__(struct __pyx_obj_8thriftrw_4spec_7service_FunctionSpec *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_8thriftrw_4spec_7service_12FunctionSpec_8__repr__(struct __pyx_obj_8thriftrw_4spec_7service_FunctionSpec *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_8thriftrw_4spec_7service_12FunctionSpec_4name___get__(struct __pyx_obj_8thriftrw_4spec_7service_FunctionSpec *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_8thriftrw_4spec_7service_12FunctionSpec_9args_spec___get__(struct __pyx_obj_8thriftrw_4spec_7service_FunctionSpec *__pyx_v_self); /* proto */
static int __pyx_pf_8thriftrw_4spec_7service_12FunctionSpec_9args_spec_2__set__(struct __pyx_obj_8thriftrw_4spec_7service_FunctionSpec *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_8thriftrw_4spec_7service_12FunctionSpec_9args_spec_4__del__(struct __pyx_obj_8thriftrw_4spec_7service_FunctionSpec *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_8thriftrw_4spec_7service_12FunctionSpec_11result_spec___get__(struct __pyx_obj_8thriftrw_4spec_7service_FunctionSpec *__pyx_v_self); /* proto */
static int __pyx_pf_8thriftrw_4spec_7service_12FunctionSpec_11result_spec_2__set__(struct __pyx_obj_8thriftrw_4spec_7service_FunctionSpec *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_8thriftrw_4spec_7service_12FunctionSpec_11result_spec_4__del__(struct __pyx_obj_8thriftrw_4spec_7service_FunctionSpec *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_8thriftrw_4spec_7service_12FunctionSpec_6oneway___get__(struct __pyx_obj_8thriftrw_4spec_7service_FunctionSpec *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_8thriftrw_4spec_7service_12FunctionSpec_6linked___get__(struct __pyx_obj_8thriftrw_4spec_7service_FunctionSpec *__pyx_v_self); /* proto */
static int __pyx_pf_8thriftrw_4spec_7service_12FunctionSpec_6linked_2__set__(struct __pyx_obj_8thriftrw_4spec_7service_FunctionSpec *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_8thriftrw_4spec_7service_12FunctionSpec_7surface___get__(struct __pyx_obj_8thriftrw_4spec_7service_FunctionSpec *__pyx_v_self); /* proto */
static int __pyx_pf_8thriftrw_4spec_7service_12FunctionSpec_7surface_2__set__(struct __pyx_obj_8thriftrw_4spec_7service_FunctionSpec *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_8thriftrw_4spec_7service_12FunctionSpec_7surface_4__del__(struct __pyx_obj_8thriftrw_4spec_7service_FunctionSpec *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_8thriftrw_4spec_7service_12FunctionSpec_7service___get__(struct __pyx_obj_8thriftrw_4spec_7service_FunctionSpec *__pyx_v_self); /* proto */
static int __pyx_pf_8thriftrw_4spec_7service_12FunctionSpec_7service_2__set__(struct __pyx_obj_8thriftrw_4spec_7service_FunctionSpec *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_8thriftrw_4spec_7service_12FunctionSpec_7service_4__del__(struct __pyx_obj_8thriftrw_4spec_7service_FunctionSpec *__pyx_v_self); /* proto */
static int __pyx_pf_8thriftrw_4spec_7service_11ServiceSpec___init__(struct __pyx_obj_8thriftrw_4spec_7service_ServiceSpec *__pyx_v_self, PyObject *__pyx_v_name, PyObject *__pyx_v_functions, PyObject *__pyx_v_parent); /* proto */
static PyObject *__pyx_pf_8thriftrw_4spec_7service_11ServiceSpec_2compile(PyTypeObject *__pyx_v_cls, PyObject *__pyx_v_service); /* proto */
static PyObject *__pyx_pf_8thriftrw_4spec_7service_11ServiceSpec_4link(struct __pyx_obj_8thriftrw_4spec_7service_ServiceSpec *__pyx_v_self, PyObject *__pyx_v_scope); /* proto */
static PyObject *__pyx_pf_8thriftrw_4spec_7service_11ServiceSpec_6lookup(struct __pyx_obj_8thriftrw_4spec_7service_ServiceSpec *__pyx_v_self, PyObject *__pyx_v_name); /* proto */
static PyObject *__pyx_pf_8thriftrw_4spec_7service_11ServiceSpec_8__str__(struct __pyx_obj_8thriftrw_4spec_7service_ServiceSpec *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_8thriftrw_4spec_7service_11ServiceSpec_10__repr__(struct __pyx_obj_8thriftrw_4spec_7service_ServiceSpec *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_8thriftrw_4spec_7service_11ServiceSpec_4name___get__(struct __pyx_obj_8thriftrw_4spec_7service_ServiceSpec *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_8thriftrw_4spec_7service_11ServiceSpec_9functions___get__(struct __pyx_obj_8thriftrw_4spec_7service_ServiceSpec *__pyx_v_self); /* proto */
static int __pyx_pf_8thriftrw_4spec_7service_11ServiceSpec_9functions_2__set__(struct __pyx_obj_8thriftrw_4spec_7service_ServiceSpec *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_8thriftrw_4spec_7service_11ServiceSpec_9functions_4__del__(struct __pyx_obj_8thriftrw_4spec_7service_ServiceSpec *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_8thriftrw_4spec_7service_11ServiceSpec_6parent___get__(struct __pyx_obj_8thriftrw_4spec_7service_ServiceSpec *__pyx_v_self); /* proto */
static int __pyx_pf_8thriftrw_4spec_7service_11ServiceSpec_6parent_2__set__(struct __pyx_obj_8thriftrw_4spec_7service_ServiceSpec *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_8thriftrw_4spec_7service_11ServiceSpec_6parent_4__del__(struct __pyx_obj_8thriftrw_4spec_7service_ServiceSpec *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_8thriftrw_4spec_7service_11ServiceSpec_10_functions___get__(struct __pyx_obj_8thriftrw_4spec_7service_ServiceSpec *__pyx_v_self); /* proto */
static int __pyx_pf_8thriftrw_4spec_7service_11ServiceSpec_10_functions_2__set__(struct __pyx_obj_8thriftrw_4spec_7service_ServiceSpec *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_8thriftrw_4spec_7service_11ServiceSpec_10_functions_4__del__(struct __pyx_obj_8thriftrw_4spec_7service_ServiceSpec *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_8thriftrw_4spec_7service_11ServiceSpec_6linked___get__(struct __pyx_obj_8thriftrw_4spec_7service_ServiceSpec *__pyx_v_self); /* proto */
static int __pyx_pf_8thriftrw_4spec_7service_11ServiceSpec_6linked_2__set__(struct __pyx_obj_8thriftrw_4spec_7service_ServiceSpec *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_8thriftrw_4spec_7service_11ServiceSpec_7surface___get__(struct __pyx_obj_8thriftrw_4spec_7service_ServiceSpec *__pyx_v_self); /* proto */
static int __pyx_pf_8thriftrw_4spec_7service_11ServiceSpec_7surface_2__set__(struct __pyx_obj_8thriftrw_4spec_7service_ServiceSpec *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_8thriftrw_4spec_7service_11ServiceSpec_7surface_4__del__(struct __pyx_obj_8thriftrw_4spec_7service_ServiceSpec *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_8thriftrw_4spec_7service_service_cls(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_service_spec, CYTHON_UNUSED PyObject *__pyx_v_scope); /* proto */
static PyObject *__pyx_tp_new_8thriftrw_4spec_7service_FunctionArgsSpec(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_8thriftrw_4spec_7service_FunctionResultSpec(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_8thriftrw_4spec_7service_FunctionSpec(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_8thriftrw_4spec_7service_ServiceSpec(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_int_0;
static PyObject *__pyx_tuple_;
static PyObject *__pyx_tuple__2;
static PyObject *__pyx_tuple__3;
static PyObject *__pyx_tuple__4;
static PyObject *__pyx_codeobj__5;

/* "thriftrw/spec/service.pyx":72
 *     """
 * 
 *     def __init__(self, name, params):             # <<<<<<<<<<<<<<
 *         super(FunctionArgsSpec, self).__init__(name, params)
 *         self.function = None
 */

/* Python wrapper */
static int __pyx_pw_8thriftrw_4spec_7service_16FunctionArgsSpec_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_8thriftrw_4spec_7service_16FunctionArgsSpec_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_name = 0;
  PyObject *__pyx_v_params = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_name,&__pyx_n_s_params,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_name)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_params)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__init__", 1, 2, 2, 1); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 72; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__init__") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 72; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_name = values[0];
    __pyx_v_params = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 72; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("thriftrw.spec.service.FunctionArgsSpec.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_8thriftrw_4spec_7service_16FunctionArgsSpec___init__(((struct __pyx_obj_8thriftrw_4spec_7service_FunctionArgsSpec *)__pyx_v_self), __pyx_v_name, __pyx_v_params);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_8thriftrw_4spec_7service_16FunctionArgsSpec___init__(struct __pyx_obj_8thriftrw_4spec_7service_FunctionArgsSpec *__pyx_v_self, PyObject *__pyx_v_name, PyObject *__pyx_v_params) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  Py_ssize_t __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "thriftrw/spec/service.pyx":73
 * 
 *     def __init__(self, name, params):
 *         super(FunctionArgsSpec, self).__init__(name, params)             # <<<<<<<<<<<<<<
 *         self.function = None
 * 
 */
  __pyx_t_2 = PyTuple_New(2); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 73; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(((PyObject *)__pyx_ptype_8thriftrw_4spec_7service_FunctionArgsSpec));
  __Pyx_GIVEREF(((PyObject *)__pyx_ptype_8thriftrw_4spec_7service_FunctionArgsSpec));
  PyTuple_SET_ITEM(__pyx_t_2, 0, ((PyObject *)__pyx_ptype_8thriftrw_4spec_7service_FunctionArgsSpec));
  __Pyx_INCREF(((PyObject *)__pyx_v_self));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self));
  PyTuple_SET_ITEM(__pyx_t_2, 1, ((PyObject *)__pyx_v_self));
  __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_super, __pyx_t_2, NULL); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 73; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_init); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 73; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = NULL;
  __pyx_t_4 = 0;
  if (CYTHON_COMPILING_IN_CPYTHON && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_4 = 1;
    }
  }
  __pyx_t_5 = PyTuple_New(2+__pyx_t_4); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 73; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  if (__pyx_t_3) {
    __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_3); __pyx_t_3 = NULL;
  }
  __Pyx_INCREF(__pyx_v_name);
  __Pyx_GIVEREF(__pyx_v_name);
  PyTuple_SET_ITEM(__pyx_t_5, 0+__pyx_t_4, __pyx_v_name);
  __Pyx_INCREF(__pyx_v_params);
  __Pyx_GIVEREF(__pyx_v_params);
  PyTuple_SET_ITEM(__pyx_t_5, 1+__pyx_t_4, __pyx_v_params);
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_5, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 73; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "thriftrw/spec/service.pyx":74
 *     def __init__(self, name, params):
 *         super(FunctionArgsSpec, self).__init__(name, params)
 *         self.function = None             # <<<<<<<<<<<<<<
 * 
 *     @classmethod
 */
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->function);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->function));
  __pyx_v_self->function = ((struct __pyx_obj_8thriftrw_4spec_7service_FunctionSpec *)Py_None);

  /* "thriftrw/spec/service.pyx":72
 *     """
 * 
 *     def __init__(self, name, params):             # <<<<<<<<<<<<<<
 *         super(FunctionArgsSpec, self).__init__(name, params)
 *         self.function = None
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("thriftrw.spec.service.FunctionArgsSpec.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "thriftrw/spec/service.pyx":77
 * 
 *     @classmethod
 *     def compile(cls, parameters, service_name, function_name):             # <<<<<<<<<<<<<<
 *         """Compiles a parameter list into a FunctionArgsSpec.
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_8thriftrw_4spec_7service_16FunctionArgsSpec_3compile(PyObject *__pyx_v_cls, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_8thriftrw_4spec_7service_16FunctionArgsSpec_2compile[] = "Compiles a parameter list into a FunctionArgsSpec.\n\n        :param parameters:\n            Collection of :py:class:`thriftrw.idl.Field` objects.\n        :param str service_name:\n            Name of the service under which the function was defined.\n        :param str function_name:\n            Name of the function whose parameter list is represented by this\n            object.\n        ";
static PyObject *__pyx_pw_8thriftrw_4spec_7service_16FunctionArgsSpec_3compile(PyObject *__pyx_v_cls, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_parameters = 0;
  PyObject *__pyx_v_service_name = 0;
  PyObject *__pyx_v_function_name = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("compile (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_parameters,&__pyx_n_s_service_name,&__pyx_n_s_function_name,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_parameters)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_service_name)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("compile", 1, 3, 3, 1); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 77; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_function_name)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("compile", 1, 3, 3, 2); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 77; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "compile") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 77; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v_parameters = values[0];
    __pyx_v_service_name = values[1];
    __pyx_v_function_name = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("compile", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 77; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("thriftrw.spec.service.FunctionArgsSpec.compile", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_8thriftrw_4spec_7service_16FunctionArgsSpec_2compile(((PyTypeObject*)__pyx_v_cls), __pyx_v_parameters, __pyx_v_service_name, __pyx_v_function_name);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_8thriftrw_4spec_7service_16FunctionArgsSpec_2compile(PyTypeObject *__pyx_v_cls, PyObject *__pyx_v_parameters, PyObject *__pyx_v_service_name, PyObject *__pyx_v_function_name) {
  PyObject *__pyx_v_args_name = NULL;
  PyObject *__pyx_v_param_specs = NULL;
  PyObject *__pyx_v_param = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  Py_ssize_t __pyx_t_3;
  PyObject *(*__pyx_t_4)(PyObject *);
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("compile", 0);

  /* "thriftrw/spec/service.pyx":88
 *             object.
 *         """
 *         args_name = str('%s_%s_request' % (service_name, function_name))             # <<<<<<<<<<<<<<
 *         param_specs = [
 *             FieldSpec.compile(
 */
  __pyx_t_1 = PyTuple_New(2); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 88; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_v_service_name);
  __Pyx_GIVEREF(__pyx_v_service_name);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_v_service_name);
  __Pyx_INCREF(__pyx_v_function_name);
  __Pyx_GIVEREF(__pyx_v_function_name);
  PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_v_function_name);
  __pyx_t_2 = PyUnicode_Format(__pyx_kp_u_s__s_request, __pyx_t_1); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 88; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyTuple_New(1); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 88; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_2);
  __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_Call(((PyObject *)(&PyString_Type)), __pyx_t_1, NULL); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 88; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_args_name = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "thriftrw/spec/service.pyx":89
 *         """
 *         args_name = str('%s_%s_request' % (service_name, function_name))
 *         param_specs = [             # <<<<<<<<<<<<<<
 *             FieldSpec.compile(
 *                 field=param,
 */
  __pyx_t_2 = PyList_New(0); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 89; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);

  /* "thriftrw/spec/service.pyx":94
 *                 struct_name=args_name,
 *                 require_requiredness=False,
 *             ) for param in parameters             # <<<<<<<<<<<<<<
 *         ]
 * 
 */
  if (likely(PyList_CheckExact(__pyx_v_parameters)) || PyTuple_CheckExact(__pyx_v_parameters)) {
    __pyx_t_1 = __pyx_v_parameters; __Pyx_INCREF(__pyx_t_1); __pyx_t_3 = 0;
    __pyx_t_4 = NULL;
  } else {
    __pyx_t_3 = -1; __pyx_t_1 = PyObject_GetIter(__pyx_v_parameters); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 94; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_4 = Py_TYPE(__pyx_t_1)->tp_iternext; if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 94; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  for (;;) {
    if (likely(!__pyx_t_4)) {
      if (likely(PyList_CheckExact(__pyx_t_1))) {
        if (__pyx_t_3 >= PyList_GET_SIZE(__pyx_t_1)) break;
        #if CYTHON_COMPILING_IN_CPYTHON
        __pyx_t_5 = PyList_GET_ITEM(__pyx_t_1, __pyx_t_3); __Pyx_INCREF(__pyx_t_5); __pyx_t_3++; if (unlikely(0 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 94; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        #else
        __pyx_t_5 = PySequence_ITEM(__pyx_t_1, __pyx_t_3); __pyx_t_3++; if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 94; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_5);
        #endif
      } else {
        if (__pyx_t_3 >= PyTuple_GET_SIZE(__pyx_t_1)) break;
        #if CYTHON_COMPILING_IN_CPYTHON
        __pyx_t_5 = PyTuple_GET_ITEM(__pyx_t_1, __pyx_t_3); __Pyx_INCREF(__pyx_t_5); __pyx_t_3++; if (unlikely(0 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 94; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        #else
        __pyx_t_5 = PySequence_ITEM(__pyx_t_1, __pyx_t_3); __pyx_t_3++; if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 94; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_5);
        #endif
      }
    } else {
      __pyx_t_5 = __pyx_t_4(__pyx_t_1);
      if (unlikely(!__pyx_t_5)) {
        PyObject* exc_type = PyErr_Occurred();
        if (exc_type) {
          if (likely(exc_type == PyExc_StopIteration || PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
          else {__pyx_filename = __pyx_f[0]; __pyx_lineno = 94; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        }
        break;
      }
      __Pyx_GOTREF(__pyx_t_5);
    }
    __Pyx_XDECREF_SET(__pyx_v_param, __pyx_t_5);
    __pyx_t_5 = 0;

    /* "thriftrw/spec/service.pyx":90
 *         args_name = str('%s_%s_request' % (service_name, function_name))
 *         param_specs = [
 *             FieldSpec.compile(             # <<<<<<<<<<<<<<
 *                 field=param,
 *                 struct_name=args_name,
 */
    __pyx_t_5 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_8thriftrw_4spec_5field_FieldSpec), __pyx_n_s_compile); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 90; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_5);

    /* "thriftrw/spec/service.pyx":91
 *         param_specs = [
 *             FieldSpec.compile(
 *                 field=param,             # <<<<<<<<<<<<<<
 *                 struct_name=args_name,
 *                 require_requiredness=False,
 */
    __pyx_t_6 = PyDict_New(); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 91; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_6);
    if (PyDict_SetItem(__pyx_t_6, __pyx_n_s_field, __pyx_v_param) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 91; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "thriftrw/spec/service.pyx":92
 *             FieldSpec.compile(
 *                 field=param,
 *                 struct_name=args_name,             # <<<<<<<<<<<<<<
 *                 require_requiredness=False,
 *             ) for param in parameters
 */
    if (PyDict_SetItem(__pyx_t_6, __pyx_n_s_struct_name, __pyx_v_args_name) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 91; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "thriftrw/spec/service.pyx":93
 *                 field=param,
 *                 struct_name=args_name,
 *                 require_requiredness=False,             # <<<<<<<<<<<<<<
 *             ) for param in parameters
 *         ]
 */
    if (PyDict_SetItem(__pyx_t_6, __pyx_n_s_require_requiredness, Py_False) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 91; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "thriftrw/spec/service.pyx":90
 *         args_name = str('%s_%s_request' % (service_name, function_name))
 *         param_specs = [
 *             FieldSpec.compile(             # <<<<<<<<<<<<<<
 *                 field=param,
 *                 struct_name=args_name,
 */
    __pyx_t_7 = __Pyx_PyObject_Call(__pyx_t_5, __pyx_empty_tuple, __pyx_t_6); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 90; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    if (unlikely(__Pyx_ListComp_Append(__pyx_t_2, (PyObject*)__pyx_t_7))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 89; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

    /* "thriftrw/spec/service.pyx":94
 *                 struct_name=args_name,
 *                 require_requiredness=False,
 *             ) for param in parameters             # <<<<<<<<<<<<<<
 *         ]
 * 
 */
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_param_specs = ((PyObject*)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "thriftrw/spec/service.pyx":97
 *         ]
 * 
 *         return cls(args_name, param_specs)             # <<<<<<<<<<<<<<
 * 
 *     def link(self, scope, function):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = PyTuple_New(2); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 97; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(__pyx_v_args_name);
  __Pyx_GIVEREF(__pyx_v_args_name);
  PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_v_args_name);
  __Pyx_INCREF(__pyx_v_param_specs);
  __Pyx_GIVEREF(__pyx_v_param_specs);
  PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_v_param_specs);
  __pyx_t_1 = __Pyx_PyObject_Call(((PyObject *)__pyx_v_cls), __pyx_t_2, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 97; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "thriftrw/spec/service.pyx":77
 * 
 *     @classmethod
 *     def compile(cls, parameters, service_name, function_name):             # <<<<<<<<<<<<<<
 *         """Compiles a parameter list into a FunctionArgsSpec.
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_AddTraceback("thriftrw.spec.service.FunctionArgsSpec.compile", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_args_name);
  __Pyx_XDECREF(__pyx_v_param_specs);
  __Pyx_XDECREF(__pyx_v_param);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "thriftrw/spec/service.pyx":99
 *         return cls(args_name, param_specs)
 * 
 *     def link(self, scope, function):             # <<<<<<<<<<<<<<
 *         self.function = function
 *         spec = super(FunctionArgsSpec, self).link(scope)
 */

/* Python wrapper */
static PyObject *__pyx_pw_8thriftrw_4spec_7service_16FunctionArgsSpec_5link(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_pw_8thriftrw_4spec_7service_16FunctionArgsSpec_5link(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_scope = 0;
  PyObject *__pyx_v_function = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("link (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_scope,&__pyx_n_s_function,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_scope)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_function)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("link", 1, 2, 2, 1); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 99; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "link") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 99; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_scope = values[0];
    __pyx_v_function = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("link", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 99; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("thriftrw.spec.service.FunctionArgsSpec.link", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_8thriftrw_4spec_7service_16FunctionArgsSpec_4link(((struct __pyx_obj_8thriftrw_4spec_7service_FunctionArgsSpec *)__pyx_v_self), __pyx_v_scope, __pyx_v_function);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_8thriftrw_4spec_7service_16FunctionArgsSpec_4link(struct __pyx_obj_8thriftrw_4spec_7service_FunctionArgsSpec *__pyx_v_self, PyObject *__pyx_v_scope, PyObject *__pyx_v_function) {
  PyObject *__pyx_v_spec = NULL;
  PyObject *__pyx_v_result_type = NULL;
  PyObject *__pyx_v_result_spec = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  int __pyx_t_6;
  Py_ssize_t __pyx_t_7;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("link", 0);

  /* "thriftrw/spec/service.pyx":100
 * 
 *     def link(self, scope, function):
 *         self.function = function             # <<<<<<<<<<<<<<
 *         spec = super(FunctionArgsSpec, self).link(scope)
 * 
 */
  if (!(likely(((__pyx_v_function) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_function, __pyx_ptype_8thriftrw_4spec_7service_FunctionSpec))))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 100; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_1 = __pyx_v_function;
  __Pyx_INCREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->function);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->function));
  __pyx_v_self->function = ((struct __pyx_obj_8thriftrw_4spec_7service_FunctionSpec *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "thriftrw/spec/service.pyx":101
 *     def link(self, scope, function):
 *         self.function = function
 *         spec = super(FunctionArgsSpec, self).link(scope)             # <<<<<<<<<<<<<<
 * 
 *         result_type = None
 */
  __pyx_t_2 = PyTuple_New(2); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 101; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(((PyObject *)__pyx_ptype_8thriftrw_4spec_7service_FunctionArgsSpec));
  __Pyx_GIVEREF(((PyObject *)__pyx_ptype_8thriftrw_4spec_7service_FunctionArgsSpec));
  PyTuple_SET_ITEM(__pyx_t_2, 0, ((PyObject *)__pyx_ptype_8thriftrw_4spec_7service_FunctionArgsSpec));
  __Pyx_INCREF(((PyObject *)__pyx_v_self));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self));
  PyTuple_SET_ITEM(__pyx_t_2, 1, ((PyObject *)__pyx_v_self));
  __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_super, __pyx_t_2, NULL); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 101; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_link); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 101; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = NULL;
  if (CYTHON_COMPILING_IN_CPYTHON && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  if (!__pyx_t_3) {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v_scope); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 101; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
  } else {
    __pyx_t_4 = PyTuple_New(1+1); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 101; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_3); __pyx_t_3 = NULL;
    __Pyx_INCREF(__pyx_v_scope);
    __Pyx_GIVEREF(__pyx_v_scope);
    PyTuple_SET_ITEM(__pyx_t_4, 0+1, __pyx_v_scope);
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_4, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 101; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_spec = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "thriftrw/spec/service.pyx":103
 *         spec = super(FunctionArgsSpec, self).link(scope)
 * 
 *         result_type = None             # <<<<<<<<<<<<<<
 *         if self.function.result_spec is not None:
 *             result_spec = self.function.result_spec.link(scope, function)
 */
  __Pyx_INCREF(Py_None);
  __pyx_v_result_type = Py_None;

  /* "thriftrw/spec/service.pyx":104
 * 
 *         result_type = None
 *         if self.function.result_spec is not None:             # <<<<<<<<<<<<<<
 *             result_spec = self.function.result_spec.link(scope, function)
 *             result_type = result_spec.surface
 */
  __pyx_t_5 = (((PyObject *)__pyx_v_self->function->result_spec) != Py_None);
  __pyx_t_6 = (__pyx_t_5 != 0);
  if (__pyx_t_6) {

    /* "thriftrw/spec/service.pyx":105
 *         result_type = None
 *         if self.function.result_spec is not None:
 *             result_spec = self.function.result_spec.link(scope, function)             # <<<<<<<<<<<<<<
 *             result_type = result_spec.surface
 * 
 */
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self->function->result_spec), __pyx_n_s_link); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 105; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = NULL;
    __pyx_t_7 = 0;
    if (CYTHON_COMPILING_IN_CPYTHON && likely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_4)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_4);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
        __pyx_t_7 = 1;
      }
    }
    __pyx_t_3 = PyTuple_New(2+__pyx_t_7); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 105; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    if (__pyx_t_4) {
      __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_4); __pyx_t_4 = NULL;
    }
    __Pyx_INCREF(__pyx_v_scope);
    __Pyx_GIVEREF(__pyx_v_scope);
    PyTuple_SET_ITEM(__pyx_t_3, 0+__pyx_t_7, __pyx_v_scope);
    __Pyx_INCREF(__pyx_v_function);
    __Pyx_GIVEREF(__pyx_v_function);
    PyTuple_SET_ITEM(__pyx_t_3, 1+__pyx_t_7, __pyx_v_function);
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_3, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 105; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_v_result_spec = __pyx_t_1;
    __pyx_t_1 = 0;

    /* "thriftrw/spec/service.pyx":106
 *         if self.function.result_spec is not None:
 *             result_spec = self.function.result_spec.link(scope, function)
 *             result_type = result_spec.surface             # <<<<<<<<<<<<<<
 * 
 *         spec.surface.result_type = result_type
 */
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_result_spec, __pyx_n_s_surface); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 106; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF_SET(__pyx_v_result_type, __pyx_t_1);
    __pyx_t_1 = 0;

    /* "thriftrw/spec/service.pyx":104
 * 
 *         result_type = None
 *         if self.function.result_spec is not None:             # <<<<<<<<<<<<<<
 *             result_spec = self.function.result_spec.link(scope, function)
 *             result_type = result_spec.surface
 */
  }

  /* "thriftrw/spec/service.pyx":108
 *             result_type = result_spec.surface
 * 
 *         spec.surface.result_type = result_type             # <<<<<<<<<<<<<<
 *         return spec
 * 
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_spec, __pyx_n_s_surface); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 108; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  if (__Pyx_PyObject_SetAttrStr(__pyx_t_1, __pyx_n_s_result_type, __pyx_v_result_type) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 108; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "thriftrw/spec/service.pyx":109
 * 
 *         spec.surface.result_type = result_type
 *         return spec             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_spec);
  __pyx_r = __pyx_v_spec;
  goto __pyx_L0;

  /* "thriftrw/spec/service.pyx":99
 *         return cls(args_name, param_specs)
 * 
 *     def link(self, scope, function):             # <<<<<<<<<<<<<<
 *         self.function = function
 *         spec = super(FunctionArgsSpec, self).link(scope)
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("thriftrw.spec.service.FunctionArgsSpec.link", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_spec);
  __Pyx_XDECREF(__pyx_v_result_type);
  __Pyx_XDECREF(__pyx_v_result_spec);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "thriftrw/spec/service.pxd":29
 * 
 * cdef class FunctionArgsSpec(StructTypeSpec):
 *     cdef public FunctionSpec function             # <<<<<<<<<<<<<<
 * 
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_8thriftrw_4spec_7service_16FunctionArgsSpec_8function_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_8thriftrw_4spec_7service_16FunctionArgsSpec_8function_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_8thriftrw_4spec_7service_16FunctionArgsSpec_8function___get__(((struct __pyx_obj_8thriftrw_4spec_7service_FunctionArgsSpec *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_8thriftrw_4spec_7service_16FunctionArgsSpec_8function___get__(struct __pyx_obj_8thriftrw_4spec_7service_FunctionArgsSpec *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->function));
  __pyx_r = ((PyObject *)__pyx_v_self->function);
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_8thriftrw_4spec_7service_16FunctionArgsSpec_8function_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_8thriftrw_4spec_7service_16FunctionArgsSpec_8function_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_8thriftrw_4spec_7service_16FunctionArgsSpec_8function_2__set__(((struct __pyx_obj_8thriftrw_4spec_7service_FunctionArgsSpec *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_8thriftrw_4spec_7service_16FunctionArgsSpec_8function_2__set__(struct __pyx_obj_8thriftrw_4spec_7service_FunctionArgsSpec *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__set__", 0);
  if (!(likely(((__pyx_v_value) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_value, __pyx_ptype_8thriftrw_4spec_7service_FunctionSpec))))) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 29; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_1 = __pyx_v_value;
  __Pyx_INCREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->function);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->function));
  __pyx_v_self->function = ((struct __pyx_obj_8thriftrw_4spec_7service_FunctionSpec *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("thriftrw.spec.service.FunctionArgsSpec.function.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_8thriftrw_4spec_7service_16FunctionArgsSpec_8function_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_8thriftrw_4spec_7service_16FunctionArgsSpec_8function_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_8thriftrw_4spec_7service_16FunctionArgsSpec_8function_4__del__(((struct __pyx_obj_8thriftrw_4spec_7service_FunctionArgsSpec *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_8thriftrw_4spec_7service_16FunctionArgsSpec_8function_4__del__(struct __pyx_obj_8thriftrw_4spec_7service_FunctionArgsSpec *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->function);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->function));
  __pyx_v_self->function = ((struct __pyx_obj_8thriftrw_4spec_7service_FunctionSpec *)Py_None);

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "thriftrw/spec/service.pyx":149
 *     """
 * 
 *     def __init__(self, name, return_spec, exceptions):             # <<<<<<<<<<<<<<
 *         self.return_spec = return_spec
 *         self.exception_specs = exceptions
 */

/* Python wrapper */
static int __pyx_pw_8thriftrw_4spec_7service_18FunctionResultSpec_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_8thriftrw_4spec_7service_18FunctionResultSpec_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_name = 0;
  PyObject *__pyx_v_return_spec = 0;
  PyObject *__pyx_v_exceptions = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_name,&__pyx_n_s_return_spec,&__pyx_n_s_exceptions,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_name)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_return_spec)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__init__", 1, 3, 3, 1); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 149; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_exceptions)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__init__", 1, 3, 3, 2); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 149; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__init__") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 149; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v_name = values[0];
    __pyx_v_return_spec = values[1];
    __pyx_v_exceptions = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 149; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("thriftrw.spec.service.FunctionResultSpec.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_8thriftrw_4spec_7service_18FunctionResultSpec___init__(((struct __pyx_obj_8thriftrw_4spec_7service_FunctionResultSpec *)__pyx_v_self), __pyx_v_name, __pyx_v_return_spec, __pyx_v_exceptions);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_8thriftrw_4spec_7service_18FunctionResultSpec___init__(struct __pyx_obj_8thriftrw_4spec_7service_FunctionResultSpec *__pyx_v_self, PyObject *__pyx_v_name, PyObject *__pyx_v_return_spec, PyObject *__pyx_v_exceptions) {
  PyObject *__pyx_v_result_specs = NULL;
  PyObject *__pyx_v_f = NULL;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  Py_ssize_t __pyx_t_6;
  PyObject *(*__pyx_t_7)(PyObject *);
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "thriftrw/spec/service.pyx":150
 * 
 *     def __init__(self, name, return_spec, exceptions):
 *         self.return_spec = return_spec             # <<<<<<<<<<<<<<
 *         self.exception_specs = exceptions
 *         self.function = None
 */
  if (!(likely(((__pyx_v_return_spec) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_return_spec, __pyx_ptype_8thriftrw_4spec_4base_TypeSpec))))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 150; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_1 = __pyx_v_return_spec;
  __Pyx_INCREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->return_spec);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->return_spec));
  __pyx_v_self->return_spec = ((struct __pyx_obj_8thriftrw_4spec_4base_TypeSpec *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "thriftrw/spec/service.pyx":151
 *     def __init__(self, name, return_spec, exceptions):
 *         self.return_spec = return_spec
 *         self.exception_specs = exceptions             # <<<<<<<<<<<<<<
 *         self.function = None
 * 
 */
  if (!(likely(PyList_CheckExact(__pyx_v_exceptions))||((__pyx_v_exceptions) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_v_exceptions)->tp_name), 0))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 151; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_1 = __pyx_v_exceptions;
  __Pyx_INCREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->exception_specs);
  __Pyx_DECREF(__pyx_v_self->exception_specs);
  __pyx_v_self->exception_specs = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "thriftrw/spec/service.pyx":152
 *         self.return_spec = return_spec
 *         self.exception_specs = exceptions
 *         self.function = None             # <<<<<<<<<<<<<<
 * 
 *         result_specs = []
 */
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->function);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->function));
  __pyx_v_self->function = ((struct __pyx_obj_8thriftrw_4spec_7service_FunctionSpec *)Py_None);

  /* "thriftrw/spec/service.pyx":154
 *         self.function = None
 * 
 *         result_specs = []             # <<<<<<<<<<<<<<
 *         if return_spec is not None:
 *             result_specs.append(
 */
  __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 154; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_result_specs = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "thriftrw/spec/service.pyx":155
 * 
 *         result_specs = []
 *         if return_spec is not None:             # <<<<<<<<<<<<<<
 *             result_specs.append(
 *                 FieldSpec(
 */
  __pyx_t_2 = (__pyx_v_return_spec != Py_None);
  __pyx_t_3 = (__pyx_t_2 != 0);
  if (__pyx_t_3) {

    /* "thriftrw/spec/service.pyx":158
 *             result_specs.append(
 *                 FieldSpec(
 *                     id=0,             # <<<<<<<<<<<<<<
 *                     name='success',
 *                     spec=return_spec,
 */
    __pyx_t_1 = PyDict_New(); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 158; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_id, __pyx_int_0) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 158; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_name, __pyx_n_u_success) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 158; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "thriftrw/spec/service.pyx":160
 *                     id=0,
 *                     name='success',
 *                     spec=return_spec,             # <<<<<<<<<<<<<<
 *                     required=False,
 *                     default_value=None
 */
    if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_spec, __pyx_v_return_spec) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 158; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "thriftrw/spec/service.pyx":161
 *                     name='success',
 *                     spec=return_spec,
 *                     required=False,             # <<<<<<<<<<<<<<
 *                     default_value=None
 *                 )
 */
    if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_required, Py_False) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 158; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "thriftrw/spec/service.pyx":162
 *                     spec=return_spec,
 *                     required=False,
 *                     default_value=None             # <<<<<<<<<<<<<<
 *                 )
 *             )
 */
    if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_default_value, Py_None) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 158; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "thriftrw/spec/service.pyx":157
 *         if return_spec is not None:
 *             result_specs.append(
 *                 FieldSpec(             # <<<<<<<<<<<<<<
 *                     id=0,
 *                     name='success',
 */
    __pyx_t_4 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_8thriftrw_4spec_5field_FieldSpec), __pyx_empty_tuple, __pyx_t_1); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 157; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "thriftrw/spec/service.pyx":156
 *         result_specs = []
 *         if return_spec is not None:
 *             result_specs.append(             # <<<<<<<<<<<<<<
 *                 FieldSpec(
 *                     id=0,
 */
    __pyx_t_5 = __Pyx_PyList_Append(__pyx_v_result_specs, __pyx_t_4); if (unlikely(__pyx_t_5 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 156; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

    /* "thriftrw/spec/service.pyx":155
 * 
 *         result_specs = []
 *         if return_spec is not None:             # <<<<<<<<<<<<<<
 *             result_specs.append(
 *                 FieldSpec(
 */
  }

  /* "thriftrw/spec/service.pyx":166
 *             )
 * 
 *         result_specs.extend(exceptions)             # <<<<<<<<<<<<<<
 * 
 *         # Set of field IDs for exceptions recognized by this result. If the
 */
  __pyx_t_5 = __Pyx_PyList_Extend(__pyx_v_result_specs, __pyx_v_exceptions); if (unlikely(__pyx_t_5 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 166; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "thriftrw/spec/service.pyx":171
 *         # system tries to read a field with an unrecognized exception ID, an
 *         # ``UnknownExceptionError`` will be raised.
 *         self.exception_ids = frozenset([f.id for f in exceptions])             # <<<<<<<<<<<<<<
 * 
 *         super(FunctionResultSpec, self).__init__(
 */
  __pyx_t_4 = PyList_New(0); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 171; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  if (likely(PyList_CheckExact(__pyx_v_exceptions)) || PyTuple_CheckExact(__pyx_v_exceptions)) {
    __pyx_t_1 = __pyx_v_exceptions; __Pyx_INCREF(__pyx_t_1); __pyx_t_6 = 0;
    __pyx_t_7 = NULL;
  } else {
    __pyx_t_6 = -1; __pyx_t_1 = PyObject_GetIter(__pyx_v_exceptions); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 171; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_7 = Py_TYPE(__pyx_t_1)->tp_iternext; if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 171; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  for (;;) {
    if (likely(!__pyx_t_7)) {
      if (likely(PyList_CheckExact(__pyx_t_1))) {
        if (__pyx_t_6 >= PyList_GET_SIZE(__pyx_t_1)) break;
        #if CYTHON_COMPILING_IN_CPYTHON
        __pyx_t_8 = PyList_GET_ITEM(__pyx_t_1, __pyx_t_6); __Pyx_INCREF(__pyx_t_8); __pyx_t_6++; if (unlikely(0 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 171; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        #else
        __pyx_t_8 = PySequence_ITEM(__pyx_t_1, __pyx_t_6); __pyx_t_6++; if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 171; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_8);
        #endif
      } else {
        if (__pyx_t_6 >= PyTuple_GET_SIZE(__pyx_t_1)) break;
        #if CYTHON_COMPILING_IN_CPYTHON
        __pyx_t_8 = PyTuple_GET_ITEM(__pyx_t_1, __pyx_t_6); __Pyx_INCREF(__pyx_t_8); __pyx_t_6++; if (unlikely(0 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 171; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        #else
        __pyx_t_8 = PySequence_ITEM(__pyx_t_1, __pyx_t_6); __pyx_t_6++; if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 171; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_8);
        #endif
      }
    } else {
      __pyx_t_8 = __pyx_t_7(__pyx_t_1);
      if (unlikely(!__pyx_t_8)) {
        PyObject* exc_type = PyErr_Occurred();
        if (exc_type) {
          if (likely(exc_type == PyExc_StopIteration || PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
          else {__pyx_filename = __pyx_f[0]; __pyx_lineno = 171; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        }
        break;
      }
      __Pyx_GOTREF(__pyx_t_8);
    }
    __Pyx_XDECREF_SET(__pyx_v_f, __pyx_t_8);
    __pyx_t_8 = 0;
    __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_f, __pyx_n_s_id); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 171; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_8);
    if (unlikely(__Pyx_ListComp_Append(__pyx_t_4, (PyObject*)__pyx_t_8))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 171; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyFrozenSet_New(__pyx_t_4); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 171; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->exception_ids);
  __Pyx_DECREF(__pyx_v_self->exception_ids);
  __pyx_v_self->exception_ids = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "thriftrw/spec/service.pyx":173
 *         self.exception_ids = frozenset([f.id for f in exceptions])
 * 
 *         super(FunctionResultSpec, self).__init__(             # <<<<<<<<<<<<<<
 *             name, result_specs, allow_empty=(return_spec is None)
 *         )
 */
  __pyx_t_1 = PyTuple_New(2); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 173; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(((PyObject *)__pyx_ptype_8thriftrw_4spec_7service_FunctionResultSpec));
  __Pyx_GIVEREF(((PyObject *)__pyx_ptype_8thriftrw_4spec_7service_FunctionResultSpec));
  PyTuple_SET_ITEM(__pyx_t_1, 0, ((PyObject *)__pyx_ptype_8thriftrw_4spec_7service_FunctionResultSpec));
  __Pyx_INCREF(((PyObject *)__pyx_v_self));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self));
  PyTuple_SET_ITEM(__pyx_t_1, 1, ((PyObject *)__pyx_v_self));
  __pyx_t_4 = __Pyx_PyObject_Call(__pyx_builtin_super, __pyx_t_1, NULL); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 173; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_init); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 173; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "thriftrw/spec/service.pyx":174
 * 
 *         super(FunctionResultSpec, self).__init__(
 *             name, result_specs, allow_empty=(return_spec is None)             # <<<<<<<<<<<<<<
 *         )
 * 
 */
  __pyx_t_4 = PyTuple_New(2); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 173; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_INCREF(__pyx_v_name);
  __Pyx_GIVEREF(__pyx_v_name);
  PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_v_name);
  __Pyx_INCREF(__pyx_v_result_specs);
  __Pyx_GIVEREF(__pyx_v_result_specs);
  PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_v_result_specs);
  __pyx_t_8 = PyDict_New(); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 174; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_8);
  __pyx_t_3 = (__pyx_v_return_spec == Py_None);
  __pyx_t_9 = __Pyx_PyBool_FromLong(__pyx_t_3); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 174; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_9);
  if (PyDict_SetItem(__pyx_t_8, __pyx_n_s_allow_empty, __pyx_t_9) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 174; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;

  /* "thriftrw/spec/service.pyx":173
 *         self.exception_ids = frozenset([f.id for f in exceptions])
 * 
 *         super(FunctionResultSpec, self).__init__(             # <<<<<<<<<<<<<<
 *             name, result_specs, allow_empty=(return_spec is None)
 *         )
 */
  __pyx_t_9 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_t_4, __pyx_t_8); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 173; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_9);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;

  /* "thriftrw/spec/service.pyx":149
 *     """
 * 
 *     def __init__(self, name, return_spec, exceptions):             # <<<<<<<<<<<<<<
 *         self.return_spec = return_spec
 *         self.exception_specs = exceptions
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_AddTraceback("thriftrw.spec.service.FunctionResultSpec.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_result_specs);
  __Pyx_XDECREF(__pyx_v_f);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "thriftrw/spec/service.pyx":177
 *         )
 * 
 *     cpdef object from_wire(self, Value wire_value):             # <<<<<<<<<<<<<<
 *         check.type_code_matches(self, wire_value)
 * 
 */

static PyObject *__pyx_pw_8thriftrw_4spec_7service_18FunctionResultSpec_3from_wire(PyObject *__pyx_v_self, PyObject *__pyx_v_wire_value); /*proto*/
static PyObject *__pyx_f_8thriftrw_4spec_7service_18FunctionResultSpec_from_wire(struct __pyx_obj_8thriftrw_4spec_7service_FunctionResultSpec *__pyx_v_self, struct __pyx_obj_8thriftrw_4wire_5value_Value *__pyx_v_wire_value, int __pyx_skip_dispatch) {
  PyObject *__pyx_v_field = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  Py_ssize_t __pyx_t_6;
  PyObject *(*__pyx_t_7)(PyObject *);
  int __pyx_t_8;
  int __pyx_t_9;
  int __pyx_t_10;
  Py_ssize_t __pyx_t_11;
  PyObject *__pyx_t_12 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("from_wire", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0)) {
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_from_wire); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 177; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)__pyx_pw_8thriftrw_4spec_7service_18FunctionResultSpec_3from_wire)) {
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF(__pyx_t_1);
      __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
      if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_3))) {
        __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
        if (likely(__pyx_t_4)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
          __Pyx_INCREF(__pyx_t_4);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_3, function);
        }
      }
      if (!__pyx_t_4) {
        __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_3, ((PyObject *)__pyx_v_wire_value)); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 177; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_2);
      } else {
        __pyx_t_5 = PyTuple_New(1+1); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 177; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_5);
        __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_4); __pyx_t_4 = NULL;
        __Pyx_INCREF(((PyObject *)__pyx_v_wire_value));
        __Pyx_GIVEREF(((PyObject *)__pyx_v_wire_value));
        PyTuple_SET_ITEM(__pyx_t_5, 0+1, ((PyObject *)__pyx_v_wire_value));
        __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_5, NULL); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 177; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      }
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_r = __pyx_t_2;
      __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      goto __pyx_L0;
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  }

  /* "thriftrw/spec/service.pyx":178
 * 
 *     cpdef object from_wire(self, Value wire_value):
 *         check.type_code_matches(self, wire_value)             # <<<<<<<<<<<<<<
 * 
 *         for field in wire_value.fields:
 */
  __pyx_t_1 = __pyx_f_8thriftrw_4spec_5check_type_code_matches(((struct __pyx_obj_8thriftrw_4spec_4base_TypeSpec *)__pyx_v_self), __pyx_v_wire_value, 0); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 178; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "thriftrw/spec/service.pyx":180
 *         check.type_code_matches(self, wire_value)
 * 
 *         for field in wire_value.fields:             # <<<<<<<<<<<<<<
 *             if field.id != 0 and field.id not in self.exception_ids:
 *                 raise UnknownExceptionError(
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_wire_value), __pyx_n_s_fields); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 180; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  if (likely(PyList_CheckExact(__pyx_t_1)) || PyTuple_CheckExact(__pyx_t_1)) {
    __pyx_t_2 = __pyx_t_1; __Pyx_INCREF(__pyx_t_2); __pyx_t_6 = 0;
    __pyx_t_7 = NULL;
  } else {
    __pyx_t_6 = -1; __pyx_t_2 = PyObject_GetIter(__pyx_t_1); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 180; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_7 = Py_TYPE(__pyx_t_2)->tp_iternext; if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 180; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  for (;;) {
    if (likely(!__pyx_t_7)) {
      if (likely(PyList_CheckExact(__pyx_t_2))) {
        if (__pyx_t_6 >= PyList_GET_SIZE(__pyx_t_2)) break;
        #if CYTHON_COMPILING_IN_CPYTHON
        __pyx_t_1 = PyList_GET_ITEM(__pyx_t_2, __pyx_t_6); __Pyx_INCREF(__pyx_t_1); __pyx_t_6++; if (unlikely(0 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 180; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        #else
        __pyx_t_1 = PySequence_ITEM(__pyx_t_2, __pyx_t_6); __pyx_t_6++; if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 180; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_1);
        #endif
      } else {
        if (__pyx_t_6 >= PyTuple_GET_SIZE(__pyx_t_2)) break;
        #if CYTHON_COMPILING_IN_CPYTHON
        __pyx_t_1 = PyTuple_GET_ITEM(__pyx_t_2, __pyx_t_6); __Pyx_INCREF(__pyx_t_1); __pyx_t_6++; if (unlikely(0 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 180; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        #else
        __pyx_t_1 = PySequence_ITEM(__pyx_t_2, __pyx_t_6); __pyx_t_6++; if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 180; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_1);
        #endif
      }
    } else {
      __pyx_t_1 = __pyx_t_7(__pyx_t_2);
      if (unlikely(!__pyx_t_1)) {
        PyObject* exc_type = PyErr_Occurred();
        if (exc_type) {
          if (likely(exc_type == PyExc_StopIteration || PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
          else {__pyx_filename = __pyx_f[0]; __pyx_lineno = 180; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        }
        break;
      }
      __Pyx_GOTREF(__pyx_t_1);
    }
    __Pyx_XDECREF_SET(__pyx_v_field, __pyx_t_1);
    __pyx_t_1 = 0;

    /* "thriftrw/spec/service.pyx":181
 * 
 *         for field in wire_value.fields:
 *             if field.id != 0 and field.id not in self.exception_ids:             # <<<<<<<<<<<<<<
 *                 raise UnknownExceptionError(
 *                     (
 */
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_field, __pyx_n_s_id); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 181; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_3 = PyObject_RichCompare(__pyx_t_1, __pyx_int_0, Py_NE); __Pyx_XGOTREF(__pyx_t_3); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 181; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_9 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_9 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 181; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (__pyx_t_9) {
    } else {
      __pyx_t_8 = __pyx_t_9;
      goto __pyx_L6_bool_binop_done;
    }
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_field, __pyx_n_s_id); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 181; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_9 = (__Pyx_PySequence_ContainsTF(__pyx_t_3, __pyx_v_self->exception_ids, Py_NE)); if (unlikely(__pyx_t_9 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 181; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_10 = (__pyx_t_9 != 0);
    __pyx_t_8 = __pyx_t_10;
    __pyx_L6_bool_binop_done:;
    if (__pyx_t_8) {

      /* "thriftrw/spec/service.pyx":182
 *         for field in wire_value.fields:
 *             if field.id != 0 and field.id not in self.exception_ids:
 *                 raise UnknownExceptionError(             # <<<<<<<<<<<<<<
 *                     (
 *                         '"%s" received an unrecognized exception. '
 */
      __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_UnknownExceptionError); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 182; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_1);

      /* "thriftrw/spec/service.pyx":187
 *                         'Make sure your Thrift IDL is up-to-date with '
 *                         'what the remote host is using.'
 *                     ) % self.name,             # <<<<<<<<<<<<<<
 *                     wire_value,
 *                 )
 */
      __pyx_t_5 = PyUnicode_Format(__pyx_kp_u_s_received_an_unrecognized_exce, __pyx_v_self->__pyx_base.name); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 187; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_5);

      /* "thriftrw/spec/service.pyx":188
 *                         'what the remote host is using.'
 *                     ) % self.name,
 *                     wire_value,             # <<<<<<<<<<<<<<
 *                 )
 * 
 */
      __pyx_t_4 = NULL;
      __pyx_t_11 = 0;
      if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_1))) {
        __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_1);
        if (likely(__pyx_t_4)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
          __Pyx_INCREF(__pyx_t_4);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_1, function);
          __pyx_t_11 = 1;
        }
      }
      __pyx_t_12 = PyTuple_New(2+__pyx_t_11); if (unlikely(!__pyx_t_12)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 182; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_12);
      if (__pyx_t_4) {
        __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_12, 0, __pyx_t_4); __pyx_t_4 = NULL;
      }
      __Pyx_GIVEREF(__pyx_t_5);
      PyTuple_SET_ITEM(__pyx_t_12, 0+__pyx_t_11, __pyx_t_5);
      __Pyx_INCREF(((PyObject *)__pyx_v_wire_value));
      __Pyx_GIVEREF(((PyObject *)__pyx_v_wire_value));
      PyTuple_SET_ITEM(__pyx_t_12, 1+__pyx_t_11, ((PyObject *)__pyx_v_wire_value));
      __pyx_t_5 = 0;
      __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_t_12, NULL); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 182; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_Raise(__pyx_t_3, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      {__pyx_filename = __pyx_f[0]; __pyx_lineno = 182; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

      /* "thriftrw/spec/service.pyx":181
 * 
 *         for field in wire_value.fields:
 *             if field.id != 0 and field.id not in self.exception_ids:             # <<<<<<<<<<<<<<
 *                 raise UnknownExceptionError(
 *                     (
 */
    }

    /* "thriftrw/spec/service.pyx":180
 *         check.type_code_matches(self, wire_value)
 * 
 *         for field in wire_value.fields:             # <<<<<<<<<<<<<<
 *             if field.id != 0 and field.id not in self.exception_ids:
 *                 raise UnknownExceptionError(
 */
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "thriftrw/spec/service.pyx":191
 *                 )
 * 
 *         return super(FunctionResultSpec, self).from_wire(wire_value)             # <<<<<<<<<<<<<<
 * 
 *     def link(self, scope, function):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_3 = PyTuple_New(2); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 191; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_INCREF(((PyObject *)__pyx_ptype_8thriftrw_4spec_7service_FunctionResultSpec));
  __Pyx_GIVEREF(((PyObject *)__pyx_ptype_8thriftrw_4spec_7service_FunctionResultSpec));
  PyTuple_SET_ITEM(__pyx_t_3, 0, ((PyObject *)__pyx_ptype_8thriftrw_4spec_7service_FunctionResultSpec));
  __Pyx_INCREF(((PyObject *)__pyx_v_self));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self));
  PyTuple_SET_ITEM(__pyx_t_3, 1, ((PyObject *)__pyx_v_self));
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_super, __pyx_t_3, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 191; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_from_wire); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 191; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = NULL;
  if (CYTHON_COMPILING_IN_CPYTHON && likely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_1 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_1)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_1);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  if (!__pyx_t_1) {
    __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_3, ((PyObject *)__pyx_v_wire_value)); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 191; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_2);
  } else {
    __pyx_t_12 = PyTuple_New(1+1); if (unlikely(!__pyx_t_12)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 191; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_12);
    __Pyx_GIVEREF(__pyx_t_1); PyTuple_SET_ITEM(__pyx_t_12, 0, __pyx_t_1); __pyx_t_1 = NULL;
    __Pyx_INCREF(((PyObject *)__pyx_v_wire_value));
    __Pyx_GIVEREF(((PyObject *)__pyx_v_wire_value));
    PyTuple_SET_ITEM(__pyx_t_12, 0+1, ((PyObject *)__pyx_v_wire_value));
    __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_12, NULL); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 191; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
  }
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "thriftrw/spec/service.pyx":177
 *         )
 * 
 *     cpdef object from_wire(self, Value wire_value):             # <<<<<<<<<<<<<<
 *         check.type_code_matches(self, wire_value)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_12);
  __Pyx_AddTraceback("thriftrw.spec.service.FunctionResultSpec.from_wire", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_field);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_8thriftrw_4spec_7service_18FunctionResultSpec_3from_wire(PyObject *__pyx_v_self, PyObject *__pyx_v_wire_value); /*proto*/
static PyObject *__pyx_pw_8thriftrw_4spec_7service_18FunctionResultSpec_3from_wire(PyObject *__pyx_v_self, PyObject *__pyx_v_wire_value) {
  CYTHON_UNUSED int __pyx_lineno = 0;
  CYTHON_UNUSED const char *__pyx_filename = NULL;
  CYTHON_UNUSED int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("from_wire (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_wire_value), __pyx_ptype_8thriftrw_4wire_5value_Value, 1, "wire_value", 0))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 177; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_r = __pyx_pf_8thriftrw_4spec_7service_18FunctionResultSpec_2from_wire(((struct __pyx_obj_8thriftrw_4spec_7service_FunctionResultSpec *)__pyx_v_self), ((struct __pyx_obj_8thriftrw_4wire_5value_Value *)__pyx_v_wire_value));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_8thriftrw_4spec_7service_18FunctionResultSpec_2from_wire(struct __pyx_obj_8thriftrw_4spec_7service_FunctionResultSpec *__pyx_v_self, struct __pyx_obj_8thriftrw_4wire_5value_Value *__pyx_v_wire_value) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("from_wire", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_8thriftrw_4spec_7service_18FunctionResultSpec_from_wire(__pyx_v_self, __pyx_v_wire_value, 1); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 177; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("thriftrw.spec.service.FunctionResultSpec.from_wire", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "thriftrw/spec/service.pyx":193
 *         return super(FunctionResultSpec, self).from_wire(wire_value)
 * 
 *     def link(self, scope, function):             # <<<<<<<<<<<<<<
 *         self.function = function
 *         if self.return_spec is not None:
 */

/* Python wrapper */
static PyObject *__pyx_pw_8thriftrw_4spec_7service_18FunctionResultSpec_5link(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_pw_8thriftrw_4spec_7service_18FunctionResultSpec_5link(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_scope = 0;
  PyObject *__pyx_v_function = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("link (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_scope,&__pyx_n_s_function,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_scope)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_function)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("link", 1, 2, 2, 1); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 193; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "link") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 193; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_scope = values[0];
    __pyx_v_function = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("link", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 193; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("thriftrw.spec.service.FunctionResultSpec.link", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_8thriftrw_4spec_7service_18FunctionResultSpec_4link(((struct __pyx_obj_8thriftrw_4spec_7service_FunctionResultSpec *)__pyx_v_self), __pyx_v_scope, __pyx_v_function);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_8thriftrw_4spec_7service_18FunctionResultSpec_4link(struct __pyx_obj_8thriftrw_4spec_7service_FunctionResultSpec *__pyx_v_self, PyObject *__pyx_v_scope, PyObject *__pyx_v_function) {
  PyObject *__pyx_v_e = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  Py_ssize_t __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("link", 0);

  /* "thriftrw/spec/service.pyx":194
 * 
 *     def link(self, scope, function):
 *         self.function = function             # <<<<<<<<<<<<<<
 *         if self.return_spec is not None:
 *             self.return_spec = self.return_spec.link(scope)
 */
  if (!(likely(((__pyx_v_function) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_function, __pyx_ptype_8thriftrw_4spec_7service_FunctionSpec))))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 194; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_1 = __pyx_v_function;
  __Pyx_INCREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->function);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->function));
  __pyx_v_self->function = ((struct __pyx_obj_8thriftrw_4spec_7service_FunctionSpec *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "thriftrw/spec/service.pyx":195
 *     def link(self, scope, function):
 *         self.function = function
 *         if self.return_spec is not None:             # <<<<<<<<<<<<<<
 *             self.return_spec = self.return_spec.link(scope)
 *         self.exception_specs = [e.link(scope) for e in self.exception_specs]
 */
  __pyx_t_2 = (((PyObject *)__pyx_v_self->return_spec) != Py_None);
  __pyx_t_3 = (__pyx_t_2 != 0);
  if (__pyx_t_3) {

    /* "thriftrw/spec/service.pyx":196
 *         self.function = function
 *         if self.return_spec is not None:
 *             self.return_spec = self.return_spec.link(scope)             # <<<<<<<<<<<<<<
 *         self.exception_specs = [e.link(scope) for e in self.exception_specs]
 *         return super(FunctionResultSpec, self).link(scope)
 */
    __pyx_t_1 = ((PyObject *)((struct __pyx_vtabstruct_8thriftrw_4spec_4base_TypeSpec *)__pyx_v_self->return_spec->__pyx_vtab)->link(__pyx_v_self->return_spec, __pyx_v_scope, 0)); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 196; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_GIVEREF(__pyx_t_1);
    __Pyx_GOTREF(__pyx_v_self->return_spec);
    __Pyx_DECREF(((PyObject *)__pyx_v_self->return_spec));
    __pyx_v_self->return_spec = ((struct __pyx_obj_8thriftrw_4spec_4base_TypeSpec *)__pyx_t_1);
    __pyx_t_1 = 0;

    /* "thriftrw/spec/service.pyx":195
 *     def link(self, scope, function):
 *         self.function = function
 *         if self.return_spec is not None:             # <<<<<<<<<<<<<<
 *             self.return_spec = self.return_spec.link(scope)
 *         self.exception_specs = [e.link(scope) for e in self.exception_specs]
 */
  }

  /* "thriftrw/spec/service.pyx":197
 *         if self.return_spec is not None:
 *             self.return_spec = self.return_spec.link(scope)
 *         self.exception_specs = [e.link(scope) for e in self.exception_specs]             # <<<<<<<<<<<<<<
 *         return super(FunctionResultSpec, self).link(scope)
 * 
 */
  __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 197; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  if (unlikely(__pyx_v_self->exception_specs == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 197; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_t_4 = __pyx_v_self->exception_specs; __Pyx_INCREF(__pyx_t_4); __pyx_t_5 = 0;
  for (;;) {
    if (__pyx_t_5 >= PyList_GET_SIZE(__pyx_t_4)) break;
    #if CYTHON_COMPILING_IN_CPYTHON
    __pyx_t_6 = PyList_GET_ITEM(__pyx_t_4, __pyx_t_5); __Pyx_INCREF(__pyx_t_6); __pyx_t_5++; if (unlikely(0 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 197; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    #else
    __pyx_t_6 = PySequence_ITEM(__pyx_t_4, __pyx_t_5); __pyx_t_5++; if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 197; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_6);
    #endif
    __Pyx_XDECREF_SET(__pyx_v_e, __pyx_t_6);
    __pyx_t_6 = 0;
    __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_v_e, __pyx_n_s_link); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 197; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_8 = NULL;
    if (CYTHON_COMPILING_IN_CPYTHON && likely(PyMethod_Check(__pyx_t_7))) {
      __pyx_t_8 = PyMethod_GET_SELF(__pyx_t_7);
      if (likely(__pyx_t_8)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_7);
        __Pyx_INCREF(__pyx_t_8);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_7, function);
      }
    }
    if (!__pyx_t_8) {
      __pyx_t_6 = __Pyx_PyObject_CallOneArg(__pyx_t_7, __pyx_v_scope); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 197; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_6);
    } else {
      __pyx_t_9 = PyTuple_New(1+1); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 197; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_9);
      __Pyx_GIVEREF(__pyx_t_8); PyTuple_SET_ITEM(__pyx_t_9, 0, __pyx_t_8); __pyx_t_8 = NULL;
      __Pyx_INCREF(__pyx_v_scope);
      __Pyx_GIVEREF(__pyx_v_scope);
      PyTuple_SET_ITEM(__pyx_t_9, 0+1, __pyx_v_scope);
      __pyx_t_6 = __Pyx_PyObject_Call(__pyx_t_7, __pyx_t_9, NULL); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 197; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    }
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    if (unlikely(__Pyx_ListComp_Append(__pyx_t_1, (PyObject*)__pyx_t_6))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 197; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  }
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->exception_specs);
  __Pyx_DECREF(__pyx_v_self->exception_specs);
  __pyx_v_self->exception_specs = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "thriftrw/spec/service.pyx":198
 *             self.return_spec = self.return_spec.link(scope)
 *         self.exception_specs = [e.link(scope) for e in self.exception_specs]
 *         return super(FunctionResultSpec, self).link(scope)             # <<<<<<<<<<<<<<
 * 
 *     @classmethod
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_4 = PyTuple_New(2); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 198; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_INCREF(((PyObject *)__pyx_ptype_8thriftrw_4spec_7service_FunctionResultSpec));
  __Pyx_GIVEREF(((PyObject *)__pyx_ptype_8thriftrw_4spec_7service_FunctionResultSpec));
  PyTuple_SET_ITEM(__pyx_t_4, 0, ((PyObject *)__pyx_ptype_8thriftrw_4spec_7service_FunctionResultSpec));
  __Pyx_INCREF(((PyObject *)__pyx_v_self));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self));
  PyTuple_SET_ITEM(__pyx_t_4, 1, ((PyObject *)__pyx_v_self));
  __pyx_t_6 = __Pyx_PyObject_Call(__pyx_builtin_super, __pyx_t_4, NULL); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 198; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_6, __pyx_n_s_link); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 198; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __pyx_t_6 = NULL;
  if (CYTHON_COMPILING_IN_CPYTHON && likely(PyMethod_Check(__pyx_t_4))) {
    __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_4);
    if (likely(__pyx_t_6)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
      __Pyx_INCREF(__pyx_t_6);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_4, function);
    }
  }
  if (!__pyx_t_6) {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_v_scope); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 198; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
  } else {
    __pyx_t_7 = PyTuple_New(1+1); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 198; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_GIVEREF(__pyx_t_6); PyTuple_SET_ITEM(__pyx_t_7, 0, __pyx_t_6); __pyx_t_6 = NULL;
    __Pyx_INCREF(__pyx_v_scope);
    __Pyx_GIVEREF(__pyx_v_scope);
    PyTuple_SET_ITEM(__pyx_t_7, 0+1, __pyx_v_scope);
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_t_7, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 198; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  }
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "thriftrw/spec/service.pyx":193
 *         return super(FunctionResultSpec, self).from_wire(wire_value)
 * 
 *     def link(self, scope, function):             # <<<<<<<<<<<<<<
 *         self.function = function
 *         if self.return_spec is not None:
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_AddTraceback("thriftrw.spec.service.FunctionResultSpec.link", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_e);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "thriftrw/spec/service.pyx":201
 * 
 *     @classmethod
 *     def compile(cls, return_type, exceptions, service_name, function_name):             # <<<<<<<<<<<<<<
 *         """Compiles information from the AST into a FunctionResultSpec.
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_8thriftrw_4spec_7service_18FunctionResultSpec_7compile(PyObject *__pyx_v_cls, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_8thriftrw_4spec_7service_18FunctionResultSpec_6compile[] = "Compiles information from the AST into a FunctionResultSpec.\n\n        :param return_type:\n            A :py:class:`thriftrw.idl.Type` representing the return type or\n            None if the function doesn't return anything.\n        :param exceptions:\n            Collection of :py:class:`thriftrw.idl.Field` objects representing\n            raised by the function.\n        :param str service_name:\n            Name of the service under which the function was defined.\n        :param str function_name:\n            Name of the function whose result this object represents.\n        ";
static PyObject *__pyx_pw_8thriftrw_4spec_7service_18FunctionResultSpec_7compile(PyObject *__pyx_v_cls, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_return_type = 0;
  PyObject *__pyx_v_exceptions = 0;
  PyObject *__pyx_v_service_name = 0;
  PyObject *__pyx_v_function_name = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("compile (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_return_type,&__pyx_n_s_exceptions,&__pyx_n_s_service_name,&__pyx_n_s_function_name,0};
    PyObject* values[4] = {0,0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_return_type)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_exceptions)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("compile", 1, 4, 4, 1); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 201; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_service_name)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("compile", 1, 4, 4, 2); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 201; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  3:
        if (likely((values[3] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_function_name)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("compile", 1, 4, 4, 3); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 201; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "compile") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 201; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 4) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
      values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
    }
    __pyx_v_return_type = values[0];
    __pyx_v_exceptions = values[1];
    __pyx_v_service_name = values[2];
    __pyx_v_function_name = values[3];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("compile", 1, 4, 4, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 201; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("thriftrw.spec.service.FunctionResultSpec.compile", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_8thriftrw_4spec_7service_18FunctionResultSpec_6compile(((PyTypeObject*)__pyx_v_cls), __pyx_v_return_type, __pyx_v_exceptions, __pyx_v_service_name, __pyx_v_function_name);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_8thriftrw_4spec_7service_18FunctionResultSpec_6compile(PyTypeObject *__pyx_v_cls, PyObject *__pyx_v_return_type, PyObject *__pyx_v_exceptions, PyObject *__pyx_v_service_name, PyObject *__pyx_v_function_name) {
  PyObject *__pyx_v_result_name = NULL;
  PyObject *__pyx_v_return_spec = NULL;
  PyObject *__pyx_v_exc_specs = NULL;
  PyObject *__pyx_v_exc = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  Py_ssize_t __pyx_t_7;
  PyObject *(*__pyx_t_8)(PyObject *);
  int __pyx_t_9;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("compile", 0);
  __Pyx_INCREF(__pyx_v_exceptions);

  /* "thriftrw/spec/service.pyx":215
 *             Name of the function whose result this object represents.
 *         """
 *         exceptions = exceptions or []             # <<<<<<<<<<<<<<
 *         result_name = str('%s_%s_response' % (service_name, function_name))
 * 
 */
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_v_exceptions); if (unlikely(__pyx_t_2 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 215; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (!__pyx_t_2) {
  } else {
    __Pyx_INCREF(__pyx_v_exceptions);
    __pyx_t_1 = __pyx_v_exceptions;
    goto __pyx_L3_bool_binop_done;
  }
  __pyx_t_3 = PyList_New(0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 215; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_INCREF(__pyx_t_3);
  __pyx_t_1 = __pyx_t_3;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_L3_bool_binop_done:;
  __Pyx_DECREF_SET(__pyx_v_exceptions, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "thriftrw/spec/service.pyx":216
 *         """
 *         exceptions = exceptions or []
 *         result_name = str('%s_%s_response' % (service_name, function_name))             # <<<<<<<<<<<<<<
 * 
 *         if return_type is not None:
 */
  __pyx_t_1 = PyTuple_New(2); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 216; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_v_service_name);
  __Pyx_GIVEREF(__pyx_v_service_name);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_v_service_name);
  __Pyx_INCREF(__pyx_v_function_name);
  __Pyx_GIVEREF(__pyx_v_function_name);
  PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_v_function_name);
  __pyx_t_3 = PyUnicode_Format(__pyx_kp_u_s__s_response, __pyx_t_1); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 216; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyTuple_New(1); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 216; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_3);
  __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PyObject_Call(((PyObject *)(&PyString_Type)), __pyx_t_1, NULL); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 216; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_result_name = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "thriftrw/spec/service.pyx":218
 *         result_name = str('%s_%s_response' % (service_name, function_name))
 * 
 *         if return_type is not None:             # <<<<<<<<<<<<<<
 *             return_spec = type_spec_or_ref(return_type)
 *         else:
 */
  __pyx_t_2 = (__pyx_v_return_type != Py_None);
  __pyx_t_4 = (__pyx_t_2 != 0);
  if (__pyx_t_4) {

    /* "thriftrw/spec/service.pyx":219
 * 
 *         if return_type is not None:
 *             return_spec = type_spec_or_ref(return_type)             # <<<<<<<<<<<<<<
 *         else:
 *             return_spec = None
 */
    __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_type_spec_or_ref); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 219; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_5 = NULL;
    if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_1))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_1);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_1, function);
      }
    }
    if (!__pyx_t_5) {
      __pyx_t_3 = __Pyx_PyObject_CallOneArg(__pyx_t_1, __pyx_v_return_type); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 219; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_3);
    } else {
      __pyx_t_6 = PyTuple_New(1+1); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 219; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_GIVEREF(__pyx_t_5); PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_5); __pyx_t_5 = NULL;
      __Pyx_INCREF(__pyx_v_return_type);
      __Pyx_GIVEREF(__pyx_v_return_type);
      PyTuple_SET_ITEM(__pyx_t_6, 0+1, __pyx_v_return_type);
      __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_t_6, NULL); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 219; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_v_return_spec = __pyx_t_3;
    __pyx_t_3 = 0;

    /* "thriftrw/spec/service.pyx":218
 *         result_name = str('%s_%s_response' % (service_name, function_name))
 * 
 *         if return_type is not None:             # <<<<<<<<<<<<<<
 *             return_spec = type_spec_or_ref(return_type)
 *         else:
 */
    goto __pyx_L5;
  }

  /* "thriftrw/spec/service.pyx":221
 *             return_spec = type_spec_or_ref(return_type)
 *         else:
 *             return_spec = None             # <<<<<<<<<<<<<<
 * 
 *         exc_specs = []
 */
  /*else*/ {
    __Pyx_INCREF(Py_None);
    __pyx_v_return_spec = Py_None;
  }
  __pyx_L5:;

  /* "thriftrw/spec/service.pyx":223
 *             return_spec = None
 * 
 *         exc_specs = []             # <<<<<<<<<<<<<<
 *         for exc in exceptions:
 *             exc_specs.append(
 */
  __pyx_t_3 = PyList_New(0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 223; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_v_exc_specs = ((PyObject*)__pyx_t_3);
  __pyx_t_3 = 0;

  /* "thriftrw/spec/service.pyx":224
 * 
 *         exc_specs = []
 *         for exc in exceptions:             # <<<<<<<<<<<<<<
 *             exc_specs.append(
 *                 FieldSpec.compile(
 */
  if (likely(PyList_CheckExact(__pyx_v_exceptions)) || PyTuple_CheckExact(__pyx_v_exceptions)) {
    __pyx_t_3 = __pyx_v_exceptions; __Pyx_INCREF(__pyx_t_3); __pyx_t_7 = 0;
    __pyx_t_8 = NULL;
  } else {
    __pyx_t_7 = -1; __pyx_t_3 = PyObject_GetIter(__pyx_v_exceptions); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 224; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_8 = Py_TYPE(__pyx_t_3)->tp_iternext; if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 224; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  for (;;) {
    if (likely(!__pyx_t_8)) {
      if (likely(PyList_CheckExact(__pyx_t_3))) {
        if (__pyx_t_7 >= PyList_GET_SIZE(__pyx_t_3)) break;
        #if CYTHON_COMPILING_IN_CPYTHON
        __pyx_t_1 = PyList_GET_ITEM(__pyx_t_3, __pyx_t_7); __Pyx_INCREF(__pyx_t_1); __pyx_t_7++; if (unlikely(0 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 224; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        #else
        __pyx_t_1 = PySequence_ITEM(__pyx_t_3, __pyx_t_7); __pyx_t_7++; if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 224; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_1);
        #endif
      } else {
        if (__pyx_t_7 >= PyTuple_GET_SIZE(__pyx_t_3)) break;
        #if CYTHON_COMPILING_IN_CPYTHON
        __pyx_t_1 = PyTuple_GET_ITEM(__pyx_t_3, __pyx_t_7); __Pyx_INCREF(__pyx_t_1); __pyx_t_7++; if (unlikely(0 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 224; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        #else
        __pyx_t_1 = PySequence_ITEM(__pyx_t_3, __pyx_t_7); __pyx_t_7++; if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 224; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_1);
        #endif
      }
    } else {
      __pyx_t_1 = __pyx_t_8(__pyx_t_3);
      if (unlikely(!__pyx_t_1)) {
        PyObject* exc_type = PyErr_Occurred();
        if (exc_type) {
          if (likely(exc_type == PyExc_StopIteration || PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
          else {__pyx_filename = __pyx_f[0]; __pyx_lineno = 224; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        }
        break;
      }
      __Pyx_GOTREF(__pyx_t_1);
    }
    __Pyx_XDECREF_SET(__pyx_v_exc, __pyx_t_1);
    __pyx_t_1 = 0;

    /* "thriftrw/spec/service.pyx":226
 *         for exc in exceptions:
 *             exc_specs.append(
 *                 FieldSpec.compile(             # <<<<<<<<<<<<<<
 *                     field=exc,
 *                     struct_name=result_name,
 */
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_8thriftrw_4spec_5field_FieldSpec), __pyx_n_s_compile); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 226; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);

    /* "thriftrw/spec/service.pyx":227
 *             exc_specs.append(
 *                 FieldSpec.compile(
 *                     field=exc,             # <<<<<<<<<<<<<<
 *                     struct_name=result_name,
 *                     require_requiredness=False,
 */
    __pyx_t_6 = PyDict_New(); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 227; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_6);
    if (PyDict_SetItem(__pyx_t_6, __pyx_n_s_field, __pyx_v_exc) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 227; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "thriftrw/spec/service.pyx":228
 *                 FieldSpec.compile(
 *                     field=exc,
 *                     struct_name=result_name,             # <<<<<<<<<<<<<<
 *                     require_requiredness=False,
 *                 )
 */
    if (PyDict_SetItem(__pyx_t_6, __pyx_n_s_struct_name, __pyx_v_result_name) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 227; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "thriftrw/spec/service.pyx":229
 *                     field=exc,
 *                     struct_name=result_name,
 *                     require_requiredness=False,             # <<<<<<<<<<<<<<
 *                 )
 *             )
 */
    if (PyDict_SetItem(__pyx_t_6, __pyx_n_s_require_requiredness, Py_False) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 227; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "thriftrw/spec/service.pyx":226
 *         for exc in exceptions:
 *             exc_specs.append(
 *                 FieldSpec.compile(             # <<<<<<<<<<<<<<
 *                     field=exc,
 *                     struct_name=result_name,
 */
    __pyx_t_5 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_empty_tuple, __pyx_t_6); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 226; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

    /* "thriftrw/spec/service.pyx":225
 *         exc_specs = []
 *         for exc in exceptions:
 *             exc_specs.append(             # <<<<<<<<<<<<<<
 *                 FieldSpec.compile(
 *                     field=exc,
 */
    __pyx_t_9 = __Pyx_PyList_Append(__pyx_v_exc_specs, __pyx_t_5); if (unlikely(__pyx_t_9 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 225; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

    /* "thriftrw/spec/service.pyx":224
 * 
 *         exc_specs = []
 *         for exc in exceptions:             # <<<<<<<<<<<<<<
 *             exc_specs.append(
 *                 FieldSpec.compile(
 */
  }
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "thriftrw/spec/service.pyx":233
 *             )
 * 
 *         return cls(result_name, return_spec, exc_specs)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_3 = PyTuple_New(3); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 233; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_INCREF(__pyx_v_result_name);
  __Pyx_GIVEREF(__pyx_v_result_name);
  PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_v_result_name);
  __Pyx_INCREF(__pyx_v_return_spec);
  __Pyx_GIVEREF(__pyx_v_return_spec);
  PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_v_return_spec);
  __Pyx_INCREF(__pyx_v_exc_specs);
  __Pyx_GIVEREF(__pyx_v_exc_specs);
  PyTuple_SET_ITEM(__pyx_t_3, 2, __pyx_v_exc_specs);
  __pyx_t_5 = __Pyx_PyObject_Call(((PyObject *)__pyx_v_cls), __pyx_t_3, NULL); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 233; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_r = __pyx_t_5;
  __pyx_t_5 = 0;
  goto __pyx_L0;

  /* "thriftrw/spec/service.pyx":201
 * 
 *     @classmethod
 *     def compile(cls, return_type, exceptions, service_name, function_name):             # <<<<<<<<<<<<<<
 *         """Compiles information from the AST into a FunctionResultSpec.
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("thriftrw.spec.service.FunctionResultSpec.compile", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_result_name);
  __Pyx_XDECREF(__pyx_v_return_spec);
  __Pyx_XDECREF(__pyx_v_exc_specs);
  __Pyx_XDECREF(__pyx_v_exc);
  __Pyx_XDECREF(__pyx_v_exceptions);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "thriftrw/spec/service.pxd":33
 * 
 * cdef class FunctionResultSpec(UnionTypeSpec):
 *     cdef public TypeSpec return_spec             # <<<<<<<<<<<<<<
 *     cdef public list exception_specs
 *     cdef public FunctionSpec function
 */

/* Python wrapper */
static PyObject *__pyx_pw_8thriftrw_4spec_7service_18FunctionResultSpec_11return_spec_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_8thriftrw_4spec_7service_18FunctionResultSpec_11return_spec_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_8thriftrw_4spec_7service_18FunctionResultSpec_11return_spec___get__(((struct __pyx_obj_8thriftrw_4spec_7service_FunctionResultSpec *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_8thriftrw_4spec_7service_18FunctionResultSpec_11return_spec___get__(struct __pyx_obj_8thriftrw_4spec_7service_FunctionResultSpec *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->return_spec));
  __pyx_r = ((PyObject *)__pyx_v_self->return_spec);
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_8thriftrw_4spec_7service_18FunctionResultSpec_11return_spec_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_8thriftrw_4spec_7service_18FunctionResultSpec_11return_spec_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_8thriftrw_4spec_7service_18FunctionResultSpec_11return_spec_2__set__(((struct __pyx_obj_8thriftrw_4spec_7service_FunctionResultSpec *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_8thriftrw_4spec_7service_18FunctionResultSpec_11return_spec_2__set__(struct __pyx_obj_8thriftrw_4spec_7service_FunctionResultSpec *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__set__", 0);
  if (!(likely(((__pyx_v_value) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_value, __pyx_ptype_8thriftrw_4spec_4base_TypeSpec))))) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 33; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_1 = __pyx_v_value;
  __Pyx_INCREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->return_spec);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->return_spec));
  __pyx_v_self->return_spec = ((struct __pyx_obj_8thriftrw_4spec_4base_TypeSpec *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("thriftrw.spec.service.FunctionResultSpec.return_spec.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_8thriftrw_4spec_7service_18FunctionResultSpec_11return_spec_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_8thriftrw_4spec_7service_18FunctionResultSpec_11return_spec_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_8thriftrw_4spec_7service_18FunctionResultSpec_11return_spec_4__del__(((struct __pyx_obj_8thriftrw_4spec_7service_FunctionResultSpec *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_8thriftrw_4spec_7service_18FunctionResultSpec_11return_spec_4__del__(struct __pyx_obj_8thriftrw_4spec_7service_FunctionResultSpec *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->return_spec);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->return_spec));
  __pyx_v_self->return_spec = ((struct __pyx_obj_8thriftrw_4spec_4base_TypeSpec *)Py_None);

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "thriftrw/spec/service.pxd":34
 * cdef class FunctionResultSpec(UnionTypeSpec):
 *     cdef public TypeSpec return_spec
 *     cdef public list exception_specs             # <<<<<<<<<<<<<<
 *     cdef public FunctionSpec function
 *     cdef public object exception_ids
 */

/* Python wrapper */
static PyObject *__pyx_pw_8thriftrw_4spec_7service_18FunctionResultSpec_15exception_specs_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_8thriftrw_4spec_7service_18FunctionResultSpec_15exception_specs_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_8thriftrw_4spec_7service_18FunctionResultSpec_15exception_specs___get__(((struct __pyx_obj_8thriftrw_4spec_7service_FunctionResultSpec *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_8thriftrw_4spec_7service_18FunctionResultSpec_15exception_specs___get__(struct __pyx_obj_8thriftrw_4spec_7service_FunctionResultSpec *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->exception_specs);
  __pyx_r = __pyx_v_self->exception_specs;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_8thriftrw_4spec_7service_18FunctionResultSpec_15exception_specs_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_8thriftrw_4spec_7service_18FunctionResultSpec_15exception_specs_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_8thriftrw_4spec_7service_18FunctionResultSpec_15exception_specs_2__set__(((struct __pyx_obj_8thriftrw_4spec_7service_FunctionResultSpec *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_8thriftrw_4spec_7service_18FunctionResultSpec_15exception_specs_2__set__(struct __pyx_obj_8thriftrw_4spec_7service_FunctionResultSpec *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__set__", 0);
  if (!(likely(PyList_CheckExact(__pyx_v_value))||((__pyx_v_value) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_v_value)->tp_name), 0))) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 34; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_1 = __pyx_v_value;
  __Pyx_INCREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->exception_specs);
  __Pyx_DECREF(__pyx_v_self->exception_specs);
  __pyx_v_self->exception_specs = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("thriftrw.spec.service.FunctionResultSpec.exception_specs.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_8thriftrw_4spec_7service_18FunctionResultSpec_15exception_specs_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_8thriftrw_4spec_7service_18FunctionResultSpec_15exception_specs_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_8thriftrw_4spec_7service_18FunctionResultSpec_15exception_specs_4__del__(((struct __pyx_obj_8thriftrw_4spec_7service_FunctionResultSpec *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_8thriftrw_4spec_7service_18FunctionResultSpec_15exception_specs_4__del__(struct __pyx_obj_8thriftrw_4spec_7service_FunctionResultSpec *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->exception_specs);
  __Pyx_DECREF(__pyx_v_self->exception_specs);
  __pyx_v_self->exception_specs = ((PyObject*)Py_None);

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "thriftrw/spec/service.pxd":35
 *     cdef public TypeSpec return_spec
 *     cdef public list exception_specs
 *     cdef public FunctionSpec function             # <<<<<<<<<<<<<<
 *     cdef public object exception_ids
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_8thriftrw_4spec_7service_18FunctionResultSpec_8function_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_8thriftrw_4spec_7service_18FunctionResultSpec_8function_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_8thriftrw_4spec_7service_18FunctionResultSpec_8function___get__(((struct __pyx_obj_8thriftrw_4spec_7service_FunctionResultSpec *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_8thriftrw_4spec_7service_18FunctionResultSpec_8function___get__(struct __pyx_obj_8thriftrw_4spec_7service_FunctionResultSpec *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->function));
  __pyx_r = ((PyObject *)__pyx_v_self->function);
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_8thriftrw_4spec_7service_18FunctionResultSpec_8function_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_8thriftrw_4spec_7service_18FunctionResultSpec_8function_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_8thriftrw_4spec_7service_18FunctionResultSpec_8function_2__set__(((struct __pyx_obj_8thriftrw_4spec_7service_FunctionResultSpec *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_8thriftrw_4spec_7service_18FunctionResultSpec_8function_2__set__(struct __pyx_obj_8thriftrw_4spec_7service_FunctionResultSpec *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__set__", 0);
  if (!(likely(((__pyx_v_value) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_value, __pyx_ptype_8thriftrw_4spec_7service_FunctionSpec))))) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 35; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_1 = __pyx_v_value;
  __Pyx_INCREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->function);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->function));
  __pyx_v_self->function = ((struct __pyx_obj_8thriftrw_4spec_7service_FunctionSpec *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("thriftrw.spec.service.FunctionResultSpec.function.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_8thriftrw_4spec_7service_18FunctionResultSpec_8function_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_8thriftrw_4spec_7service_18FunctionResultSpec_8function_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_8thriftrw_4spec_7service_18FunctionResultSpec_8function_4__del__(((struct __pyx_obj_8thriftrw_4spec_7service_FunctionResultSpec *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_8thriftrw_4spec_7service_18FunctionResultSpec_8function_4__del__(struct __pyx_obj_8thriftrw_4spec_7service_FunctionResultSpec *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->function);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->function));
  __pyx_v_self->function = ((struct __pyx_obj_8thriftrw_4spec_7service_FunctionSpec *)Py_None);

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "thriftrw/spec/service.pxd":36
 *     cdef public list exception_specs
 *     cdef public FunctionSpec function
 *     cdef public object exception_ids             # <<<<<<<<<<<<<<
 * 
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_8thriftrw_4spec_7service_18FunctionResultSpec_13exception_ids_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_8thriftrw_4spec_7service_18FunctionResultSpec_13exception_ids_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_8thriftrw_4spec_7service_18FunctionResultSpec_13exception_ids___get__(((struct __pyx_obj_8thriftrw_4spec_7service_FunctionResultSpec *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_8thriftrw_4spec_7service_18FunctionResultSpec_13exception_ids___get__(struct __pyx_obj_8thriftrw_4spec_7service_FunctionResultSpec *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->exception_ids);
  __pyx_r = __pyx_v_self->exception_ids;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_8thriftrw_4spec_7service_18FunctionResultSpec_13exception_ids_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_8thriftrw_4spec_7service_18FunctionResultSpec_13exception_ids_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_8thriftrw_4spec_7service_18FunctionResultSpec_13exception_ids_2__set__(((struct __pyx_obj_8thriftrw_4spec_7service_FunctionResultSpec *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_8thriftrw_4spec_7service_18FunctionResultSpec_13exception_ids_2__set__(struct __pyx_obj_8thriftrw_4spec_7service_FunctionResultSpec *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 0);
  __Pyx_INCREF(__pyx_v_value);
  __Pyx_GIVEREF(__pyx_v_value);
  __Pyx_GOTREF(__pyx_v_self->exception_ids);
  __Pyx_DECREF(__pyx_v_self->exception_ids);
  __pyx_v_self->exception_ids = __pyx_v_value;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_8thriftrw_4spec_7service_18FunctionResultSpec_13exception_ids_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_8thriftrw_4spec_7service_18FunctionResultSpec_13exception_ids_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_8thriftrw_4spec_7service_18FunctionResultSpec_13exception_ids_4__del__(((struct __pyx_obj_8thriftrw_4spec_7service_FunctionResultSpec *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_8thriftrw_4spec_7service_18FunctionResultSpec_13exception_ids_4__del__(struct __pyx_obj_8thriftrw_4spec_7service_FunctionResultSpec *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->exception_ids);
  __Pyx_DECREF(__pyx_v_self->exception_ids);
  __pyx_v_self->exception_ids = Py_None;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "thriftrw/spec/service.pyx":279
 *     """
 * 
 *     def __init__(self, name, args_spec, result_spec, oneway):             # <<<<<<<<<<<<<<
 *         self.name = str(name)
 *         self.args_spec = args_spec
 */

/* Python wrapper */
static int __pyx_pw_8thriftrw_4spec_7service_12FunctionSpec_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_8thriftrw_4spec_7service_12FunctionSpec_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_name = 0;
  PyObject *__pyx_v_args_spec = 0;
  PyObject *__pyx_v_result_spec = 0;
  PyObject *__pyx_v_oneway = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_name,&__pyx_n_s_args_spec,&__pyx_n_s_result_spec,&__pyx_n_s_oneway,0};
    PyObject* values[4] = {0,0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_name)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_args_spec)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__init__", 1, 4, 4, 1); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 279; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_result_spec)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__init__", 1, 4, 4, 2); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 279; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  3:
        if (likely((values[3] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_oneway)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__init__", 1, 4, 4, 3); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 279; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__init__") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 279; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 4) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
      values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
    }
    __pyx_v_name = values[0];
    __pyx_v_args_spec = values[1];
    __pyx_v_result_spec = values[2];
    __pyx_v_oneway = values[3];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 1, 4, 4, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 279; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("thriftrw.spec.service.FunctionSpec.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_8thriftrw_4spec_7service_12FunctionSpec___init__(((struct __pyx_obj_8thriftrw_4spec_7service_FunctionSpec *)__pyx_v_self), __pyx_v_name, __pyx_v_args_spec, __pyx_v_result_spec, __pyx_v_oneway);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_8thriftrw_4spec_7service_12FunctionSpec___init__(struct __pyx_obj_8thriftrw_4spec_7service_FunctionSpec *__pyx_v_self, PyObject *__pyx_v_name, PyObject *__pyx_v_args_spec, PyObject *__pyx_v_result_spec, PyObject *__pyx_v_oneway) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_t_3;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "thriftrw/spec/service.pyx":280
 * 
 *     def __init__(self, name, args_spec, result_spec, oneway):
 *         self.name = str(name)             # <<<<<<<<<<<<<<
 *         self.args_spec = args_spec
 *         self.result_spec = result_spec
 */
  __pyx_t_1 = PyTuple_New(1); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 280; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_v_name);
  __Pyx_GIVEREF(__pyx_v_name);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_v_name);
  __pyx_t_2 = __Pyx_PyObject_Call(((PyObject *)(&PyString_Type)), __pyx_t_1, NULL); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 280; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (!(likely(PyString_CheckExact(__pyx_t_2))||((__pyx_t_2) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "str", Py_TYPE(__pyx_t_2)->tp_name), 0))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 280; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GIVEREF(__pyx_t_2);
  __Pyx_GOTREF(__pyx_v_self->name);
  __Pyx_DECREF(__pyx_v_self->name);
  __pyx_v_self->name = ((PyObject*)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "thriftrw/spec/service.pyx":281
 *     def __init__(self, name, args_spec, result_spec, oneway):
 *         self.name = str(name)
 *         self.args_spec = args_spec             # <<<<<<<<<<<<<<
 *         self.result_spec = result_spec
 *         self.oneway = oneway
 */
  if (!(likely(((__pyx_v_args_spec) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_args_spec, __pyx_ptype_8thriftrw_4spec_7service_FunctionArgsSpec))))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 281; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_2 = __pyx_v_args_spec;
  __Pyx_INCREF(__pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_2);
  __Pyx_GOTREF(__pyx_v_self->args_spec);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->args_spec));
  __pyx_v_self->args_spec = ((struct __pyx_obj_8thriftrw_4spec_7service_FunctionArgsSpec *)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "thriftrw/spec/service.pyx":282
 *         self.name = str(name)
 *         self.args_spec = args_spec
 *         self.result_spec = result_spec             # <<<<<<<<<<<<<<
 *         self.oneway = oneway
 *         self.linked = False
 */
  if (!(likely(((__pyx_v_result_spec) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_result_spec, __pyx_ptype_8thriftrw_4spec_7service_FunctionResultSpec))))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 282; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_2 = __pyx_v_result_spec;
  __Pyx_INCREF(__pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_2);
  __Pyx_GOTREF(__pyx_v_self->result_spec);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->result_spec));
  __pyx_v_self->result_spec = ((struct __pyx_obj_8thriftrw_4spec_7service_FunctionResultSpec *)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "thriftrw/spec/service.pyx":283
 *         self.args_spec = args_spec
 *         self.result_spec = result_spec
 *         self.oneway = oneway             # <<<<<<<<<<<<<<
 *         self.linked = False
 *         self.surface = None
 */
  __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_v_oneway); if (unlikely((__pyx_t_3 == (int)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 283; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_v_self->oneway = __pyx_t_3;

  /* "thriftrw/spec/service.pyx":284
 *         self.result_spec = result_spec
 *         self.oneway = oneway
 *         self.linked = False             # <<<<<<<<<<<<<<
 *         self.surface = None
 *         self.service = None
 */
  __pyx_v_self->linked = 0;

  /* "thriftrw/spec/service.pyx":285
 *         self.oneway = oneway
 *         self.linked = False
 *         self.surface = None             # <<<<<<<<<<<<<<
 *         self.service = None
 * 
 */
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->surface);
  __Pyx_DECREF(__pyx_v_self->surface);
  __pyx_v_self->surface = Py_None;

  /* "thriftrw/spec/service.pyx":286
 *         self.linked = False
 *         self.surface = None
 *         self.service = None             # <<<<<<<<<<<<<<
 * 
 *     @classmethod
 */
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->service);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->service));
  __pyx_v_self->service = ((struct __pyx_obj_8thriftrw_4spec_7service_ServiceSpec *)Py_None);

  /* "thriftrw/spec/service.pyx":279
 *     """
 * 
 *     def __init__(self, name, args_spec, result_spec, oneway):             # <<<<<<<<<<<<<<
 *         self.name = str(name)
 *         self.args_spec = args_spec
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("thriftrw.spec.service.FunctionSpec.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "thriftrw/spec/service.pyx":289
 * 
 *     @classmethod
 *     def compile(cls, func, service_name):             # <<<<<<<<<<<<<<
 *         args_spec = FunctionArgsSpec.compile(
 *             parameters=func.parameters,
 */

/* Python wrapper */
static PyObject *__pyx_pw_8thriftrw_4spec_7service_12FunctionSpec_3compile(PyObject *__pyx_v_cls, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_pw_8thriftrw_4spec_7service_12FunctionSpec_3compile(PyObject *__pyx_v_cls, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_func = 0;
  PyObject *__pyx_v_service_name = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("compile (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_func,&__pyx_n_s_service_name,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_func)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_service_name)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("compile", 1, 2, 2, 1); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 289; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "compile") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 289; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_func = values[0];
    __pyx_v_service_name = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("compile", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 289; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("thriftrw.spec.service.FunctionSpec.compile", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_8thriftrw_4spec_7service_12FunctionSpec_2compile(((PyTypeObject*)__pyx_v_cls), __pyx_v_func, __pyx_v_service_name);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_8thriftrw_4spec_7service_12FunctionSpec_2compile(PyTypeObject *__pyx_v_cls, PyObject *__pyx_v_func, PyObject *__pyx_v_service_name) {
  PyObject *__pyx_v_args_spec = NULL;
  PyObject *__pyx_v_result_spec = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  int __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("compile", 0);

  /* "thriftrw/spec/service.pyx":290
 *     @classmethod
 *     def compile(cls, func, service_name):
 *         args_spec = FunctionArgsSpec.compile(             # <<<<<<<<<<<<<<
 *             parameters=func.parameters,
 *             service_name=service_name,
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_8thriftrw_4spec_7service_FunctionArgsSpec), __pyx_n_s_compile); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 290; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);

  /* "thriftrw/spec/service.pyx":291
 *     def compile(cls, func, service_name):
 *         args_spec = FunctionArgsSpec.compile(
 *             parameters=func.parameters,             # <<<<<<<<<<<<<<
 *             service_name=service_name,
 *             function_name=func.name,
 */
  __pyx_t_2 = PyDict_New(); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 291; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_func, __pyx_n_s_parameters); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 291; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_parameters, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 291; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "thriftrw/spec/service.pyx":292
 *         args_spec = FunctionArgsSpec.compile(
 *             parameters=func.parameters,
 *             service_name=service_name,             # <<<<<<<<<<<<<<
 *             function_name=func.name,
 *         )
 */
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_service_name, __pyx_v_service_name) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 291; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "thriftrw/spec/service.pyx":293
 *             parameters=func.parameters,
 *             service_name=service_name,
 *             function_name=func.name,             # <<<<<<<<<<<<<<
 *         )
 * 
 */
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_func, __pyx_n_s_name); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 293; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_function_name, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 291; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "thriftrw/spec/service.pyx":290
 *     @classmethod
 *     def compile(cls, func, service_name):
 *         args_spec = FunctionArgsSpec.compile(             # <<<<<<<<<<<<<<
 *             parameters=func.parameters,
 *             service_name=service_name,
 */
  __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_empty_tuple, __pyx_t_2); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 290; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_args_spec = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "thriftrw/spec/service.pyx":296
 *         )
 * 
 *         if func.oneway:             # <<<<<<<<<<<<<<
 *             result_spec = None
 *             if func.return_type is not None:
 */
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_func, __pyx_n_s_oneway); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 296; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_4 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 296; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (__pyx_t_4) {

    /* "thriftrw/spec/service.pyx":297
 * 
 *         if func.oneway:
 *             result_spec = None             # <<<<<<<<<<<<<<
 *             if func.return_type is not None:
 *                 raise ThriftCompilerError(
 */
    __Pyx_INCREF(Py_None);
    __pyx_v_result_spec = Py_None;

    /* "thriftrw/spec/service.pyx":298
 *         if func.oneway:
 *             result_spec = None
 *             if func.return_type is not None:             # <<<<<<<<<<<<<<
 *                 raise ThriftCompilerError(
 *                     'Function "%s.%s" is oneway. It cannot return a value.'
 */
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_func, __pyx_n_s_return_type); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 298; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = (__pyx_t_3 != Py_None);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_5 = (__pyx_t_4 != 0);
    if (__pyx_t_5) {

      /* "thriftrw/spec/service.pyx":299
 *             result_spec = None
 *             if func.return_type is not None:
 *                 raise ThriftCompilerError(             # <<<<<<<<<<<<<<
 *                     'Function "%s.%s" is oneway. It cannot return a value.'
 *                     % (service_name, func.name)
 */
      __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_ThriftCompilerError); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 299; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_2);

      /* "thriftrw/spec/service.pyx":301
 *                 raise ThriftCompilerError(
 *                     'Function "%s.%s" is oneway. It cannot return a value.'
 *                     % (service_name, func.name)             # <<<<<<<<<<<<<<
 *                 )
 *             if func.exceptions:
 */
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_func, __pyx_n_s_name); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 301; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_t_6 = PyTuple_New(2); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 301; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_INCREF(__pyx_v_service_name);
      __Pyx_GIVEREF(__pyx_v_service_name);
      PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_v_service_name);
      __Pyx_GIVEREF(__pyx_t_1);
      PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_t_1);
      __pyx_t_1 = 0;
      __pyx_t_1 = PyUnicode_Format(__pyx_kp_u_Function_s_s_is_oneway_It_cannot, __pyx_t_6); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 301; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __pyx_t_6 = NULL;
      if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_2))) {
        __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_2);
        if (likely(__pyx_t_6)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
          __Pyx_INCREF(__pyx_t_6);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_2, function);
        }
      }
      if (!__pyx_t_6) {
        __pyx_t_3 = __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_1); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 299; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        __Pyx_GOTREF(__pyx_t_3);
      } else {
        __pyx_t_7 = PyTuple_New(1+1); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 299; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_7);
        __Pyx_GIVEREF(__pyx_t_6); PyTuple_SET_ITEM(__pyx_t_7, 0, __pyx_t_6); __pyx_t_6 = NULL;
        __Pyx_GIVEREF(__pyx_t_1);
        PyTuple_SET_ITEM(__pyx_t_7, 0+1, __pyx_t_1);
        __pyx_t_1 = 0;
        __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_7, NULL); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 299; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      }
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_Raise(__pyx_t_3, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      {__pyx_filename = __pyx_f[0]; __pyx_lineno = 299; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

      /* "thriftrw/spec/service.pyx":298
 *         if func.oneway:
 *             result_spec = None
 *             if func.return_type is not None:             # <<<<<<<<<<<<<<
 *                 raise ThriftCompilerError(
 *                     'Function "%s.%s" is oneway. It cannot return a value.'
 */
    }

    /* "thriftrw/spec/service.pyx":303
 *                     % (service_name, func.name)
 *                 )
 *             if func.exceptions:             # <<<<<<<<<<<<<<
 *                 raise ThriftCompilerError(
 *                     'Function "%s.%s" is oneway. It cannot raise exceptions.'
 */
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_func, __pyx_n_s_exceptions); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 303; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_5 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_5 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 303; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (__pyx_t_5) {

      /* "thriftrw/spec/service.pyx":304
 *                 )
 *             if func.exceptions:
 *                 raise ThriftCompilerError(             # <<<<<<<<<<<<<<
 *                     'Function "%s.%s" is oneway. It cannot raise exceptions.'
 *                     % (service_name, func.name)
 */
      __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_ThriftCompilerError); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 304; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_2);

      /* "thriftrw/spec/service.pyx":306
 *                 raise ThriftCompilerError(
 *                     'Function "%s.%s" is oneway. It cannot raise exceptions.'
 *                     % (service_name, func.name)             # <<<<<<<<<<<<<<
 *                 )
 *         else:
 */
      __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_v_func, __pyx_n_s_name); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 306; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_7);
      __pyx_t_1 = PyTuple_New(2); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 306; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_INCREF(__pyx_v_service_name);
      __Pyx_GIVEREF(__pyx_v_service_name);
      PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_v_service_name);
      __Pyx_GIVEREF(__pyx_t_7);
      PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_t_7);
      __pyx_t_7 = 0;
      __pyx_t_7 = PyUnicode_Format(__pyx_kp_u_Function_s_s_is_oneway_It_cannot_2, __pyx_t_1); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 306; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_7);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __pyx_t_1 = NULL;
      if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_2))) {
        __pyx_t_1 = PyMethod_GET_SELF(__pyx_t_2);
        if (likely(__pyx_t_1)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
          __Pyx_INCREF(__pyx_t_1);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_2, function);
        }
      }
      if (!__pyx_t_1) {
        __pyx_t_3 = __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_7); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 304; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        __Pyx_GOTREF(__pyx_t_3);
      } else {
        __pyx_t_6 = PyTuple_New(1+1); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 304; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_6);
        __Pyx_GIVEREF(__pyx_t_1); PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_1); __pyx_t_1 = NULL;
        __Pyx_GIVEREF(__pyx_t_7);
        PyTuple_SET_ITEM(__pyx_t_6, 0+1, __pyx_t_7);
        __pyx_t_7 = 0;
        __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_6, NULL); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 304; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      }
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_Raise(__pyx_t_3, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      {__pyx_filename = __pyx_f[0]; __pyx_lineno = 304; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

      /* "thriftrw/spec/service.pyx":303
 *                     % (service_name, func.name)
 *                 )
 *             if func.exceptions:             # <<<<<<<<<<<<<<
 *                 raise ThriftCompilerError(
 *                     'Function "%s.%s" is oneway. It cannot raise exceptions.'
 */
    }

    /* "thriftrw/spec/service.pyx":296
 *         )
 * 
 *         if func.oneway:             # <<<<<<<<<<<<<<
 *             result_spec = None
 *             if func.return_type is not None:
 */
    goto __pyx_L3;
  }

  /* "thriftrw/spec/service.pyx":309
 *                 )
 *         else:
 *             result_spec = FunctionResultSpec.compile(             # <<<<<<<<<<<<<<
 *                 return_type=func.return_type,
 *                 exceptions=func.exceptions,
 */
  /*else*/ {
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_8thriftrw_4spec_7service_FunctionResultSpec), __pyx_n_s_compile); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 309; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);

    /* "thriftrw/spec/service.pyx":310
 *         else:
 *             result_spec = FunctionResultSpec.compile(
 *                 return_type=func.return_type,             # <<<<<<<<<<<<<<
 *                 exceptions=func.exceptions,
 *                 service_name=service_name,
 */
    __pyx_t_2 = PyDict_New(); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 310; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_v_func, __pyx_n_s_return_type); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 310; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_6);
    if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_return_type, __pyx_t_6) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 310; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

    /* "thriftrw/spec/service.pyx":311
 *             result_spec = FunctionResultSpec.compile(
 *                 return_type=func.return_type,
 *                 exceptions=func.exceptions,             # <<<<<<<<<<<<<<
 *                 service_name=service_name,
 *                 function_name=func.name,
 */
    __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_v_func, __pyx_n_s_exceptions); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 311; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_6);
    if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_exceptions, __pyx_t_6) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 310; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

    /* "thriftrw/spec/service.pyx":312
 *                 return_type=func.return_type,
 *                 exceptions=func.exceptions,
 *                 service_name=service_name,             # <<<<<<<<<<<<<<
 *                 function_name=func.name,
 *             )
 */
    if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_service_name, __pyx_v_service_name) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 310; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "thriftrw/spec/service.pyx":313
 *                 exceptions=func.exceptions,
 *                 service_name=service_name,
 *                 function_name=func.name,             # <<<<<<<<<<<<<<
 *             )
 * 
 */
    __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_v_func, __pyx_n_s_name); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 313; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_6);
    if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_function_name, __pyx_t_6) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 310; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

    /* "thriftrw/spec/service.pyx":309
 *                 )
 *         else:
 *             result_spec = FunctionResultSpec.compile(             # <<<<<<<<<<<<<<
 *                 return_type=func.return_type,
 *                 exceptions=func.exceptions,
 */
    __pyx_t_6 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_empty_tuple, __pyx_t_2); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 309; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_v_result_spec = __pyx_t_6;
    __pyx_t_6 = 0;
  }
  __pyx_L3:;

  /* "thriftrw/spec/service.pyx":316
 *             )
 * 
 *         return cls(func.name, args_spec, result_spec, func.oneway)             # <<<<<<<<<<<<<<
 * 
 *     cpdef FunctionSpec link(self, scope, ServiceSpec service):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_v_func, __pyx_n_s_name); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 316; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_func, __pyx_n_s_oneway); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 316; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyTuple_New(4); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 316; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_6);
  PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_6);
  __Pyx_INCREF(__pyx_v_args_spec);
  __Pyx_GIVEREF(__pyx_v_args_spec);
  PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_v_args_spec);
  __Pyx_INCREF(__pyx_v_result_spec);
  __Pyx_GIVEREF(__pyx_v_result_spec);
  PyTuple_SET_ITEM(__pyx_t_3, 2, __pyx_v_result_spec);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_3, 3, __pyx_t_2);
  __pyx_t_6 = 0;
  __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_Call(((PyObject *)__pyx_v_cls), __pyx_t_3, NULL); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 316; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "thriftrw/spec/service.pyx":289
 * 
 *     @classmethod
 *     def compile(cls, func, service_name):             # <<<<<<<<<<<<<<
 *         args_spec = FunctionArgsSpec.compile(
 *             parameters=func.parameters,
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_AddTraceback("thriftrw.spec.service.FunctionSpec.compile", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_args_spec);
  __Pyx_XDECREF(__pyx_v_result_spec);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "thriftrw/spec/service.pyx":318
 *         return cls(func.name, args_spec, result_spec, func.oneway)
 * 
 *     cpdef FunctionSpec link(self, scope, ServiceSpec service):             # <<<<<<<<<<<<<<
 *         if not self.linked:
 *             self.service = service
 */

static PyObject *__pyx_pw_8thriftrw_4spec_7service_12FunctionSpec_5link(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static struct __pyx_obj_8thriftrw_4spec_7service_FunctionSpec *__pyx_f_8thriftrw_4spec_7service_12FunctionSpec_link(struct __pyx_obj_8thriftrw_4spec_7service_FunctionSpec *__pyx_v_self, PyObject *__pyx_v_scope, struct __pyx_obj_8thriftrw_4spec_7service_ServiceSpec *__pyx_v_service, int __pyx_skip_dispatch) {
  PyObject *__pyx_v_result_spec_surface = NULL;
  struct __pyx_obj_8thriftrw_4spec_7service_FunctionSpec *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  Py_ssize_t __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("link", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0)) {
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_link); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 318; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)__pyx_pw_8thriftrw_4spec_7service_12FunctionSpec_5link)) {
      __Pyx_XDECREF(((PyObject *)__pyx_r));
      __Pyx_INCREF(__pyx_t_1);
      __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
      __pyx_t_5 = 0;
      if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_3))) {
        __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
        if (likely(__pyx_t_4)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
          __Pyx_INCREF(__pyx_t_4);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_3, function);
          __pyx_t_5 = 1;
        }
      }
      __pyx_t_6 = PyTuple_New(2+__pyx_t_5); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 318; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_6);
      if (__pyx_t_4) {
        __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_4); __pyx_t_4 = NULL;
      }
      __Pyx_INCREF(__pyx_v_scope);
      __Pyx_GIVEREF(__pyx_v_scope);
      PyTuple_SET_ITEM(__pyx_t_6, 0+__pyx_t_5, __pyx_v_scope);
      __Pyx_INCREF(((PyObject *)__pyx_v_service));
      __Pyx_GIVEREF(((PyObject *)__pyx_v_service));
      PyTuple_SET_ITEM(__pyx_t_6, 1+__pyx_t_5, ((PyObject *)__pyx_v_service));
      __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_6, NULL); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 318; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (!(likely(((__pyx_t_2) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_2, __pyx_ptype_8thriftrw_4spec_7service_FunctionSpec))))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 318; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __pyx_r = ((struct __pyx_obj_8thriftrw_4spec_7service_FunctionSpec *)__pyx_t_2);
      __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      goto __pyx_L0;
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  }

  /* "thriftrw/spec/service.pyx":319
 * 
 *     cpdef FunctionSpec link(self, scope, ServiceSpec service):
 *         if not self.linked:             # <<<<<<<<<<<<<<
 *             self.service = service
 *             self.linked = True
 */
  __pyx_t_7 = ((!(__pyx_v_self->linked != 0)) != 0);
  if (__pyx_t_7) {

    /* "thriftrw/spec/service.pyx":320
 *     cpdef FunctionSpec link(self, scope, ServiceSpec service):
 *         if not self.linked:
 *             self.service = service             # <<<<<<<<<<<<<<
 *             self.linked = True
 *             if self.result_spec:
 */
    __Pyx_INCREF(((PyObject *)__pyx_v_service));
    __Pyx_GIVEREF(((PyObject *)__pyx_v_service));
    __Pyx_GOTREF(__pyx_v_self->service);
    __Pyx_DECREF(((PyObject *)__pyx_v_self->service));
    __pyx_v_self->service = __pyx_v_service;

    /* "thriftrw/spec/service.pyx":321
 *         if not self.linked:
 *             self.service = service
 *             self.linked = True             # <<<<<<<<<<<<<<
 *             if self.result_spec:
 *                 self.result_spec = self.result_spec.link(scope, self)
 */
    __pyx_v_self->linked = 1;

    /* "thriftrw/spec/service.pyx":322
 *             self.service = service
 *             self.linked = True
 *             if self.result_spec:             # <<<<<<<<<<<<<<
 *                 self.result_spec = self.result_spec.link(scope, self)
 *                 result_spec_surface = self.result_spec.surface
 */
    __pyx_t_7 = __Pyx_PyObject_IsTrue(((PyObject *)__pyx_v_self->result_spec)); if (unlikely(__pyx_t_7 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 322; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    if (__pyx_t_7) {

      /* "thriftrw/spec/service.pyx":323
 *             self.linked = True
 *             if self.result_spec:
 *                 self.result_spec = self.result_spec.link(scope, self)             # <<<<<<<<<<<<<<
 *                 result_spec_surface = self.result_spec.surface
 *             else:
 */
      __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self->result_spec), __pyx_n_s_link); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 323; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_t_3 = NULL;
      __pyx_t_5 = 0;
      if (CYTHON_COMPILING_IN_CPYTHON && likely(PyMethod_Check(__pyx_t_2))) {
        __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
        if (likely(__pyx_t_3)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
          __Pyx_INCREF(__pyx_t_3);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_2, function);
          __pyx_t_5 = 1;
        }
      }
      __pyx_t_6 = PyTuple_New(2+__pyx_t_5); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 323; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_6);
      if (__pyx_t_3) {
        __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_3); __pyx_t_3 = NULL;
      }
      __Pyx_INCREF(__pyx_v_scope);
      __Pyx_GIVEREF(__pyx_v_scope);
      PyTuple_SET_ITEM(__pyx_t_6, 0+__pyx_t_5, __pyx_v_scope);
      __Pyx_INCREF(((PyObject *)__pyx_v_self));
      __Pyx_GIVEREF(((PyObject *)__pyx_v_self));
      PyTuple_SET_ITEM(__pyx_t_6, 1+__pyx_t_5, ((PyObject *)__pyx_v_self));
      __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_6, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 323; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_8thriftrw_4spec_7service_FunctionResultSpec))))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 323; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GIVEREF(__pyx_t_1);
      __Pyx_GOTREF(__pyx_v_self->result_spec);
      __Pyx_DECREF(((PyObject *)__pyx_v_self->result_spec));
      __pyx_v_self->result_spec = ((struct __pyx_obj_8thriftrw_4spec_7service_FunctionResultSpec *)__pyx_t_1);
      __pyx_t_1 = 0;

      /* "thriftrw/spec/service.pyx":324
 *             if self.result_spec:
 *                 self.result_spec = self.result_spec.link(scope, self)
 *                 result_spec_surface = self.result_spec.surface             # <<<<<<<<<<<<<<
 *             else:
 *                 result_spec_surface = None
 */
      __pyx_t_1 = __pyx_v_self->result_spec->__pyx_base.surface;
      __Pyx_INCREF(__pyx_t_1);
      __pyx_v_result_spec_surface = __pyx_t_1;
      __pyx_t_1 = 0;

      /* "thriftrw/spec/service.pyx":322
 *             self.service = service
 *             self.linked = True
 *             if self.result_spec:             # <<<<<<<<<<<<<<
 *                 self.result_spec = self.result_spec.link(scope, self)
 *                 result_spec_surface = self.result_spec.surface
 */
      goto __pyx_L4;
    }

    /* "thriftrw/spec/service.pyx":326
 *                 result_spec_surface = self.result_spec.surface
 *             else:
 *                 result_spec_surface = None             # <<<<<<<<<<<<<<
 *             self.args_spec = self.args_spec.link(scope, self)
 * 
 */
    /*else*/ {
      __Pyx_INCREF(Py_None);
      __pyx_v_result_spec_surface = Py_None;
    }
    __pyx_L4:;

    /* "thriftrw/spec/service.pyx":327
 *             else:
 *                 result_spec_surface = None
 *             self.args_spec = self.args_spec.link(scope, self)             # <<<<<<<<<<<<<<
 * 
 *             self.surface = ServiceFunction(
 */
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self->args_spec), __pyx_n_s_link); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 327; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_6 = NULL;
    __pyx_t_5 = 0;
    if (CYTHON_COMPILING_IN_CPYTHON && likely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_6)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_6);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
        __pyx_t_5 = 1;
      }
    }
    __pyx_t_3 = PyTuple_New(2+__pyx_t_5); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 327; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    if (__pyx_t_6) {
      __Pyx_GIVEREF(__pyx_t_6); PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_6); __pyx_t_6 = NULL;
    }
    __Pyx_INCREF(__pyx_v_scope);
    __Pyx_GIVEREF(__pyx_v_scope);
    PyTuple_SET_ITEM(__pyx_t_3, 0+__pyx_t_5, __pyx_v_scope);
    __Pyx_INCREF(((PyObject *)__pyx_v_self));
    __Pyx_GIVEREF(((PyObject *)__pyx_v_self));
    PyTuple_SET_ITEM(__pyx_t_3, 1+__pyx_t_5, ((PyObject *)__pyx_v_self));
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_3, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 327; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_8thriftrw_4spec_7service_FunctionArgsSpec))))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 327; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GIVEREF(__pyx_t_1);
    __Pyx_GOTREF(__pyx_v_self->args_spec);
    __Pyx_DECREF(((PyObject *)__pyx_v_self->args_spec));
    __pyx_v_self->args_spec = ((struct __pyx_obj_8thriftrw_4spec_7service_FunctionArgsSpec *)__pyx_t_1);
    __pyx_t_1 = 0;

    /* "thriftrw/spec/service.pyx":329
 *             self.args_spec = self.args_spec.link(scope, self)
 * 
 *             self.surface = ServiceFunction(             # <<<<<<<<<<<<<<
 *                 self.name,
 *                 self.args_spec.surface,
 */
    __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_ServiceFunction); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 329; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_2);

    /* "thriftrw/spec/service.pyx":333
 *                 self.args_spec.surface,
 *                 result_spec_surface,
 *                 self,             # <<<<<<<<<<<<<<
 *             )
 *         return self
 */
    __pyx_t_3 = NULL;
    __pyx_t_5 = 0;
    if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_3)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_3);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
        __pyx_t_5 = 1;
      }
    }
    __pyx_t_6 = PyTuple_New(4+__pyx_t_5); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 329; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_6);
    if (__pyx_t_3) {
      __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_3); __pyx_t_3 = NULL;
    }
    __Pyx_INCREF(__pyx_v_self->name);
    __Pyx_GIVEREF(__pyx_v_self->name);
    PyTuple_SET_ITEM(__pyx_t_6, 0+__pyx_t_5, __pyx_v_self->name);
    __Pyx_INCREF(__pyx_v_self->args_spec->__pyx_base.surface);
    __Pyx_GIVEREF(__pyx_v_self->args_spec->__pyx_base.surface);
    PyTuple_SET_ITEM(__pyx_t_6, 1+__pyx_t_5, __pyx_v_self->args_spec->__pyx_base.surface);
    __Pyx_INCREF(__pyx_v_result_spec_surface);
    __Pyx_GIVEREF(__pyx_v_result_spec_surface);
    PyTuple_SET_ITEM(__pyx_t_6, 2+__pyx_t_5, __pyx_v_result_spec_surface);
    __Pyx_INCREF(((PyObject *)__pyx_v_self));
    __Pyx_GIVEREF(((PyObject *)__pyx_v_self));
    PyTuple_SET_ITEM(__pyx_t_6, 3+__pyx_t_5, ((PyObject *)__pyx_v_self));
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_6, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 329; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "thriftrw/spec/service.pyx":329
 *             self.args_spec = self.args_spec.link(scope, self)
 * 
 *             self.surface = ServiceFunction(             # <<<<<<<<<<<<<<
 *                 self.name,
 *                 self.args_spec.surface,
 */
    __Pyx_GIVEREF(__pyx_t_1);
    __Pyx_GOTREF(__pyx_v_self->surface);
    __Pyx_DECREF(__pyx_v_self->surface);
    __pyx_v_self->surface = __pyx_t_1;
    __pyx_t_1 = 0;

    /* "thriftrw/spec/service.pyx":319
 * 
 *     cpdef FunctionSpec link(self, scope, ServiceSpec service):
 *         if not self.linked:             # <<<<<<<<<<<<<<
 *             self.service = service
 *             self.linked = True
 */
  }

  /* "thriftrw/spec/service.pyx":335
 *                 self,
 *             )
 *         return self             # <<<<<<<<<<<<<<
 * 
 *     def __str__(self):
 */
  __Pyx_XDECREF(((PyObject *)__pyx_r));
  __Pyx_INCREF(((PyObject *)__pyx_v_self));
  __pyx_r = __pyx_v_self;
  goto __pyx_L0;

  /* "thriftrw/spec/service.pyx":318
 *         return cls(func.name, args_spec, result_spec, func.oneway)
 * 
 *     cpdef FunctionSpec link(self, scope, ServiceSpec service):             # <<<<<<<<<<<<<<
 *         if not self.linked:
 *             self.service = service
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("thriftrw.spec.service.FunctionSpec.link", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_result_spec_surface);
  __Pyx_XGIVEREF((PyObject *)__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_8thriftrw_4spec_7service_12FunctionSpec_5link(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_pw_8thriftrw_4spec_7service_12FunctionSpec_5link(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_scope = 0;
  struct __pyx_obj_8thriftrw_4spec_7service_ServiceSpec *__pyx_v_service = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("link (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_scope,&__pyx_n_s_service,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_scope)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_service)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("link", 1, 2, 2, 1); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 318; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "link") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 318; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_scope = values[0];
    __pyx_v_service = ((struct __pyx_obj_8thriftrw_4spec_7service_ServiceSpec *)values[1]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("link", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 318; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("thriftrw.spec.service.FunctionSpec.link", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_service), __pyx_ptype_8thriftrw_4spec_7service_ServiceSpec, 1, "service", 0))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 318; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_r = __pyx_pf_8thriftrw_4spec_7service_12FunctionSpec_4link(((struct __pyx_obj_8thriftrw_4spec_7service_FunctionSpec *)__pyx_v_self), __pyx_v_scope, __pyx_v_service);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_8thriftrw_4spec_7service_12FunctionSpec_4link(struct __pyx_obj_8thriftrw_4spec_7service_FunctionSpec *__pyx_v_self, PyObject *__pyx_v_scope, struct __pyx_obj_8thriftrw_4spec_7service_ServiceSpec *__pyx_v_service) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("link", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = ((PyObject *)__pyx_f_8thriftrw_4spec_7service_12FunctionSpec_link(__pyx_v_self, __pyx_v_scope, __pyx_v_service, 1)); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 318; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("thriftrw.spec.service.FunctionSpec.link", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "thriftrw/spec/service.pyx":337
 *         return self
 * 
 *     def __str__(self):             # <<<<<<<<<<<<<<
 *         return 'FunctionSpec(name=%r, args_spec=%r, result_spec=%r)' % (
 *             self.name, self.args_spec, self.result_spec
 */

/* Python wrapper */
static PyObject *__pyx_pw_8thriftrw_4spec_7service_12FunctionSpec_7__str__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_8thriftrw_4spec_7service_12FunctionSpec_7__str__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__str__ (wrapper)", 0);
  __pyx_r = __pyx_pf_8thriftrw_4spec_7service_12FunctionSpec_6__str__(((struct __pyx_obj_8thriftrw_4spec_7service_FunctionSpec *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_8thriftrw_4spec_7service_12FunctionSpec_6__str__(struct __pyx_obj_8thriftrw_4spec_7service_FunctionSpec *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__str__", 0);

  /* "thriftrw/spec/service.pyx":338
 * 
 *     def __str__(self):
 *         return 'FunctionSpec(name=%r, args_spec=%r, result_spec=%r)' % (             # <<<<<<<<<<<<<<
 *             self.name, self.args_spec, self.result_spec
 *         )
 */
  __Pyx_XDECREF(__pyx_r);

  /* "thriftrw/spec/service.pyx":339
 *     def __str__(self):
 *         return 'FunctionSpec(name=%r, args_spec=%r, result_spec=%r)' % (
 *             self.name, self.args_spec, self.result_spec             # <<<<<<<<<<<<<<
 *         )
 * 
 */
  __pyx_t_1 = PyTuple_New(3); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 339; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_v_self->name);
  __Pyx_GIVEREF(__pyx_v_self->name);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_v_self->name);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->args_spec));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->args_spec));
  PyTuple_SET_ITEM(__pyx_t_1, 1, ((PyObject *)__pyx_v_self->args_spec));
  __Pyx_INCREF(((PyObject *)__pyx_v_self->result_spec));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->result_spec));
  PyTuple_SET_ITEM(__pyx_t_1, 2, ((PyObject *)__pyx_v_self->result_spec));

  /* "thriftrw/spec/service.pyx":338
 * 
 *     def __str__(self):
 *         return 'FunctionSpec(name=%r, args_spec=%r, result_spec=%r)' % (             # <<<<<<<<<<<<<<
 *             self.name, self.args_spec, self.result_spec
 *         )
 */
  __pyx_t_2 = PyUnicode_Format(__pyx_kp_u_FunctionSpec_name_r_args_spec_r, __pyx_t_1); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 338; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "thriftrw/spec/service.pyx":337
 *         return self
 * 
 *     def __str__(self):             # <<<<<<<<<<<<<<
 *         return 'FunctionSpec(name=%r, args_spec=%r, result_spec=%r)' % (
 *             self.name, self.args_spec, self.result_spec
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("thriftrw.spec.service.FunctionSpec.__str__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "thriftrw/spec/service.pyx":342
 *         )
 * 
 *     def __repr__(self):             # <<<<<<<<<<<<<<
 *         return str(self)
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_8thriftrw_4spec_7service_12FunctionSpec_9__repr__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_8thriftrw_4spec_7service_12FunctionSpec_9__repr__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__repr__ (wrapper)", 0);
  __pyx_r = __pyx_pf_8thriftrw_4spec_7service_12FunctionSpec_8__repr__(((struct __pyx_obj_8thriftrw_4spec_7service_FunctionSpec *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_8thriftrw_4spec_7service_12FunctionSpec_8__repr__(struct __pyx_obj_8thriftrw_4spec_7service_FunctionSpec *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__repr__", 0);

  /* "thriftrw/spec/service.pyx":343
 * 
 *     def __repr__(self):
 *         return str(self)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyTuple_New(1); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 343; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(((PyObject *)__pyx_v_self));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self));
  PyTuple_SET_ITEM(__pyx_t_1, 0, ((PyObject *)__pyx_v_self));
  __pyx_t_2 = __Pyx_PyObject_Call(((PyObject *)(&PyString_Type)), __pyx_t_1, NULL); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 343; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "thriftrw/spec/service.pyx":342
 *         )
 * 
 *     def __repr__(self):             # <<<<<<<<<<<<<<
 *         return str(self)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("thriftrw.spec.service.FunctionSpec.__repr__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "thriftrw/spec/service.pxd":40
 * 
 * cdef class FunctionSpec(object):
 *     cdef readonly str name             # <<<<<<<<<<<<<<
 *     cdef public FunctionArgsSpec args_spec
 *     cdef public FunctionResultSpec result_spec
 */

/* Python wrapper */
static PyObject *__pyx_pw_8thriftrw_4spec_7service_12FunctionSpec_4name_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_8thriftrw_4spec_7service_12FunctionSpec_4name_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_8thriftrw_4spec_7service_12FunctionSpec_4name___get__(((struct __pyx_obj_8thriftrw_4spec_7service_FunctionSpec *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_8thriftrw_4spec_7service_12FunctionSpec_4name___get__(struct __pyx_obj_8thriftrw_4spec_7service_FunctionSpec *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->name);
  __pyx_r = __pyx_v_self->name;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "thriftrw/spec/service.pxd":41
 * cdef class FunctionSpec(object):
 *     cdef readonly str name
 *     cdef public FunctionArgsSpec args_spec             # <<<<<<<<<<<<<<
 *     cdef public FunctionResultSpec result_spec
 *     cdef readonly bint oneway
 */

/* Python wrapper */
static PyObject *__pyx_pw_8thriftrw_4spec_7service_12FunctionSpec_9args_spec_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_8thriftrw_4spec_7service_12FunctionSpec_9args_spec_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_8thriftrw_4spec_7service_12FunctionSpec_9args_spec___get__(((struct __pyx_obj_8thriftrw_4spec_7service_FunctionSpec *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_8thriftrw_4spec_7service_12FunctionSpec_9args_spec___get__(struct __pyx_obj_8thriftrw_4spec_7service_FunctionSpec *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->args_spec));
  __pyx_r = ((PyObject *)__pyx_v_self->args_spec);
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_8thriftrw_4spec_7service_12FunctionSpec_9args_spec_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_8thriftrw_4spec_7service_12FunctionSpec_9args_spec_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_8thriftrw_4spec_7service_12FunctionSpec_9args_spec_2__set__(((struct __pyx_obj_8thriftrw_4spec_7service_FunctionSpec *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_8thriftrw_4spec_7service_12FunctionSpec_9args_spec_2__set__(struct __pyx_obj_8thriftrw_4spec_7service_FunctionSpec *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__set__", 0);
  if (!(likely(((__pyx_v_value) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_value, __pyx_ptype_8thriftrw_4spec_7service_FunctionArgsSpec))))) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 41; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_1 = __pyx_v_value;
  __Pyx_INCREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->args_spec);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->args_spec));
  __pyx_v_self->args_spec = ((struct __pyx_obj_8thriftrw_4spec_7service_FunctionArgsSpec *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("thriftrw.spec.service.FunctionSpec.args_spec.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_8thriftrw_4spec_7service_12FunctionSpec_9args_spec_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_8thriftrw_4spec_7service_12FunctionSpec_9args_spec_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_8thriftrw_4spec_7service_12FunctionSpec_9args_spec_4__del__(((struct __pyx_obj_8thriftrw_4spec_7service_FunctionSpec *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_8thriftrw_4spec_7service_12FunctionSpec_9args_spec_4__del__(struct __pyx_obj_8thriftrw_4spec_7service_FunctionSpec *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->args_spec);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->args_spec));
  __pyx_v_self->args_spec = ((struct __pyx_obj_8thriftrw_4spec_7service_FunctionArgsSpec *)Py_None);

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "thriftrw/spec/service.pxd":42
 *     cdef readonly str name
 *     cdef public FunctionArgsSpec args_spec
 *     cdef public FunctionResultSpec result_spec             # <<<<<<<<<<<<<<
 *     cdef readonly bint oneway
 *     cdef public bint linked
 */

/* Python wrapper */
static PyObject *__pyx_pw_8thriftrw_4spec_7service_12FunctionSpec_11result_spec_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_8thriftrw_4spec_7service_12FunctionSpec_11result_spec_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_8thriftrw_4spec_7service_12FunctionSpec_11result_spec___get__(((struct __pyx_obj_8thriftrw_4spec_7service_FunctionSpec *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_8thriftrw_4spec_7service_12FunctionSpec_11result_spec___get__(struct __pyx_obj_8thriftrw_4spec_7service_FunctionSpec *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->result_spec));
  __pyx_r = ((PyObject *)__pyx_v_self->result_spec);
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_8thriftrw_4spec_7service_12FunctionSpec_11result_spec_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_8thriftrw_4spec_7service_12FunctionSpec_11result_spec_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_8thriftrw_4spec_7service_12FunctionSpec_11result_spec_2__set__(((struct __pyx_obj_8thriftrw_4spec_7service_FunctionSpec *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_8thriftrw_4spec_7service_12FunctionSpec_11result_spec_2__set__(struct __pyx_obj_8thriftrw_4spec_7service_FunctionSpec *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__set__", 0);
  if (!(likely(((__pyx_v_value) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_value, __pyx_ptype_8thriftrw_4spec_7service_FunctionResultSpec))))) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 42; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_1 = __pyx_v_value;
  __Pyx_INCREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->result_spec);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->result_spec));
  __pyx_v_self->result_spec = ((struct __pyx_obj_8thriftrw_4spec_7service_FunctionResultSpec *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("thriftrw.spec.service.FunctionSpec.result_spec.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_8thriftrw_4spec_7service_12FunctionSpec_11result_spec_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_8thriftrw_4spec_7service_12FunctionSpec_11result_spec_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_8thriftrw_4spec_7service_12FunctionSpec_11result_spec_4__del__(((struct __pyx_obj_8thriftrw_4spec_7service_FunctionSpec *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_8thriftrw_4spec_7service_12FunctionSpec_11result_spec_4__del__(struct __pyx_obj_8thriftrw_4spec_7service_FunctionSpec *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->result_spec);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->result_spec));
  __pyx_v_self->result_spec = ((struct __pyx_obj_8thriftrw_4spec_7service_FunctionResultSpec *)Py_None);

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "thriftrw/spec/service.pxd":43
 *     cdef public FunctionArgsSpec args_spec
 *     cdef public FunctionResultSpec result_spec
 *     cdef readonly bint oneway             # <<<<<<<<<<<<<<
 *     cdef public bint linked
 *     cdef public object surface
 */

/* Python wrapper */
static PyObject *__pyx_pw_8thriftrw_4spec_7service_12FunctionSpec_6oneway_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_8thriftrw_4spec_7service_12FunctionSpec_6oneway_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_8thriftrw_4spec_7service_12FunctionSpec_6oneway___get__(((struct __pyx_obj_8thriftrw_4spec_7service_FunctionSpec *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_8thriftrw_4spec_7service_12FunctionSpec_6oneway___get__(struct __pyx_obj_8thriftrw_4spec_7service_FunctionSpec *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyBool_FromLong(__pyx_v_self->oneway); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 43; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("thriftrw.spec.service.FunctionSpec.oneway.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "thriftrw/spec/service.pxd":44
 *     cdef public FunctionResultSpec result_spec
 *     cdef readonly bint oneway
 *     cdef public bint linked             # <<<<<<<<<<<<<<
 *     cdef public object surface
 *     cdef public ServiceSpec service
 */

/* Python wrapper */
static PyObject *__pyx_pw_8thriftrw_4spec_7service_12FunctionSpec_6linked_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_8thriftrw_4spec_7service_12FunctionSpec_6linked_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_8thriftrw_4spec_7service_12FunctionSpec_6linked___get__(((struct __pyx_obj_8thriftrw_4spec_7service_FunctionSpec *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_8thriftrw_4spec_7service_12FunctionSpec_6linked___get__(struct __pyx_obj_8thriftrw_4spec_7service_FunctionSpec *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyBool_FromLong(__pyx_v_self->linked); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 44; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("thriftrw.spec.service.FunctionSpec.linked.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_8thriftrw_4spec_7service_12FunctionSpec_6linked_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_8thriftrw_4spec_7service_12FunctionSpec_6linked_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_8thriftrw_4spec_7service_12FunctionSpec_6linked_2__set__(((struct __pyx_obj_8thriftrw_4spec_7service_FunctionSpec *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_8thriftrw_4spec_7service_12FunctionSpec_6linked_2__set__(struct __pyx_obj_8thriftrw_4spec_7service_FunctionSpec *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__set__", 0);
  __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_v_value); if (unlikely((__pyx_t_1 == (int)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 44; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_v_self->linked = __pyx_t_1;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("thriftrw.spec.service.FunctionSpec.linked.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "thriftrw/spec/service.pxd":45
 *     cdef readonly bint oneway
 *     cdef public bint linked
 *     cdef public object surface             # <<<<<<<<<<<<<<
 *     cdef public ServiceSpec service
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_8thriftrw_4spec_7service_12FunctionSpec_7surface_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_8thriftrw_4spec_7service_12FunctionSpec_7surface_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_8thriftrw_4spec_7service_12FunctionSpec_7surface___get__(((struct __pyx_obj_8thriftrw_4spec_7service_FunctionSpec *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_8thriftrw_4spec_7service_12FunctionSpec_7surface___get__(struct __pyx_obj_8thriftrw_4spec_7service_FunctionSpec *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->surface);
  __pyx_r = __pyx_v_self->surface;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_8thriftrw_4spec_7service_12FunctionSpec_7surface_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_8thriftrw_4spec_7service_12FunctionSpec_7surface_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_8thriftrw_4spec_7service_12FunctionSpec_7surface_2__set__(((struct __pyx_obj_8thriftrw_4spec_7service_FunctionSpec *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_8thriftrw_4spec_7service_12FunctionSpec_7surface_2__set__(struct __pyx_obj_8thriftrw_4spec_7service_FunctionSpec *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 0);
  __Pyx_INCREF(__pyx_v_value);
  __Pyx_GIVEREF(__pyx_v_value);
  __Pyx_GOTREF(__pyx_v_self->surface);
  __Pyx_DECREF(__pyx_v_self->surface);
  __pyx_v_self->surface = __pyx_v_value;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_8thriftrw_4spec_7service_12FunctionSpec_7surface_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_8thriftrw_4spec_7service_12FunctionSpec_7surface_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_8thriftrw_4spec_7service_12FunctionSpec_7surface_4__del__(((struct __pyx_obj_8thriftrw_4spec_7service_FunctionSpec *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_8thriftrw_4spec_7service_12FunctionSpec_7surface_4__del__(struct __pyx_obj_8thriftrw_4spec_7service_FunctionSpec *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->surface);
  __Pyx_DECREF(__pyx_v_self->surface);
  __pyx_v_self->surface = Py_None;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "thriftrw/spec/service.pxd":46
 *     cdef public bint linked
 *     cdef public object surface
 *     cdef public ServiceSpec service             # <<<<<<<<<<<<<<
 * 
 *     cpdef FunctionSpec link(self, scope, ServiceSpec service)
 */

/* Python wrapper */
static PyObject *__pyx_pw_8thriftrw_4spec_7service_12FunctionSpec_7service_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_8thriftrw_4spec_7service_12FunctionSpec_7service_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_8thriftrw_4spec_7service_12FunctionSpec_7service___get__(((struct __pyx_obj_8thriftrw_4spec_7service_FunctionSpec *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_8thriftrw_4spec_7service_12FunctionSpec_7service___get__(struct __pyx_obj_8thriftrw_4spec_7service_FunctionSpec *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->service));
  __pyx_r = ((PyObject *)__pyx_v_self->service);
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_8thriftrw_4spec_7service_12FunctionSpec_7service_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_8thriftrw_4spec_7service_12FunctionSpec_7service_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_8thriftrw_4spec_7service_12FunctionSpec_7service_2__set__(((struct __pyx_obj_8thriftrw_4spec_7service_FunctionSpec *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_8thriftrw_4spec_7service_12FunctionSpec_7service_2__set__(struct __pyx_obj_8thriftrw_4spec_7service_FunctionSpec *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__set__", 0);
  if (!(likely(((__pyx_v_value) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_value, __pyx_ptype_8thriftrw_4spec_7service_ServiceSpec))))) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 46; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_1 = __pyx_v_value;
  __Pyx_INCREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->service);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->service));
  __pyx_v_self->service = ((struct __pyx_obj_8thriftrw_4spec_7service_ServiceSpec *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("thriftrw.spec.service.FunctionSpec.service.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_8thriftrw_4spec_7service_12FunctionSpec_7service_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_8thriftrw_4spec_7service_12FunctionSpec_7service_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_8thriftrw_4spec_7service_12FunctionSpec_7service_4__del__(((struct __pyx_obj_8thriftrw_4spec_7service_FunctionSpec *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_8thriftrw_4spec_7service_12FunctionSpec_7service_4__del__(struct __pyx_obj_8thriftrw_4spec_7service_FunctionSpec *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->service);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->service));
  __pyx_v_self->service = ((struct __pyx_obj_8thriftrw_4spec_7service_ServiceSpec *)Py_None);

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "thriftrw/spec/service.pyx":373
 *     """
 * 
 *     def __init__(self, name, functions, parent):             # <<<<<<<<<<<<<<
 *         self.name = str(name)
 *         self.functions = functions
 */

/* Python wrapper */
static int __pyx_pw_8thriftrw_4spec_7service_11ServiceSpec_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_8thriftrw_4spec_7service_11ServiceSpec_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_name = 0;
  PyObject *__pyx_v_functions = 0;
  PyObject *__pyx_v_parent = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_name,&__pyx_n_s_functions,&__pyx_n_s_parent,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_name)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_functions)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__init__", 1, 3, 3, 1); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 373; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_parent)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__init__", 1, 3, 3, 2); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 373; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__init__") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 373; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v_name = values[0];
    __pyx_v_functions = values[1];
    __pyx_v_parent = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 373; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("thriftrw.spec.service.ServiceSpec.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_8thriftrw_4spec_7service_11ServiceSpec___init__(((struct __pyx_obj_8thriftrw_4spec_7service_ServiceSpec *)__pyx_v_self), __pyx_v_name, __pyx_v_functions, __pyx_v_parent);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_8thriftrw_4spec_7service_11ServiceSpec___init__(struct __pyx_obj_8thriftrw_4spec_7service_ServiceSpec *__pyx_v_self, PyObject *__pyx_v_name, PyObject *__pyx_v_functions, PyObject *__pyx_v_parent) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "thriftrw/spec/service.pyx":374
 * 
 *     def __init__(self, name, functions, parent):
 *         self.name = str(name)             # <<<<<<<<<<<<<<
 *         self.functions = functions
 *         self.parent = parent
 */
  __pyx_t_1 = PyTuple_New(1); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 374; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_v_name);
  __Pyx_GIVEREF(__pyx_v_name);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_v_name);
  __pyx_t_2 = __Pyx_PyObject_Call(((PyObject *)(&PyString_Type)), __pyx_t_1, NULL); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 374; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (!(likely(PyString_CheckExact(__pyx_t_2))||((__pyx_t_2) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "str", Py_TYPE(__pyx_t_2)->tp_name), 0))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 374; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GIVEREF(__pyx_t_2);
  __Pyx_GOTREF(__pyx_v_self->name);
  __Pyx_DECREF(__pyx_v_self->name);
  __pyx_v_self->name = ((PyObject*)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "thriftrw/spec/service.pyx":375
 *     def __init__(self, name, functions, parent):
 *         self.name = str(name)
 *         self.functions = functions             # <<<<<<<<<<<<<<
 *         self.parent = parent
 * 
 */
  if (!(likely(PyList_CheckExact(__pyx_v_functions))||((__pyx_v_functions) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_v_functions)->tp_name), 0))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 375; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_2 = __pyx_v_functions;
  __Pyx_INCREF(__pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_2);
  __Pyx_GOTREF(__pyx_v_self->functions);
  __Pyx_DECREF(__pyx_v_self->functions);
  __pyx_v_self->functions = ((PyObject*)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "thriftrw/spec/service.pyx":376
 *         self.name = str(name)
 *         self.functions = functions
 *         self.parent = parent             # <<<<<<<<<<<<<<
 * 
 *         # For quick function lookups.
 */
  __Pyx_INCREF(__pyx_v_parent);
  __Pyx_GIVEREF(__pyx_v_parent);
  __Pyx_GOTREF(__pyx_v_self->parent);
  __Pyx_DECREF(__pyx_v_self->parent);
  __pyx_v_self->parent = __pyx_v_parent;

  /* "thriftrw/spec/service.pyx":379
 * 
 *         # For quick function lookups.
 *         self._functions = None             # <<<<<<<<<<<<<<
 *         self.linked = False
 *         self.surface = None
 */
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->_functions);
  __Pyx_DECREF(__pyx_v_self->_functions);
  __pyx_v_self->_functions = ((PyObject*)Py_None);

  /* "thriftrw/spec/service.pyx":380
 *         # For quick function lookups.
 *         self._functions = None
 *         self.linked = False             # <<<<<<<<<<<<<<
 *         self.surface = None
 * 
 */
  __pyx_v_self->linked = 0;

  /* "thriftrw/spec/service.pyx":381
 *         self._functions = None
 *         self.linked = False
 *         self.surface = None             # <<<<<<<<<<<<<<
 * 
 *     @classmethod
 */
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->surface);
  __Pyx_DECREF(__pyx_v_self->surface);
  __pyx_v_self->surface = Py_None;

  /* "thriftrw/spec/service.pyx":373
 *     """
 * 
 *     def __init__(self, name, functions, parent):             # <<<<<<<<<<<<<<
 *         self.name = str(name)
 *         self.functions = functions
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("thriftrw.spec.service.ServiceSpec.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "thriftrw/spec/service.pyx":384
 * 
 *     @classmethod
 *     def compile(cls, service):             # <<<<<<<<<<<<<<
 *         functions = []
 *         names = set()
 */

/* Python wrapper */
static PyObject *__pyx_pw_8thriftrw_4spec_7service_11ServiceSpec_3compile(PyObject *__pyx_v_cls, PyObject *__pyx_v_service); /*proto*/
static PyObject *__pyx_pw_8thriftrw_4spec_7service_11ServiceSpec_3compile(PyObject *__pyx_v_cls, PyObject *__pyx_v_service) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("compile (wrapper)", 0);
  __pyx_r = __pyx_pf_8thriftrw_4spec_7service_11ServiceSpec_2compile(((PyTypeObject*)__pyx_v_cls), ((PyObject *)__pyx_v_service));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_8thriftrw_4spec_7service_11ServiceSpec_2compile(PyTypeObject *__pyx_v_cls, PyObject *__pyx_v_service) {
  PyObject *__pyx_v_functions = NULL;
  PyObject *__pyx_v_names = NULL;
  PyObject *__pyx_v_func = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  Py_ssize_t __pyx_t_3;
  PyObject *(*__pyx_t_4)(PyObject *);
  int __pyx_t_5;
  int __pyx_t_6;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  int __pyx_t_11;
  Py_ssize_t __pyx_t_12;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("compile", 0);

  /* "thriftrw/spec/service.pyx":385
 *     @classmethod
 *     def compile(cls, service):
 *         functions = []             # <<<<<<<<<<<<<<
 *         names = set()
 * 
 */
  __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 385; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_functions = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "thriftrw/spec/service.pyx":386
 *     def compile(cls, service):
 *         functions = []
 *         names = set()             # <<<<<<<<<<<<<<
 * 
 *         for func in service.functions:
 */
  __pyx_t_1 = PySet_New(0); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 386; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_names = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "thriftrw/spec/service.pyx":388
 *         names = set()
 * 
 *         for func in service.functions:             # <<<<<<<<<<<<<<
 *             if func.name in names:
 *                 raise ThriftCompilerError(
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_service, __pyx_n_s_functions); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 388; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  if (likely(PyList_CheckExact(__pyx_t_1)) || PyTuple_CheckExact(__pyx_t_1)) {
    __pyx_t_2 = __pyx_t_1; __Pyx_INCREF(__pyx_t_2); __pyx_t_3 = 0;
    __pyx_t_4 = NULL;
  } else {
    __pyx_t_3 = -1; __pyx_t_2 = PyObject_GetIter(__pyx_t_1); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 388; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = Py_TYPE(__pyx_t_2)->tp_iternext; if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 388; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  for (;;) {
    if (likely(!__pyx_t_4)) {
      if (likely(PyList_CheckExact(__pyx_t_2))) {
        if (__pyx_t_3 >= PyList_GET_SIZE(__pyx_t_2)) break;
        #if CYTHON_COMPILING_IN_CPYTHON
        __pyx_t_1 = PyList_GET_ITEM(__pyx_t_2, __pyx_t_3); __Pyx_INCREF(__pyx_t_1); __pyx_t_3++; if (unlikely(0 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 388; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        #else
        __pyx_t_1 = PySequence_ITEM(__pyx_t_2, __pyx_t_3); __pyx_t_3++; if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 388; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_1);
        #endif
      } else {
        if (__pyx_t_3 >= PyTuple_GET_SIZE(__pyx_t_2)) break;
        #if CYTHON_COMPILING_IN_CPYTHON
        __pyx_t_1 = PyTuple_GET_ITEM(__pyx_t_2, __pyx_t_3); __Pyx_INCREF(__pyx_t_1); __pyx_t_3++; if (unlikely(0 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 388; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        #else
        __pyx_t_1 = PySequence_ITEM(__pyx_t_2, __pyx_t_3); __pyx_t_3++; if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 388; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_1);
        #endif
      }
    } else {
      __pyx_t_1 = __pyx_t_4(__pyx_t_2);
      if (unlikely(!__pyx_t_1)) {
        PyObject* exc_type = PyErr_Occurred();
        if (exc_type) {
          if (likely(exc_type == PyExc_StopIteration || PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
          else {__pyx_filename = __pyx_f[0]; __pyx_lineno = 388; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        }
        break;
      }
      __Pyx_GOTREF(__pyx_t_1);
    }
    __Pyx_XDECREF_SET(__pyx_v_func, __pyx_t_1);
    __pyx_t_1 = 0;

    /* "thriftrw/spec/service.pyx":389
 * 
 *         for func in service.functions:
 *             if func.name in names:             # <<<<<<<<<<<<<<
 *                 raise ThriftCompilerError(
 *                     'Function "%s.%s" cannot be defined. '
 */
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_func, __pyx_n_s_name); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 389; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_5 = (__Pyx_PySequence_ContainsTF(__pyx_t_1, __pyx_v_names, Py_EQ)); if (unlikely(__pyx_t_5 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 389; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_6 = (__pyx_t_5 != 0);
    if (__pyx_t_6) {

      /* "thriftrw/spec/service.pyx":390
 *         for func in service.functions:
 *             if func.name in names:
 *                 raise ThriftCompilerError(             # <<<<<<<<<<<<<<
 *                     'Function "%s.%s" cannot be defined. '
 *                     'That name is already taken.'
 */
      __pyx_t_7 = __Pyx_GetModuleGlobalName(__pyx_n_s_ThriftCompilerError); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 390; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_7);

      /* "thriftrw/spec/service.pyx":393
 *                     'Function "%s.%s" cannot be defined. '
 *                     'That name is already taken.'
 *                     % (service.name, func.name)             # <<<<<<<<<<<<<<
 *                 )
 *             names.add(func.name)
 */
      __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_service, __pyx_n_s_name); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 393; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_8);
      __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_v_func, __pyx_n_s_name); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 393; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_9);
      __pyx_t_10 = PyTuple_New(2); if (unlikely(!__pyx_t_10)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 393; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_10);
      __Pyx_GIVEREF(__pyx_t_8);
      PyTuple_SET_ITEM(__pyx_t_10, 0, __pyx_t_8);
      __Pyx_GIVEREF(__pyx_t_9);
      PyTuple_SET_ITEM(__pyx_t_10, 1, __pyx_t_9);
      __pyx_t_8 = 0;
      __pyx_t_9 = 0;
      __pyx_t_9 = PyUnicode_Format(__pyx_kp_u_Function_s_s_cannot_be_defined_T, __pyx_t_10); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 393; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_9);
      __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
      __pyx_t_10 = NULL;
      if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_7))) {
        __pyx_t_10 = PyMethod_GET_SELF(__pyx_t_7);
        if (likely(__pyx_t_10)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_7);
          __Pyx_INCREF(__pyx_t_10);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_7, function);
        }
      }
      if (!__pyx_t_10) {
        __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_7, __pyx_t_9); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 390; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
        __Pyx_GOTREF(__pyx_t_1);
      } else {
        __pyx_t_8 = PyTuple_New(1+1); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 390; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_8);
        __Pyx_GIVEREF(__pyx_t_10); PyTuple_SET_ITEM(__pyx_t_8, 0, __pyx_t_10); __pyx_t_10 = NULL;
        __Pyx_GIVEREF(__pyx_t_9);
        PyTuple_SET_ITEM(__pyx_t_8, 0+1, __pyx_t_9);
        __pyx_t_9 = 0;
        __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_7, __pyx_t_8, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 390; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      }
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      __Pyx_Raise(__pyx_t_1, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      {__pyx_filename = __pyx_f[0]; __pyx_lineno = 390; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

      /* "thriftrw/spec/service.pyx":389
 * 
 *         for func in service.functions:
 *             if func.name in names:             # <<<<<<<<<<<<<<
 *                 raise ThriftCompilerError(
 *                     'Function "%s.%s" cannot be defined. '
 */
    }

    /* "thriftrw/spec/service.pyx":395
 *                     % (service.name, func.name)
 *                 )
 *             names.add(func.name)             # <<<<<<<<<<<<<<
 *             functions.append(
 *                 FunctionSpec.compile(func, service.name)
 */
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_func, __pyx_n_s_name); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 395; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_11 = PySet_Add(__pyx_v_names, __pyx_t_1); if (unlikely(__pyx_t_11 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 395; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "thriftrw/spec/service.pyx":397
 *             names.add(func.name)
 *             functions.append(
 *                 FunctionSpec.compile(func, service.name)             # <<<<<<<<<<<<<<
 *             )
 * 
 */
    __pyx_t_7 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_8thriftrw_4spec_7service_FunctionSpec), __pyx_n_s_compile); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 397; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_service, __pyx_n_s_name); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 397; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_t_9 = NULL;
    __pyx_t_12 = 0;
    if (CYTHON_COMPILING_IN_CPYTHON && likely(PyMethod_Check(__pyx_t_7))) {
      __pyx_t_9 = PyMethod_GET_SELF(__pyx_t_7);
      if (likely(__pyx_t_9)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_7);
        __Pyx_INCREF(__pyx_t_9);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_7, function);
        __pyx_t_12 = 1;
      }
    }
    __pyx_t_10 = PyTuple_New(2+__pyx_t_12); if (unlikely(!__pyx_t_10)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 397; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_10);
    if (__pyx_t_9) {
      __Pyx_GIVEREF(__pyx_t_9); PyTuple_SET_ITEM(__pyx_t_10, 0, __pyx_t_9); __pyx_t_9 = NULL;
    }
    __Pyx_INCREF(__pyx_v_func);
    __Pyx_GIVEREF(__pyx_v_func);
    PyTuple_SET_ITEM(__pyx_t_10, 0+__pyx_t_12, __pyx_v_func);
    __Pyx_GIVEREF(__pyx_t_8);
    PyTuple_SET_ITEM(__pyx_t_10, 1+__pyx_t_12, __pyx_t_8);
    __pyx_t_8 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_7, __pyx_t_10, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 397; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

    /* "thriftrw/spec/service.pyx":396
 *                 )
 *             names.add(func.name)
 *             functions.append(             # <<<<<<<<<<<<<<
 *                 FunctionSpec.compile(func, service.name)
 *             )
 */
    __pyx_t_11 = __Pyx_PyList_Append(__pyx_v_functions, __pyx_t_1); if (unlikely(__pyx_t_11 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 396; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "thriftrw/spec/service.pyx":388
 *         names = set()
 * 
 *         for func in service.functions:             # <<<<<<<<<<<<<<
 *             if func.name in names:
 *                 raise ThriftCompilerError(
 */
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "thriftrw/spec/service.pyx":400
 *             )
 * 
 *         return cls(service.name, functions, service.parent)             # <<<<<<<<<<<<<<
 * 
 *     cpdef ServiceSpec link(self, scope):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_service, __pyx_n_s_name); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 400; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_service, __pyx_n_s_parent); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 400; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_7 = PyTuple_New(3); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 400; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_7);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_7, 0, __pyx_t_2);
  __Pyx_INCREF(__pyx_v_functions);
  __Pyx_GIVEREF(__pyx_v_functions);
  PyTuple_SET_ITEM(__pyx_t_7, 1, __pyx_v_functions);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_7, 2, __pyx_t_1);
  __pyx_t_2 = 0;
  __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(((PyObject *)__pyx_v_cls), __pyx_t_7, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 400; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "thriftrw/spec/service.pyx":384
 * 
 *     @classmethod
 *     def compile(cls, service):             # <<<<<<<<<<<<<<
 *         functions = []
 *         names = set()
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_AddTraceback("thriftrw.spec.service.ServiceSpec.compile", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_functions);
  __Pyx_XDECREF(__pyx_v_names);
  __Pyx_XDECREF(__pyx_v_func);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "thriftrw/spec/service.pyx":402
 *         return cls(service.name, functions, service.parent)
 * 
 *     cpdef ServiceSpec link(self, scope):             # <<<<<<<<<<<<<<
 *         if not self.linked:
 *             self.linked = True
 */

static PyObject *__pyx_pw_8thriftrw_4spec_7service_11ServiceSpec_5link(PyObject *__pyx_v_self, PyObject *__pyx_v_scope); /*proto*/
static struct __pyx_obj_8thriftrw_4spec_7service_ServiceSpec *__pyx_f_8thriftrw_4spec_7service_11ServiceSpec_link(struct __pyx_obj_8thriftrw_4spec_7service_ServiceSpec *__pyx_v_self, PyObject *__pyx_v_scope, int __pyx_skip_dispatch) {
  PyObject *__pyx_v_f = NULL;
  PyObject *__pyx_v_name = NULL;
  PyObject *__pyx_v_func = NULL;
  struct __pyx_obj_8thriftrw_4spec_7service_ServiceSpec *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  int __pyx_t_7;
  Py_ssize_t __pyx_t_8;
  PyObject *__pyx_t_9 = NULL;
  Py_ssize_t __pyx_t_10;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("link", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0)) {
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_link); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 402; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)__pyx_pw_8thriftrw_4spec_7service_11ServiceSpec_5link)) {
      __Pyx_XDECREF(((PyObject *)__pyx_r));
      __Pyx_INCREF(__pyx_t_1);
      __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
      if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_3))) {
        __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
        if (likely(__pyx_t_4)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
          __Pyx_INCREF(__pyx_t_4);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_3, function);
        }
      }
      if (!__pyx_t_4) {
        __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_v_scope); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 402; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_2);
      } else {
        __pyx_t_5 = PyTuple_New(1+1); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 402; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_5);
        __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_4); __pyx_t_4 = NULL;
        __Pyx_INCREF(__pyx_v_scope);
        __Pyx_GIVEREF(__pyx_v_scope);
        PyTuple_SET_ITEM(__pyx_t_5, 0+1, __pyx_v_scope);
        __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_5, NULL); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 402; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      }
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (!(likely(((__pyx_t_2) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_2, __pyx_ptype_8thriftrw_4spec_7service_ServiceSpec))))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 402; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __pyx_r = ((struct __pyx_obj_8thriftrw_4spec_7service_ServiceSpec *)__pyx_t_2);
      __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      goto __pyx_L0;
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  }

  /* "thriftrw/spec/service.pyx":403
 * 
 *     cpdef ServiceSpec link(self, scope):
 *         if not self.linked:             # <<<<<<<<<<<<<<
 *             self.linked = True
 * 
 */
  __pyx_t_6 = ((!(__pyx_v_self->linked != 0)) != 0);
  if (__pyx_t_6) {

    /* "thriftrw/spec/service.pyx":404
 *     cpdef ServiceSpec link(self, scope):
 *         if not self.linked:
 *             self.linked = True             # <<<<<<<<<<<<<<
 * 
 *             if self.parent is not None:
 */
    __pyx_v_self->linked = 1;

    /* "thriftrw/spec/service.pyx":406
 *             self.linked = True
 * 
 *             if self.parent is not None:             # <<<<<<<<<<<<<<
 *                 self.parent = scope.resolve_service_spec(
 *                     self.parent.name, self.parent.lineno
 */
    __pyx_t_6 = (__pyx_v_self->parent != Py_None);
    __pyx_t_7 = (__pyx_t_6 != 0);
    if (__pyx_t_7) {

      /* "thriftrw/spec/service.pyx":407
 * 
 *             if self.parent is not None:
 *                 self.parent = scope.resolve_service_spec(             # <<<<<<<<<<<<<<
 *                     self.parent.name, self.parent.lineno
 *                 )
 */
      __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_scope, __pyx_n_s_resolve_service_spec); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 407; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_2);

      /* "thriftrw/spec/service.pyx":408
 *             if self.parent is not None:
 *                 self.parent = scope.resolve_service_spec(
 *                     self.parent.name, self.parent.lineno             # <<<<<<<<<<<<<<
 *                 )
 * 
 */
      __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->parent, __pyx_n_s_name); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 408; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->parent, __pyx_n_s_lineno); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 408; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_5);
      __pyx_t_4 = NULL;
      __pyx_t_8 = 0;
      if (CYTHON_COMPILING_IN_CPYTHON && likely(PyMethod_Check(__pyx_t_2))) {
        __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
        if (likely(__pyx_t_4)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
          __Pyx_INCREF(__pyx_t_4);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_2, function);
          __pyx_t_8 = 1;
        }
      }
      __pyx_t_9 = PyTuple_New(2+__pyx_t_8); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 407; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_9);
      if (__pyx_t_4) {
        __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_9, 0, __pyx_t_4); __pyx_t_4 = NULL;
      }
      __Pyx_GIVEREF(__pyx_t_3);
      PyTuple_SET_ITEM(__pyx_t_9, 0+__pyx_t_8, __pyx_t_3);
      __Pyx_GIVEREF(__pyx_t_5);
      PyTuple_SET_ITEM(__pyx_t_9, 1+__pyx_t_8, __pyx_t_5);
      __pyx_t_3 = 0;
      __pyx_t_5 = 0;
      __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_9, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 407; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

      /* "thriftrw/spec/service.pyx":407
 * 
 *             if self.parent is not None:
 *                 self.parent = scope.resolve_service_spec(             # <<<<<<<<<<<<<<
 *                     self.parent.name, self.parent.lineno
 *                 )
 */
      __Pyx_GIVEREF(__pyx_t_1);
      __Pyx_GOTREF(__pyx_v_self->parent);
      __Pyx_DECREF(__pyx_v_self->parent);
      __pyx_v_self->parent = __pyx_t_1;
      __pyx_t_1 = 0;

      /* "thriftrw/spec/service.pyx":406
 *             self.linked = True
 * 
 *             if self.parent is not None:             # <<<<<<<<<<<<<<
 *                 self.parent = scope.resolve_service_spec(
 *                     self.parent.name, self.parent.lineno
 */
    }

    /* "thriftrw/spec/service.pyx":411
 *                 )
 * 
 *             self.functions = [func.link(scope, self) for func in self.functions]             # <<<<<<<<<<<<<<
 *             self.surface = service_cls(self, scope)
 * 
 */
    __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 411; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    if (unlikely(__pyx_v_self->functions == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
      {__pyx_filename = __pyx_f[0]; __pyx_lineno = 411; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __pyx_t_2 = __pyx_v_self->functions; __Pyx_INCREF(__pyx_t_2); __pyx_t_8 = 0;
    for (;;) {
      if (__pyx_t_8 >= PyList_GET_SIZE(__pyx_t_2)) break;
      #if CYTHON_COMPILING_IN_CPYTHON
      __pyx_t_9 = PyList_GET_ITEM(__pyx_t_2, __pyx_t_8); __Pyx_INCREF(__pyx_t_9); __pyx_t_8++; if (unlikely(0 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 411; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      #else
      __pyx_t_9 = PySequence_ITEM(__pyx_t_2, __pyx_t_8); __pyx_t_8++; if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 411; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_9);
      #endif
      __Pyx_XDECREF_SET(__pyx_v_func, __pyx_t_9);
      __pyx_t_9 = 0;
      __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_v_func, __pyx_n_s_link); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 411; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_5);
      __pyx_t_3 = NULL;
      __pyx_t_10 = 0;
      if (CYTHON_COMPILING_IN_CPYTHON && likely(PyMethod_Check(__pyx_t_5))) {
        __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_5);
        if (likely(__pyx_t_3)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
          __Pyx_INCREF(__pyx_t_3);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_5, function);
          __pyx_t_10 = 1;
        }
      }
      __pyx_t_4 = PyTuple_New(2+__pyx_t_10); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 411; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_4);
      if (__pyx_t_3) {
        __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_3); __pyx_t_3 = NULL;
      }
      __Pyx_INCREF(__pyx_v_scope);
      __Pyx_GIVEREF(__pyx_v_scope);
      PyTuple_SET_ITEM(__pyx_t_4, 0+__pyx_t_10, __pyx_v_scope);
      __Pyx_INCREF(((PyObject *)__pyx_v_self));
      __Pyx_GIVEREF(((PyObject *)__pyx_v_self));
      PyTuple_SET_ITEM(__pyx_t_4, 1+__pyx_t_10, ((PyObject *)__pyx_v_self));
      __pyx_t_9 = __Pyx_PyObject_Call(__pyx_t_5, __pyx_t_4, NULL); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 411; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_9);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      if (unlikely(__Pyx_ListComp_Append(__pyx_t_1, (PyObject*)__pyx_t_9))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 411; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    }
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_GIVEREF(__pyx_t_1);
    __Pyx_GOTREF(__pyx_v_self->functions);
    __Pyx_DECREF(__pyx_v_self->functions);
    __pyx_v_self->functions = ((PyObject*)__pyx_t_1);
    __pyx_t_1 = 0;

    /* "thriftrw/spec/service.pyx":412
 * 
 *             self.functions = [func.link(scope, self) for func in self.functions]
 *             self.surface = service_cls(self, scope)             # <<<<<<<<<<<<<<
 * 
 *             # Build index for lookup. Normalize type of names to bytes since
 */
    __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_service_cls); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 412; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_9 = NULL;
    __pyx_t_8 = 0;
    if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_9 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_9)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_9);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
        __pyx_t_8 = 1;
      }
    }
    __pyx_t_5 = PyTuple_New(2+__pyx_t_8); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 412; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_5);
    if (__pyx_t_9) {
      __Pyx_GIVEREF(__pyx_t_9); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_9); __pyx_t_9 = NULL;
    }
    __Pyx_INCREF(((PyObject *)__pyx_v_self));
    __Pyx_GIVEREF(((PyObject *)__pyx_v_self));
    PyTuple_SET_ITEM(__pyx_t_5, 0+__pyx_t_8, ((PyObject *)__pyx_v_self));
    __Pyx_INCREF(__pyx_v_scope);
    __Pyx_GIVEREF(__pyx_v_scope);
    PyTuple_SET_ITEM(__pyx_t_5, 1+__pyx_t_8, __pyx_v_scope);
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_5, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 412; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_GIVEREF(__pyx_t_1);
    __Pyx_GOTREF(__pyx_v_self->surface);
    __Pyx_DECREF(__pyx_v_self->surface);
    __pyx_v_self->surface = __pyx_t_1;
    __pyx_t_1 = 0;

    /* "thriftrw/spec/service.pyx":416
 *             # Build index for lookup. Normalize type of names to bytes since
 *             # it may be bytes or unicode.
 *             self._functions = {}             # <<<<<<<<<<<<<<
 *             for f in self.functions:
 *                 name = f.name
 */
    __pyx_t_1 = PyDict_New(); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 416; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_GIVEREF(__pyx_t_1);
    __Pyx_GOTREF(__pyx_v_self->_functions);
    __Pyx_DECREF(__pyx_v_self->_functions);
    __pyx_v_self->_functions = ((PyObject*)__pyx_t_1);
    __pyx_t_1 = 0;

    /* "thriftrw/spec/service.pyx":417
 *             # it may be bytes or unicode.
 *             self._functions = {}
 *             for f in self.functions:             # <<<<<<<<<<<<<<
 *                 name = f.name
 *                 if isinstance(name, unicode):
 */
    if (unlikely(__pyx_v_self->functions == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
      {__pyx_filename = __pyx_f[0]; __pyx_lineno = 417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __pyx_t_1 = __pyx_v_self->functions; __Pyx_INCREF(__pyx_t_1); __pyx_t_8 = 0;
    for (;;) {
      if (__pyx_t_8 >= PyList_GET_SIZE(__pyx_t_1)) break;
      #if CYTHON_COMPILING_IN_CPYTHON
      __pyx_t_2 = PyList_GET_ITEM(__pyx_t_1, __pyx_t_8); __Pyx_INCREF(__pyx_t_2); __pyx_t_8++; if (unlikely(0 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      #else
      __pyx_t_2 = PySequence_ITEM(__pyx_t_1, __pyx_t_8); __pyx_t_8++; if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_2);
      #endif
      __Pyx_XDECREF_SET(__pyx_v_f, __pyx_t_2);
      __pyx_t_2 = 0;

      /* "thriftrw/spec/service.pyx":418
 *             self._functions = {}
 *             for f in self.functions:
 *                 name = f.name             # <<<<<<<<<<<<<<
 *                 if isinstance(name, unicode):
 *                     name = name.encode('utf-8')
 */
      __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_f, __pyx_n_s_name); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 418; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_XDECREF_SET(__pyx_v_name, __pyx_t_2);
      __pyx_t_2 = 0;

      /* "thriftrw/spec/service.pyx":419
 *             for f in self.functions:
 *                 name = f.name
 *                 if isinstance(name, unicode):             # <<<<<<<<<<<<<<
 *                     name = name.encode('utf-8')
 *                 self._functions[name] = f
 */
      __pyx_t_7 = PyUnicode_Check(__pyx_v_name); 
      __pyx_t_6 = (__pyx_t_7 != 0);
      if (__pyx_t_6) {

        /* "thriftrw/spec/service.pyx":420
 *                 name = f.name
 *                 if isinstance(name, unicode):
 *                     name = name.encode('utf-8')             # <<<<<<<<<<<<<<
 *                 self._functions[name] = f
 * 
 */
        __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_name, __pyx_n_s_encode); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 420; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_2);
        __pyx_t_5 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_tuple_, NULL); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 420; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_5);
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __Pyx_DECREF_SET(__pyx_v_name, __pyx_t_5);
        __pyx_t_5 = 0;

        /* "thriftrw/spec/service.pyx":419
 *             for f in self.functions:
 *                 name = f.name
 *                 if isinstance(name, unicode):             # <<<<<<<<<<<<<<
 *                     name = name.encode('utf-8')
 *                 self._functions[name] = f
 */
      }

      /* "thriftrw/spec/service.pyx":421
 *                 if isinstance(name, unicode):
 *                     name = name.encode('utf-8')
 *                 self._functions[name] = f             # <<<<<<<<<<<<<<
 * 
 *         return self
 */
      if (unlikely(__pyx_v_self->_functions == Py_None)) {
        PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
        {__pyx_filename = __pyx_f[0]; __pyx_lineno = 421; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      }
      if (unlikely(PyDict_SetItem(__pyx_v_self->_functions, __pyx_v_name, __pyx_v_f) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 421; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

      /* "thriftrw/spec/service.pyx":417
 *             # it may be bytes or unicode.
 *             self._functions = {}
 *             for f in self.functions:             # <<<<<<<<<<<<<<
 *                 name = f.name
 *                 if isinstance(name, unicode):
 */
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "thriftrw/spec/service.pyx":403
 * 
 *     cpdef ServiceSpec link(self, scope):
 *         if not self.linked:             # <<<<<<<<<<<<<<
 *             self.linked = True
 * 
 */
  }

  /* "thriftrw/spec/service.pyx":423
 *                 self._functions[name] = f
 * 
 *         return self             # <<<<<<<<<<<<<<
 * 
 *     def lookup(self, name):
 */
  __Pyx_XDECREF(((PyObject *)__pyx_r));
  __Pyx_INCREF(((PyObject *)__pyx_v_self));
  __pyx_r = __pyx_v_self;
  goto __pyx_L0;

  /* "thriftrw/spec/service.pyx":402
 *         return cls(service.name, functions, service.parent)
 * 
 *     cpdef ServiceSpec link(self, scope):             # <<<<<<<<<<<<<<
 *         if not self.linked:
 *             self.linked = True
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_AddTraceback("thriftrw.spec.service.ServiceSpec.link", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_f);
  __Pyx_XDECREF(__pyx_v_name);
  __Pyx_XDECREF(__pyx_v_func);
  __Pyx_XGIVEREF((PyObject *)__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_8thriftrw_4spec_7service_11ServiceSpec_5link(PyObject *__pyx_v_self, PyObject *__pyx_v_scope); /*proto*/
static PyObject *__pyx_pw_8thriftrw_4spec_7service_11ServiceSpec_5link(PyObject *__pyx_v_self, PyObject *__pyx_v_scope) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("link (wrapper)", 0);
  __pyx_r = __pyx_pf_8thriftrw_4spec_7service_11ServiceSpec_4link(((struct __pyx_obj_8thriftrw_4spec_7service_ServiceSpec *)__pyx_v_self), ((PyObject *)__pyx_v_scope));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_8thriftrw_4spec_7service_11ServiceSpec_4link(struct __pyx_obj_8thriftrw_4spec_7service_ServiceSpec *__pyx_v_self, PyObject *__pyx_v_scope) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("link", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = ((PyObject *)__pyx_f_8thriftrw_4spec_7service_11ServiceSpec_link(__pyx_v_self, __pyx_v_scope, 1)); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 402; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("thriftrw.spec.service.ServiceSpec.link", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "thriftrw/spec/service.pyx":425
 *         return self
 * 
 *     def lookup(self, name):             # <<<<<<<<<<<<<<
 *         """Look up a function with the given name.
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_8thriftrw_4spec_7service_11ServiceSpec_7lookup(PyObject *__pyx_v_self, PyObject *__pyx_v_name); /*proto*/
static char __pyx_doc_8thriftrw_4spec_7service_11ServiceSpec_6lookup[] = "Look up a function with the given name.\n\n        Returns the function spec or None if no such function exists.\n\n        .. versionadded:: 1.0\n        ";
static PyObject *__pyx_pw_8thriftrw_4spec_7service_11ServiceSpec_7lookup(PyObject *__pyx_v_self, PyObject *__pyx_v_name) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("lookup (wrapper)", 0);
  __pyx_r = __pyx_pf_8thriftrw_4spec_7service_11ServiceSpec_6lookup(((struct __pyx_obj_8thriftrw_4spec_7service_ServiceSpec *)__pyx_v_self), ((PyObject *)__pyx_v_name));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_8thriftrw_4spec_7service_11ServiceSpec_6lookup(struct __pyx_obj_8thriftrw_4spec_7service_ServiceSpec *__pyx_v_self, PyObject *__pyx_v_name) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("lookup", 0);
  __Pyx_INCREF(__pyx_v_name);

  /* "thriftrw/spec/service.pyx":432
 *         .. versionadded:: 1.0
 *         """
 *         if isinstance(name, unicode):             # <<<<<<<<<<<<<<
 *             name = name.encode('utf-8')
 *         return self._functions.get(name, None)
 */
  __pyx_t_1 = PyUnicode_Check(__pyx_v_name); 
  __pyx_t_2 = (__pyx_t_1 != 0);
  if (__pyx_t_2) {

    /* "thriftrw/spec/service.pyx":433
 *         """
 *         if isinstance(name, unicode):
 *             name = name.encode('utf-8')             # <<<<<<<<<<<<<<
 *         return self._functions.get(name, None)
 * 
 */
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_name, __pyx_n_s_encode); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 433; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_tuple__2, NULL); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 433; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF_SET(__pyx_v_name, __pyx_t_4);
    __pyx_t_4 = 0;

    /* "thriftrw/spec/service.pyx":432
 *         .. versionadded:: 1.0
 *         """
 *         if isinstance(name, unicode):             # <<<<<<<<<<<<<<
 *             name = name.encode('utf-8')
 *         return self._functions.get(name, None)
 */
  }

  /* "thriftrw/spec/service.pyx":434
 *         if isinstance(name, unicode):
 *             name = name.encode('utf-8')
 *         return self._functions.get(name, None)             # <<<<<<<<<<<<<<
 * 
 *     def __str__(self):
 */
  __Pyx_XDECREF(__pyx_r);
  if (unlikely(__pyx_v_self->_functions == Py_None)) {
    PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%s'", "get");
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 434; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_t_4 = __Pyx_PyDict_GetItemDefault(__pyx_v_self->_functions, __pyx_v_name, Py_None); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 434; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_r = __pyx_t_4;
  __pyx_t_4 = 0;
  goto __pyx_L0;

  /* "thriftrw/spec/service.pyx":425
 *         return self
 * 
 *     def lookup(self, name):             # <<<<<<<<<<<<<<
 *         """Look up a function with the given name.
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("thriftrw.spec.service.ServiceSpec.lookup", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_name);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "thriftrw/spec/service.pyx":436
 *         return self._functions.get(name, None)
 * 
 *     def __str__(self):             # <<<<<<<<<<<<<<
 *         return 'ServiceSpec(name=%r, functions=%r, parent=%r)' % (
 *             self.name, self.functions, self.parent
 */

/* Python wrapper */
static PyObject *__pyx_pw_8thriftrw_4spec_7service_11ServiceSpec_9__str__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_8thriftrw_4spec_7service_11ServiceSpec_9__str__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__str__ (wrapper)", 0);
  __pyx_r = __pyx_pf_8thriftrw_4spec_7service_11ServiceSpec_8__str__(((struct __pyx_obj_8thriftrw_4spec_7service_ServiceSpec *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_8thriftrw_4spec_7service_11ServiceSpec_8__str__(struct __pyx_obj_8thriftrw_4spec_7service_ServiceSpec *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__str__", 0);

  /* "thriftrw/spec/service.pyx":437
 * 
 *     def __str__(self):
 *         return 'ServiceSpec(name=%r, functions=%r, parent=%r)' % (             # <<<<<<<<<<<<<<
 *             self.name, self.functions, self.parent
 *         )
 */
  __Pyx_XDECREF(__pyx_r);

  /* "thriftrw/spec/service.pyx":438
 *     def __str__(self):
 *         return 'ServiceSpec(name=%r, functions=%r, parent=%r)' % (
 *             self.name, self.functions, self.parent             # <<<<<<<<<<<<<<
 *         )
 * 
 */
  __pyx_t_1 = PyTuple_New(3); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 438; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_v_self->name);
  __Pyx_GIVEREF(__pyx_v_self->name);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_v_self->name);
  __Pyx_INCREF(__pyx_v_self->functions);
  __Pyx_GIVEREF(__pyx_v_self->functions);
  PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_v_self->functions);
  __Pyx_INCREF(__pyx_v_self->parent);
  __Pyx_GIVEREF(__pyx_v_self->parent);
  PyTuple_SET_ITEM(__pyx_t_1, 2, __pyx_v_self->parent);

  /* "thriftrw/spec/service.pyx":437
 * 
 *     def __str__(self):
 *         return 'ServiceSpec(name=%r, functions=%r, parent=%r)' % (             # <<<<<<<<<<<<<<
 *             self.name, self.functions, self.parent
 *         )
 */
  __pyx_t_2 = PyUnicode_Format(__pyx_kp_u_ServiceSpec_name_r_functions_r_p, __pyx_t_1); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 437; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "thriftrw/spec/service.pyx":436
 *         return self._functions.get(name, None)
 * 
 *     def __str__(self):             # <<<<<<<<<<<<<<
 *         return 'ServiceSpec(name=%r, functions=%r, parent=%r)' % (
 *             self.name, self.functions, self.parent
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("thriftrw.spec.service.ServiceSpec.__str__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "thriftrw/spec/service.pyx":441
 *         )
 * 
 *     def __repr__(self):             # <<<<<<<<<<<<<<
 *         return str(self)
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_8thriftrw_4spec_7service_11ServiceSpec_11__repr__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_8thriftrw_4spec_7service_11ServiceSpec_11__repr__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__repr__ (wrapper)", 0);
  __pyx_r = __pyx_pf_8thriftrw_4spec_7service_11ServiceSpec_10__repr__(((struct __pyx_obj_8thriftrw_4spec_7service_ServiceSpec *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_8thriftrw_4spec_7service_11ServiceSpec_10__repr__(struct __pyx_obj_8thriftrw_4spec_7service_ServiceSpec *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__repr__", 0);

  /* "thriftrw/spec/service.pyx":442
 * 
 *     def __repr__(self):
 *         return str(self)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyTuple_New(1); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 442; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(((PyObject *)__pyx_v_self));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self));
  PyTuple_SET_ITEM(__pyx_t_1, 0, ((PyObject *)__pyx_v_self));
  __pyx_t_2 = __Pyx_PyObject_Call(((PyObject *)(&PyString_Type)), __pyx_t_1, NULL); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 442; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "thriftrw/spec/service.pyx":441
 *         )
 * 
 *     def __repr__(self):             # <<<<<<<<<<<<<<
 *         return str(self)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("thriftrw.spec.service.ServiceSpec.__repr__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "thriftrw/spec/service.pxd":52
 * 
 * cdef class ServiceSpec(object):
 *     cdef readonly str name             # <<<<<<<<<<<<<<
 *     cdef public list functions
 *     cdef public object parent
 */

/* Python wrapper */
static PyObject *__pyx_pw_8thriftrw_4spec_7service_11ServiceSpec_4name_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_8thriftrw_4spec_7service_11ServiceSpec_4name_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_8thriftrw_4spec_7service_11ServiceSpec_4name___get__(((struct __pyx_obj_8thriftrw_4spec_7service_ServiceSpec *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_8thriftrw_4spec_7service_11ServiceSpec_4name___get__(struct __pyx_obj_8thriftrw_4spec_7service_ServiceSpec *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->name);
  __pyx_r = __pyx_v_self->name;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "thriftrw/spec/service.pxd":53
 * cdef class ServiceSpec(object):
 *     cdef readonly str name
 *     cdef public list functions             # <<<<<<<<<<<<<<
 *     cdef public object parent
 *     cdef public dict _functions
 */

/* Python wrapper */
static PyObject *__pyx_pw_8thriftrw_4spec_7service_11ServiceSpec_9functions_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_8thriftrw_4spec_7service_11ServiceSpec_9functions_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_8thriftrw_4spec_7service_11ServiceSpec_9functions___get__(((struct __pyx_obj_8thriftrw_4spec_7service_ServiceSpec *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_8thriftrw_4spec_7service_11ServiceSpec_9functions___get__(struct __pyx_obj_8thriftrw_4spec_7service_ServiceSpec *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->functions);
  __pyx_r = __pyx_v_self->functions;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_8thriftrw_4spec_7service_11ServiceSpec_9functions_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_8thriftrw_4spec_7service_11ServiceSpec_9functions_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_8thriftrw_4spec_7service_11ServiceSpec_9functions_2__set__(((struct __pyx_obj_8thriftrw_4spec_7service_ServiceSpec *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_8thriftrw_4spec_7service_11ServiceSpec_9functions_2__set__(struct __pyx_obj_8thriftrw_4spec_7service_ServiceSpec *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__set__", 0);
  if (!(likely(PyList_CheckExact(__pyx_v_value))||((__pyx_v_value) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_v_value)->tp_name), 0))) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 53; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_1 = __pyx_v_value;
  __Pyx_INCREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->functions);
  __Pyx_DECREF(__pyx_v_self->functions);
  __pyx_v_self->functions = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("thriftrw.spec.service.ServiceSpec.functions.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_8thriftrw_4spec_7service_11ServiceSpec_9functions_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_8thriftrw_4spec_7service_11ServiceSpec_9functions_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_8thriftrw_4spec_7service_11ServiceSpec_9functions_4__del__(((struct __pyx_obj_8thriftrw_4spec_7service_ServiceSpec *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_8thriftrw_4spec_7service_11ServiceSpec_9functions_4__del__(struct __pyx_obj_8thriftrw_4spec_7service_ServiceSpec *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->functions);
  __Pyx_DECREF(__pyx_v_self->functions);
  __pyx_v_self->functions = ((PyObject*)Py_None);

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "thriftrw/spec/service.pxd":54
 *     cdef readonly str name
 *     cdef public list functions
 *     cdef public object parent             # <<<<<<<<<<<<<<
 *     cdef public dict _functions
 *     cdef public bint linked
 */

/* Python wrapper */
static PyObject *__pyx_pw_8thriftrw_4spec_7service_11ServiceSpec_6parent_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_8thriftrw_4spec_7service_11ServiceSpec_6parent_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_8thriftrw_4spec_7service_11ServiceSpec_6parent___get__(((struct __pyx_obj_8thriftrw_4spec_7service_ServiceSpec *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_8thriftrw_4spec_7service_11ServiceSpec_6parent___get__(struct __pyx_obj_8thriftrw_4spec_7service_ServiceSpec *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->parent);
  __pyx_r = __pyx_v_self->parent;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_8thriftrw_4spec_7service_11ServiceSpec_6parent_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_8thriftrw_4spec_7service_11ServiceSpec_6parent_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_8thriftrw_4spec_7service_11ServiceSpec_6parent_2__set__(((struct __pyx_obj_8thriftrw_4spec_7service_ServiceSpec *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_8thriftrw_4spec_7service_11ServiceSpec_6parent_2__set__(struct __pyx_obj_8thriftrw_4spec_7service_ServiceSpec *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 0);
  __Pyx_INCREF(__pyx_v_value);
  __Pyx_GIVEREF(__pyx_v_value);
  __Pyx_GOTREF(__pyx_v_self->parent);
  __Pyx_DECREF(__pyx_v_self->parent);
  __pyx_v_self->parent = __pyx_v_value;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_8thriftrw_4spec_7service_11ServiceSpec_6parent_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_8thriftrw_4spec_7service_11ServiceSpec_6parent_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_8thriftrw_4spec_7service_11ServiceSpec_6parent_4__del__(((struct __pyx_obj_8thriftrw_4spec_7service_ServiceSpec *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_8thriftrw_4spec_7service_11ServiceSpec_6parent_4__del__(struct __pyx_obj_8thriftrw_4spec_7service_ServiceSpec *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->parent);
  __Pyx_DECREF(__pyx_v_self->parent);
  __pyx_v_self->parent = Py_None;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "thriftrw/spec/service.pxd":55
 *     cdef public list functions
 *     cdef public object parent
 *     cdef public dict _functions             # <<<<<<<<<<<<<<
 *     cdef public bint linked
 *     cdef public object surface
 */

/* Python wrapper */
static PyObject *__pyx_pw_8thriftrw_4spec_7service_11ServiceSpec_10_functions_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_8thriftrw_4spec_7service_11ServiceSpec_10_functions_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_8thriftrw_4spec_7service_11ServiceSpec_10_functions___get__(((struct __pyx_obj_8thriftrw_4spec_7service_ServiceSpec *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_8thriftrw_4spec_7service_11ServiceSpec_10_functions___get__(struct __pyx_obj_8thriftrw_4spec_7service_ServiceSpec *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->_functions);
  __pyx_r = __pyx_v_self->_functions;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_8thriftrw_4spec_7service_11ServiceSpec_10_functions_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_8thriftrw_4spec_7service_11ServiceSpec_10_functions_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_8thriftrw_4spec_7service_11ServiceSpec_10_functions_2__set__(((struct __pyx_obj_8thriftrw_4spec_7service_ServiceSpec *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_8thriftrw_4spec_7service_11ServiceSpec_10_functions_2__set__(struct __pyx_obj_8thriftrw_4spec_7service_ServiceSpec *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__set__", 0);
  if (!(likely(PyDict_CheckExact(__pyx_v_value))||((__pyx_v_value) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "dict", Py_TYPE(__pyx_v_value)->tp_name), 0))) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 55; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_1 = __pyx_v_value;
  __Pyx_INCREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->_functions);
  __Pyx_DECREF(__pyx_v_self->_functions);
  __pyx_v_self->_functions = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("thriftrw.spec.service.ServiceSpec._functions.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_8thriftrw_4spec_7service_11ServiceSpec_10_functions_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_8thriftrw_4spec_7service_11ServiceSpec_10_functions_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_8thriftrw_4spec_7service_11ServiceSpec_10_functions_4__del__(((struct __pyx_obj_8thriftrw_4spec_7service_ServiceSpec *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_8thriftrw_4spec_7service_11ServiceSpec_10_functions_4__del__(struct __pyx_obj_8thriftrw_4spec_7service_ServiceSpec *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->_functions);
  __Pyx_DECREF(__pyx_v_self->_functions);
  __pyx_v_self->_functions = ((PyObject*)Py_None);

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "thriftrw/spec/service.pxd":56
 *     cdef public object parent
 *     cdef public dict _functions
 *     cdef public bint linked             # <<<<<<<<<<<<<<
 *     cdef public object surface
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_8thriftrw_4spec_7service_11ServiceSpec_6linked_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_8thriftrw_4spec_7service_11ServiceSpec_6linked_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_8thriftrw_4spec_7service_11ServiceSpec_6linked___get__(((struct __pyx_obj_8thriftrw_4spec_7service_ServiceSpec *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_8thriftrw_4spec_7service_11ServiceSpec_6linked___get__(struct __pyx_obj_8thriftrw_4spec_7service_ServiceSpec *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyBool_FromLong(__pyx_v_self->linked); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 56; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("thriftrw.spec.service.ServiceSpec.linked.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_8thriftrw_4spec_7service_11ServiceSpec_6linked_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_8thriftrw_4spec_7service_11ServiceSpec_6linked_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_8thriftrw_4spec_7service_11ServiceSpec_6linked_2__set__(((struct __pyx_obj_8thriftrw_4spec_7service_ServiceSpec *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_8thriftrw_4spec_7service_11ServiceSpec_6linked_2__set__(struct __pyx_obj_8thriftrw_4spec_7service_ServiceSpec *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__set__", 0);
  __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_v_value); if (unlikely((__pyx_t_1 == (int)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 56; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_v_self->linked = __pyx_t_1;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("thriftrw.spec.service.ServiceSpec.linked.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "thriftrw/spec/service.pxd":57
 *     cdef public dict _functions
 *     cdef public bint linked
 *     cdef public object surface             # <<<<<<<<<<<<<<
 * 
 *     cpdef ServiceSpec link(self, scope)
 */

/* Python wrapper */
static PyObject *__pyx_pw_8thriftrw_4spec_7service_11ServiceSpec_7surface_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_8thriftrw_4spec_7service_11ServiceSpec_7surface_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_8thriftrw_4spec_7service_11ServiceSpec_7surface___get__(((struct __pyx_obj_8thriftrw_4spec_7service_ServiceSpec *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_8thriftrw_4spec_7service_11ServiceSpec_7surface___get__(struct __pyx_obj_8thriftrw_4spec_7service_ServiceSpec *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->surface);
  __pyx_r = __pyx_v_self->surface;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_8thriftrw_4spec_7service_11ServiceSpec_7surface_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_8thriftrw_4spec_7service_11ServiceSpec_7surface_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_8thriftrw_4spec_7service_11ServiceSpec_7surface_2__set__(((struct __pyx_obj_8thriftrw_4spec_7service_ServiceSpec *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_8thriftrw_4spec_7service_11ServiceSpec_7surface_2__set__(struct __pyx_obj_8thriftrw_4spec_7service_ServiceSpec *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 0);
  __Pyx_INCREF(__pyx_v_value);
  __Pyx_GIVEREF(__pyx_v_value);
  __Pyx_GOTREF(__pyx_v_self->surface);
  __Pyx_DECREF(__pyx_v_self->surface);
  __pyx_v_self->surface = __pyx_v_value;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_8thriftrw_4spec_7service_11ServiceSpec_7surface_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_8thriftrw_4spec_7service_11ServiceSpec_7surface_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_8thriftrw_4spec_7service_11ServiceSpec_7surface_4__del__(((struct __pyx_obj_8thriftrw_4spec_7service_ServiceSpec *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_8thriftrw_4spec_7service_11ServiceSpec_7surface_4__del__(struct __pyx_obj_8thriftrw_4spec_7service_ServiceSpec *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->surface);
  __Pyx_DECREF(__pyx_v_self->surface);
  __pyx_v_self->surface = Py_None;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "thriftrw/spec/service.pyx":473
 * 
 * 
 * def service_cls(service_spec, scope):             # <<<<<<<<<<<<<<
 *     """Generates a class from the given service spec.
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_8thriftrw_4spec_7service_1service_cls(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_8thriftrw_4spec_7service_service_cls[] = "Generates a class from the given service spec.\n\n    :param ServiceSpec service_spec:\n        Specification of the service.\n    :param scope:\n        Compilation scope.\n    ";
static PyMethodDef __pyx_mdef_8thriftrw_4spec_7service_1service_cls = {"service_cls", (PyCFunction)__pyx_pw_8thriftrw_4spec_7service_1service_cls, METH_VARARGS|METH_KEYWORDS, __pyx_doc_8thriftrw_4spec_7service_service_cls};
static PyObject *__pyx_pw_8thriftrw_4spec_7service_1service_cls(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_service_spec = 0;
  CYTHON_UNUSED PyObject *__pyx_v_scope = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("service_cls (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_service_spec,&__pyx_n_s_scope,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_service_spec)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_scope)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("service_cls", 1, 2, 2, 1); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 473; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "service_cls") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 473; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_service_spec = values[0];
    __pyx_v_scope = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("service_cls", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 473; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("thriftrw.spec.service.service_cls", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_8thriftrw_4spec_7service_service_cls(__pyx_self, __pyx_v_service_spec, __pyx_v_scope);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_8thriftrw_4spec_7service_service_cls(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_service_spec, CYTHON_UNUSED PyObject *__pyx_v_scope) {
  PyObject *__pyx_v_parent_cls = NULL;
  PyObject *__pyx_v_service_dct = NULL;
  PyObject *__pyx_v_function = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  Py_ssize_t __pyx_t_5;
  PyObject *(*__pyx_t_6)(PyObject *);
  PyObject *__pyx_t_7 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("service_cls", 0);

  /* "thriftrw/spec/service.pyx":481
 *         Compilation scope.
 *     """
 *     parent_cls = object             # <<<<<<<<<<<<<<
 *     if service_spec.parent is not None:
 *         parent_cls = service_spec.parent.surface
 */
  __Pyx_INCREF(__pyx_builtin_object);
  __pyx_v_parent_cls = __pyx_builtin_object;

  /* "thriftrw/spec/service.pyx":482
 *     """
 *     parent_cls = object
 *     if service_spec.parent is not None:             # <<<<<<<<<<<<<<
 *         parent_cls = service_spec.parent.surface
 * 
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_service_spec, __pyx_n_s_parent); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 482; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = (__pyx_t_1 != Py_None);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_3 = (__pyx_t_2 != 0);
  if (__pyx_t_3) {

    /* "thriftrw/spec/service.pyx":483
 *     parent_cls = object
 *     if service_spec.parent is not None:
 *         parent_cls = service_spec.parent.surface             # <<<<<<<<<<<<<<
 * 
 *     service_dct = {}
 */
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_service_spec, __pyx_n_s_parent); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 483; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_surface); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 483; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF_SET(__pyx_v_parent_cls, __pyx_t_4);
    __pyx_t_4 = 0;

    /* "thriftrw/spec/service.pyx":482
 *     """
 *     parent_cls = object
 *     if service_spec.parent is not None:             # <<<<<<<<<<<<<<
 *         parent_cls = service_spec.parent.surface
 * 
 */
  }

  /* "thriftrw/spec/service.pyx":485
 *         parent_cls = service_spec.parent.surface
 * 
 *     service_dct = {}             # <<<<<<<<<<<<<<
 *     for function in service_spec.functions:
 *         service_dct[function.name] = function.surface
 */
  __pyx_t_4 = PyDict_New(); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 485; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_v_service_dct = ((PyObject*)__pyx_t_4);
  __pyx_t_4 = 0;

  /* "thriftrw/spec/service.pyx":486
 * 
 *     service_dct = {}
 *     for function in service_spec.functions:             # <<<<<<<<<<<<<<
 *         service_dct[function.name] = function.surface
 * 
 */
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_service_spec, __pyx_n_s_functions); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 486; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  if (likely(PyList_CheckExact(__pyx_t_4)) || PyTuple_CheckExact(__pyx_t_4)) {
    __pyx_t_1 = __pyx_t_4; __Pyx_INCREF(__pyx_t_1); __pyx_t_5 = 0;
    __pyx_t_6 = NULL;
  } else {
    __pyx_t_5 = -1; __pyx_t_1 = PyObject_GetIter(__pyx_t_4); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 486; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_6 = Py_TYPE(__pyx_t_1)->tp_iternext; if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 486; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  for (;;) {
    if (likely(!__pyx_t_6)) {
      if (likely(PyList_CheckExact(__pyx_t_1))) {
        if (__pyx_t_5 >= PyList_GET_SIZE(__pyx_t_1)) break;
        #if CYTHON_COMPILING_IN_CPYTHON
        __pyx_t_4 = PyList_GET_ITEM(__pyx_t_1, __pyx_t_5); __Pyx_INCREF(__pyx_t_4); __pyx_t_5++; if (unlikely(0 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 486; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        #else
        __pyx_t_4 = PySequence_ITEM(__pyx_t_1, __pyx_t_5); __pyx_t_5++; if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 486; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_4);
        #endif
      } else {
        if (__pyx_t_5 >= PyTuple_GET_SIZE(__pyx_t_1)) break;
        #if CYTHON_COMPILING_IN_CPYTHON
        __pyx_t_4 = PyTuple_GET_ITEM(__pyx_t_1, __pyx_t_5); __Pyx_INCREF(__pyx_t_4); __pyx_t_5++; if (unlikely(0 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 486; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        #else
        __pyx_t_4 = PySequence_ITEM(__pyx_t_1, __pyx_t_5); __pyx_t_5++; if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 486; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_4);
        #endif
      }
    } else {
      __pyx_t_4 = __pyx_t_6(__pyx_t_1);
      if (unlikely(!__pyx_t_4)) {
        PyObject* exc_type = PyErr_Occurred();
        if (exc_type) {
          if (likely(exc_type == PyExc_StopIteration || PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
          else {__pyx_filename = __pyx_f[0]; __pyx_lineno = 486; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        }
        break;
      }
      __Pyx_GOTREF(__pyx_t_4);
    }
    __Pyx_XDECREF_SET(__pyx_v_function, __pyx_t_4);
    __pyx_t_4 = 0;

    /* "thriftrw/spec/service.pyx":487
 *     service_dct = {}
 *     for function in service_spec.functions:
 *         service_dct[function.name] = function.surface             # <<<<<<<<<<<<<<
 * 
 *     service_dct['service_spec'] = service_spec
 */
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_function, __pyx_n_s_surface); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 487; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_v_function, __pyx_n_s_name); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 487; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_7);
    if (unlikely(PyDict_SetItem(__pyx_v_service_dct, __pyx_t_7, __pyx_t_4) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 487; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

    /* "thriftrw/spec/service.pyx":486
 * 
 *     service_dct = {}
 *     for function in service_spec.functions:             # <<<<<<<<<<<<<<
 *         service_dct[function.name] = function.surface
 * 
 */
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "thriftrw/spec/service.pyx":489
 *         service_dct[function.name] = function.surface
 * 
 *     service_dct['service_spec'] = service_spec             # <<<<<<<<<<<<<<
 *     service_dct['__slots__'] = ()
 * 
 */
  if (unlikely(PyDict_SetItem(__pyx_v_service_dct, __pyx_n_u_service_spec, __pyx_v_service_spec) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 489; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "thriftrw/spec/service.pyx":490
 * 
 *     service_dct['service_spec'] = service_spec
 *     service_dct['__slots__'] = ()             # <<<<<<<<<<<<<<
 * 
 *     return type(str(service_spec.name), (parent_cls,), service_dct)
 */
  if (unlikely(PyDict_SetItem(__pyx_v_service_dct, __pyx_n_u_slots, __pyx_empty_tuple) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 490; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "thriftrw/spec/service.pyx":492
 *     service_dct['__slots__'] = ()
 * 
 *     return type(str(service_spec.name), (parent_cls,), service_dct)             # <<<<<<<<<<<<<<
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_service_spec, __pyx_n_s_name); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 492; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_4 = PyTuple_New(1); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 492; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_1);
  __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(((PyObject *)(&PyString_Type)), __pyx_t_4, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 492; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = PyTuple_New(1); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 492; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_INCREF(__pyx_v_parent_cls);
  __Pyx_GIVEREF(__pyx_v_parent_cls);
  PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_v_parent_cls);
  __pyx_t_7 = PyTuple_New(3); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 492; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_7);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_7, 0, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_7, 1, __pyx_t_4);
  __Pyx_INCREF(__pyx_v_service_dct);
  __Pyx_GIVEREF(__pyx_v_service_dct);
  PyTuple_SET_ITEM(__pyx_t_7, 2, __pyx_v_service_dct);
  __pyx_t_1 = 0;
  __pyx_t_4 = 0;
  __pyx_t_4 = __Pyx_PyObject_Call(((PyObject *)(&PyType_Type)), __pyx_t_7, NULL); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 492; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __pyx_r = __pyx_t_4;
  __pyx_t_4 = 0;
  goto __pyx_L0;

  /* "thriftrw/spec/service.pyx":473
 * 
 * 
 * def service_cls(service_spec, scope):             # <<<<<<<<<<<<<<
 *     """Generates a class from the given service spec.
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_AddTraceback("thriftrw.spec.service.service_cls", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_parent_cls);
  __Pyx_XDECREF(__pyx_v_service_dct);
  __Pyx_XDECREF(__pyx_v_function);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}
static struct __pyx_vtabstruct_8thriftrw_4spec_7service_FunctionArgsSpec __pyx_vtable_8thriftrw_4spec_7service_FunctionArgsSpec;

static PyObject *__pyx_tp_new_8thriftrw_4spec_7service_FunctionArgsSpec(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_8thriftrw_4spec_7service_FunctionArgsSpec *p;
  PyObject *o = __pyx_ptype_8thriftrw_4spec_6struct_StructTypeSpec->tp_new(t, a, k);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_8thriftrw_4spec_7service_FunctionArgsSpec *)o);
  p->__pyx_base.__pyx_base.__pyx_vtab = (struct __pyx_vtabstruct_8thriftrw_4spec_4base_TypeSpec*)__pyx_vtabptr_8thriftrw_4spec_7service_FunctionArgsSpec;
  p->function = ((struct __pyx_obj_8thriftrw_4spec_7service_FunctionSpec *)Py_None); Py_INCREF(Py_None);
  return o;
}

static void __pyx_tp_dealloc_8thriftrw_4spec_7service_FunctionArgsSpec(PyObject *o) {
  struct __pyx_obj_8thriftrw_4spec_7service_FunctionArgsSpec *p = (struct __pyx_obj_8thriftrw_4spec_7service_FunctionArgsSpec *)o;
  #if PY_VERSION_HEX >= 0x030400a1
  if (unlikely(Py_TYPE(o)->tp_finalize) && !_PyGC_FINALIZED(o)) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->function);
  PyObject_GC_Track(o);
  if (likely(__pyx_ptype_8thriftrw_4spec_6struct_StructTypeSpec)) __pyx_ptype_8thriftrw_4spec_6struct_StructTypeSpec->tp_dealloc(o); else __Pyx_call_next_tp_dealloc(o, __pyx_tp_dealloc_8thriftrw_4spec_7service_FunctionArgsSpec);
}

static int __pyx_tp_traverse_8thriftrw_4spec_7service_FunctionArgsSpec(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_8thriftrw_4spec_7service_FunctionArgsSpec *p = (struct __pyx_obj_8thriftrw_4spec_7service_FunctionArgsSpec *)o;
  e = ((likely(__pyx_ptype_8thriftrw_4spec_6struct_StructTypeSpec)) ? ((__pyx_ptype_8thriftrw_4spec_6struct_StructTypeSpec->tp_traverse) ? __pyx_ptype_8thriftrw_4spec_6struct_StructTypeSpec->tp_traverse(o, v, a) : 0) : __Pyx_call_next_tp_traverse(o, v, a, __pyx_tp_traverse_8thriftrw_4spec_7service_FunctionArgsSpec)); if (e) return e;
  if (p->function) {
    e = (*v)(((PyObject*)p->function), a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_8thriftrw_4spec_7service_FunctionArgsSpec(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_8thriftrw_4spec_7service_FunctionArgsSpec *p = (struct __pyx_obj_8thriftrw_4spec_7service_FunctionArgsSpec *)o;
  if (likely(__pyx_ptype_8thriftrw_4spec_6struct_StructTypeSpec)) { if (__pyx_ptype_8thriftrw_4spec_6struct_StructTypeSpec->tp_clear) __pyx_ptype_8thriftrw_4spec_6struct_StructTypeSpec->tp_clear(o); } else __Pyx_call_next_tp_clear(o, __pyx_tp_clear_8thriftrw_4spec_7service_FunctionArgsSpec);
  tmp = ((PyObject*)p->function);
  p->function = ((struct __pyx_obj_8thriftrw_4spec_7service_FunctionSpec *)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyObject *__pyx_getprop_8thriftrw_4spec_7service_16FunctionArgsSpec_function(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_8thriftrw_4spec_7service_16FunctionArgsSpec_8function_1__get__(o);
}

static int __pyx_setprop_8thriftrw_4spec_7service_16FunctionArgsSpec_function(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_8thriftrw_4spec_7service_16FunctionArgsSpec_8function_3__set__(o, v);
  }
  else {
    return __pyx_pw_8thriftrw_4spec_7service_16FunctionArgsSpec_8function_5__del__(o);
  }
}

static PyMethodDef __pyx_methods_8thriftrw_4spec_7service_FunctionArgsSpec[] = {
  {"compile", (PyCFunction)__pyx_pw_8thriftrw_4spec_7service_16FunctionArgsSpec_3compile, METH_VARARGS|METH_KEYWORDS, __pyx_doc_8thriftrw_4spec_7service_16FunctionArgsSpec_2compile},
  {"link", (PyCFunction)__pyx_pw_8thriftrw_4spec_7service_16FunctionArgsSpec_5link, METH_VARARGS|METH_KEYWORDS, 0},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_8thriftrw_4spec_7service_FunctionArgsSpec[] = {
  {(char *)"function", __pyx_getprop_8thriftrw_4spec_7service_16FunctionArgsSpec_function, __pyx_setprop_8thriftrw_4spec_7service_16FunctionArgsSpec_function, 0, 0},
  {0, 0, 0, 0, 0}
};

static PyTypeObject __pyx_type_8thriftrw_4spec_7service_FunctionArgsSpec = {
  PyVarObject_HEAD_INIT(0, 0)
  "thriftrw.spec.service.FunctionArgsSpec", /*tp_name*/
  sizeof(struct __pyx_obj_8thriftrw_4spec_7service_FunctionArgsSpec), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_8thriftrw_4spec_7service_FunctionArgsSpec, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  "Represents the parameters of a service function.\n\n    .. py:attribute:: function\n\n        The :py:class:`FunctionSpec` whose arguments this spec represents.\n        This value is available only after the spec has been linked.\n\n    The parameters of a function implicitly form a struct which contains the\n    parameters as its fields, which are optional by default.\n\n    The surface for this is the same as :py:class:`StructTypeSpec` except that\n    the generated class also includes the following class attributes.\n\n    .. py:attribute:: result_type\n\n        A reference to the class representing the result type for\n        ``function``, or None if the function was oneway.\n\n    .. versionchanged:: 1.0\n\n        Added the ``function`` attribute.\n\n    .. versionchanged:: 1.1\n\n        Added ``result_type`` class attribute to the surface.\n    ", /*tp_doc*/
  __pyx_tp_traverse_8thriftrw_4spec_7service_FunctionArgsSpec, /*tp_traverse*/
  __pyx_tp_clear_8thriftrw_4spec_7service_FunctionArgsSpec, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_8thriftrw_4spec_7service_FunctionArgsSpec, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_8thriftrw_4spec_7service_FunctionArgsSpec, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  __pyx_pw_8thriftrw_4spec_7service_16FunctionArgsSpec_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_8thriftrw_4spec_7service_FunctionArgsSpec, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};
static struct __pyx_vtabstruct_8thriftrw_4spec_7service_FunctionResultSpec __pyx_vtable_8thriftrw_4spec_7service_FunctionResultSpec;

static PyObject *__pyx_tp_new_8thriftrw_4spec_7service_FunctionResultSpec(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_8thriftrw_4spec_7service_FunctionResultSpec *p;
  PyObject *o = __pyx_ptype_8thriftrw_4spec_5union_UnionTypeSpec->tp_new(t, a, k);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_8thriftrw_4spec_7service_FunctionResultSpec *)o);
  p->__pyx_base.__pyx_base.__pyx_vtab = (struct __pyx_vtabstruct_8thriftrw_4spec_4base_TypeSpec*)__pyx_vtabptr_8thriftrw_4spec_7service_FunctionResultSpec;
  p->return_spec = ((struct __pyx_obj_8thriftrw_4spec_4base_TypeSpec *)Py_None); Py_INCREF(Py_None);
  p->exception_specs = ((PyObject*)Py_None); Py_INCREF(Py_None);
  p->function = ((struct __pyx_obj_8thriftrw_4spec_7service_FunctionSpec *)Py_None); Py_INCREF(Py_None);
  p->exception_ids = Py_None; Py_INCREF(Py_None);
  return o;
}

static void __pyx_tp_dealloc_8thriftrw_4spec_7service_FunctionResultSpec(PyObject *o) {
  struct __pyx_obj_8thriftrw_4spec_7service_FunctionResultSpec *p = (struct __pyx_obj_8thriftrw_4spec_7service_FunctionResultSpec *)o;
  #if PY_VERSION_HEX >= 0x030400a1
  if (unlikely(Py_TYPE(o)->tp_finalize) && !_PyGC_FINALIZED(o)) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->return_spec);
  Py_CLEAR(p->exception_specs);
  Py_CLEAR(p->function);
  Py_CLEAR(p->exception_ids);
  PyObject_GC_Track(o);
  if (likely(__pyx_ptype_8thriftrw_4spec_5union_UnionTypeSpec)) __pyx_ptype_8thriftrw_4spec_5union_UnionTypeSpec->tp_dealloc(o); else __Pyx_call_next_tp_dealloc(o, __pyx_tp_dealloc_8thriftrw_4spec_7service_FunctionResultSpec);
}

static int __pyx_tp_traverse_8thriftrw_4spec_7service_FunctionResultSpec(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_8thriftrw_4spec_7service_FunctionResultSpec *p = (struct __pyx_obj_8thriftrw_4spec_7service_FunctionResultSpec *)o;
  e = ((likely(__pyx_ptype_8thriftrw_4spec_5union_UnionTypeSpec)) ? ((__pyx_ptype_8thriftrw_4spec_5union_UnionTypeSpec->tp_traverse) ? __pyx_ptype_8thriftrw_4spec_5union_UnionTypeSpec->tp_traverse(o, v, a) : 0) : __Pyx_call_next_tp_traverse(o, v, a, __pyx_tp_traverse_8thriftrw_4spec_7service_FunctionResultSpec)); if (e) return e;
  if (p->return_spec) {
    e = (*v)(((PyObject*)p->return_spec), a); if (e) return e;
  }
  if (p->exception_specs) {
    e = (*v)(p->exception_specs, a); if (e) return e;
  }
  if (p->function) {
    e = (*v)(((PyObject*)p->function), a); if (e) return e;
  }
  if (p->exception_ids) {
    e = (*v)(p->exception_ids, a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_8thriftrw_4spec_7service_FunctionResultSpec(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_8thriftrw_4spec_7service_FunctionResultSpec *p = (struct __pyx_obj_8thriftrw_4spec_7service_FunctionResultSpec *)o;
  if (likely(__pyx_ptype_8thriftrw_4spec_5union_UnionTypeSpec)) { if (__pyx_ptype_8thriftrw_4spec_5union_UnionTypeSpec->tp_clear) __pyx_ptype_8thriftrw_4spec_5union_UnionTypeSpec->tp_clear(o); } else __Pyx_call_next_tp_clear(o, __pyx_tp_clear_8thriftrw_4spec_7service_FunctionResultSpec);
  tmp = ((PyObject*)p->return_spec);
  p->return_spec = ((struct __pyx_obj_8thriftrw_4spec_4base_TypeSpec *)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->exception_specs);
  p->exception_specs = ((PyObject*)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->function);
  p->function = ((struct __pyx_obj_8thriftrw_4spec_7service_FunctionSpec *)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->exception_ids);
  p->exception_ids = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyObject *__pyx_getprop_8thriftrw_4spec_7service_18FunctionResultSpec_return_spec(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_8thriftrw_4spec_7service_18FunctionResultSpec_11return_spec_1__get__(o);
}

static int __pyx_setprop_8thriftrw_4spec_7service_18FunctionResultSpec_return_spec(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_8thriftrw_4spec_7service_18FunctionResultSpec_11return_spec_3__set__(o, v);
  }
  else {
    return __pyx_pw_8thriftrw_4spec_7service_18FunctionResultSpec_11return_spec_5__del__(o);
  }
}

static PyObject *__pyx_getprop_8thriftrw_4spec_7service_18FunctionResultSpec_exception_specs(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_8thriftrw_4spec_7service_18FunctionResultSpec_15exception_specs_1__get__(o);
}

static int __pyx_setprop_8thriftrw_4spec_7service_18FunctionResultSpec_exception_specs(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_8thriftrw_4spec_7service_18FunctionResultSpec_15exception_specs_3__set__(o, v);
  }
  else {
    return __pyx_pw_8thriftrw_4spec_7service_18FunctionResultSpec_15exception_specs_5__del__(o);
  }
}

static PyObject *__pyx_getprop_8thriftrw_4spec_7service_18FunctionResultSpec_function(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_8thriftrw_4spec_7service_18FunctionResultSpec_8function_1__get__(o);
}

static int __pyx_setprop_8thriftrw_4spec_7service_18FunctionResultSpec_function(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_8thriftrw_4spec_7service_18FunctionResultSpec_8function_3__set__(o, v);
  }
  else {
    return __pyx_pw_8thriftrw_4spec_7service_18FunctionResultSpec_8function_5__del__(o);
  }
}

static PyObject *__pyx_getprop_8thriftrw_4spec_7service_18FunctionResultSpec_exception_ids(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_8thriftrw_4spec_7service_18FunctionResultSpec_13exception_ids_1__get__(o);
}

static int __pyx_setprop_8thriftrw_4spec_7service_18FunctionResultSpec_exception_ids(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_8thriftrw_4spec_7service_18FunctionResultSpec_13exception_ids_3__set__(o, v);
  }
  else {
    return __pyx_pw_8thriftrw_4spec_7service_18FunctionResultSpec_13exception_ids_5__del__(o);
  }
}

static PyMethodDef __pyx_methods_8thriftrw_4spec_7service_FunctionResultSpec[] = {
  {"from_wire", (PyCFunction)__pyx_pw_8thriftrw_4spec_7service_18FunctionResultSpec_3from_wire, METH_O, 0},
  {"link", (PyCFunction)__pyx_pw_8thriftrw_4spec_7service_18FunctionResultSpec_5link, METH_VARARGS|METH_KEYWORDS, 0},
  {"compile", (PyCFunction)__pyx_pw_8thriftrw_4spec_7service_18FunctionResultSpec_7compile, METH_VARARGS|METH_KEYWORDS, __pyx_doc_8thriftrw_4spec_7service_18FunctionResultSpec_6compile},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_8thriftrw_4spec_7service_FunctionResultSpec[] = {
  {(char *)"return_spec", __pyx_getprop_8thriftrw_4spec_7service_18FunctionResultSpec_return_spec, __pyx_setprop_8thriftrw_4spec_7service_18FunctionResultSpec_return_spec, 0, 0},
  {(char *)"exception_specs", __pyx_getprop_8thriftrw_4spec_7service_18FunctionResultSpec_exception_specs, __pyx_setprop_8thriftrw_4spec_7service_18FunctionResultSpec_exception_specs, 0, 0},
  {(char *)"function", __pyx_getprop_8thriftrw_4spec_7service_18FunctionResultSpec_function, __pyx_setprop_8thriftrw_4spec_7service_18FunctionResultSpec_function, 0, 0},
  {(char *)"exception_ids", __pyx_getprop_8thriftrw_4spec_7service_18FunctionResultSpec_exception_ids, __pyx_setprop_8thriftrw_4spec_7service_18FunctionResultSpec_exception_ids, 0, 0},
  {0, 0, 0, 0, 0}
};

static PyTypeObject __pyx_type_8thriftrw_4spec_7service_FunctionResultSpec = {
  PyVarObject_HEAD_INIT(0, 0)
  "thriftrw.spec.service.FunctionResultSpec", /*tp_name*/
  sizeof(struct __pyx_obj_8thriftrw_4spec_7service_FunctionResultSpec), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_8thriftrw_4spec_7service_FunctionResultSpec, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  "Represents the result of a service function.\n\n    .. py:attribute:: return_spec\n\n        :py:class:`thriftrw.spec.TypeSpec` of the return type or None if the\n        function does not return anything.\n\n    .. py:attribute:: exception_specs\n\n        Collection of :py:class:`thriftrw.spec.FieldSpec` objects defining the\n        exceptions that this function can raise.\n\n    .. py:attribute:: function\n\n        The :py:class:`FunctionSpec` whose result type this spec represents.\n        This value is available only after the spec has been linked.\n\n    The return value of a function and the exceptions raised by it implicitly\n    form a union which contains the return value at field ID ``0`` and the\n    exceptions on the remaining field IDs.\n\n    .. versionchanged:: 0.2\n\n        Expose the class and add the ``return_spec`` and ``exception_specs``\n        attributes.\n\n    .. versionchanged:: 0.5\n\n        When deserializing, if an unrecognized exception is found, a\n        :py:class:`thriftrw.errors.UnknownExceptionError` is raised.\n\n    .. versionchanged:: 1.0\n\n        Added the ``function`` attribute.\n    ", /*tp_doc*/
  __pyx_tp_traverse_8thriftrw_4spec_7service_FunctionResultSpec, /*tp_traverse*/
  __pyx_tp_clear_8thriftrw_4spec_7service_FunctionResultSpec, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_8thriftrw_4spec_7service_FunctionResultSpec, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_8thriftrw_4spec_7service_FunctionResultSpec, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  __pyx_pw_8thriftrw_4spec_7service_18FunctionResultSpec_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_8thriftrw_4spec_7service_FunctionResultSpec, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};
static struct __pyx_vtabstruct_8thriftrw_4spec_7service_FunctionSpec __pyx_vtable_8thriftrw_4spec_7service_FunctionSpec;

static PyObject *__pyx_tp_new_8thriftrw_4spec_7service_FunctionSpec(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  struct __pyx_obj_8thriftrw_4spec_7service_FunctionSpec *p;
  PyObject *o;
  if (likely((t->tp_flags & Py_TPFLAGS_IS_ABSTRACT) == 0)) {
    o = (*t->tp_alloc)(t, 0);
  } else {
    o = (PyObject *) PyBaseObject_Type.tp_new(t, __pyx_empty_tuple, 0);
  }
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_8thriftrw_4spec_7service_FunctionSpec *)o);
  p->__pyx_vtab = __pyx_vtabptr_8thriftrw_4spec_7service_FunctionSpec;
  p->name = ((PyObject*)Py_None); Py_INCREF(Py_None);
  p->args_spec = ((struct __pyx_obj_8thriftrw_4spec_7service_FunctionArgsSpec *)Py_None); Py_INCREF(Py_None);
  p->result_spec = ((struct __pyx_obj_8thriftrw_4spec_7service_FunctionResultSpec *)Py_None); Py_INCREF(Py_None);
  p->surface = Py_None; Py_INCREF(Py_None);
  p->service = ((struct __pyx_obj_8thriftrw_4spec_7service_ServiceSpec *)Py_None); Py_INCREF(Py_None);
  return o;
}

static void __pyx_tp_dealloc_8thriftrw_4spec_7service_FunctionSpec(PyObject *o) {
  struct __pyx_obj_8thriftrw_4spec_7service_FunctionSpec *p = (struct __pyx_obj_8thriftrw_4spec_7service_FunctionSpec *)o;
  #if PY_VERSION_HEX >= 0x030400a1
  if (unlikely(Py_TYPE(o)->tp_finalize) && !_PyGC_FINALIZED(o)) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->name);
  Py_CLEAR(p->args_spec);
  Py_CLEAR(p->result_spec);
  Py_CLEAR(p->surface);
  Py_CLEAR(p->service);
  (*Py_TYPE(o)->tp_free)(o);
}

static int __pyx_tp_traverse_8thriftrw_4spec_7service_FunctionSpec(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_8thriftrw_4spec_7service_FunctionSpec *p = (struct __pyx_obj_8thriftrw_4spec_7service_FunctionSpec *)o;
  if (p->args_spec) {
    e = (*v)(((PyObject*)p->args_spec), a); if (e) return e;
  }
  if (p->result_spec) {
    e = (*v)(((PyObject*)p->result_spec), a); if (e) return e;
  }
  if (p->surface) {
    e = (*v)(p->surface, a); if (e) return e;
  }
  if (p->service) {
    e = (*v)(((PyObject*)p->service), a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_8thriftrw_4spec_7service_FunctionSpec(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_8thriftrw_4spec_7service_FunctionSpec *p = (struct __pyx_obj_8thriftrw_4spec_7service_FunctionSpec *)o;
  tmp = ((PyObject*)p->args_spec);
  p->args_spec = ((struct __pyx_obj_8thriftrw_4spec_7service_FunctionArgsSpec *)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->result_spec);
  p->result_spec = ((struct __pyx_obj_8thriftrw_4spec_7service_FunctionResultSpec *)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->surface);
  p->surface = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->service);
  p->service = ((struct __pyx_obj_8thriftrw_4spec_7service_ServiceSpec *)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyObject *__pyx_getprop_8thriftrw_4spec_7service_12FunctionSpec_name(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_8thriftrw_4spec_7service_12FunctionSpec_4name_1__get__(o);
}

static PyObject *__pyx_getprop_8thriftrw_4spec_7service_12FunctionSpec_args_spec(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_8thriftrw_4spec_7service_12FunctionSpec_9args_spec_1__get__(o);
}

static int __pyx_setprop_8thriftrw_4spec_7service_12FunctionSpec_args_spec(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_8thriftrw_4spec_7service_12FunctionSpec_9args_spec_3__set__(o, v);
  }
  else {
    return __pyx_pw_8thriftrw_4spec_7service_12FunctionSpec_9args_spec_5__del__(o);
  }
}

static PyObject *__pyx_getprop_8thriftrw_4spec_7service_12FunctionSpec_result_spec(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_8thriftrw_4spec_7service_12FunctionSpec_11result_spec_1__get__(o);
}

static int __pyx_setprop_8thriftrw_4spec_7service_12FunctionSpec_result_spec(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_8thriftrw_4spec_7service_12FunctionSpec_11result_spec_3__set__(o, v);
  }
  else {
    return __pyx_pw_8thriftrw_4spec_7service_12FunctionSpec_11result_spec_5__del__(o);
  }
}

static PyObject *__pyx_getprop_8thriftrw_4spec_7service_12FunctionSpec_oneway(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_8thriftrw_4spec_7service_12FunctionSpec_6oneway_1__get__(o);
}

static PyObject *__pyx_getprop_8thriftrw_4spec_7service_12FunctionSpec_linked(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_8thriftrw_4spec_7service_12FunctionSpec_6linked_1__get__(o);
}

static int __pyx_setprop_8thriftrw_4spec_7service_12FunctionSpec_linked(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_8thriftrw_4spec_7service_12FunctionSpec_6linked_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_8thriftrw_4spec_7service_12FunctionSpec_surface(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_8thriftrw_4spec_7service_12FunctionSpec_7surface_1__get__(o);
}

static int __pyx_setprop_8thriftrw_4spec_7service_12FunctionSpec_surface(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_8thriftrw_4spec_7service_12FunctionSpec_7surface_3__set__(o, v);
  }
  else {
    return __pyx_pw_8thriftrw_4spec_7service_12FunctionSpec_7surface_5__del__(o);
  }
}

static PyObject *__pyx_getprop_8thriftrw_4spec_7service_12FunctionSpec_service(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_8thriftrw_4spec_7service_12FunctionSpec_7service_1__get__(o);
}

static int __pyx_setprop_8thriftrw_4spec_7service_12FunctionSpec_service(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_8thriftrw_4spec_7service_12FunctionSpec_7service_3__set__(o, v);
  }
  else {
    return __pyx_pw_8thriftrw_4spec_7service_12FunctionSpec_7service_5__del__(o);
  }
}

static PyMethodDef __pyx_methods_8thriftrw_4spec_7service_FunctionSpec[] = {
  {"compile", (PyCFunction)__pyx_pw_8thriftrw_4spec_7service_12FunctionSpec_3compile, METH_VARARGS|METH_KEYWORDS, 0},
  {"link", (PyCFunction)__pyx_pw_8thriftrw_4spec_7service_12FunctionSpec_5link, METH_VARARGS|METH_KEYWORDS, 0},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_8thriftrw_4spec_7service_FunctionSpec[] = {
  {(char *)"name", __pyx_getprop_8thriftrw_4spec_7service_12FunctionSpec_name, 0, 0, 0},
  {(char *)"args_spec", __pyx_getprop_8thriftrw_4spec_7service_12FunctionSpec_args_spec, __pyx_setprop_8thriftrw_4spec_7service_12FunctionSpec_args_spec, 0, 0},
  {(char *)"result_spec", __pyx_getprop_8thriftrw_4spec_7service_12FunctionSpec_result_spec, __pyx_setprop_8thriftrw_4spec_7service_12FunctionSpec_result_spec, 0, 0},
  {(char *)"oneway", __pyx_getprop_8thriftrw_4spec_7service_12FunctionSpec_oneway, 0, 0, 0},
  {(char *)"linked", __pyx_getprop_8thriftrw_4spec_7service_12FunctionSpec_linked, __pyx_setprop_8thriftrw_4spec_7service_12FunctionSpec_linked, 0, 0},
  {(char *)"surface", __pyx_getprop_8thriftrw_4spec_7service_12FunctionSpec_surface, __pyx_setprop_8thriftrw_4spec_7service_12FunctionSpec_surface, 0, 0},
  {(char *)"service", __pyx_getprop_8thriftrw_4spec_7service_12FunctionSpec_service, __pyx_setprop_8thriftrw_4spec_7service_12FunctionSpec_service, 0, 0},
  {0, 0, 0, 0, 0}
};

static PyTypeObject __pyx_type_8thriftrw_4spec_7service_FunctionSpec = {
  PyVarObject_HEAD_INIT(0, 0)
  "thriftrw.spec.service.FunctionSpec", /*tp_name*/
  sizeof(struct __pyx_obj_8thriftrw_4spec_7service_FunctionSpec), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_8thriftrw_4spec_7service_FunctionSpec, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  __pyx_pw_8thriftrw_4spec_7service_12FunctionSpec_9__repr__, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  __pyx_pw_8thriftrw_4spec_7service_12FunctionSpec_7__str__, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  "Specification of a single function on a service.\n\n    .. py:attribute:: name\n\n        Name of the function.\n\n    .. py:attribute:: service\n\n        The :py:class:`ServiceSpec` that contains this function.\n        This value is available only after the spec has been linked.\n\n        .. versionadded:: 1.1\n\n    .. py:attribute:: args_spec\n\n        :py:class:`FunctionArgsSpec` specifying the arguments accepted by this\n        function as a struct.\n\n    .. py:attribute:: result_spec\n\n        :py:class:`FunctionResultSpec` specifying the output of this\n        function as a  union of the return type and the exceptions raised\n        by this function.\n\n        The return type of the function (if any) is a field in the union\n        with field ID 0 and name 'success'.\n\n        This value is None if the function is oneway.\n\n    .. py:attribute:: oneway\n\n        Whether this function is oneway or not.\n\n    The ``surface`` for a FunctionSpec is a :py:class:`ServiceFunction`\n    object. Unlike the ``surface`` for other specs, a FunctionSpec's surface\n    is attached to the service class as a class attribute.\n\n    .. versionchanged:: 0.3\n\n        Added the ``oneway`` attribute.\n    ", /*tp_doc*/
  __pyx_tp_traverse_8thriftrw_4spec_7service_FunctionSpec, /*tp_traverse*/
  __pyx_tp_clear_8thriftrw_4spec_7service_FunctionSpec, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_8thriftrw_4spec_7service_FunctionSpec, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_8thriftrw_4spec_7service_FunctionSpec, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  __pyx_pw_8thriftrw_4spec_7service_12FunctionSpec_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_8thriftrw_4spec_7service_FunctionSpec, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};
static struct __pyx_vtabstruct_8thriftrw_4spec_7service_ServiceSpec __pyx_vtable_8thriftrw_4spec_7service_ServiceSpec;

static PyObject *__pyx_tp_new_8thriftrw_4spec_7service_ServiceSpec(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  struct __pyx_obj_8thriftrw_4spec_7service_ServiceSpec *p;
  PyObject *o;
  if (likely((t->tp_flags & Py_TPFLAGS_IS_ABSTRACT) == 0)) {
    o = (*t->tp_alloc)(t, 0);
  } else {
    o = (PyObject *) PyBaseObject_Type.tp_new(t, __pyx_empty_tuple, 0);
  }
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_8thriftrw_4spec_7service_ServiceSpec *)o);
  p->__pyx_vtab = __pyx_vtabptr_8thriftrw_4spec_7service_ServiceSpec;
  p->name = ((PyObject*)Py_None); Py_INCREF(Py_None);
  p->functions = ((PyObject*)Py_None); Py_INCREF(Py_None);
  p->parent = Py_None; Py_INCREF(Py_None);
  p->_functions = ((PyObject*)Py_None); Py_INCREF(Py_None);
  p->surface = Py_None; Py_INCREF(Py_None);
  return o;
}

static void __pyx_tp_dealloc_8thriftrw_4spec_7service_ServiceSpec(PyObject *o) {
  struct __pyx_obj_8thriftrw_4spec_7service_ServiceSpec *p = (struct __pyx_obj_8thriftrw_4spec_7service_ServiceSpec *)o;
  #if PY_VERSION_HEX >= 0x030400a1
  if (unlikely(Py_TYPE(o)->tp_finalize) && !_PyGC_FINALIZED(o)) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->name);
  Py_CLEAR(p->functions);
  Py_CLEAR(p->parent);
  Py_CLEAR(p->_functions);
  Py_CLEAR(p->surface);
  (*Py_TYPE(o)->tp_free)(o);
}

static int __pyx_tp_traverse_8thriftrw_4spec_7service_ServiceSpec(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_8thriftrw_4spec_7service_ServiceSpec *p = (struct __pyx_obj_8thriftrw_4spec_7service_ServiceSpec *)o;
  if (p->functions) {
    e = (*v)(p->functions, a); if (e) return e;
  }
  if (p->parent) {
    e = (*v)(p->parent, a); if (e) return e;
  }
  if (p->_functions) {
    e = (*v)(p->_functions, a); if (e) return e;
  }
  if (p->surface) {
    e = (*v)(p->surface, a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_8thriftrw_4spec_7service_ServiceSpec(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_8thriftrw_4spec_7service_ServiceSpec *p = (struct __pyx_obj_8thriftrw_4spec_7service_ServiceSpec *)o;
  tmp = ((PyObject*)p->functions);
  p->functions = ((PyObject*)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->parent);
  p->parent = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->_functions);
  p->_functions = ((PyObject*)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->surface);
  p->surface = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyObject *__pyx_getprop_8thriftrw_4spec_7service_11ServiceSpec_name(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_8thriftrw_4spec_7service_11ServiceSpec_4name_1__get__(o);
}

static PyObject *__pyx_getprop_8thriftrw_4spec_7service_11ServiceSpec_functions(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_8thriftrw_4spec_7service_11ServiceSpec_9functions_1__get__(o);
}

static int __pyx_setprop_8thriftrw_4spec_7service_11ServiceSpec_functions(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_8thriftrw_4spec_7service_11ServiceSpec_9functions_3__set__(o, v);
  }
  else {
    return __pyx_pw_8thriftrw_4spec_7service_11ServiceSpec_9functions_5__del__(o);
  }
}

static PyObject *__pyx_getprop_8thriftrw_4spec_7service_11ServiceSpec_parent(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_8thriftrw_4spec_7service_11ServiceSpec_6parent_1__get__(o);
}

static int __pyx_setprop_8thriftrw_4spec_7service_11ServiceSpec_parent(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_8thriftrw_4spec_7service_11ServiceSpec_6parent_3__set__(o, v);
  }
  else {
    return __pyx_pw_8thriftrw_4spec_7service_11ServiceSpec_6parent_5__del__(o);
  }
}

static PyObject *__pyx_getprop_8thriftrw_4spec_7service_11ServiceSpec__functions(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_8thriftrw_4spec_7service_11ServiceSpec_10_functions_1__get__(o);
}

static int __pyx_setprop_8thriftrw_4spec_7service_11ServiceSpec__functions(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_8thriftrw_4spec_7service_11ServiceSpec_10_functions_3__set__(o, v);
  }
  else {
    return __pyx_pw_8thriftrw_4spec_7service_11ServiceSpec_10_functions_5__del__(o);
  }
}

static PyObject *__pyx_getprop_8thriftrw_4spec_7service_11ServiceSpec_linked(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_8thriftrw_4spec_7service_11ServiceSpec_6linked_1__get__(o);
}

static int __pyx_setprop_8thriftrw_4spec_7service_11ServiceSpec_linked(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_8thriftrw_4spec_7service_11ServiceSpec_6linked_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_8thriftrw_4spec_7service_11ServiceSpec_surface(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_8thriftrw_4spec_7service_11ServiceSpec_7surface_1__get__(o);
}

static int __pyx_setprop_8thriftrw_4spec_7service_11ServiceSpec_surface(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_8thriftrw_4spec_7service_11ServiceSpec_7surface_3__set__(o, v);
  }
  else {
    return __pyx_pw_8thriftrw_4spec_7service_11ServiceSpec_7surface_5__del__(o);
  }
}

static PyMethodDef __pyx_methods_8thriftrw_4spec_7service_ServiceSpec[] = {
  {"compile", (PyCFunction)__pyx_pw_8thriftrw_4spec_7service_11ServiceSpec_3compile, METH_O, 0},
  {"link", (PyCFunction)__pyx_pw_8thriftrw_4spec_7service_11ServiceSpec_5link, METH_O, 0},
  {"lookup", (PyCFunction)__pyx_pw_8thriftrw_4spec_7service_11ServiceSpec_7lookup, METH_O, __pyx_doc_8thriftrw_4spec_7service_11ServiceSpec_6lookup},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_8thriftrw_4spec_7service_ServiceSpec[] = {
  {(char *)"name", __pyx_getprop_8thriftrw_4spec_7service_11ServiceSpec_name, 0, 0, 0},
  {(char *)"functions", __pyx_getprop_8thriftrw_4spec_7service_11ServiceSpec_functions, __pyx_setprop_8thriftrw_4spec_7service_11ServiceSpec_functions, 0, 0},
  {(char *)"parent", __pyx_getprop_8thriftrw_4spec_7service_11ServiceSpec_parent, __pyx_setprop_8thriftrw_4spec_7service_11ServiceSpec_parent, 0, 0},
  {(char *)"_functions", __pyx_getprop_8thriftrw_4spec_7service_11ServiceSpec__functions, __pyx_setprop_8thriftrw_4spec_7service_11ServiceSpec__functions, 0, 0},
  {(char *)"linked", __pyx_getprop_8thriftrw_4spec_7service_11ServiceSpec_linked, __pyx_setprop_8thriftrw_4spec_7service_11ServiceSpec_linked, 0, 0},
  {(char *)"surface", __pyx_getprop_8thriftrw_4spec_7service_11ServiceSpec_surface, __pyx_setprop_8thriftrw_4spec_7service_11ServiceSpec_surface, 0, 0},
  {0, 0, 0, 0, 0}
};

static PyTypeObject __pyx_type_8thriftrw_4spec_7service_ServiceSpec = {
  PyVarObject_HEAD_INIT(0, 0)
  "thriftrw.spec.service.ServiceSpec", /*tp_name*/
  sizeof(struct __pyx_obj_8thriftrw_4spec_7service_ServiceSpec), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_8thriftrw_4spec_7service_ServiceSpec, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  __pyx_pw_8thriftrw_4spec_7service_11ServiceSpec_11__repr__, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  __pyx_pw_8thriftrw_4spec_7service_11ServiceSpec_9__str__, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  "Spec for a single service.\n\n    .. py:attribute:: name\n\n        Name of the service.\n\n    .. py:attribute:: functions\n\n        Collection of :py:class:`FunctionSpec` objects.\n\n    .. py:attribute:: parent\n\n        ``ServiceSpec`` of the parent service or None if this service does not\n        inherit any service.\n\n    The ``surface`` for a ``ServiceSpec`` is a class that has the following\n    attributes:\n\n    .. py:attribute:: service_spec\n\n        :py:class:`ServiceSpec` for this service.\n\n    And a reference to one :py:class:`ServiceFunction` object for each\n    function defined in the service.\n    ", /*tp_doc*/
  __pyx_tp_traverse_8thriftrw_4spec_7service_ServiceSpec, /*tp_traverse*/
  __pyx_tp_clear_8thriftrw_4spec_7service_ServiceSpec, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_8thriftrw_4spec_7service_ServiceSpec, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_8thriftrw_4spec_7service_ServiceSpec, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  __pyx_pw_8thriftrw_4spec_7service_11ServiceSpec_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_8thriftrw_4spec_7service_ServiceSpec, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};

static PyMethodDef __pyx_methods[] = {
  {0, 0, 0, 0}
};

#if PY_MAJOR_VERSION >= 3
static struct PyModuleDef __pyx_moduledef = {
  #if PY_VERSION_HEX < 0x03020000
    { PyObject_HEAD_INIT(NULL) NULL, 0, NULL },
  #else
    PyModuleDef_HEAD_INIT,
  #endif
    "service",
    0, /* m_doc */
    -1, /* m_size */
    __pyx_methods /* m_methods */,
    NULL, /* m_reload */
    NULL, /* m_traverse */
    NULL, /* m_clear */
    NULL /* m_free */
};
#endif

static __Pyx_StringTabEntry __pyx_string_tab[] = {
  {&__pyx_n_u_FunctionArgsSpec, __pyx_k_FunctionArgsSpec, sizeof(__pyx_k_FunctionArgsSpec), 0, 1, 0, 1},
  {&__pyx_n_u_FunctionResultSpec, __pyx_k_FunctionResultSpec, sizeof(__pyx_k_FunctionResultSpec), 0, 1, 0, 1},
  {&__pyx_n_u_FunctionSpec, __pyx_k_FunctionSpec, sizeof(__pyx_k_FunctionSpec), 0, 1, 0, 1},
  {&__pyx_kp_u_FunctionSpec_name_r_args_spec_r, __pyx_k_FunctionSpec_name_r_args_spec_r, sizeof(__pyx_k_FunctionSpec_name_r_args_spec_r), 0, 1, 0, 0},
  {&__pyx_kp_u_Function_s_s_cannot_be_defined_T, __pyx_k_Function_s_s_cannot_be_defined_T, sizeof(__pyx_k_Function_s_s_cannot_be_defined_T), 0, 1, 0, 0},
  {&__pyx_kp_u_Function_s_s_is_oneway_It_cannot, __pyx_k_Function_s_s_is_oneway_It_cannot, sizeof(__pyx_k_Function_s_s_is_oneway_It_cannot), 0, 1, 0, 0},
  {&__pyx_kp_u_Function_s_s_is_oneway_It_cannot_2, __pyx_k_Function_s_s_is_oneway_It_cannot_2, sizeof(__pyx_k_Function_s_s_is_oneway_It_cannot_2), 0, 1, 0, 0},
  {&__pyx_kp_s_Represents_a_single_function_on, __pyx_k_Represents_a_single_function_on, sizeof(__pyx_k_Represents_a_single_function_on), 0, 0, 1, 0},
  {&__pyx_n_s_ServiceFunction, __pyx_k_ServiceFunction, sizeof(__pyx_k_ServiceFunction), 0, 0, 1, 1},
  {&__pyx_n_u_ServiceFunction, __pyx_k_ServiceFunction, sizeof(__pyx_k_ServiceFunction), 0, 1, 0, 1},
  {&__pyx_n_u_ServiceSpec, __pyx_k_ServiceSpec, sizeof(__pyx_k_ServiceSpec), 0, 1, 0, 1},
  {&__pyx_kp_u_ServiceSpec_name_r_functions_r_p, __pyx_k_ServiceSpec_name_r_functions_r_p, sizeof(__pyx_k_ServiceSpec_name_r_functions_r_p), 0, 1, 0, 0},
  {&__pyx_n_s_ThriftCompilerError, __pyx_k_ThriftCompilerError, sizeof(__pyx_k_ThriftCompilerError), 0, 0, 1, 1},
  {&__pyx_n_s_UnknownExceptionError, __pyx_k_UnknownExceptionError, sizeof(__pyx_k_UnknownExceptionError), 0, 0, 1, 1},
  {&__pyx_n_s_all, __pyx_k_all, sizeof(__pyx_k_all), 0, 0, 1, 1},
  {&__pyx_n_s_allow_empty, __pyx_k_allow_empty, sizeof(__pyx_k_allow_empty), 0, 0, 1, 1},
  {&__pyx_n_s_args_spec, __pyx_k_args_spec, sizeof(__pyx_k_args_spec), 0, 0, 1, 1},
  {&__pyx_n_s_collections, __pyx_k_collections, sizeof(__pyx_k_collections), 0, 0, 1, 1},
  {&__pyx_n_s_compile, __pyx_k_compile, sizeof(__pyx_k_compile), 0, 0, 1, 1},
  {&__pyx_n_s_default_value, __pyx_k_default_value, sizeof(__pyx_k_default_value), 0, 0, 1, 1},
  {&__pyx_n_s_doc, __pyx_k_doc, sizeof(__pyx_k_doc), 0, 0, 1, 1},
  {&__pyx_n_s_encode, __pyx_k_encode, sizeof(__pyx_k_encode), 0, 0, 1, 1},
  {&__pyx_n_s_errors, __pyx_k_errors, sizeof(__pyx_k_errors), 0, 0, 1, 1},
  {&__pyx_n_s_exceptions, __pyx_k_exceptions, sizeof(__pyx_k_exceptions), 0, 0, 1, 1},
  {&__pyx_n_s_field, __pyx_k_field, sizeof(__pyx_k_field), 0, 0, 1, 1},
  {&__pyx_n_s_fields, __pyx_k_fields, sizeof(__pyx_k_fields), 0, 0, 1, 1},
  {&__pyx_n_s_from_wire, __pyx_k_from_wire, sizeof(__pyx_k_from_wire), 0, 0, 1, 1},
  {&__pyx_n_s_func, __pyx_k_func, sizeof(__pyx_k_func), 0, 0, 1, 1},
  {&__pyx_n_s_function, __pyx_k_function, sizeof(__pyx_k_function), 0, 0, 1, 1},
  {&__pyx_n_s_function_name, __pyx_k_function_name, sizeof(__pyx_k_function_name), 0, 0, 1, 1},
  {&__pyx_n_s_functions, __pyx_k_functions, sizeof(__pyx_k_functions), 0, 0, 1, 1},
  {&__pyx_n_s_get, __pyx_k_get, sizeof(__pyx_k_get), 0, 0, 1, 1},
  {&__pyx_n_s_id, __pyx_k_id, sizeof(__pyx_k_id), 0, 0, 1, 1},
  {&__pyx_n_s_import, __pyx_k_import, sizeof(__pyx_k_import), 0, 0, 1, 1},
  {&__pyx_n_s_init, __pyx_k_init, sizeof(__pyx_k_init), 0, 0, 1, 1},
  {&__pyx_n_s_lineno, __pyx_k_lineno, sizeof(__pyx_k_lineno), 0, 0, 1, 1},
  {&__pyx_n_s_link, __pyx_k_link, sizeof(__pyx_k_link), 0, 0, 1, 1},
  {&__pyx_n_s_main, __pyx_k_main, sizeof(__pyx_k_main), 0, 0, 1, 1},
  {&__pyx_n_s_metaclass, __pyx_k_metaclass, sizeof(__pyx_k_metaclass), 0, 0, 1, 1},
  {&__pyx_n_s_module, __pyx_k_module, sizeof(__pyx_k_module), 0, 0, 1, 1},
  {&__pyx_n_s_name, __pyx_k_name, sizeof(__pyx_k_name), 0, 0, 1, 1},
  {&__pyx_kp_u_name_request_response_spec, __pyx_k_name_request_response_spec, sizeof(__pyx_k_name_request_response_spec), 0, 1, 0, 0},
  {&__pyx_n_s_namedtuple, __pyx_k_namedtuple, sizeof(__pyx_k_namedtuple), 0, 0, 1, 1},
  {&__pyx_n_s_object, __pyx_k_object, sizeof(__pyx_k_object), 0, 0, 1, 1},
  {&__pyx_n_s_oneway, __pyx_k_oneway, sizeof(__pyx_k_oneway), 0, 0, 1, 1},
  {&__pyx_n_s_parameters, __pyx_k_parameters, sizeof(__pyx_k_parameters), 0, 0, 1, 1},
  {&__pyx_n_s_params, __pyx_k_params, sizeof(__pyx_k_params), 0, 0, 1, 1},
  {&__pyx_n_s_parent, __pyx_k_parent, sizeof(__pyx_k_parent), 0, 0, 1, 1},
  {&__pyx_n_s_parent_cls, __pyx_k_parent_cls, sizeof(__pyx_k_parent_cls), 0, 0, 1, 1},
  {&__pyx_n_s_prepare, __pyx_k_prepare, sizeof(__pyx_k_prepare), 0, 0, 1, 1},
  {&__pyx_kp_s_private_var_folders_8z_qdzjsr3n, __pyx_k_private_var_folders_8z_qdzjsr3n, sizeof(__pyx_k_private_var_folders_8z_qdzjsr3n), 0, 0, 1, 0},
  {&__pyx_n_s_pyx_vtable, __pyx_k_pyx_vtable, sizeof(__pyx_k_pyx_vtable), 0, 0, 1, 1},
  {&__pyx_n_s_qualname, __pyx_k_qualname, sizeof(__pyx_k_qualname), 0, 0, 1, 1},
  {&__pyx_n_s_require_requiredness, __pyx_k_require_requiredness, sizeof(__pyx_k_require_requiredness), 0, 0, 1, 1},
  {&__pyx_n_s_required, __pyx_k_required, sizeof(__pyx_k_required), 0, 0, 1, 1},
  {&__pyx_n_s_resolve_service_spec, __pyx_k_resolve_service_spec, sizeof(__pyx_k_resolve_service_spec), 0, 0, 1, 1},
  {&__pyx_n_s_result_spec, __pyx_k_result_spec, sizeof(__pyx_k_result_spec), 0, 0, 1, 1},
  {&__pyx_n_s_result_type, __pyx_k_result_type, sizeof(__pyx_k_result_type), 0, 0, 1, 1},
  {&__pyx_n_s_return_spec, __pyx_k_return_spec, sizeof(__pyx_k_return_spec), 0, 0, 1, 1},
  {&__pyx_n_s_return_type, __pyx_k_return_type, sizeof(__pyx_k_return_type), 0, 0, 1, 1},
  {&__pyx_kp_u_s__s_request, __pyx_k_s__s_request, sizeof(__pyx_k_s__s_request), 0, 1, 0, 0},
  {&__pyx_kp_u_s__s_response, __pyx_k_s__s_response, sizeof(__pyx_k_s__s_response), 0, 1, 0, 0},
  {&__pyx_kp_u_s_received_an_unrecognized_exce, __pyx_k_s_received_an_unrecognized_exce, sizeof(__pyx_k_s_received_an_unrecognized_exce), 0, 1, 0, 0},
  {&__pyx_n_s_scope, __pyx_k_scope, sizeof(__pyx_k_scope), 0, 0, 1, 1},
  {&__pyx_n_s_service, __pyx_k_service, sizeof(__pyx_k_service), 0, 0, 1, 1},
  {&__pyx_n_s_service_cls, __pyx_k_service_cls, sizeof(__pyx_k_service_cls), 0, 0, 1, 1},
  {&__pyx_n_s_service_dct, __pyx_k_service_dct, sizeof(__pyx_k_service_dct), 0, 0, 1, 1},
  {&__pyx_n_s_service_name, __pyx_k_service_name, sizeof(__pyx_k_service_name), 0, 0, 1, 1},
  {&__pyx_n_s_service_spec, __pyx_k_service_spec, sizeof(__pyx_k_service_spec), 0, 0, 1, 1},
  {&__pyx_n_u_service_spec, __pyx_k_service_spec, sizeof(__pyx_k_service_spec), 0, 1, 0, 1},
  {&__pyx_n_u_slots, __pyx_k_slots, sizeof(__pyx_k_slots), 0, 1, 0, 1},
  {&__pyx_n_s_spec, __pyx_k_spec, sizeof(__pyx_k_spec), 0, 0, 1, 1},
  {&__pyx_n_s_spec_mapper, __pyx_k_spec_mapper, sizeof(__pyx_k_spec_mapper), 0, 0, 1, 1},
  {&__pyx_n_s_struct_name, __pyx_k_struct_name, sizeof(__pyx_k_struct_name), 0, 0, 1, 1},
  {&__pyx_n_u_success, __pyx_k_success, sizeof(__pyx_k_success), 0, 1, 0, 1},
  {&__pyx_n_s_super, __pyx_k_super, sizeof(__pyx_k_super), 0, 0, 1, 1},
  {&__pyx_n_s_surface, __pyx_k_surface, sizeof(__pyx_k_surface), 0, 0, 1, 1},
  {&__pyx_n_s_test, __pyx_k_test, sizeof(__pyx_k_test), 0, 0, 1, 1},
  {&__pyx_n_s_thriftrw_spec_service, __pyx_k_thriftrw_spec_service, sizeof(__pyx_k_thriftrw_spec_service), 0, 0, 1, 1},
  {&__pyx_n_s_type_spec_or_ref, __pyx_k_type_spec_or_ref, sizeof(__pyx_k_type_spec_or_ref), 0, 0, 1, 1},
  {&__pyx_kp_u_utf_8, __pyx_k_utf_8, sizeof(__pyx_k_utf_8), 0, 1, 0, 0},
  {0, 0, 0, 0, 0, 0, 0}
};
static int __Pyx_InitCachedBuiltins(void) {
  __pyx_builtin_super = __Pyx_GetBuiltinName(__pyx_n_s_super); if (!__pyx_builtin_super) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 73; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_builtin_object = __Pyx_GetBuiltinName(__pyx_n_s_object); if (!__pyx_builtin_object) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 481; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  return 0;
  __pyx_L1_error:;
  return -1;
}

static int __Pyx_InitCachedConstants(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_InitCachedConstants", 0);

  /* "thriftrw/spec/service.pyx":420
 *                 name = f.name
 *                 if isinstance(name, unicode):
 *                     name = name.encode('utf-8')             # <<<<<<<<<<<<<<
 *                 self._functions[name] = f
 * 
 */
  __pyx_tuple_ = PyTuple_Pack(1, __pyx_kp_u_utf_8); if (unlikely(!__pyx_tuple_)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 420; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple_);
  __Pyx_GIVEREF(__pyx_tuple_);

  /* "thriftrw/spec/service.pyx":433
 *         """
 *         if isinstance(name, unicode):
 *             name = name.encode('utf-8')             # <<<<<<<<<<<<<<
 *         return self._functions.get(name, None)
 * 
 */
  __pyx_tuple__2 = PyTuple_Pack(1, __pyx_kp_u_utf_8); if (unlikely(!__pyx_tuple__2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 433; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__2);
  __Pyx_GIVEREF(__pyx_tuple__2);

  /* "thriftrw/spec/service.pyx":446
 * 
 * class ServiceFunction(
 *     namedtuple('ServiceFunction', 'name request response spec')             # <<<<<<<<<<<<<<
 * ):
 *     """Represents a single function on a service.
 */
  __pyx_tuple__3 = PyTuple_Pack(2, __pyx_n_u_ServiceFunction, __pyx_kp_u_name_request_response_spec); if (unlikely(!__pyx_tuple__3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 446; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__3);
  __Pyx_GIVEREF(__pyx_tuple__3);

  /* "thriftrw/spec/service.pyx":473
 * 
 * 
 * def service_cls(service_spec, scope):             # <<<<<<<<<<<<<<
 *     """Generates a class from the given service spec.
 * 
 */
  __pyx_tuple__4 = PyTuple_Pack(5, __pyx_n_s_service_spec, __pyx_n_s_scope, __pyx_n_s_parent_cls, __pyx_n_s_service_dct, __pyx_n_s_function); if (unlikely(!__pyx_tuple__4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 473; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__4);
  __Pyx_GIVEREF(__pyx_tuple__4);
  __pyx_codeobj__5 = (PyObject*)__Pyx_PyCode_New(2, 0, 5, 0, 0, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__4, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_private_var_folders_8z_qdzjsr3n, __pyx_n_s_service_cls, 473, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 473; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_RefNannyFinishContext();
  return -1;
}

static int __Pyx_InitGlobals(void) {
  if (__Pyx_InitStrings(__pyx_string_tab) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;};
  __pyx_int_0 = PyInt_FromLong(0); if (unlikely(!__pyx_int_0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  return 0;
  __pyx_L1_error:;
  return -1;
}

#if PY_MAJOR_VERSION < 3
PyMODINIT_FUNC initservice(void); /*proto*/
PyMODINIT_FUNC initservice(void)
#else
PyMODINIT_FUNC PyInit_service(void); /*proto*/
PyMODINIT_FUNC PyInit_service(void)
#endif
{
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannyDeclarations
  #if CYTHON_REFNANNY
  __Pyx_RefNanny = __Pyx_RefNannyImportAPI("refnanny");
  if (!__Pyx_RefNanny) {
      PyErr_Clear();
      __Pyx_RefNanny = __Pyx_RefNannyImportAPI("Cython.Runtime.refnanny");
      if (!__Pyx_RefNanny)
          Py_FatalError("failed to import 'refnanny' module");
  }
  #endif
  __Pyx_RefNannySetupContext("PyMODINIT_FUNC PyInit_service(void)", 0);
  if (__Pyx_check_binary_version() < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_empty_tuple = PyTuple_New(0); if (unlikely(!__pyx_empty_tuple)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_empty_bytes = PyBytes_FromStringAndSize("", 0); if (unlikely(!__pyx_empty_bytes)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  #ifdef __Pyx_CyFunction_USED
  if (__pyx_CyFunction_init() < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  #endif
  #ifdef __Pyx_FusedFunction_USED
  if (__pyx_FusedFunction_init() < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  #endif
  #ifdef __Pyx_Coroutine_USED
  if (__pyx_Coroutine_init() < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  #endif
  #ifdef __Pyx_Generator_USED
  if (__pyx_Generator_init() < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  #endif
  #ifdef __Pyx_StopAsyncIteration_USED
  if (__pyx_StopAsyncIteration_init() < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  #endif
  /*--- Library function declarations ---*/
  /*--- Threads initialization code ---*/
  #if defined(__PYX_FORCE_INIT_THREADS) && __PYX_FORCE_INIT_THREADS
  #ifdef WITH_THREAD /* Python build with threading support? */
  PyEval_InitThreads();
  #endif
  #endif
  /*--- Module creation code ---*/
  #if PY_MAJOR_VERSION < 3
  __pyx_m = Py_InitModule4("service", __pyx_methods, 0, 0, PYTHON_API_VERSION); Py_XINCREF(__pyx_m);
  #else
  __pyx_m = PyModule_Create(&__pyx_moduledef);
  #endif
  if (unlikely(!__pyx_m)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_d = PyModule_GetDict(__pyx_m); if (unlikely(!__pyx_d)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  Py_INCREF(__pyx_d);
  __pyx_b = PyImport_AddModule(__Pyx_BUILTIN_MODULE_NAME); if (unlikely(!__pyx_b)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  #if CYTHON_COMPILING_IN_PYPY
  Py_INCREF(__pyx_b);
  #endif
  if (PyObject_SetAttrString(__pyx_m, "__builtins__", __pyx_b) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;};
  /*--- Initialize various global constants etc. ---*/
  if (__Pyx_InitGlobals() < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  #if PY_MAJOR_VERSION < 3 && (__PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT)
  if (__Pyx_init_sys_getdefaultencoding_params() < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  #endif
  if (__pyx_module_is_main_thriftrw__spec__service) {
    if (PyObject_SetAttrString(__pyx_m, "__name__", __pyx_n_s_main) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  #if PY_MAJOR_VERSION >= 3
  {
    PyObject *modules = PyImport_GetModuleDict(); if (unlikely(!modules)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    if (!PyDict_GetItemString(modules, "thriftrw.spec.service")) {
      if (unlikely(PyDict_SetItemString(modules, "thriftrw.spec.service", __pyx_m) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
  }
  #endif
  /*--- Builtin init code ---*/
  if (__Pyx_InitCachedBuiltins() < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  /*--- Constants init code ---*/
  if (__Pyx_InitCachedConstants() < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  /*--- Global init code ---*/
  /*--- Variable export code ---*/
  /*--- Function export code ---*/
  /*--- Type init code ---*/
  __pyx_ptype_8thriftrw_4spec_6struct_StructTypeSpec = __Pyx_ImportType("thriftrw.spec.struct", "StructTypeSpec", sizeof(struct __pyx_obj_8thriftrw_4spec_6struct_StructTypeSpec), 1); if (unlikely(!__pyx_ptype_8thriftrw_4spec_6struct_StructTypeSpec)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_vtabptr_8thriftrw_4spec_6struct_StructTypeSpec = (struct __pyx_vtabstruct_8thriftrw_4spec_6struct_StructTypeSpec*)__Pyx_GetVtable(__pyx_ptype_8thriftrw_4spec_6struct_StructTypeSpec->tp_dict); if (unlikely(!__pyx_vtabptr_8thriftrw_4spec_6struct_StructTypeSpec)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_vtabptr_8thriftrw_4spec_7service_FunctionArgsSpec = &__pyx_vtable_8thriftrw_4spec_7service_FunctionArgsSpec;
  __pyx_vtable_8thriftrw_4spec_7service_FunctionArgsSpec.__pyx_base = *__pyx_vtabptr_8thriftrw_4spec_6struct_StructTypeSpec;
  __pyx_type_8thriftrw_4spec_7service_FunctionArgsSpec.tp_base = __pyx_ptype_8thriftrw_4spec_6struct_StructTypeSpec;
  if (PyType_Ready(&__pyx_type_8thriftrw_4spec_7service_FunctionArgsSpec) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 44; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_type_8thriftrw_4spec_7service_FunctionArgsSpec.tp_print = 0;
  if (__Pyx_SetVtable(__pyx_type_8thriftrw_4spec_7service_FunctionArgsSpec.tp_dict, __pyx_vtabptr_8thriftrw_4spec_7service_FunctionArgsSpec) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 44; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (PyObject_SetAttrString(__pyx_m, "FunctionArgsSpec", (PyObject *)&__pyx_type_8thriftrw_4spec_7service_FunctionArgsSpec) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 44; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_ptype_8thriftrw_4spec_7service_FunctionArgsSpec = &__pyx_type_8thriftrw_4spec_7service_FunctionArgsSpec;
  __pyx_ptype_8thriftrw_4spec_5union_UnionTypeSpec = __Pyx_ImportType("thriftrw.spec.union", "UnionTypeSpec", sizeof(struct __pyx_obj_8thriftrw_4spec_5union_UnionTypeSpec), 1); if (unlikely(!__pyx_ptype_8thriftrw_4spec_5union_UnionTypeSpec)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_vtabptr_8thriftrw_4spec_5union_UnionTypeSpec = (struct __pyx_vtabstruct_8thriftrw_4spec_5union_UnionTypeSpec*)__Pyx_GetVtable(__pyx_ptype_8thriftrw_4spec_5union_UnionTypeSpec->tp_dict); if (unlikely(!__pyx_vtabptr_8thriftrw_4spec_5union_UnionTypeSpec)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_vtabptr_8thriftrw_4spec_7service_FunctionResultSpec = &__pyx_vtable_8thriftrw_4spec_7service_FunctionResultSpec;
  __pyx_vtable_8thriftrw_4spec_7service_FunctionResultSpec.__pyx_base = *__pyx_vtabptr_8thriftrw_4spec_5union_UnionTypeSpec;
  __pyx_vtable_8thriftrw_4spec_7service_FunctionResultSpec.__pyx_base.__pyx_base.from_wire = (PyObject *(*)(struct __pyx_obj_8thriftrw_4spec_4base_TypeSpec *, struct __pyx_obj_8thriftrw_4wire_5value_Value *, int __pyx_skip_dispatch))__pyx_f_8thriftrw_4spec_7service_18FunctionResultSpec_from_wire;
  __pyx_type_8thriftrw_4spec_7service_FunctionResultSpec.tp_base = __pyx_ptype_8thriftrw_4spec_5union_UnionTypeSpec;
  if (PyType_Ready(&__pyx_type_8thriftrw_4spec_7service_FunctionResultSpec) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 112; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_type_8thriftrw_4spec_7service_FunctionResultSpec.tp_print = 0;
  if (__Pyx_SetVtable(__pyx_type_8thriftrw_4spec_7service_FunctionResultSpec.tp_dict, __pyx_vtabptr_8thriftrw_4spec_7service_FunctionResultSpec) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 112; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (PyObject_SetAttrString(__pyx_m, "FunctionResultSpec", (PyObject *)&__pyx_type_8thriftrw_4spec_7service_FunctionResultSpec) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 112; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_ptype_8thriftrw_4spec_7service_FunctionResultSpec = &__pyx_type_8thriftrw_4spec_7service_FunctionResultSpec;
  __pyx_vtabptr_8thriftrw_4spec_7service_FunctionSpec = &__pyx_vtable_8thriftrw_4spec_7service_FunctionSpec;
  __pyx_vtable_8thriftrw_4spec_7service_FunctionSpec.link = (struct __pyx_obj_8thriftrw_4spec_7service_FunctionSpec *(*)(struct __pyx_obj_8thriftrw_4spec_7service_FunctionSpec *, PyObject *, struct __pyx_obj_8thriftrw_4spec_7service_ServiceSpec *, int __pyx_skip_dispatch))__pyx_f_8thriftrw_4spec_7service_12FunctionSpec_link;
  if (PyType_Ready(&__pyx_type_8thriftrw_4spec_7service_FunctionSpec) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 236; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_type_8thriftrw_4spec_7service_FunctionSpec.tp_print = 0;
  if (__Pyx_SetVtable(__pyx_type_8thriftrw_4spec_7service_FunctionSpec.tp_dict, __pyx_vtabptr_8thriftrw_4spec_7service_FunctionSpec) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 236; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (PyObject_SetAttrString(__pyx_m, "FunctionSpec", (PyObject *)&__pyx_type_8thriftrw_4spec_7service_FunctionSpec) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 236; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_ptype_8thriftrw_4spec_7service_FunctionSpec = &__pyx_type_8thriftrw_4spec_7service_FunctionSpec;
  __pyx_vtabptr_8thriftrw_4spec_7service_ServiceSpec = &__pyx_vtable_8thriftrw_4spec_7service_ServiceSpec;
  __pyx_vtable_8thriftrw_4spec_7service_ServiceSpec.link = (struct __pyx_obj_8thriftrw_4spec_7service_ServiceSpec *(*)(struct __pyx_obj_8thriftrw_4spec_7service_ServiceSpec *, PyObject *, int __pyx_skip_dispatch))__pyx_f_8thriftrw_4spec_7service_11ServiceSpec_link;
  if (PyType_Ready(&__pyx_type_8thriftrw_4spec_7service_ServiceSpec) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 346; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_type_8thriftrw_4spec_7service_ServiceSpec.tp_print = 0;
  if (__Pyx_SetVtable(__pyx_type_8thriftrw_4spec_7service_ServiceSpec.tp_dict, __pyx_vtabptr_8thriftrw_4spec_7service_ServiceSpec) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 346; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (PyObject_SetAttrString(__pyx_m, "ServiceSpec", (PyObject *)&__pyx_type_8thriftrw_4spec_7service_ServiceSpec) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 346; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_ptype_8thriftrw_4spec_7service_ServiceSpec = &__pyx_type_8thriftrw_4spec_7service_ServiceSpec;
  /*--- Type import code ---*/
  __pyx_ptype_8thriftrw_4wire_5value_ValueVisitor = __Pyx_ImportType("thriftrw.wire.value", "ValueVisitor", sizeof(struct __pyx_obj_8thriftrw_4wire_5value_ValueVisitor), 1); if (unlikely(!__pyx_ptype_8thriftrw_4wire_5value_ValueVisitor)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 31; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_vtabptr_8thriftrw_4wire_5value_ValueVisitor = (struct __pyx_vtabstruct_8thriftrw_4wire_5value_ValueVisitor*)__Pyx_GetVtable(__pyx_ptype_8thriftrw_4wire_5value_ValueVisitor->tp_dict); if (unlikely(!__pyx_vtabptr_8thriftrw_4wire_5value_ValueVisitor)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 31; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_ptype_8thriftrw_4wire_5value__ValueVisitorArgs = __Pyx_ImportType("thriftrw.wire.value", "_ValueVisitorArgs", sizeof(struct __pyx_obj_8thriftrw_4wire_5value__ValueVisitorArgs), 1); if (unlikely(!__pyx_ptype_8thriftrw_4wire_5value__ValueVisitorArgs)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 56; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_vtabptr_8thriftrw_4wire_5value__ValueVisitorArgs = (struct __pyx_vtabstruct_8thriftrw_4wire_5value__ValueVisitorArgs*)__Pyx_GetVtable(__pyx_ptype_8thriftrw_4wire_5value__ValueVisitorArgs->tp_dict); if (unlikely(!__pyx_vtabptr_8thriftrw_4wire_5value__ValueVisitorArgs)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 56; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_ptype_8thriftrw_4wire_5value_Value = __Pyx_ImportType("thriftrw.wire.value", "Value", sizeof(struct __pyx_obj_8thriftrw_4wire_5value_Value), 1); if (unlikely(!__pyx_ptype_8thriftrw_4wire_5value_Value)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 60; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_vtabptr_8thriftrw_4wire_5value_Value = (struct __pyx_vtabstruct_8thriftrw_4wire_5value_Value*)__Pyx_GetVtable(__pyx_ptype_8thriftrw_4wire_5value_Value->tp_dict); if (unlikely(!__pyx_vtabptr_8thriftrw_4wire_5value_Value)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 60; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_ptype_8thriftrw_4wire_5value_BoolValue = __Pyx_ImportType("thriftrw.wire.value", "BoolValue", sizeof(struct __pyx_obj_8thriftrw_4wire_5value_BoolValue), 1); if (unlikely(!__pyx_ptype_8thriftrw_4wire_5value_BoolValue)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 65; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_vtabptr_8thriftrw_4wire_5value_BoolValue = (struct __pyx_vtabstruct_8thriftrw_4wire_5value_BoolValue*)__Pyx_GetVtable(__pyx_ptype_8thriftrw_4wire_5value_BoolValue->tp_dict); if (unlikely(!__pyx_vtabptr_8thriftrw_4wire_5value_BoolValue)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 65; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_ptype_8thriftrw_4wire_5value_ByteValue = __Pyx_ImportType("thriftrw.wire.value", "ByteValue", sizeof(struct __pyx_obj_8thriftrw_4wire_5value_ByteValue), 1); if (unlikely(!__pyx_ptype_8thriftrw_4wire_5value_ByteValue)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 69; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_vtabptr_8thriftrw_4wire_5value_ByteValue = (struct __pyx_vtabstruct_8thriftrw_4wire_5value_ByteValue*)__Pyx_GetVtable(__pyx_ptype_8thriftrw_4wire_5value_ByteValue->tp_dict); if (unlikely(!__pyx_vtabptr_8thriftrw_4wire_5value_ByteValue)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 69; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_ptype_8thriftrw_4wire_5value_DoubleValue = __Pyx_ImportType("thriftrw.wire.value", "DoubleValue", sizeof(struct __pyx_obj_8thriftrw_4wire_5value_DoubleValue), 1); if (unlikely(!__pyx_ptype_8thriftrw_4wire_5value_DoubleValue)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 73; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_vtabptr_8thriftrw_4wire_5value_DoubleValue = (struct __pyx_vtabstruct_8thriftrw_4wire_5value_DoubleValue*)__Pyx_GetVtable(__pyx_ptype_8thriftrw_4wire_5value_DoubleValue->tp_dict); if (unlikely(!__pyx_vtabptr_8thriftrw_4wire_5value_DoubleValue)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 73; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_ptype_8thriftrw_4wire_5value_I16Value = __Pyx_ImportType("thriftrw.wire.value", "I16Value", sizeof(struct __pyx_obj_8thriftrw_4wire_5value_I16Value), 1); if (unlikely(!__pyx_ptype_8thriftrw_4wire_5value_I16Value)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 77; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_vtabptr_8thriftrw_4wire_5value_I16Value = (struct __pyx_vtabstruct_8thriftrw_4wire_5value_I16Value*)__Pyx_GetVtable(__pyx_ptype_8thriftrw_4wire_5value_I16Value->tp_dict); if (unlikely(!__pyx_vtabptr_8thriftrw_4wire_5value_I16Value)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 77; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_ptype_8thriftrw_4wire_5value_I32Value = __Pyx_ImportType("thriftrw.wire.value", "I32Value", sizeof(struct __pyx_obj_8thriftrw_4wire_5value_I32Value), 1); if (unlikely(!__pyx_ptype_8thriftrw_4wire_5value_I32Value)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 81; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_vtabptr_8thriftrw_4wire_5value_I32Value = (struct __pyx_vtabstruct_8thriftrw_4wire_5value_I32Value*)__Pyx_GetVtable(__pyx_ptype_8thriftrw_4wire_5value_I32Value->tp_dict); if (unlikely(!__pyx_vtabptr_8thriftrw_4wire_5value_I32Value)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 81; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_ptype_8thriftrw_4wire_5value_I64Value = __Pyx_ImportType("thriftrw.wire.value", "I64Value", sizeof(struct __pyx_obj_8thriftrw_4wire_5value_I64Value), 1); if (unlikely(!__pyx_ptype_8thriftrw_4wire_5value_I64Value)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 85; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_vtabptr_8thriftrw_4wire_5value_I64Value = (struct __pyx_vtabstruct_8thriftrw_4wire_5value_I64Value*)__Pyx_GetVtable(__pyx_ptype_8thriftrw_4wire_5value_I64Value->tp_dict); if (unlikely(!__pyx_vtabptr_8thriftrw_4wire_5value_I64Value)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 85; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_ptype_8thriftrw_4wire_5value_BinaryValue = __Pyx_ImportType("thriftrw.wire.value", "BinaryValue", sizeof(struct __pyx_obj_8thriftrw_4wire_5value_BinaryValue), 1); if (unlikely(!__pyx_ptype_8thriftrw_4wire_5value_BinaryValue)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 89; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_vtabptr_8thriftrw_4wire_5value_BinaryValue = (struct __pyx_vtabstruct_8thriftrw_4wire_5value_BinaryValue*)__Pyx_GetVtable(__pyx_ptype_8thriftrw_4wire_5value_BinaryValue->tp_dict); if (unlikely(!__pyx_vtabptr_8thriftrw_4wire_5value_BinaryValue)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 89; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_ptype_8thriftrw_4wire_5value_FieldValue = __Pyx_ImportType("thriftrw.wire.value", "FieldValue", sizeof(struct __pyx_obj_8thriftrw_4wire_5value_FieldValue), 1); if (unlikely(!__pyx_ptype_8thriftrw_4wire_5value_FieldValue)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 94; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_ptype_8thriftrw_4wire_5value_StructValue = __Pyx_ImportType("thriftrw.wire.value", "StructValue", sizeof(struct __pyx_obj_8thriftrw_4wire_5value_StructValue), 1); if (unlikely(!__pyx_ptype_8thriftrw_4wire_5value_StructValue)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 100; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_vtabptr_8thriftrw_4wire_5value_StructValue = (struct __pyx_vtabstruct_8thriftrw_4wire_5value_StructValue*)__Pyx_GetVtable(__pyx_ptype_8thriftrw_4wire_5value_StructValue->tp_dict); if (unlikely(!__pyx_vtabptr_8thriftrw_4wire_5value_StructValue)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 100; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_ptype_8thriftrw_4wire_5value_MapItem = __Pyx_ImportType("thriftrw.wire.value", "MapItem", sizeof(struct __pyx_obj_8thriftrw_4wire_5value_MapItem), 1); if (unlikely(!__pyx_ptype_8thriftrw_4wire_5value_MapItem)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 105; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_ptype_8thriftrw_4wire_5value_MapValue = __Pyx_ImportType("thriftrw.wire.value", "MapValue", sizeof(struct __pyx_obj_8thriftrw_4wire_5value_MapValue), 1); if (unlikely(!__pyx_ptype_8thriftrw_4wire_5value_MapValue)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 110; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_vtabptr_8thriftrw_4wire_5value_MapValue = (struct __pyx_vtabstruct_8thriftrw_4wire_5value_MapValue*)__Pyx_GetVtable(__pyx_ptype_8thriftrw_4wire_5value_MapValue->tp_dict); if (unlikely(!__pyx_vtabptr_8thriftrw_4wire_5value_MapValue)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 110; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_ptype_8thriftrw_4wire_5value_SetValue = __Pyx_ImportType("thriftrw.wire.value", "SetValue", sizeof(struct __pyx_obj_8thriftrw_4wire_5value_SetValue), 1); if (unlikely(!__pyx_ptype_8thriftrw_4wire_5value_SetValue)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 116; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_vtabptr_8thriftrw_4wire_5value_SetValue = (struct __pyx_vtabstruct_8thriftrw_4wire_5value_SetValue*)__Pyx_GetVtable(__pyx_ptype_8thriftrw_4wire_5value_SetValue->tp_dict); if (unlikely(!__pyx_vtabptr_8thriftrw_4wire_5value_SetValue)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 116; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_ptype_8thriftrw_4wire_5value_ListValue = __Pyx_ImportType("thriftrw.wire.value", "ListValue", sizeof(struct __pyx_obj_8thriftrw_4wire_5value_ListValue), 1); if (unlikely(!__pyx_ptype_8thriftrw_4wire_5value_ListValue)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 121; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_vtabptr_8thriftrw_4wire_5value_ListValue = (struct __pyx_vtabstruct_8thriftrw_4wire_5value_ListValue*)__Pyx_GetVtable(__pyx_ptype_8thriftrw_4wire_5value_ListValue->tp_dict); if (unlikely(!__pyx_vtabptr_8thriftrw_4wire_5value_ListValue)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 121; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_ptype_8thriftrw_4spec_4base_TypeSpec = __Pyx_ImportType("thriftrw.spec.base", "TypeSpec", sizeof(struct __pyx_obj_8thriftrw_4spec_4base_TypeSpec), 1); if (unlikely(!__pyx_ptype_8thriftrw_4spec_4base_TypeSpec)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 26; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_vtabptr_8thriftrw_4spec_4base_TypeSpec = (struct __pyx_vtabstruct_8thriftrw_4spec_4base_TypeSpec*)__Pyx_GetVtable(__pyx_ptype_8thriftrw_4spec_4base_TypeSpec->tp_dict); if (unlikely(!__pyx_vtabptr_8thriftrw_4spec_4base_TypeSpec)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 26; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_ptype_8thriftrw_4spec_5field_FieldSpec = __Pyx_ImportType("thriftrw.spec.field", "FieldSpec", sizeof(struct __pyx_obj_8thriftrw_4spec_5field_FieldSpec), 1); if (unlikely(!__pyx_ptype_8thriftrw_4spec_5field_FieldSpec)) {__pyx_filename = __pyx_f[4]; __pyx_lineno = 28; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  /*--- Variable import code ---*/
  /*--- Function import code ---*/
  __pyx_t_1 = __Pyx_ImportModule("thriftrw.spec.check"); if (!__pyx_t_1) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ImportFunction(__pyx_t_1, "type_code_matches", (void (**)(void))&__pyx_f_8thriftrw_4spec_5check_type_code_matches, "PyObject *(struct __pyx_obj_8thriftrw_4spec_4base_TypeSpec *, struct __pyx_obj_8thriftrw_4wire_5value_Value *, int __pyx_skip_dispatch)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  Py_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  /*--- Execution code ---*/
  #if defined(__Pyx_Generator_USED) || defined(__Pyx_Coroutine_USED)
  if (__Pyx_patch_abc() < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  #endif

  /* "thriftrw/spec/service.pyx":23
 * from __future__ import absolute_import, unicode_literals, print_function
 * 
 * from collections import namedtuple             # <<<<<<<<<<<<<<
 * 
 * from thriftrw.wire.value cimport Value
 */
  __pyx_t_2 = PyList_New(1); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 23; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(__pyx_n_s_namedtuple);
  __Pyx_GIVEREF(__pyx_n_s_namedtuple);
  PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_s_namedtuple);
  __pyx_t_3 = __Pyx_Import(__pyx_n_s_collections, __pyx_t_2, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 23; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_3, __pyx_n_s_namedtuple); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 23; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_namedtuple, __pyx_t_2) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 23; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "thriftrw/spec/service.pyx":31
 * from . cimport check
 * 
 * from .spec_mapper import type_spec_or_ref             # <<<<<<<<<<<<<<
 * from ..errors import ThriftCompilerError
 * from ..errors import UnknownExceptionError
 */
  __pyx_t_3 = PyList_New(1); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 31; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_INCREF(__pyx_n_s_type_spec_or_ref);
  __Pyx_GIVEREF(__pyx_n_s_type_spec_or_ref);
  PyList_SET_ITEM(__pyx_t_3, 0, __pyx_n_s_type_spec_or_ref);
  __pyx_t_2 = __Pyx_Import(__pyx_n_s_spec_mapper, __pyx_t_3, 1); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 31; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_ImportFrom(__pyx_t_2, __pyx_n_s_type_spec_or_ref); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 31; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_type_spec_or_ref, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 31; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "thriftrw/spec/service.pyx":32
 * 
 * from .spec_mapper import type_spec_or_ref
 * from ..errors import ThriftCompilerError             # <<<<<<<<<<<<<<
 * from ..errors import UnknownExceptionError
 * 
 */
  __pyx_t_2 = PyList_New(1); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 32; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(__pyx_n_s_ThriftCompilerError);
  __Pyx_GIVEREF(__pyx_n_s_ThriftCompilerError);
  PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_s_ThriftCompilerError);
  __pyx_t_3 = __Pyx_Import(__pyx_n_s_errors, __pyx_t_2, 2); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 32; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_3, __pyx_n_s_ThriftCompilerError); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 32; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_ThriftCompilerError, __pyx_t_2) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 32; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "thriftrw/spec/service.pyx":33
 * from .spec_mapper import type_spec_or_ref
 * from ..errors import ThriftCompilerError
 * from ..errors import UnknownExceptionError             # <<<<<<<<<<<<<<
 * 
 * __all__ = [
 */
  __pyx_t_3 = PyList_New(1); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 33; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_INCREF(__pyx_n_s_UnknownExceptionError);
  __Pyx_GIVEREF(__pyx_n_s_UnknownExceptionError);
  PyList_SET_ITEM(__pyx_t_3, 0, __pyx_n_s_UnknownExceptionError);
  __pyx_t_2 = __Pyx_Import(__pyx_n_s_errors, __pyx_t_3, 2); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 33; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_ImportFrom(__pyx_t_2, __pyx_n_s_UnknownExceptionError); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 33; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_UnknownExceptionError, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 33; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "thriftrw/spec/service.pyx":35
 * from ..errors import UnknownExceptionError
 * 
 * __all__ = [             # <<<<<<<<<<<<<<
 *     'ServiceSpec',
 *     'FunctionSpec',
 */
  __pyx_t_2 = PyList_New(5); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 35; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(__pyx_n_u_ServiceSpec);
  __Pyx_GIVEREF(__pyx_n_u_ServiceSpec);
  PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_u_ServiceSpec);
  __Pyx_INCREF(__pyx_n_u_FunctionSpec);
  __Pyx_GIVEREF(__pyx_n_u_FunctionSpec);
  PyList_SET_ITEM(__pyx_t_2, 1, __pyx_n_u_FunctionSpec);
  __Pyx_INCREF(__pyx_n_u_ServiceFunction);
  __Pyx_GIVEREF(__pyx_n_u_ServiceFunction);
  PyList_SET_ITEM(__pyx_t_2, 2, __pyx_n_u_ServiceFunction);
  __Pyx_INCREF(__pyx_n_u_FunctionArgsSpec);
  __Pyx_GIVEREF(__pyx_n_u_FunctionArgsSpec);
  PyList_SET_ITEM(__pyx_t_2, 3, __pyx_n_u_FunctionArgsSpec);
  __Pyx_INCREF(__pyx_n_u_FunctionResultSpec);
  __Pyx_GIVEREF(__pyx_n_u_FunctionResultSpec);
  PyList_SET_ITEM(__pyx_t_2, 4, __pyx_n_u_FunctionResultSpec);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_all, __pyx_t_2) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 35; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "thriftrw/spec/service.pyx":77
 * 
 *     @classmethod
 *     def compile(cls, parameters, service_name, function_name):             # <<<<<<<<<<<<<<
 *         """Compiles a parameter list into a FunctionArgsSpec.
 * 
 */
  __pyx_t_2 = __Pyx_GetNameInClass((PyObject *)__pyx_ptype_8thriftrw_4spec_7service_FunctionArgsSpec, __pyx_n_s_compile); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 77; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);

  /* "thriftrw/spec/service.pyx":76
 *         self.function = None
 * 
 *     @classmethod             # <<<<<<<<<<<<<<
 *     def compile(cls, parameters, service_name, function_name):
 *         """Compiles a parameter list into a FunctionArgsSpec.
 */
  __pyx_t_3 = __Pyx_Method_ClassMethod(__pyx_t_2); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 76; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_8thriftrw_4spec_7service_FunctionArgsSpec->tp_dict, __pyx_n_s_compile, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 77; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  PyType_Modified(__pyx_ptype_8thriftrw_4spec_7service_FunctionArgsSpec);

  /* "thriftrw/spec/service.pyx":201
 * 
 *     @classmethod
 *     def compile(cls, return_type, exceptions, service_name, function_name):             # <<<<<<<<<<<<<<
 *         """Compiles information from the AST into a FunctionResultSpec.
 * 
 */
  __pyx_t_3 = __Pyx_GetNameInClass((PyObject *)__pyx_ptype_8thriftrw_4spec_7service_FunctionResultSpec, __pyx_n_s_compile); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 201; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);

  /* "thriftrw/spec/service.pyx":200
 *         return super(FunctionResultSpec, self).link(scope)
 * 
 *     @classmethod             # <<<<<<<<<<<<<<
 *     def compile(cls, return_type, exceptions, service_name, function_name):
 *         """Compiles information from the AST into a FunctionResultSpec.
 */
  __pyx_t_2 = __Pyx_Method_ClassMethod(__pyx_t_3); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 200; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_8thriftrw_4spec_7service_FunctionResultSpec->tp_dict, __pyx_n_s_compile, __pyx_t_2) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 201; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_8thriftrw_4spec_7service_FunctionResultSpec);

  /* "thriftrw/spec/service.pyx":289
 * 
 *     @classmethod
 *     def compile(cls, func, service_name):             # <<<<<<<<<<<<<<
 *         args_spec = FunctionArgsSpec.compile(
 *             parameters=func.parameters,
 */
  __pyx_t_2 = __Pyx_GetNameInClass((PyObject *)__pyx_ptype_8thriftrw_4spec_7service_FunctionSpec, __pyx_n_s_compile); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 289; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);

  /* "thriftrw/spec/service.pyx":288
 *         self.service = None
 * 
 *     @classmethod             # <<<<<<<<<<<<<<
 *     def compile(cls, func, service_name):
 *         args_spec = FunctionArgsSpec.compile(
 */
  __pyx_t_3 = __Pyx_Method_ClassMethod(__pyx_t_2); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 288; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_8thriftrw_4spec_7service_FunctionSpec->tp_dict, __pyx_n_s_compile, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 289; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  PyType_Modified(__pyx_ptype_8thriftrw_4spec_7service_FunctionSpec);

  /* "thriftrw/spec/service.pyx":384
 * 
 *     @classmethod
 *     def compile(cls, service):             # <<<<<<<<<<<<<<
 *         functions = []
 *         names = set()
 */
  __pyx_t_3 = __Pyx_GetNameInClass((PyObject *)__pyx_ptype_8thriftrw_4spec_7service_ServiceSpec, __pyx_n_s_compile); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 384; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);

  /* "thriftrw/spec/service.pyx":383
 *         self.surface = None
 * 
 *     @classmethod             # <<<<<<<<<<<<<<
 *     def compile(cls, service):
 *         functions = []
 */
  __pyx_t_2 = __Pyx_Method_ClassMethod(__pyx_t_3); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 383; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_8thriftrw_4spec_7service_ServiceSpec->tp_dict, __pyx_n_s_compile, __pyx_t_2) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 384; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_8thriftrw_4spec_7service_ServiceSpec);

  /* "thriftrw/spec/service.pyx":446
 * 
 * class ServiceFunction(
 *     namedtuple('ServiceFunction', 'name request response spec')             # <<<<<<<<<<<<<<
 * ):
 *     """Represents a single function on a service.
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_namedtuple); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 446; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_tuple__3, NULL); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 446; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "thriftrw/spec/service.pyx":445
 * 
 * 
 * class ServiceFunction(             # <<<<<<<<<<<<<<
 *     namedtuple('ServiceFunction', 'name request response spec')
 * ):
 */
  __pyx_t_2 = PyTuple_New(1); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 445; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_3);
  __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_CalculateMetaclass(NULL, __pyx_t_2); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 445; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_Py3MetaclassPrepare(__pyx_t_3, __pyx_t_2, __pyx_n_s_ServiceFunction, __pyx_n_s_ServiceFunction, (PyObject *) NULL, __pyx_n_s_thriftrw_spec_service, __pyx_kp_s_Represents_a_single_function_on); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 445; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = __Pyx_Py3ClassCreate(__pyx_t_3, __pyx_n_s_ServiceFunction, __pyx_t_2, __pyx_t_4, NULL, 0, 1); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 445; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_ServiceFunction, __pyx_t_5) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 445; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "thriftrw/spec/service.pyx":473
 * 
 * 
 * def service_cls(service_spec, scope):             # <<<<<<<<<<<<<<
 *     """Generates a class from the given service spec.
 * 
 */
  __pyx_t_2 = PyCFunction_NewEx(&__pyx_mdef_8thriftrw_4spec_7service_1service_cls, NULL, __pyx_n_s_thriftrw_spec_service); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 473; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_service_cls, __pyx_t_2) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 473; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "thriftrw/spec/service.pyx":1
 * # Copyright (c) 2015 Uber Technologies, Inc.             # <<<<<<<<<<<<<<
 * #
 * # Permission is hereby granted, free of charge, to any person obtaining a copy
 */
  __pyx_t_2 = PyDict_New(); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_test, __pyx_t_2) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /*--- Wrapped vars code ---*/

  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  if (__pyx_m) {
    if (__pyx_d) {
      __Pyx_AddTraceback("init thriftrw.spec.service", __pyx_clineno, __pyx_lineno, __pyx_filename);
    }
    Py_DECREF(__pyx_m); __pyx_m = 0;
  } else if (!PyErr_Occurred()) {
    PyErr_SetString(PyExc_ImportError, "init thriftrw.spec.service");
  }
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  #if PY_MAJOR_VERSION < 3
  return;
  #else
  return __pyx_m;
  #endif
}

/* --- Runtime support code --- */
#if CYTHON_REFNANNY
static __Pyx_RefNannyAPIStruct *__Pyx_RefNannyImportAPI(const char *modname) {
    PyObject *m = NULL, *p = NULL;
    void *r = NULL;
    m = PyImport_ImportModule((char *)modname);
    if (!m) goto end;
    p = PyObject_GetAttrString(m, (char *)"RefNannyAPI");
    if (!p) goto end;
    r = PyLong_AsVoidPtr(p);
end:
    Py_XDECREF(p);
    Py_XDECREF(m);
    return (__Pyx_RefNannyAPIStruct *)r;
}
#endif

static PyObject *__Pyx_GetBuiltinName(PyObject *name) {
    PyObject* result = __Pyx_PyObject_GetAttrStr(__pyx_b, name);
    if (unlikely(!result)) {
        PyErr_Format(PyExc_NameError,
#if PY_MAJOR_VERSION >= 3
            "name '%U' is not defined", name);
#else
            "name '%.200s' is not defined", PyString_AS_STRING(name));
#endif
    }
    return result;
}

static void __Pyx_RaiseArgtupleInvalid(
    const char* func_name,
    int exact,
    Py_ssize_t num_min,
    Py_ssize_t num_max,
    Py_ssize_t num_found)
{
    Py_ssize_t num_expected;
    const char *more_or_less;
    if (num_found < num_min) {
        num_expected = num_min;
        more_or_less = "at least";
    } else {
        num_expected = num_max;
        more_or_less = "at most";
    }
    if (exact) {
        more_or_less = "exactly";
    }
    PyErr_Format(PyExc_TypeError,
                 "%.200s() takes %.8s %" CYTHON_FORMAT_SSIZE_T "d positional argument%.1s (%" CYTHON_FORMAT_SSIZE_T "d given)",
                 func_name, more_or_less, num_expected,
                 (num_expected == 1) ? "" : "s", num_found);
}

static void __Pyx_RaiseDoubleKeywordsError(
    const char* func_name,
    PyObject* kw_name)
{
    PyErr_Format(PyExc_TypeError,
        #if PY_MAJOR_VERSION >= 3
        "%s() got multiple values for keyword argument '%U'", func_name, kw_name);
        #else
        "%s() got multiple values for keyword argument '%s'", func_name,
        PyString_AsString(kw_name));
        #endif
}

static int __Pyx_ParseOptionalKeywords(
    PyObject *kwds,
    PyObject **argnames[],
    PyObject *kwds2,
    PyObject *values[],
    Py_ssize_t num_pos_args,
    const char* function_name)
{
    PyObject *key = 0, *value = 0;
    Py_ssize_t pos = 0;
    PyObject*** name;
    PyObject*** first_kw_arg = argnames + num_pos_args;
    while (PyDict_Next(kwds, &pos, &key, &value)) {
        name = first_kw_arg;
        while (*name && (**name != key)) name++;
        if (*name) {
            values[name-argnames] = value;
            continue;
        }
        name = first_kw_arg;
        #if PY_MAJOR_VERSION < 3
        if (likely(PyString_CheckExact(key)) || likely(PyString_Check(key))) {
            while (*name) {
                if ((CYTHON_COMPILING_IN_PYPY || PyString_GET_SIZE(**name) == PyString_GET_SIZE(key))
                        && _PyString_Eq(**name, key)) {
                    values[name-argnames] = value;
                    break;
                }
                name++;
            }
            if (*name) continue;
            else {
                PyObject*** argname = argnames;
                while (argname != first_kw_arg) {
                    if ((**argname == key) || (
                            (CYTHON_COMPILING_IN_PYPY || PyString_GET_SIZE(**argname) == PyString_GET_SIZE(key))
                             && _PyString_Eq(**argname, key))) {
                        goto arg_passed_twice;
                    }
                    argname++;
                }
            }
        } else
        #endif
        if (likely(PyUnicode_Check(key))) {
            while (*name) {
                int cmp = (**name == key) ? 0 :
                #if !CYTHON_COMPILING_IN_PYPY && PY_MAJOR_VERSION >= 3
                    (PyUnicode_GET_SIZE(**name) != PyUnicode_GET_SIZE(key)) ? 1 :
                #endif
                    PyUnicode_Compare(**name, key);
                if (cmp < 0 && unlikely(PyErr_Occurred())) goto bad;
                if (cmp == 0) {
                    values[name-argnames] = value;
                    break;
                }
                name++;
            }
            if (*name) continue;
            else {
                PyObject*** argname = argnames;
                while (argname != first_kw_arg) {
                    int cmp = (**argname == key) ? 0 :
                    #if !CYTHON_COMPILING_IN_PYPY && PY_MAJOR_VERSION >= 3
                        (PyUnicode_GET_SIZE(**argname) != PyUnicode_GET_SIZE(key)) ? 1 :
                    #endif
                        PyUnicode_Compare(**argname, key);
                    if (cmp < 0 && unlikely(PyErr_Occurred())) goto bad;
                    if (cmp == 0) goto arg_passed_twice;
                    argname++;
                }
            }
        } else
            goto invalid_keyword_type;
        if (kwds2) {
            if (unlikely(PyDict_SetItem(kwds2, key, value))) goto bad;
        } else {
            goto invalid_keyword;
        }
    }
    return 0;
arg_passed_twice:
    __Pyx_RaiseDoubleKeywordsError(function_name, key);
    goto bad;
invalid_keyword_type:
    PyErr_Format(PyExc_TypeError,
        "%.200s() keywords must be strings", function_name);
    goto bad;
invalid_keyword:
    PyErr_Format(PyExc_TypeError,
    #if PY_MAJOR_VERSION < 3
        "%.200s() got an unexpected keyword argument '%.200s'",
        function_name, PyString_AsString(key));
    #else
        "%s() got an unexpected keyword argument '%U'",
        function_name, key);
    #endif
bad:
    return -1;
}

#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_Call(PyObject *func, PyObject *arg, PyObject *kw) {
    PyObject *result;
    ternaryfunc call = func->ob_type->tp_call;
    if (unlikely(!call))
        return PyObject_Call(func, arg, kw);
    if (unlikely(Py_EnterRecursiveCall((char*)" while calling a Python object")))
        return NULL;
    result = (*call)(func, arg, kw);
    Py_LeaveRecursiveCall();
    if (unlikely(!result) && unlikely(!PyErr_Occurred())) {
        PyErr_SetString(
            PyExc_SystemError,
            "NULL result without error in PyObject_Call");
    }
    return result;
}
#endif

static CYTHON_INLINE int __Pyx_TypeTest(PyObject *obj, PyTypeObject *type) {
    if (unlikely(!type)) {
        PyErr_SetString(PyExc_SystemError, "Missing type object");
        return 0;
    }
    if (likely(PyObject_TypeCheck(obj, type)))
        return 1;
    PyErr_Format(PyExc_TypeError, "Cannot convert %.200s to %.200s",
                 Py_TYPE(obj)->tp_name, type->tp_name);
    return 0;
}

#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallMethO(PyObject *func, PyObject *arg) {
    PyObject *self, *result;
    PyCFunction cfunc;
    cfunc = PyCFunction_GET_FUNCTION(func);
    self = PyCFunction_GET_SELF(func);
    if (unlikely(Py_EnterRecursiveCall((char*)" while calling a Python object")))
        return NULL;
    result = cfunc(self, arg);
    Py_LeaveRecursiveCall();
    if (unlikely(!result) && unlikely(!PyErr_Occurred())) {
        PyErr_SetString(
            PyExc_SystemError,
            "NULL result without error in PyObject_Call");
    }
    return result;
}
#endif

#if CYTHON_COMPILING_IN_CPYTHON
static PyObject* __Pyx__PyObject_CallOneArg(PyObject *func, PyObject *arg) {
    PyObject *result;
    PyObject *args = PyTuple_New(1);
    if (unlikely(!args)) return NULL;
    Py_INCREF(arg);
    PyTuple_SET_ITEM(args, 0, arg);
    result = __Pyx_PyObject_Call(func, args, NULL);
    Py_DECREF(args);
    return result;
}
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg) {
#ifdef __Pyx_CyFunction_USED
    if (likely(PyCFunction_Check(func) || PyObject_TypeCheck(func, __pyx_CyFunctionType))) {
#else
    if (likely(PyCFunction_Check(func))) {
#endif
        if (likely(PyCFunction_GET_FLAGS(func) & METH_O)) {
            return __Pyx_PyObject_CallMethO(func, arg);
        }
    }
    return __Pyx__PyObject_CallOneArg(func, arg);
}
#else
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg) {
    PyObject *result;
    PyObject *args = PyTuple_Pack(1, arg);
    if (unlikely(!args)) return NULL;
    result = __Pyx_PyObject_Call(func, args, NULL);
    Py_DECREF(args);
    return result;
}
#endif

static CYTHON_INLINE PyObject *__Pyx_GetModuleGlobalName(PyObject *name) {
    PyObject *result;
#if CYTHON_COMPILING_IN_CPYTHON
    result = PyDict_GetItem(__pyx_d, name);
    if (likely(result)) {
        Py_INCREF(result);
    } else {
#else
    result = PyObject_GetItem(__pyx_d, name);
    if (!result) {
        PyErr_Clear();
#endif
        result = __Pyx_GetBuiltinName(name);
    }
    return result;
}

static CYTHON_INLINE void __Pyx_ErrRestore(PyObject *type, PyObject *value, PyObject *tb) {
#if CYTHON_COMPILING_IN_CPYTHON
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    PyThreadState *tstate = PyThreadState_GET();
    tmp_type = tstate->curexc_type;
    tmp_value = tstate->curexc_value;
    tmp_tb = tstate->curexc_traceback;
    tstate->curexc_type = type;
    tstate->curexc_value = value;
    tstate->curexc_traceback = tb;
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
#else
    PyErr_Restore(type, value, tb);
#endif
}
static CYTHON_INLINE void __Pyx_ErrFetch(PyObject **type, PyObject **value, PyObject **tb) {
#if CYTHON_COMPILING_IN_CPYTHON
    PyThreadState *tstate = PyThreadState_GET();
    *type = tstate->curexc_type;
    *value = tstate->curexc_value;
    *tb = tstate->curexc_traceback;
    tstate->curexc_type = 0;
    tstate->curexc_value = 0;
    tstate->curexc_traceback = 0;
#else
    PyErr_Fetch(type, value, tb);
#endif
}

#if PY_MAJOR_VERSION < 3
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb,
                        CYTHON_UNUSED PyObject *cause) {
    Py_XINCREF(type);
    if (!value || value == Py_None)
        value = NULL;
    else
        Py_INCREF(value);
    if (!tb || tb == Py_None)
        tb = NULL;
    else {
        Py_INCREF(tb);
        if (!PyTraceBack_Check(tb)) {
            PyErr_SetString(PyExc_TypeError,
                "raise: arg 3 must be a traceback or None");
            goto raise_error;
        }
    }
    if (PyType_Check(type)) {
#if CYTHON_COMPILING_IN_PYPY
        if (!value) {
            Py_INCREF(Py_None);
            value = Py_None;
        }
#endif
        PyErr_NormalizeException(&type, &value, &tb);
    } else {
        if (value) {
            PyErr_SetString(PyExc_TypeError,
                "instance exception may not have a separate value");
            goto raise_error;
        }
        value = type;
        type = (PyObject*) Py_TYPE(type);
        Py_INCREF(type);
        if (!PyType_IsSubtype((PyTypeObject *)type, (PyTypeObject *)PyExc_BaseException)) {
            PyErr_SetString(PyExc_TypeError,
                "raise: exception class must be a subclass of BaseException");
            goto raise_error;
        }
    }
    __Pyx_ErrRestore(type, value, tb);
    return;
raise_error:
    Py_XDECREF(value);
    Py_XDECREF(type);
    Py_XDECREF(tb);
    return;
}
#else
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb, PyObject *cause) {
    PyObject* owned_instance = NULL;
    if (tb == Py_None) {
        tb = 0;
    } else if (tb && !PyTraceBack_Check(tb)) {
        PyErr_SetString(PyExc_TypeError,
            "raise: arg 3 must be a traceback or None");
        goto bad;
    }
    if (value == Py_None)
        value = 0;
    if (PyExceptionInstance_Check(type)) {
        if (value) {
            PyErr_SetString(PyExc_TypeError,
                "instance exception may not have a separate value");
            goto bad;
        }
        value = type;
        type = (PyObject*) Py_TYPE(value);
    } else if (PyExceptionClass_Check(type)) {
        PyObject *instance_class = NULL;
        if (value && PyExceptionInstance_Check(value)) {
            instance_class = (PyObject*) Py_TYPE(value);
            if (instance_class != type) {
                int is_subclass = PyObject_IsSubclass(instance_class, type);
                if (!is_subclass) {
                    instance_class = NULL;
                } else if (unlikely(is_subclass == -1)) {
                    goto bad;
                } else {
                    type = instance_class;
                }
            }
        }
        if (!instance_class) {
            PyObject *args;
            if (!value)
                args = PyTuple_New(0);
            else if (PyTuple_Check(value)) {
                Py_INCREF(value);
                args = value;
            } else
                args = PyTuple_Pack(1, value);
            if (!args)
                goto bad;
            owned_instance = PyObject_Call(type, args, NULL);
            Py_DECREF(args);
            if (!owned_instance)
                goto bad;
            value = owned_instance;
            if (!PyExceptionInstance_Check(value)) {
                PyErr_Format(PyExc_TypeError,
                             "calling %R should have returned an instance of "
                             "BaseException, not %R",
                             type, Py_TYPE(value));
                goto bad;
            }
        }
    } else {
        PyErr_SetString(PyExc_TypeError,
            "raise: exception class must be a subclass of BaseException");
        goto bad;
    }
#if PY_VERSION_HEX >= 0x03030000
    if (cause) {
#else
    if (cause && cause != Py_None) {
#endif
        PyObject *fixed_cause;
        if (cause == Py_None) {
            fixed_cause = NULL;
        } else if (PyExceptionClass_Check(cause)) {
            fixed_cause = PyObject_CallObject(cause, NULL);
            if (fixed_cause == NULL)
                goto bad;
        } else if (PyExceptionInstance_Check(cause)) {
            fixed_cause = cause;
            Py_INCREF(fixed_cause);
        } else {
            PyErr_SetString(PyExc_TypeError,
                            "exception causes must derive from "
                            "BaseException");
            goto bad;
        }
        PyException_SetCause(value, fixed_cause);
    }
    PyErr_SetObject(type, value);
    if (tb) {
#if CYTHON_COMPILING_IN_PYPY
        PyObject *tmp_type, *tmp_value, *tmp_tb;
        PyErr_Fetch(&tmp_type, &tmp_value, &tmp_tb);
        Py_INCREF(tb);
        PyErr_Restore(tmp_type, tmp_value, tb);
        Py_XDECREF(tmp_tb);
#else
        PyThreadState *tstate = PyThreadState_GET();
        PyObject* tmp_tb = tstate->curexc_traceback;
        if (tb != tmp_tb) {
            Py_INCREF(tb);
            tstate->curexc_traceback = tb;
            Py_XDECREF(tmp_tb);
        }
#endif
    }
bad:
    Py_XDECREF(owned_instance);
    return;
}
#endif

static void __Pyx_RaiseArgumentTypeInvalid(const char* name, PyObject *obj, PyTypeObject *type) {
    PyErr_Format(PyExc_TypeError,
        "Argument '%.200s' has incorrect type (expected %.200s, got %.200s)",
        name, type->tp_name, Py_TYPE(obj)->tp_name);
}
static CYTHON_INLINE int __Pyx_ArgTypeTest(PyObject *obj, PyTypeObject *type, int none_allowed,
    const char *name, int exact)
{
    if (unlikely(!type)) {
        PyErr_SetString(PyExc_SystemError, "Missing type object");
        return 0;
    }
    if (none_allowed && obj == Py_None) return 1;
    else if (exact) {
        if (likely(Py_TYPE(obj) == type)) return 1;
        #if PY_MAJOR_VERSION == 2
        else if ((type == &PyBaseString_Type) && likely(__Pyx_PyBaseString_CheckExact(obj))) return 1;
        #endif
    }
    else {
        if (likely(PyObject_TypeCheck(obj, type))) return 1;
    }
    __Pyx_RaiseArgumentTypeInvalid(name, obj, type);
    return 0;
}

static PyObject* __Pyx_PyDict_GetItemDefault(PyObject* d, PyObject* key, PyObject* default_value) {
    PyObject* value;
#if PY_MAJOR_VERSION >= 3 && !CYTHON_COMPILING_IN_PYPY
    value = PyDict_GetItemWithError(d, key);
    if (unlikely(!value)) {
        if (unlikely(PyErr_Occurred()))
            return NULL;
        value = default_value;
    }
    Py_INCREF(value);
#else
    if (PyString_CheckExact(key) || PyUnicode_CheckExact(key) || PyInt_CheckExact(key)) {
        value = PyDict_GetItem(d, key);
        if (unlikely(!value)) {
            value = default_value;
        }
        Py_INCREF(value);
    } else {
        if (default_value == Py_None)
            default_value = NULL;
        value = PyObject_CallMethodObjArgs(
            d, __pyx_n_s_get, key, default_value, NULL);
    }
#endif
    return value;
}

static void __Pyx_call_next_tp_dealloc(PyObject* obj, destructor current_tp_dealloc) {
    PyTypeObject* type = Py_TYPE(obj);
    while (type && type->tp_dealloc != current_tp_dealloc)
        type = type->tp_base;
    while (type && type->tp_dealloc == current_tp_dealloc)
        type = type->tp_base;
    if (type)
        type->tp_dealloc(obj);
}

static int __Pyx_call_next_tp_traverse(PyObject* obj, visitproc v, void *a, traverseproc current_tp_traverse) {
    PyTypeObject* type = Py_TYPE(obj);
    while (type && type->tp_traverse != current_tp_traverse)
        type = type->tp_base;
    while (type && type->tp_traverse == current_tp_traverse)
        type = type->tp_base;
    if (type && type->tp_traverse)
        return type->tp_traverse(obj, v, a);
    return 0;
}

static void __Pyx_call_next_tp_clear(PyObject* obj, inquiry current_tp_clear) {
    PyTypeObject* type = Py_TYPE(obj);
    while (type && type->tp_clear != current_tp_clear)
        type = type->tp_base;
    while (type && type->tp_clear == current_tp_clear)
        type = type->tp_base;
    if (type && type->tp_clear)
        type->tp_clear(obj);
}

static void* __Pyx_GetVtable(PyObject *dict) {
    void* ptr;
    PyObject *ob = PyObject_GetItem(dict, __pyx_n_s_pyx_vtable);
    if (!ob)
        goto bad;
#if PY_VERSION_HEX >= 0x02070000
    ptr = PyCapsule_GetPointer(ob, 0);
#else
    ptr = PyCObject_AsVoidPtr(ob);
#endif
    if (!ptr && !PyErr_Occurred())
        PyErr_SetString(PyExc_RuntimeError, "invalid vtable found for imported type");
    Py_DECREF(ob);
    return ptr;
bad:
    Py_XDECREF(ob);
    return NULL;
}

static int __Pyx_SetVtable(PyObject *dict, void *vtable) {
#if PY_VERSION_HEX >= 0x02070000
    PyObject *ob = PyCapsule_New(vtable, 0, 0);
#else
    PyObject *ob = PyCObject_FromVoidPtr(vtable, 0);
#endif
    if (!ob)
        goto bad;
    if (PyDict_SetItem(dict, __pyx_n_s_pyx_vtable, ob) < 0)
        goto bad;
    Py_DECREF(ob);
    return 0;
bad:
    Py_XDECREF(ob);
    return -1;
}

static PyObject *__Pyx_Import(PyObject *name, PyObject *from_list, int level) {
    PyObject *empty_list = 0;
    PyObject *module = 0;
    PyObject *global_dict = 0;
    PyObject *empty_dict = 0;
    PyObject *list;
    #if PY_VERSION_HEX < 0x03030000
    PyObject *py_import;
    py_import = __Pyx_PyObject_GetAttrStr(__pyx_b, __pyx_n_s_import);
    if (!py_import)
        goto bad;
    #endif
    if (from_list)
        list = from_list;
    else {
        empty_list = PyList_New(0);
        if (!empty_list)
            goto bad;
        list = empty_list;
    }
    global_dict = PyModule_GetDict(__pyx_m);
    if (!global_dict)
        goto bad;
    empty_dict = PyDict_New();
    if (!empty_dict)
        goto bad;
    {
        #if PY_MAJOR_VERSION >= 3
        if (level == -1) {
            if (strchr(__Pyx_MODULE_NAME, '.')) {
                #if PY_VERSION_HEX < 0x03030000
                PyObject *py_level = PyInt_FromLong(1);
                if (!py_level)
                    goto bad;
                module = PyObject_CallFunctionObjArgs(py_import,
                    name, global_dict, empty_dict, list, py_level, NULL);
                Py_DECREF(py_level);
                #else
                module = PyImport_ImportModuleLevelObject(
                    name, global_dict, empty_dict, list, 1);
                #endif
                if (!module) {
                    if (!PyErr_ExceptionMatches(PyExc_ImportError))
                        goto bad;
                    PyErr_Clear();
                }
            }
            level = 0;
        }
        #endif
        if (!module) {
            #if PY_VERSION_HEX < 0x03030000
            PyObject *py_level = PyInt_FromLong(level);
            if (!py_level)
                goto bad;
            module = PyObject_CallFunctionObjArgs(py_import,
                name, global_dict, empty_dict, list, py_level, NULL);
            Py_DECREF(py_level);
            #else
            module = PyImport_ImportModuleLevelObject(
                name, global_dict, empty_dict, list, level);
            #endif
        }
    }
bad:
    #if PY_VERSION_HEX < 0x03030000
    Py_XDECREF(py_import);
    #endif
    Py_XDECREF(empty_list);
    Py_XDECREF(empty_dict);
    return module;
}

static PyObject* __Pyx_ImportFrom(PyObject* module, PyObject* name) {
    PyObject* value = __Pyx_PyObject_GetAttrStr(module, name);
    if (unlikely(!value) && PyErr_ExceptionMatches(PyExc_AttributeError)) {
        PyErr_Format(PyExc_ImportError,
        #if PY_MAJOR_VERSION < 3
            "cannot import name %.230s", PyString_AS_STRING(name));
        #else
            "cannot import name %S", name);
        #endif
    }
    return value;
}

static PyObject *__Pyx_GetNameInClass(PyObject *nmspace, PyObject *name) {
    PyObject *result;
    result = __Pyx_PyObject_GetAttrStr(nmspace, name);
    if (!result)
        result = __Pyx_GetModuleGlobalName(name);
    return result;
}

static PyObject *__Pyx_CalculateMetaclass(PyTypeObject *metaclass, PyObject *bases) {
    Py_ssize_t i, nbases = PyTuple_GET_SIZE(bases);
    for (i=0; i < nbases; i++) {
        PyTypeObject *tmptype;
        PyObject *tmp = PyTuple_GET_ITEM(bases, i);
        tmptype = Py_TYPE(tmp);
#if PY_MAJOR_VERSION < 3
        if (tmptype == &PyClass_Type)
            continue;
#endif
        if (!metaclass) {
            metaclass = tmptype;
            continue;
        }
        if (PyType_IsSubtype(metaclass, tmptype))
            continue;
        if (PyType_IsSubtype(tmptype, metaclass)) {
            metaclass = tmptype;
            continue;
        }
        PyErr_SetString(PyExc_TypeError,
                        "metaclass conflict: "
                        "the metaclass of a derived class "
                        "must be a (non-strict) subclass "
                        "of the metaclasses of all its bases");
        return NULL;
    }
    if (!metaclass) {
#if PY_MAJOR_VERSION < 3
        metaclass = &PyClass_Type;
#else
        metaclass = &PyType_Type;
#endif
    }
    Py_INCREF((PyObject*) metaclass);
    return (PyObject*) metaclass;
}

static PyObject *__Pyx_Py3MetaclassPrepare(PyObject *metaclass, PyObject *bases, PyObject *name,
                                           PyObject *qualname, PyObject *mkw, PyObject *modname, PyObject *doc) {
    PyObject *ns;
    if (metaclass) {
        PyObject *prep = __Pyx_PyObject_GetAttrStr(metaclass, __pyx_n_s_prepare);
        if (prep) {
            PyObject *pargs = PyTuple_Pack(2, name, bases);
            if (unlikely(!pargs)) {
                Py_DECREF(prep);
                return NULL;
            }
            ns = PyObject_Call(prep, pargs, mkw);
            Py_DECREF(prep);
            Py_DECREF(pargs);
        } else {
            if (unlikely(!PyErr_ExceptionMatches(PyExc_AttributeError)))
                return NULL;
            PyErr_Clear();
            ns = PyDict_New();
        }
    } else {
        ns = PyDict_New();
    }
    if (unlikely(!ns))
        return NULL;
    if (unlikely(PyObject_SetItem(ns, __pyx_n_s_module, modname) < 0)) goto bad;
    if (unlikely(PyObject_SetItem(ns, __pyx_n_s_qualname, qualname) < 0)) goto bad;
    if (unlikely(doc && PyObject_SetItem(ns, __pyx_n_s_doc, doc) < 0)) goto bad;
    return ns;
bad:
    Py_DECREF(ns);
    return NULL;
}
static PyObject *__Pyx_Py3ClassCreate(PyObject *metaclass, PyObject *name, PyObject *bases,
                                      PyObject *dict, PyObject *mkw,
                                      int calculate_metaclass, int allow_py2_metaclass) {
    PyObject *result, *margs;
    PyObject *owned_metaclass = NULL;
    if (allow_py2_metaclass) {
        owned_metaclass = PyObject_GetItem(dict, __pyx_n_s_metaclass);
        if (owned_metaclass) {
            metaclass = owned_metaclass;
        } else if (likely(PyErr_ExceptionMatches(PyExc_KeyError))) {
            PyErr_Clear();
        } else {
            return NULL;
        }
    }
    if (calculate_metaclass && (!metaclass || PyType_Check(metaclass))) {
        metaclass = __Pyx_CalculateMetaclass((PyTypeObject*) metaclass, bases);
        Py_XDECREF(owned_metaclass);
        if (unlikely(!metaclass))
            return NULL;
        owned_metaclass = metaclass;
    }
    margs = PyTuple_Pack(3, name, bases, dict);
    if (unlikely(!margs)) {
        result = NULL;
    } else {
        result = PyObject_Call(metaclass, margs, mkw);
        Py_DECREF(margs);
    }
    Py_XDECREF(owned_metaclass);
    return result;
}

static int __pyx_bisect_code_objects(__Pyx_CodeObjectCacheEntry* entries, int count, int code_line) {
    int start = 0, mid = 0, end = count - 1;
    if (end >= 0 && code_line > entries[end].code_line) {
        return count;
    }
    while (start < end) {
        mid = start + (end - start) / 2;
        if (code_line < entries[mid].code_line) {
            end = mid;
        } else if (code_line > entries[mid].code_line) {
             start = mid + 1;
        } else {
            return mid;
        }
    }
    if (code_line <= entries[mid].code_line) {
        return mid;
    } else {
        return mid + 1;
    }
}
static PyCodeObject *__pyx_find_code_object(int code_line) {
    PyCodeObject* code_object;
    int pos;
    if (unlikely(!code_line) || unlikely(!__pyx_code_cache.entries)) {
        return NULL;
    }
    pos = __pyx_bisect_code_objects(__pyx_code_cache.entries, __pyx_code_cache.count, code_line);
    if (unlikely(pos >= __pyx_code_cache.count) || unlikely(__pyx_code_cache.entries[pos].code_line != code_line)) {
        return NULL;
    }
    code_object = __pyx_code_cache.entries[pos].code_object;
    Py_INCREF(code_object);
    return code_object;
}
static void __pyx_insert_code_object(int code_line, PyCodeObject* code_object) {
    int pos, i;
    __Pyx_CodeObjectCacheEntry* entries = __pyx_code_cache.entries;
    if (unlikely(!code_line)) {
        return;
    }
    if (unlikely(!entries)) {
        entries = (__Pyx_CodeObjectCacheEntry*)PyMem_Malloc(64*sizeof(__Pyx_CodeObjectCacheEntry));
        if (likely(entries)) {
            __pyx_code_cache.entries = entries;
            __pyx_code_cache.max_count = 64;
            __pyx_code_cache.count = 1;
            entries[0].code_line = code_line;
            entries[0].code_object = code_object;
            Py_INCREF(code_object);
        }
        return;
    }
    pos = __pyx_bisect_code_objects(__pyx_code_cache.entries, __pyx_code_cache.count, code_line);
    if ((pos < __pyx_code_cache.count) && unlikely(__pyx_code_cache.entries[pos].code_line == code_line)) {
        PyCodeObject* tmp = entries[pos].code_object;
        entries[pos].code_object = code_object;
        Py_DECREF(tmp);
        return;
    }
    if (__pyx_code_cache.count == __pyx_code_cache.max_count) {
        int new_max = __pyx_code_cache.max_count + 64;
        entries = (__Pyx_CodeObjectCacheEntry*)PyMem_Realloc(
            __pyx_code_cache.entries, (size_t)new_max*sizeof(__Pyx_CodeObjectCacheEntry));
        if (unlikely(!entries)) {
            return;
        }
        __pyx_code_cache.entries = entries;
        __pyx_code_cache.max_count = new_max;
    }
    for (i=__pyx_code_cache.count; i>pos; i--) {
        entries[i] = entries[i-1];
    }
    entries[pos].code_line = code_line;
    entries[pos].code_object = code_object;
    __pyx_code_cache.count++;
    Py_INCREF(code_object);
}

#include "compile.h"
#include "frameobject.h"
#include "traceback.h"
static PyCodeObject* __Pyx_CreateCodeObjectForTraceback(
            const char *funcname, int c_line,
            int py_line, const char *filename) {
    PyCodeObject *py_code = 0;
    PyObject *py_srcfile = 0;
    PyObject *py_funcname = 0;
    #if PY_MAJOR_VERSION < 3
    py_srcfile = PyString_FromString(filename);
    #else
    py_srcfile = PyUnicode_FromString(filename);
    #endif
    if (!py_srcfile) goto bad;
    if (c_line) {
        #if PY_MAJOR_VERSION < 3
        py_funcname = PyString_FromFormat( "%s (%s:%d)", funcname, __pyx_cfilenm, c_line);
        #else
        py_funcname = PyUnicode_FromFormat( "%s (%s:%d)", funcname, __pyx_cfilenm, c_line);
        #endif
    }
    else {
        #if PY_MAJOR_VERSION < 3
        py_funcname = PyString_FromString(funcname);
        #else
        py_funcname = PyUnicode_FromString(funcname);
        #endif
    }
    if (!py_funcname) goto bad;
    py_code = __Pyx_PyCode_New(
        0,
        0,
        0,
        0,
        0,
        __pyx_empty_bytes, /*PyObject *code,*/
        __pyx_empty_tuple, /*PyObject *consts,*/
        __pyx_empty_tuple, /*PyObject *names,*/
        __pyx_empty_tuple, /*PyObject *varnames,*/
        __pyx_empty_tuple, /*PyObject *freevars,*/
        __pyx_empty_tuple, /*PyObject *cellvars,*/
        py_srcfile,   /*PyObject *filename,*/
        py_funcname,  /*PyObject *name,*/
        py_line,
        __pyx_empty_bytes  /*PyObject *lnotab*/
    );
    Py_DECREF(py_srcfile);
    Py_DECREF(py_funcname);
    return py_code;
bad:
    Py_XDECREF(py_srcfile);
    Py_XDECREF(py_funcname);
    return NULL;
}
static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename) {
    PyCodeObject *py_code = 0;
    PyFrameObject *py_frame = 0;
    py_code = __pyx_find_code_object(c_line ? c_line : py_line);
    if (!py_code) {
        py_code = __Pyx_CreateCodeObjectForTraceback(
            funcname, c_line, py_line, filename);
        if (!py_code) goto bad;
        __pyx_insert_code_object(c_line ? c_line : py_line, py_code);
    }
    py_frame = PyFrame_New(
        PyThreadState_GET(), /*PyThreadState *tstate,*/
        py_code,             /*PyCodeObject *code,*/
        __pyx_d,      /*PyObject *globals,*/
        0                    /*PyObject *locals*/
    );
    if (!py_frame) goto bad;
    py_frame->f_lineno = py_line;
    PyTraceBack_Here(py_frame);
bad:
    Py_XDECREF(py_code);
    Py_XDECREF(py_frame);
}

static PyObject* __Pyx_Method_ClassMethod(PyObject *method) {
#if CYTHON_COMPILING_IN_PYPY
    if (PyObject_TypeCheck(method, &PyWrapperDescr_Type)) {
        return PyClassMethod_New(method);
    }
#else
    static PyTypeObject *methoddescr_type = NULL;
    if (methoddescr_type == NULL) {
       PyObject *meth = PyObject_GetAttrString((PyObject*)&PyList_Type, "append");
       if (!meth) return NULL;
       methoddescr_type = Py_TYPE(meth);
       Py_DECREF(meth);
    }
    if (PyObject_TypeCheck(method, methoddescr_type)) {
        PyMethodDescrObject *descr = (PyMethodDescrObject *)method;
        #if PY_VERSION_HEX < 0x03020000
        PyTypeObject *d_type = descr->d_type;
        #else
        PyTypeObject *d_type = descr->d_common.d_type;
        #endif
        return PyDescr_NewClassMethod(d_type, descr->d_method);
    }
#endif
    else if (PyMethod_Check(method)) {
        return PyClassMethod_New(PyMethod_GET_FUNCTION(method));
    }
    else if (PyCFunction_Check(method)) {
        return PyClassMethod_New(method);
    }
#ifdef __Pyx_CyFunction_USED
    else if (PyObject_TypeCheck(method, __pyx_CyFunctionType)) {
        return PyClassMethod_New(method);
    }
#endif
    PyErr_SetString(PyExc_TypeError,
                   "Class-level classmethod() can only be called on "
                   "a method_descriptor or instance method.");
    return NULL;
}

static CYTHON_INLINE PyObject* __Pyx_PyInt_From_long(long value) {
    const long neg_one = (long) -1, const_zero = (long) 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(long) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(long) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
        } else if (sizeof(long) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
        }
    } else {
        if (sizeof(long) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(long) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(long),
                                     little, !is_unsigned);
    }
}

#define __PYX_VERIFY_RETURN_INT(target_type, func_type, func_value)\
    __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, 0)
#define __PYX_VERIFY_RETURN_INT_EXC(target_type, func_type, func_value)\
    __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, 1)
#define __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, exc)\
    {\
        func_type value = func_value;\
        if (sizeof(target_type) < sizeof(func_type)) {\
            if (unlikely(value != (func_type) (target_type) value)) {\
                func_type zero = 0;\
                if (exc && unlikely(value == (func_type)-1 && PyErr_Occurred()))\
                    return (target_type) -1;\
                if (is_unsigned && unlikely(value < zero))\
                    goto raise_neg_overflow;\
                else\
                    goto raise_overflow;\
            }\
        }\
        return (target_type) value;\
    }

#if CYTHON_USE_PYLONG_INTERNALS
  #include "longintrepr.h"
#endif

static CYTHON_INLINE long __Pyx_PyInt_As_long(PyObject *x) {
    const long neg_one = (long) -1, const_zero = (long) 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(long) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(long, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (long) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (long) 0;
                case  1: __PYX_VERIFY_RETURN_INT(long, digit, digits[0])
                case 2:
                    if (8 * sizeof(long) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 2 * PyLong_SHIFT) {
                            return (long) (((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(long) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 3 * PyLong_SHIFT) {
                            return (long) (((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(long) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 4 * PyLong_SHIFT) {
                            return (long) (((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (long) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(long) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, unsigned long, PyLong_AsUnsignedLong(x))
            } else if (sizeof(long) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (long) 0;
                case -1: __PYX_VERIFY_RETURN_INT(long, sdigit, -(sdigit) digits[0])
                case  1: __PYX_VERIFY_RETURN_INT(long,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(long) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(long) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                            return (long) ((((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(long) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                            return (long) ((((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(long) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                            return (long) ((((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(long) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, long, PyLong_AsLong(x))
            } else if (sizeof(long) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, PY_LONG_LONG, PyLong_AsLongLong(x))
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            long val;
            PyObject *v = __Pyx_PyNumber_Int(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (long) -1;
        }
    } else {
        long val;
        PyObject *tmp = __Pyx_PyNumber_Int(x);
        if (!tmp) return (long) -1;
        val = __Pyx_PyInt_As_long(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to long");
    return (long) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to long");
    return (long) -1;
}

static CYTHON_INLINE int __Pyx_PyInt_As_int(PyObject *x) {
    const int neg_one = (int) -1, const_zero = (int) 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(int) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(int, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (int) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (int) 0;
                case  1: __PYX_VERIFY_RETURN_INT(int, digit, digits[0])
                case 2:
                    if (8 * sizeof(int) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 2 * PyLong_SHIFT) {
                            return (int) (((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(int) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 3 * PyLong_SHIFT) {
                            return (int) (((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(int) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 4 * PyLong_SHIFT) {
                            return (int) (((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (int) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(int) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, unsigned long, PyLong_AsUnsignedLong(x))
            } else if (sizeof(int) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (int) 0;
                case -1: __PYX_VERIFY_RETURN_INT(int, sdigit, -(sdigit) digits[0])
                case  1: __PYX_VERIFY_RETURN_INT(int,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(int) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(int) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                            return (int) ((((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(int) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                            return (int) ((((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 4 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(int) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 4 * PyLong_SHIFT) {
                            return (int) ((((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(int) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, long, PyLong_AsLong(x))
            } else if (sizeof(int) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, PY_LONG_LONG, PyLong_AsLongLong(x))
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            int val;
            PyObject *v = __Pyx_PyNumber_Int(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (int) -1;
        }
    } else {
        int val;
        PyObject *tmp = __Pyx_PyNumber_Int(x);
        if (!tmp) return (int) -1;
        val = __Pyx_PyInt_As_int(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to int");
    return (int) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to int");
    return (int) -1;
}

static int __Pyx_check_binary_version(void) {
    char ctversion[4], rtversion[4];
    PyOS_snprintf(ctversion, 4, "%d.%d", PY_MAJOR_VERSION, PY_MINOR_VERSION);
    PyOS_snprintf(rtversion, 4, "%s", Py_GetVersion());
    if (ctversion[0] != rtversion[0] || ctversion[2] != rtversion[2]) {
        char message[200];
        PyOS_snprintf(message, sizeof(message),
                      "compiletime version %s of module '%.100s' "
                      "does not match runtime version %s",
                      ctversion, __Pyx_MODULE_NAME, rtversion);
        return PyErr_WarnEx(NULL, message, 1);
    }
    return 0;
}

#ifndef __PYX_HAVE_RT_ImportModule
#define __PYX_HAVE_RT_ImportModule
static PyObject *__Pyx_ImportModule(const char *name) {
    PyObject *py_name = 0;
    PyObject *py_module = 0;
    py_name = __Pyx_PyIdentifier_FromString(name);
    if (!py_name)
        goto bad;
    py_module = PyImport_Import(py_name);
    Py_DECREF(py_name);
    return py_module;
bad:
    Py_XDECREF(py_name);
    return 0;
}
#endif

#ifndef __PYX_HAVE_RT_ImportType
#define __PYX_HAVE_RT_ImportType
static PyTypeObject *__Pyx_ImportType(const char *module_name, const char *class_name,
    size_t size, int strict)
{
    PyObject *py_module = 0;
    PyObject *result = 0;
    PyObject *py_name = 0;
    char warning[200];
    Py_ssize_t basicsize;
#ifdef Py_LIMITED_API
    PyObject *py_basicsize;
#endif
    py_module = __Pyx_ImportModule(module_name);
    if (!py_module)
        goto bad;
    py_name = __Pyx_PyIdentifier_FromString(class_name);
    if (!py_name)
        goto bad;
    result = PyObject_GetAttr(py_module, py_name);
    Py_DECREF(py_name);
    py_name = 0;
    Py_DECREF(py_module);
    py_module = 0;
    if (!result)
        goto bad;
    if (!PyType_Check(result)) {
        PyErr_Format(PyExc_TypeError,
            "%.200s.%.200s is not a type object",
            module_name, class_name);
        goto bad;
    }
#ifndef Py_LIMITED_API
    basicsize = ((PyTypeObject *)result)->tp_basicsize;
#else
    py_basicsize = PyObject_GetAttrString(result, "__basicsize__");
    if (!py_basicsize)
        goto bad;
    basicsize = PyLong_AsSsize_t(py_basicsize);
    Py_DECREF(py_basicsize);
    py_basicsize = 0;
    if (basicsize == (Py_ssize_t)-1 && PyErr_Occurred())
        goto bad;
#endif
    if (!strict && (size_t)basicsize > size) {
        PyOS_snprintf(warning, sizeof(warning),
            "%s.%s size changed, may indicate binary incompatibility",
            module_name, class_name);
        if (PyErr_WarnEx(NULL, warning, 0) < 0) goto bad;
    }
    else if ((size_t)basicsize != size) {
        PyErr_Format(PyExc_ValueError,
            "%.200s.%.200s has the wrong size, try recompiling",
            module_name, class_name);
        goto bad;
    }
    return (PyTypeObject *)result;
bad:
    Py_XDECREF(py_module);
    Py_XDECREF(result);
    return NULL;
}
#endif

#ifndef __PYX_HAVE_RT_ImportFunction
#define __PYX_HAVE_RT_ImportFunction
static int __Pyx_ImportFunction(PyObject *module, const char *funcname, void (**f)(void), const char *sig) {
    PyObject *d = 0;
    PyObject *cobj = 0;
    union {
        void (*fp)(void);
        void *p;
    } tmp;
    d = PyObject_GetAttrString(module, (char *)"__pyx_capi__");
    if (!d)
        goto bad;
    cobj = PyDict_GetItemString(d, funcname);
    if (!cobj) {
        PyErr_Format(PyExc_ImportError,
            "%.200s does not export expected C function %.200s",
                PyModule_GetName(module), funcname);
        goto bad;
    }
#if PY_VERSION_HEX >= 0x02070000
    if (!PyCapsule_IsValid(cobj, sig)) {
        PyErr_Format(PyExc_TypeError,
            "C function %.200s.%.200s has wrong signature (expected %.500s, got %.500s)",
             PyModule_GetName(module), funcname, sig, PyCapsule_GetName(cobj));
        goto bad;
    }
    tmp.p = PyCapsule_GetPointer(cobj, sig);
#else
    {const char *desc, *s1, *s2;
    desc = (const char *)PyCObject_GetDesc(cobj);
    if (!desc)
        goto bad;
    s1 = desc; s2 = sig;
    while (*s1 != '\0' && *s1 == *s2) { s1++; s2++; }
    if (*s1 != *s2) {
        PyErr_Format(PyExc_TypeError,
            "C function %.200s.%.200s has wrong signature (expected %.500s, got %.500s)",
             PyModule_GetName(module), funcname, sig, desc);
        goto bad;
    }
    tmp.p = PyCObject_AsVoidPtr(cobj);}
#endif
    *f = tmp.fp;
    if (!(*f))
        goto bad;
    Py_DECREF(d);
    return 0;
bad:
    Py_XDECREF(d);
    return -1;
}
#endif

static int __Pyx_InitStrings(__Pyx_StringTabEntry *t) {
    while (t->p) {
        #if PY_MAJOR_VERSION < 3
        if (t->is_unicode) {
            *t->p = PyUnicode_DecodeUTF8(t->s, t->n - 1, NULL);
        } else if (t->intern) {
            *t->p = PyString_InternFromString(t->s);
        } else {
            *t->p = PyString_FromStringAndSize(t->s, t->n - 1);
        }
        #else
        if (t->is_unicode | t->is_str) {
            if (t->intern) {
                *t->p = PyUnicode_InternFromString(t->s);
            } else if (t->encoding) {
                *t->p = PyUnicode_Decode(t->s, t->n - 1, t->encoding, NULL);
            } else {
                *t->p = PyUnicode_FromStringAndSize(t->s, t->n - 1);
            }
        } else {
            *t->p = PyBytes_FromStringAndSize(t->s, t->n - 1);
        }
        #endif
        if (!*t->p)
            return -1;
        ++t;
    }
    return 0;
}

static CYTHON_INLINE PyObject* __Pyx_PyUnicode_FromString(const char* c_str) {
    return __Pyx_PyUnicode_FromStringAndSize(c_str, (Py_ssize_t)strlen(c_str));
}
static CYTHON_INLINE char* __Pyx_PyObject_AsString(PyObject* o) {
    Py_ssize_t ignore;
    return __Pyx_PyObject_AsStringAndSize(o, &ignore);
}
static CYTHON_INLINE char* __Pyx_PyObject_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
#if CYTHON_COMPILING_IN_CPYTHON && (__PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT)
    if (
#if PY_MAJOR_VERSION < 3 && __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
            __Pyx_sys_getdefaultencoding_not_ascii &&
#endif
            PyUnicode_Check(o)) {
#if PY_VERSION_HEX < 0x03030000
        char* defenc_c;
        PyObject* defenc = _PyUnicode_AsDefaultEncodedString(o, NULL);
        if (!defenc) return NULL;
        defenc_c = PyBytes_AS_STRING(defenc);
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
        {
            char* end = defenc_c + PyBytes_GET_SIZE(defenc);
            char* c;
            for (c = defenc_c; c < end; c++) {
                if ((unsigned char) (*c) >= 128) {
                    PyUnicode_AsASCIIString(o);
                    return NULL;
                }
            }
        }
#endif
        *length = PyBytes_GET_SIZE(defenc);
        return defenc_c;
#else
        if (__Pyx_PyUnicode_READY(o) == -1) return NULL;
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
        if (PyUnicode_IS_ASCII(o)) {
            *length = PyUnicode_GET_LENGTH(o);
            return PyUnicode_AsUTF8(o);
        } else {
            PyUnicode_AsASCIIString(o);
            return NULL;
        }
#else
        return PyUnicode_AsUTF8AndSize(o, length);
#endif
#endif
    } else
#endif
#if (!CYTHON_COMPILING_IN_PYPY) || (defined(PyByteArray_AS_STRING) && defined(PyByteArray_GET_SIZE))
    if (PyByteArray_Check(o)) {
        *length = PyByteArray_GET_SIZE(o);
        return PyByteArray_AS_STRING(o);
    } else
#endif
    {
        char* result;
        int r = PyBytes_AsStringAndSize(o, &result, length);
        if (unlikely(r < 0)) {
            return NULL;
        } else {
            return result;
        }
    }
}
static CYTHON_INLINE int __Pyx_PyObject_IsTrue(PyObject* x) {
   int is_true = x == Py_True;
   if (is_true | (x == Py_False) | (x == Py_None)) return is_true;
   else return PyObject_IsTrue(x);
}
static CYTHON_INLINE PyObject* __Pyx_PyNumber_Int(PyObject* x) {
  PyNumberMethods *m;
  const char *name = NULL;
  PyObject *res = NULL;
#if PY_MAJOR_VERSION < 3
  if (PyInt_Check(x) || PyLong_Check(x))
#else
  if (PyLong_Check(x))
#endif
    return __Pyx_NewRef(x);
  m = Py_TYPE(x)->tp_as_number;
#if PY_MAJOR_VERSION < 3
  if (m && m->nb_int) {
    name = "int";
    res = PyNumber_Int(x);
  }
  else if (m && m->nb_long) {
    name = "long";
    res = PyNumber_Long(x);
  }
#else
  if (m && m->nb_int) {
    name = "int";
    res = PyNumber_Long(x);
  }
#endif
  if (res) {
#if PY_MAJOR_VERSION < 3
    if (!PyInt_Check(res) && !PyLong_Check(res)) {
#else
    if (!PyLong_Check(res)) {
#endif
      PyErr_Format(PyExc_TypeError,
                   "__%.4s__ returned non-%.4s (type %.200s)",
                   name, name, Py_TYPE(res)->tp_name);
      Py_DECREF(res);
      return NULL;
    }
  }
  else if (!PyErr_Occurred()) {
    PyErr_SetString(PyExc_TypeError,
                    "an integer is required");
  }
  return res;
}
static CYTHON_INLINE Py_ssize_t __Pyx_PyIndex_AsSsize_t(PyObject* b) {
  Py_ssize_t ival;
  PyObject *x;
#if PY_MAJOR_VERSION < 3
  if (likely(PyInt_CheckExact(b))) {
    if (sizeof(Py_ssize_t) >= sizeof(long))
        return PyInt_AS_LONG(b);
    else
        return PyInt_AsSsize_t(x);
  }
#endif
  if (likely(PyLong_CheckExact(b))) {
    #if CYTHON_USE_PYLONG_INTERNALS
    const digit* digits = ((PyLongObject*)b)->ob_digit;
    const Py_ssize_t size = Py_SIZE(b);
    if (likely(__Pyx_sst_abs(size) <= 1)) {
        ival = likely(size) ? digits[0] : 0;
        if (size == -1) ival = -ival;
        return ival;
    } else {
      switch (size) {
         case 2:
           if (8 * sizeof(Py_ssize_t) > 2 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -2:
           if (8 * sizeof(Py_ssize_t) > 2 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case 3:
           if (8 * sizeof(Py_ssize_t) > 3 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -3:
           if (8 * sizeof(Py_ssize_t) > 3 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case 4:
           if (8 * sizeof(Py_ssize_t) > 4 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -4:
           if (8 * sizeof(Py_ssize_t) > 4 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
      }
    }
    #endif
    return PyLong_AsSsize_t(b);
  }
  x = PyNumber_Index(b);
  if (!x) return -1;
  ival = PyInt_AsSsize_t(x);
  Py_DECREF(x);
  return ival;
}
static CYTHON_INLINE PyObject * __Pyx_PyInt_FromSize_t(size_t ival) {
    return PyInt_FromSize_t(ival);
}


#endif /* Py_PYTHON_H */
